<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 034a448fd808 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-18 12:07 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_backup_wizard{

	/** @var string */
	const BACKUP = 'make_backup';

	/** @var string */
	const RECOVER = 'recover_backup';

	/** @var string */
	private $mode;

	/** @var string */
	private $frameset;

	/** @var we_fileupload_ui_base */
	private $fileUploader = null;

	/** @var array */
	private $json = [];

	function __construct(string $mode = self::BACKUP){
		if(we_base_request::int('step') < 2){
			//make sure we reset these vars on opening of this dialog
			we_backup_settings::clearSessionData();
		}
		$this->mode = $mode;
		$this->frameset = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=' . $mode;
		$this->json['mode'] = $mode;
		$this->json['modeCmd'] = $mode;
	}

	private function getJSDep(): string{
		$this->json['import_from'] = we_base_request::string('import_from');

		return we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard', 'data-backup' => setDynamicVar($this->json)]);
	}

	/**
	 *
	 * @return array<int,string>
	 */
	private static function getOptions(): array{
		return array_filter([
			10 => 'handle_' . we_modules_info::CORE,
			11 => we_modules_info::isActive(we_modules_info::OBJECT) ? 'handle_' . we_modules_info::OBJECT : '',
			12 => 'handle_' . we_backup_settings::OPTION_BACKUP_VERSIONS,
			13 => 'handle_' . we_backup_settings::OPTION_BACKUP_VERSION_BIN,
			14 => 'handle_' . we_backup_settings::OPTION_BACKUP_BINARY,
			20 => 'handle_' . we_modules_info::USERS,
			25 => we_modules_info::isActive(we_modules_info::CUSTOMER) ? 'handle_' . we_modules_info::CUSTOMER : '',
			30 => we_modules_info::isActive(we_modules_info::SHOP) ? 'handle_' . we_modules_info::SHOP : '',
			35 => we_modules_info::isActive(we_modules_info::WORKFLOW) ? 'handle_' . we_modules_info::WORKFLOW : '',
			45 => we_modules_info::isActive(we_modules_info::NEWSLETTER) ? 'handle_' . we_modules_info::NEWSLETTER : '',
			55 => we_modules_info::isActive(we_modules_info::SCHEDULER) ? 'handle_' . we_modules_info::SCHEDULER : '',
			60 => we_modules_info::isActive(we_modules_info::EXPORT) ? 'handle_' . we_modules_info::EXPORT : '',
			65 => we_modules_info::isActive(we_modules_info::VOTING) ? 'handle_' . we_modules_info::VOTING : '',
			75 => we_modules_info::isActive(we_modules_info::GLOSSARY) ? 'handle_' . we_modules_info::GLOSSARY : '',
			98 => 'handle_' . we_backup_settings::OPTION_BACKUP_HOOKS,
			99 => 'handle_' . we_backup_settings::OPTION_BACKUP_CUSTOMTAGS,
			100 => 'handle_' . we_backup_settings::OPTION_BACKUP_SETTINGS,
			102 => 'handle_' . we_backup_settings::OPTION_BACKUP_HISTORY,
			300 => 'handle_' . we_backup_settings::OPTION_BACKUP_EXTERN,
			400 => 'handle_' . we_modules_info::SEARCH
		]);
	}

	private function getHTMLFrameset(): string{
		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_' . ($this->mode === self::BACKUP ? 'backup' : 'recover') . '_title]'), '', we_html_element::body([
							'id' => 'weMainBody']
								, we_html_element::iframe('edbody', $this->frameset . '&pnt=edbody', 'position:absolute;top:0;bottom:40px;left:0;right:0;', 'border:0;width:100%;height:100%;') .
								we_html_element::iframe('busy', $this->frameset, 'position:absolute;height:40px;bottom:0;left:0;right:0;overflow: hidden') .
								we_html_element::iframe('cmd', $this->frameset . '&pnt=cmd')
						)
		);
	}

	private function getHTMLStep(int $step): string{
		switch($this->mode){
			default:
			case self::BACKUP:
				switch($step){
					default:
					case 1:
						return $this->getHTMLBackupStep1();
					case 2:
						return $this->getHTMLBackupStep2();
					case 3:
						return $this->getHTMLBackupStep3();
				}
			case self::RECOVER:
				switch($step){
					default:
					case 1:
						return $this->getHTMLRecoverStep1();
					case 2:
						return $this->getHTMLRecoverStep2();
					case 3:
						return $this->getHTMLRecoverStep3();
					case 4:
						return $this->getHTMLRecoverStep4();
				}
		}
	}

	private function getHTMLRecoverStep1(): string{
		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[save_before]'), we_html_tools::TYPE_ALERT, 600)))->noline())
				->add((new we_html_multiIconBoxContentLine('', g_l('backup', '[save_question]')))->noline());


		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
							'data-backup' => setDynamicVar($this->json)]), we_html_element::body(['class' => 'weDialogBody', 'onload' => "startStep(1)"], we_html_element::form([
									'name' => we_base_constants::WE_FORM, 'method' => 'post'], (new we_html_multiIconBox($parts, 'backup_options'))->marginLeft()->headline(g_l('backup', '[step1]'))->get()
								)
						)
		);
	}

	private function getHTMLRecoverStep2(): string{
		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', we_html_forms::radiobutton('import_server', true, 'import_from', g_l('backup', '[import][from_server]'))))->noline())
				->add((new we_html_multiIconBoxContentLine('', we_html_forms::radiobutton('import_upload', false, 'import_from', g_l('backup', '[import][from_local]'))))->noline());

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
							'data-backup' => setDynamicVar($this->json)]), we_html_element::body(['class' => 'weDialogBody', 'onload' => "startStep(2);"], we_html_element::form([
									'name' => we_base_constants::WE_FORM, 'method' => 'post'], we_html_element::hiddens(['pnt' => 'edbody', 'step' => '3']) .
										(new we_html_multiIconBox($parts, 'backup_options'))->marginLeft()->headline(g_l('backup', '[step2]'))->get()
								)
						)
		);
	}

	private function getHTMLRecoverStep3Import_upload(): we_html_multiIconBoxContent{
		$inputTypeFile = $this->fileUploader->getHTML();

		return (new we_html_multiIconBoxContent())
						->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[charset_warning]'), we_html_tools::TYPE_ALERT, 600, false)))->noline())
						//['headline' => '', 'html' => $this->fileUploader->getHtmlAlertBoxes(), 'noline' => true],
						->add((new we_html_multiIconBoxContentLine('', $inputTypeFile))->noline());
	}

	private function getHTMLRecoverStep3Import_server(): we_html_multiIconBoxContent{
		$select = new we_html_select(['name' => 'backup_select', 'size' => 7, 'style' => "width: 700px;"]);
		$files = [];
		//$extra_files = [];
		$dateformat = g_l('date', '[format][default]');
		chdir(WE_BACKUP_PATH . we_backup_util::SAVE_DIR);
		foreach(glob(we_backup_util::FILE_PREFIX . '*.zip') as $filename){
			$filesize = we_base_file::getHumanFileSize(filesize($filename) ?: 0);
			/* if(strpos($entry, 'weBackup_') !== 0){
			  $extra_files[$adddatadir . $entry] = $entry . " $filedate $filesize";
			  continue;
			  } */
			try{
				$info = (new we_backup_importZIP(WE_BACKUP_PATH . we_backup_util::SAVE_DIR . $filename))->getOnlyInfo();
				$files[$filename] = (new DateTime($info->Date))->format($dateformat) . ' - ' . $info->Url . trim(' ' . $info->Comment) . ' (WE: ' . $info->WE . ') ' . $filesize;
			}catch(we_exception_file $e){

			}

			//$extra_files[$filename] = $filename. " $filedate $filesize";
		}

		krsort($files);
		//asort($extra_files);

		/* foreach($files as $fk=>$fv)	$select->addOption($fk,$fv); */

		$default = we_html_select::getNewOptionGroup(['class' => 'bold', 'style' => 'font-style: normal; color: darkblue;', 'label' => g_l('backup', '[we_backups]')]);
		//$other = we_html_select::getNewOptionGroup(['class' => 'bold', 'style' => 'font-style: normal; color: darkblue;', 'label' => g_l('backup', '[other_files]')]);

		foreach($files as $fk => $fv){
			$default->addChild(we_html_select::getNewOption($fk, $fv));
		}
		/* foreach($extra_files as $fk => $fv){
		  if(strlen($fv) > 75){
		  $fv = addslashes(substr($fv, 0, 65) . '...' . substr($fv, -10));
		  }
		  $other->addChild(we_html_select::getNewOption($fk, $fv));
		  } */

		$select->addChild($default);
		//$select->addChild($other);

		$this->json['extra_files'] = [];
		$this->json['extra_files_desc'] = [];
		/* foreach($extra_files as $fk => $fv){
		  $this->json['extra_files'][] = $fk;
		  $this->json['extra_files_desc'][] = $fv;
		  } */

		return (new we_html_multiIconBoxContent())
						//->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[charset_warning]'), we_html_tools::TYPE_ALERT, 600, false)))->noline())
						->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[select_server_file]'), we_html_tools::TYPE_INFO, 700, false)))->noline())
						->add((new we_html_multiIconBoxContentLine('', $select->getHtml()))->noline())
						//array("headline"=>"","html"=>we_html_forms::checkbox('1', false, "show_all", g_l('backup',"[show_all]"), false, "defaultfont", "showAll()"),"space"=>0,"noline"=>1);
						->add((new we_html_multiIconBoxContentLine('', we_html_button::create_button('delete_backup', "javascript:delSelected();", '', '', false, false))));
	}

	private function getHTMLRecoverStep3(): string{
		we_backup_settings::clearSessionData();

		$this->fileUploader = new we_fileupload_ui_base('we_upload_file', [], ['.zip']);
		$this->fileUploader->setGenericFileName(TEMP_DIR . we_fileupload::REPLACE_BY_FILENAME);
		$this->fileUploader->setGenericFileName(WE_BACKUP_DIR . we_backup_util::SAVE_DIR . we_fileupload::REPLACE_BY_FILENAME);
		$this->fileUploader->setShowFilenamesFull(true);

		$maxsize = we_fileupload::getMaxUploadSizeB();
		$importUpload = (we_base_request::string('import_from') === 'import_upload');

		$parts = (new we_html_multiIconBoxContent())
				->append($importUpload ?
						$this->getHTMLRecoverStep3Import_upload() :
						$this->getHTMLRecoverStep3Import_server())
				->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkbox('1', true, 'rebuild', g_l('backup', '[rebuild]'), false))))
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[delold_notice]', false, true), we_html_tools::TYPE_QUESTION, 700, false) .
						we_html_button::create_button('delold', "javascript:delOldFiles();", '', '', false, false)))->space(we_html_multiIconBox::SPACE_SMALL));
		//->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[import][options]'), we_html_tools::TYPE_INFO, 700, false)))->noline()->space(we_html_multiIconBox::SPACE_SMALL));

		$form_properties = self::getOptions();
		ksort($form_properties, SORT_NUMERIC);
		$this->json['form_properties'] = $form_properties;
		$checked = false;
		foreach($form_properties as $k => $v){
			$v = (string) $v;
			switch($v){
				case 'extern_exp':
				case 'backup_log':
					continue 2;
				case 'handle_' . we_backup_settings::OPTION_BACKUP_EXTERN:
					continue 2;
				//$checked = false;
				default:
					$checked = true;
			}
			//$help = g_l('backup', '[' . str_replace('handle_', '', $v) . '_info]', true);
			$parts->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkbox('1', $checked, $v, g_l('backup', '[' . str_replace('handle_', 'import][', $v) . '_data]'), false, '', "doClick(" . $k . ");")))->space(we_html_multiIconBox::SPACE_SMALL)->noline());
		}

		$parts
				->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkbox('1', false, 'handle_' . we_backup_settings::OPTION_BACKUP_EXTERN, g_l('backup', '[import][extern_data]') . we_html_tools::alertAttentionBox(g_l('backup', '[extern_info]'), we_html_tools::TYPE_HELP, 0, false), false, '', "doClick(300);")))->space(we_html_multiIconBox::SPACE_SMALL)->noline())
				->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkbox('1', true, 'backup_log', g_l('backup', '[export][backup_log]') . we_html_tools::alertAttentionBox(g_l('backup', '[backup_log_exp]'), we_html_tools::TYPE_HELP, 0, false), false, '', "doClick(320);")))->space(we_html_multiIconBox::SPACE_SMALL)->noline())
		;


		$form_attribs = (we_base_request::string('import_from') === 'import_upload' ?
				['name' => we_base_constants::WE_FORM, 'method' => 'post', 'action' => $this->frameset, 'target' => 'cmd', 'enctype' => 'multipart/form-data'] :
				['name' => we_base_constants::WE_FORM, 'method' => 'post', 'action' => $this->frameset, 'target' => 'cmd']
				);

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => "startStep(3);self.focus();"], we_html_element::form($form_attribs, we_html_element::hiddens([
									'pnt' => 'cmd',
									'cmd' => 'import',
									'step' => '3',
									'MAX_FILE_SIZE' => (string) $maxsize]) .
								we_html_element::hidden('operation_mode', 'import') .
								(new we_html_multiIconBox($parts, 'backup_options'))->marginLeft()->fold(6, false, g_l('backup', '[recover_option]'))->headline(g_l('backup', '[step3]'))->get()
						)
		);

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
							'data-backup' => setDynamicVar($this->json)]) . ($importUpload ? $this->fileUploader->getCss() . $this->fileUploader->getJs() : '') .
						$this->getJSDep(), $body);
	}

	private function getHTMLRecoverStep4(): string{
		we_backup_settings::clearSessionData();

		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[finished_success]'), we_html_tools::TYPE_INFO, 600)))->noline())
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[old_backups_warning]'), we_html_tools::TYPE_ALERT, 600, false)))->noline());

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
							'data-backup' => setDynamicVar($this->json)]), we_html_element::body(['class' => 'weDialogBody', 'onload' => "stopBusy()"], we_html_element::form([
									'name' => we_base_constants::WE_FORM, 'method' => 'post', 'enctype' => 'multipart/form-data'],
										(new we_html_multiIconBox($parts, 'backup_options'))->marginLeft()->headline(g_l('backup', '[step3]'))->get()
								)
						)
		);
	}

	private function getHTMLBackupStep1(): string{
		we_backup_settings::clearSessionData();

		$form_properties = [
			1 => 'export_server',
			2 => 'export_send',
			320 => 'backup_log',
				] + self::getOptions();

		ksort($form_properties, SORT_NUMERIC);

		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('backup', '[filename]') . we_html_tools::alertAttentionBox(g_l('backup', '[filename_info]', false, true), we_html_tools::TYPE_HELP, 0, false), we_backup_util::FILE_PREFIX . we_html_tools::textInput('filename', str_replace('.', '-', $_SERVER['SERVER_NAME']) . '_' . date('Y_m_d__H_i', time()) . '_' . str_replace('.', '-', WE_VERSION), 0, [
										], 'text', '40em')))->rightHead()->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('backup', '[file][comment]'), we_html_tools::textInput('backup_comment', '', 0, [], 'text', '40em')))->rightHead()->noline())
				/* ->add($compression ?
				  (new we_html_multiIconBoxContentLine('', we_html_forms::checkbox(we_backup_util::COMPRESSION, true, "compress", g_l('backup', '[compress]'), false, '', '', false, g_l('backup', '[ftp_hint]'), we_html_tools::TYPE_ALERT, 600))) :
				  null)
				  ->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[protect_txt]'), we_html_tools::TYPE_INFO, 600, false)))->noline())
				  ->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkbox('1', false, "protect", g_l('backup', '[protect]'), false, '', ''))))
				 */
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[export][location]'), we_html_tools::TYPE_INFO, 600, false)))->noline())
				->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkbox('1', true, 'export_server', g_l('backup', '[export][location_server]'), false, '', "doClick(1)")))->noline())
				->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkbox('1', false, 'export_send', g_l('backup', '[export][location_send]'), false, '', "doClick(2)", (!we_base_permission::hasPerm([
											we_base_perm::EXIM_EXPORT]))))))
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('backup', '[export][options]'), we_html_tools::TYPE_INFO, 600, false)))->noline())
		;

		$this->json['form_properties'] = $form_properties;
		foreach($form_properties as $k => $v){
			$v = (string) $v;
			$checked = true;
			switch($v){
				case 'export_server':
				case 'export_send':
				case 'backup_log':
					continue 2;
				case 'handle_' . we_backup_settings::OPTION_BACKUP_VERSION_BIN:
				case 'handle_' . we_backup_settings::OPTION_BACKUP_EXTERN:
					$checked = false;
					break;
			}
			try{
				$help = g_l('backup', '[' . str_replace('handle_', '', $v) . '_info]', true);
			}catch(we_exception_base $e){
				$help = '';
			}
			$parts->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkbox('1', $checked, $v, g_l('backup', '[' . str_replace('handle_', 'export][', $v) . '_data]'), false, '', "doClick(" . $k . ");") . ($help ? we_html_tools::alertAttentionBox($help, we_html_tools::TYPE_HELP, 0, false) : '')))->noline());
		}

		$parts->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkbox('1', true, 'backup_log', g_l('backup', '[export][backup_log]'), false, '', "doClick(320);") . we_html_tools::alertAttentionBox(g_l('backup', '[backup_log_exp]'), we_html_tools::TYPE_HELP, 0, false)))->noline());

		$js = $this->getJSDep();

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title_export]'), we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
							'data-backup' => setDynamicVar($this->json)]) . $js, we_html_element::body(['class' => 'weDialogBody', 'onload' => "startStep(1)"], we_html_element::form([
									'name' => we_base_constants::WE_FORM, 'method' => 'post', 'onsubmit' => "return false;"], we_html_element::hiddens([
											'pnt' => 'cmd',
											'cmd' => 'export',
											'operation_mode' => 'backup',
											we_backup_settings::OPTION_BACKUP_IMPORTAFTER => (string) we_base_request::bool('do_import_after_backup')
										]) .
										(new we_html_multiIconBox($parts, 'backup_options1'))->marginLeft()->fold(7, false, g_l('backup', '[option]'))->headline(g_l('backup', '[export][step1]'))->get()
								)
						)
		);
	}

	private function getHTMLBackupStep2(): string{
		$settings = we_backup_settings::i();
		'@phan-var we_backup_settings $settings';

		if($settings->option(we_backup_settings::OPTION_BACKUP_EXPORT2SEND)){
			$content = (is_file($settings->backup_file) ?
//Note: we show a link for external download - do we need this?
					we_html_element::div([], self::getDownloadLinkText() . we_html_element::br(2) .
							we_html_element::a([
								'href' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showTemp&' . http_build_query([
									'file' => str_replace(WEBEDITION_PATH, '', $settings->backup_file),
									'binary' => 1
								]),
								'download' => $settings->filename], g_l('backup', '[download_file]'))
					) :
					we_html_element::div([], g_l('backup', '[download_failed]'))
					);
		} else {
			$content = '';
		}

		if(is_file($settings->backup_file)){
			$info = (new we_backup_importZIP())->getOnlyInfo();
			$extra = we_html_element::div([], 'SQL: ' . $info->BackupFiles . we_html_element::br() .
							g_l('backup', '[export][files]') . ': ' . $info->BinFiles . we_html_element::br() .
							g_l('fileselector', '[filesize]') . ': ' . we_base_file::getHumanFileSize(filesize(we_backup_settings::i()->backup_file) ?: 0));
		} else {
			$extra = '';
		}

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title_export]'), we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
							'data-backup' => setDynamicVar($this->json)]), we_html_element::body(['class' => 'weDialogBody', 'onload' => "startStep(3," . (!empty($settings->option(we_backup_settings::OPTION_BACKUP_IMPORTAFTER)) ? 1 : 0) . ");"], we_html_element::form([
									'name' => we_base_constants::WE_FORM, 'method' => 'post'], we_html_tools::dialogLayout(we_html_element::div(['style' => 'padding-bottom:20px;'], g_l('backup', '[finish]')) . $content, g_l('backup', '[export][step2]') . $extra)
								)
						)
		);
	}

	private function getHTMLBackupStep3(): string{
		$ret = '';
		$settings = we_backup_settings::i();
		'@phan-var we_backup_settings $settings';
		if(we_base_request::bool('backupfile')){
			$filename = urldecode(we_base_request::_(we_base_request::RAW, 'backupfile'));

			if(file_exists($filename) && str_contains(strtolower($filename), strtolower(WE_BACKUP_PATH))){ // Does file exist and does it saved in backup dir?
				$size = filesize($filename);

				header('Pragma: public');
				header('Expires: 0');
				header('Cache-control: private, max-age=0, must-revalidate');
				header('Content-Type: application/octet-stream');
				header('Content-Disposition: attachment; filename="' . trim(htmlentities(basename($filename))) . '"');
				header('Content-Description: ' . trim(htmlentities(basename($filename))));
				header('Content-Length: ' . $size);
				if(!readfile($filename)){
					$ret = $this->build_error_message();
				}
			} else {
				$ret = $this->build_error_message();
			}
		} else {
			$ret = $this->build_error_message();
		}

		if(!empty($settings->backup_file) && empty($settings->option(we_backup_settings::OPTION_BACKUP_EXPORT2SERVER)) &&
				is_file($settings->backup_file)){
			we_base_file::insertIntoCleanUp($settings->backup_file, 0);
		}

		we_backup_settings::clearSessionData();
		return $ret;
	}

	private function build_error_message(): string{
		return we_html_tools::getHtmlTop('', '', we_html_element::body(['class'=>"weDialogBody"],we_html_tools::dialogLayout(g_l('backup', '[download_failed]'), g_l('backup', '[export][step2]'))));
	}

	private function getHTMLExtern(): string{
		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), '', we_html_element::body([
							'class' => 'weEditorBody',
							'onload' => "self.focus();"
								], we_html_element::form([
									'name' => we_base_constants::WE_FORM
										], we_html_tools::yesNoCancelDialog(g_l('backup', '[extern_backup_question_' . we_base_request::string('w', 'exp') . ']'), we_html_element::span([
													'class' => "fa-stack fa-lg",
													'style' => "color:#F2F200;"
														], '<i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i>'), true, true, false, "WE().util.dialogTop(window).doClose();", "WE().util.dialogTop(window).opener.top.edbody.clearExtern();WE().util.dialogTop(window).doClose();"))
						)
		);
	}

	private function getHTMLBusy(): string{
		$head = '';

		$table = new we_html_table(['class' => 'default', 'style' => 'width:100%;text-align:right'], 1, 3);

		if(we_base_request::string('operation_mode') === 'busy'){
			$text = we_base_request::string('current_description', g_l('backup', '[working]'));
			$progress = new we_gui_progressBar(we_base_request::float('percent'), 200);
			$progress->addText($text, we_gui_progressBar::TOP, 'current_description');
			$head .= we_gui_progressBar::getJSCode();
			$table->setCol(0, 0, $progress->getHtml('', 'margin-left:15px'), ['style' => 'text-align:left;']);
		}

		$step = we_base_request::int('step');

		switch($this->mode){
			case self::BACKUP:
				switch($step){
					case 1:
						$table->setCol(0, 1, we_html_button::position_yes_no_cancel(we_html_button::create_button('make_backup', "javascript:doExport();"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")));
						break;
					case 2:
						$table->setCol(0, 1, we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"));
						break;
					case 3://recover_backup
						if(we_base_request::bool('do_import_after_backup')){
							$body = we_html_button::create_button(we_html_button::NEXT, "javascript:WE().util.dialogTop(window).busy.recoverBackupAfterMaking();");
						} else if(!empty($_SESSION['weS']['inbackup'])){
							$body = we_html_button::create_button(we_html_button::NEXT, "javascript:WE().util.dialogTop(window).opener.weiter();WE().util.dialogTop(window).doClose();");
							unset($_SESSION['weS']['inbackup']);
						} else {
							$body = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");
						}
						$table->setCol(0, 1, $body);
						break;
					default:
				}
				break;
			case self::RECOVER:
				switch($step){
					case 1:
						$buttons = we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::YES, "javascript:pressYesStep1();"), we_html_button::create_button(we_html_button::NO, "javascript:WE().util.dialogTop(window).edbody.location='" . $this->frameset . "&pnt=edbody&step=2';"), we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"));
						$table->setCol(0, 1, $buttons);
						break;
					case 2:
						$nextbuts = we_html_button::create_button(we_html_button::BACK, "javascript:WE().util.dialogTop(window).edbody.location='" . $this->frameset . "&pnt=edbody&step=1'") .
								we_html_button::create_button(we_html_button::NEXT, "javascript:WE().util.dialogTop(window).edbody.we_submitForm('edbody','" . $this->frameset . "');");

						$buttons = we_html_button::position_yes_no_cancel($nextbuts, '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"));

						$table->setCol(0, 1, $buttons);
						break;
					case 3:
						$startImportCall = 'WE().util.dialogTop(window).edbody.startImport()';
						$cancelCall = 'WE().util.dialogTop(window).doClose();';

						if(we_modules_info::isActive(we_modules_info::WORKFLOW)){
							$nextbut = (we_workflow_utility::getAllWorkflowDocs(FILE_TABLE) || (we_modules_info::isActive(we_modules_info::OBJECT) && we_workflow_utility::getAllWorkflowDocs(OBJECT_FILES_TABLE)) ?
									we_html_button::create_button('restore_backup', "javascript:if(window.confirm('" . strtr(g_l('modules_workflow', '[ask_before_recover]'), ["\n" => '\n']) . "')) " . $startImportCall . ";") :
									we_html_button::create_button('restore_backup', "javascript:" . $startImportCall));
						} else {
							$nextbut = we_html_button::create_button('restore_backup', "javascript:" . $startImportCall);
						}

						$nextprevbuts = we_html_button::create_button(we_html_button::BACK, "javascript:WE().util.dialogTop(window).edbody.location='" . $this->frameset . "&pnt=edbody&step=2';") . $nextbut;
						$buttons = we_html_button::position_yes_no_cancel($nextprevbuts, '', we_html_button::create_button(we_html_button::CANCEL, "javascript:" . $cancelCall));

						$table->setCol(0, 1, $buttons);
						break;
					case 4:
						$table->setCol(0, 1, we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"));
						break;
					case 5:
						$table->setCol(0, 1, we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();"));
						break;
					default:
				}
				break;
		}

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), we_html_element::jsScript(JS_DIR . 'we_backup_busy.js', '', ['id' => 'loadVarBackup_wizard',
							'data-backup' => setDynamicVar($this->json)]) .
						$head, we_html_element::body(['class' => "editfooter"], $table->getHtml())
		);
	}

	private function getHTMLCmd(we_base_jsCmd $jsCmd): void{
		switch(we_base_request::string('operation_mode', '-1')){
			case '-1':
				return;
			case 'rebuild':
				$jsCmd->addCmd('rebuild');
				return;
			case 'deleteall':
				$_SESSION['weS']['backup_delete'] = 1;
				$_SESSION['weS']['delete_files'] = [
					'nok' => [],
					'info' => g_l('backup', '[files_not_deleted]')
				];

				$jsCmd->addCmd('deleteall');
				return;
			case 'deletebackup':
				$bfile = we_base_request::_(we_base_request::FILE, 'bfile');
				if(str_starts_with($bfile, '..')){
					$jsCmd->addMsg(g_l('backup', '[name_notok]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if(!is_writable(WE_BACKUP_PATH . $bfile)){
					$jsCmd->addMsg(g_l('backup', '[error_delete]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if(we_base_file::deleteLocalFile(WE_BACKUP_PATH . $bfile)){
					$jsCmd->addCmd('deletebackup');
				} else {
					$jsCmd->addMsg(g_l('backup', '[error_delete]'), we_base_util::WE_MESSAGE_ERROR);
				}
				return;
			default:
				$jsCmd->addMsg(g_l('backup', '[error]'), we_base_util::WE_MESSAGE_ERROR);
				return;
		}
	}

	static function getDownloadLinkText(): string{
		switch(we_base_browserDetect::inst()->getBrowser()){
			case we_base_browserDetect::SAFARI:
			case we_base_browserDetect::APPLE:
				$out = g_l('backup', '[save_link_as_SAFARI]', false, true);
				break;
			case we_base_browserDetect::IE:
				$out = g_l('backup', '[save_link_as_IE]', false, true);
				break;
			case we_base_browserDetect::FF:
				$out = g_l('backup', '[save_link_as_FF]', false, true);
				break;
			case we_base_browserDetect::OPERA:
			default:
				$out = g_l('backup', '[save_link_as_DEFAULT]', false, true);
		}

		return nl2br(htmlspecialchars(preg_replace('#<br\s*/?\s*>#i', "\n", $out)));
	}

	public static function showLog(): void{
		$parts = new we_html_multiIconBoxContent();
		if(we_base_permission::hasPerm([we_base_perm::EXIM_BACKUPLOG])){
			$parts->add((new we_html_multiIconBoxContentLine(g_l('backup', '[view_log]'),
							(file_exists(WE_BACKUP_PATH . we_backup_util::LOG_FILE) ?
									'<pre>' . file_get_contents(WE_BACKUP_PATH . we_backup_util::LOG_FILE) . '</pre>' :
									'<p>' . g_l('backup', '[view_log_not_found]') . '</p>')))->rightHead()->space(we_html_multiIconBox::SPACE_SMALL)
			);
		} else {
			$parts->add((new we_html_multiIconBoxContentLine(g_l('backup', '[view_log]'), we_html_element::p([], g_l('backup', '[view_log_no_perm]'))))->rightHead()->space(we_html_multiIconBox::SPACE_SMALL));
		}
		$buttons = we_html_button::formatButtons(we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();"));
		echo we_html_tools::getHtmlTop(g_l('backup', '[view_log]'), we_html_element::jsScript(JS_DIR . 'closeEscape.js')
				, we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], we_html_element::div(['id' => 'info'],
								(new we_html_multiIconBox($parts))->marginLeft()->buttons($buttons)->get()
				))
		);
	}

	public static function showBackupFrameset(): void{
		$weBackupWizard = new self(we_backup_wizard::BACKUP);

		switch($what = we_base_request::string('pnt', 'frameset')){
			case 'frameset':
				echo $weBackupWizard->getHTMLFrameset();
				break;
			case 'edbody':
				echo $weBackupWizard->getHTMLStep(we_base_request::int('step', 1));
				break;
			case 'cmd':
				$jsCmd = new we_base_jsCmd();
				$weBackupWizard->getHTMLCmd($jsCmd);
				echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'backup_wizard.js') . $jsCmd->getCmds(), we_html_element::body());
				break;
			case 'busy':
				echo $weBackupWizard->getHTMLBusy();
				break;
			case 'extern':
				echo $weBackupWizard->getHTMLExtern();
				break;
			default:
				t_e(__FILE__ . ' unknown reference: ' . $what);
		}
	}

	public static function showRecoverFrameset(): void{
		$what = we_base_request::string('pnt', 'frameset');
		$step = we_base_request::int('step', 1);
		$weBackupWizard = new self(we_backup_wizard::RECOVER);

		switch($what){
			case 'frameset':
				echo $weBackupWizard->getHTMLFrameset();
				break;
			case 'edbody':
				echo $weBackupWizard->getHTMLStep($step);
				break;
			case 'cmd':
				$jsCmd = new we_base_jsCmd();
				$weBackupWizard->getHTMLCmd($jsCmd);
				if(!$jsCmd->isEmpty()){
					echo we_html_tools::getHtmlTop('webEdition', we_html_element::jsScript(JS_DIR . 'backup_wizard.js') . $jsCmd->getCmds(), we_html_element::body());
					break;
				}
			//FIXME: no break??
			case 'busy':
				echo $weBackupWizard->getHTMLBusy();
				break;
			case 'extern':
				echo $weBackupWizard->getHTMLExtern();
				break;
			default:
				t_e(__FILE__ . ' unknown reference: ' . $what);
		}
	}

	public static function getJSLangConsts(): string{
		//'make_backup' : 'recover_backup'
		return we_base_util::setLangString([
					'make_backup' => [
						'binary_data' => g_l('backup', '[export][binary_data]'),
						'binary_dep' => (g_l('backup', '[export][binary_dep]')),
						'core_data' => g_l('backup', '[export][core_data]'),
						'customer_data' => g_l('backup', '[export][customer_data]'),
						'history_data' => g_l('backup', '[export][history_data]'),
						'newsletter_data' => g_l('backup', '[export][newsletter_data]'),
						'newsletter_dep' => (g_l('backup', '[export][newsletter_dep]')),
						'object_data' => g_l('backup', '[export][object_data]'),
						'schedule_data' => g_l('backup', '[export][schedule_data]'),
						'schedule_dep' => (g_l('backup', '[export][schedule_dep]')),
						'shop_data' => g_l('backup', '[export][shop_data]'),
						'shop_dep' => (g_l('backup', '[export][shop_dep]')),
						'temporary_data' => g_l('backup', '[export][temporary_data]'),
						'temporary_dep' => (g_l('backup', '[export][temporary_dep]')),
						'user_data' => g_l('backup', '[export][user_data]'),
						'versions_binarys_data' => g_l('backup', '[export][versions_binarys_data]'),
						'versions_binarys_dep' => (g_l('backup', '[export][versions_binarys_dep]')),
						'versions_data' => g_l('backup', '[export][versions_data]'),
						'versions_dep' => (g_l('backup', '[export][versions_dep]')),
						'workflow_data' => g_l('backup', '[export][workflow_data]'),
						'workflow_dep' => (g_l('backup', '[export][workflow_dep]')),
					],
					'recover_backup' => [
						'binary_data' => g_l('backup', '[import][binary_data]'),
						'binary_dep' => (g_l('backup', '[import][binary_dep]')),
						'core_data' => g_l('backup', '[import][core_data]'),
						'customer_data' => g_l('backup', '[import][customer_data]'),
						'history_data' => g_l('backup', '[import][history_data]'),
						'newsletter_data' => g_l('backup', '[import][newsletter_data]'),
						'newsletter_dep' => (g_l('backup', '[import][newsletter_dep]')),
						'object_data' => g_l('backup', '[import][object_data]'),
						'schedule_data' => g_l('backup', '[import][schedule_data]'),
						'schedule_dep' => (g_l('backup', '[import][schedule_dep]')),
						'shop_data' => g_l('backup', '[import][shop_data]'),
						'shop_dep' => (g_l('backup', '[import][shop_dep]')),
						'temporary_data' => g_l('backup', '[import][temporary_data]'),
						'temporary_dep' => (g_l('backup', '[import][temporary_dep]')),
						'user_data' => g_l('backup', '[import][user_data]'),
						'versions_binarys_data' => g_l('backup', '[import][versions_binarys_data]'),
						'versions_binarys_dep' => (g_l('backup', '[import][versions_binarys_dep]')),
						'versions_data' => g_l('backup', '[import][versions_data]'),
						'versions_dep' => (g_l('backup', '[import][versions_dep]')),
						'workflow_data' => g_l('backup', '[import][workflow_data]'),
						'workflow_dep' => (g_l('backup', '[import][workflow_dep]')),
					],
					'del_backup_confirm' => g_l('backup', '[del_backup_confirm]'),
					'delold_confirm' => g_l('backup', '[delold_confirm]'),
					'error_timeout' => g_l('backup', '[error_timeout]'),
					'finished' => g_l('backup', '[finished]'),
					'finished_success' => g_l('backup', '[finished_success]'),
					'import_file_found' => str_replace('"', '\'', g_l('backup', '[import][file_found]') . ' \n\n' . g_l('backup', '[import][file_found_question]')),
					'nothing_selected' => g_l('backup', '[nothing_selected]'),
					'nothing_selected_fromlist' => g_l('backup', '[nothing_selected_fromlist]'),
					'recover_backup_unsaved_changes' => g_l('backup', '[recover_backup_unsaved_changes]'),
					'save_not_checked' => g_l('backup', '[save_not_checked]'),
					'unselect_dep2' => g_l('backup', '[unselect_dep2]'),
					'unselect_dep3' => g_l('backup', '[unselect_dep3]'),
		]);
	}

}
