<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 19e3d3aaf587 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-12-15 23:51 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//FIXME: handle to start different session if name doesn't match!

abstract class we_base_session{

	public static function initSession(): void{
		if(!isset($_SESSION)){
			session_name(SESSION_NAME);
			if(version_compare(PHP_VERSION, '7.3.0') >= 0){
				session_set_cookie_params(['path' => WEBEDITION_DIR, 'secure' => we_http_util::isHttps(), 'httponly' => true, 'samesite' => 'strict']); //@phan-suppress-current-line PhanTypeMismatchArgumentInternalProbablyReal
			} else {
				$cookie = session_get_cookie_params();
				//set we-cookie only on for WE-path
				session_set_cookie_params($cookie['lifetime'], WEBEDITION_DIR, $cookie['domain'], we_http_util::isHttps(), true);
			}
			we_base_WE::$session = new we_base_sessionHandler();
		}

		//FIXME: if this is not set, nothing works in WE!!!
		we_base_WE::$transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', md5(uniqID('', true)));

		if(!isset($_SESSION['weS'])){
			$_SESSION['weS'] = [
				'EditPageNr' => we_base_constants::WE_EDITPAGE_CONTENT,
				'user' => ['ID' => '', 'Username' => '', 'workSpace' => '', 'isWeSession' => false],
				'we_data' => [we_base_WE::$transaction => []],
				'hasJava' => false,
			];
		}

		//if this var is not given, no login dialog was shown, we don't need to call the hook
		if(empty($_POST['WE_LOGIN_do']) && empty($_REQUEST['jwt'])){
			return;
		}
		$_SESSION['weS']['hasJava'] = we_base_request::bool('hasJava');
		$DB_WE = DB_WE::$inst;

		if(!empty($_REQUEST['jwt'])){
			$_POST['WE_LOGIN_username'] = $_POST['WE_LOGIN_username'] ?? '';
			$_POST['WE_LOGIN_password'] = $_POST['WE_LOGIN_password'] ?? '';
		}

		$checkUserPassword = true; //may be influenced by hook
		$hook = new we_hook_handler('user_preLogin', [
			'user' => [
				'Username' => &$_POST['WE_LOGIN_username'],
				'password' => &$_POST['WE_LOGIN_password'],
			],
			'checkPassword' => &$checkUserPassword,
			'type' => 'login'
		]);
		$hookRet = $hook->executeHook();

		if(!$hookRet || !(isset($_POST['WE_LOGIN_username']) && isset($_POST['WE_LOGIN_password']))){
			if(!$hookRet){
				we_gui_login::$userLoginDenied = we_users_user::LOGIN_CREDENTIALS_INVALID_HOOK;
				we_gui_login::$userLoginDeniedErr = $hook->getErrorString();
			}
			return;
		}

		$token = we_base_request::string('securityToken');
		if($checkUserPassword && !we_captcha_captcha::check($token, 'token', true)){//@phan-suppress-current-line PhanRedundantCondition
			we_gui_login::$userLoginDenied = we_users_user::LOGIN_DENIED_SECURITY;
			return;
		}

// only if username exists !!
		if(
			!($userdata = getHash('SELECT ID,username,passwd,TOTP,LoginDenied,ParentID,CONCAT_WS(" ",First,Second) AS RealName FROM ' . USER_TABLE . ' WHERE IsFolder=0 AND username="' . $DB_WE->escape((string) $_POST['WE_LOGIN_username']) . '"')) ||
			($checkUserPassword && !we_users_user::comparePasswords($userdata['passwd'], (string) $_POST['WE_LOGIN_password']))
		){
			we_base_sessionHandler::makeNewID(true);
			return;
		}

		if($userdata['LoginDenied']){ // userlogin is denied
			we_gui_login::$userLoginDenied = we_users_user::LOGIN_DENIED;
			we_base_sessionHandler::makeNewID(true);
			return;
		}

		if($userdata['TOTP']){//need to check pin!
			$gen = new PHPGangsta_GoogleAuthenticator();
			$code = (string) ($_POST['WE_LOGIN_TOPT'] ?? '');
			if($gen->verifyCode($userdata['TOTP'], $code, 2) &&
				!we_captcha_captcha::hasTopt($code)
			){

			} else {
				we_gui_login::$userLoginDenied = we_users_user::LOGIN_DENIED_TOTP;
				we_base_sessionHandler::makeNewID(true);
				return;
			}
		}

		if($checkUserPassword && !preg_match('|^\$([^$]{2,4})\$([^$]+)\$(.+)$|', $userdata['passwd'])){ //will cause update on old php-versions every time. since md5 doesn't cost much, ignore this.
			$salted = we_users_user::makeSaltedPassword((string) $_POST['WE_LOGIN_password']);
			// UPDATE Password with SALT
			$DB_WE->query('UPDATE ' . USER_TABLE . ' SET passwd="' . $DB_WE->escape($salted) . '" WHERE IsFolder=0 AND username="' . $DB_WE->escape((string) $_POST['WE_LOGIN_username']) . '" AND ID=' . $userdata['ID']);
		}

		if($checkUserPassword && !preg_match('/' . addcslashes(SECURITY_USER_PASS_REGEX, '/') . '/', (string) $_POST['WE_LOGIN_password'])){
			$_SESSION['WE_USER_PASSWORD_NOT_SUFFICIENT'] = 1;
		}

		//set LastLogin
		$DB_WE->query('UPDATE ' . USER_TABLE . ' SET LastLogin_Date=NOW() WHERE ID=' . $userdata['ID']);

		if(!(isset($_SESSION['user']) && is_array($_SESSION['user']))){
			$_SESSION['user'] = [];
		}
		$_SESSION['user'] = array_merge($_SESSION['user'], [
			'Username' => $userdata['username'],
			'ID' => (int) $userdata['ID'],
			'RealName' => $userdata['RealName'],
			'Group' => (int) $userdata['ParentID'],
			'Aliases' => DB_WE::$inst->getAllFirstq('SELECT ID,ParentID FROM ' . USER_TABLE . ' WHERE Alias=' . $userdata['ID'], true)
		]);

		if($userdata['username'] && $userdata['ID']){
			$_SESSION['prefs'] = we_users_user::readPrefs((int) $userdata['ID'], true);
			$_SESSION['perms'] = we_users_user::getAllPermissions((int) $userdata['ID']);
			we_users_user::setEffectiveWorkspaces((int) $userdata['ID']);
		}

		$_SESSION['user']['isWeSession'] = true;

		we_base_sessionHandler::makeNewID();

		$hook = new we_hook_handler('user_Login', [
			'user' => &$_SESSION['user'],
			'perms' => &$_SESSION['perms'],
			'prefs' => &$_SESSION['prefs'],
			'type' => 'login'
		]);

		$hook->executeHook();
	}

	public static function logout(bool $isIncluded = false): void{
		$db = DB_WE::$inst;
		$db->query('UPDATE ' . LOCK_TABLE . ' SET sessionID=x\'00\',UserID=releaseRequestID WHERE releaseRequestID IS NOT NULL AND releaseRequestID!=UserID AND UserID=' . ((int) $_SESSION['user']['ID']) . ' AND sessionID=x\'' . we_base_sessionHandler::getCurrentHex() . '\'');

		$db->query('DELETE FROM ' . LOCK_TABLE . ' WHERE UserID=' . ((int) $_SESSION['user']['ID']) . ' AND sessionID=x\'' . we_base_sessionHandler::getCurrentHex() . '\'');
//FIXME: table is set to false value, if 2 sessions are open; but this is updated shortly - so ignore it now
//TODO: update to time if still locked files open
		$db->query('UPDATE ' . USER_TABLE . ' SET Ping=NULL WHERE ID=' . ((int) $_SESSION['user']['ID']));

		we_base_file::cleanTempFiles(true);

//	getJSCommand
		/* $path = (isset($_SESSION['weS']['SEEM']['startId']) ? // logout from webEdition opened with tag:linkToSuperEasyEditMode
		  $_SESSION['weS']['SEEM']['startPath'] :
		  WEBEDITION_DIR);
		 */

		we_base_sessionHandler::makeNewID(true);

		if(!$isIncluded){
			echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'logout.js'), we_html_element::body());
		}
	}

	public static function loggingOut(): void{
		self::logout(true);

		if(we_base_request::bool('isopener')){
			header('location: ' . WEBEDITION_DIR . 'index.php');
		}

		echo we_html_tools::getHtmlTop('', '', '
	<body onload="self.setTimeout(self.close, 1000);" style="background-color:#386AAB;color:white">
		' . g_l('global', '[irregular_logout]') . '
	</body>');
	}

}
