<?php
declare(strict_types=1);
/**
 * webEdition CMS
 *
 * $Rev: 6a3c6209a897 $
 * $Author: nilsole <nils.reibold@aldente-it.de> $
 * $Date: 2021-04-03 02:05 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Util Functions
 *
 * all functions in this class are static! Please use it in static form:
 *    we_base_util::function_name();
 *
 *
 * @static
 */
abstract class we_base_util{
	public const MIME_BY_HEAD_THEN_EXTENSION = 0;
	public const MIME_BY_EXTENSION = 1;
	public const MIME_BY_HEAD = 2;
	public const MIME_BY_DATA = 3;
	public const WE_MESSAGE_NOTICE = 1;
	public const WE_MESSAGE_WARNING = 2;
	public const WE_MESSAGE_ERROR = 4;
	public const WE_MESSAGE_INFO = 8;
	public const WE_MESSAGE_FRONTEND = 16;
	public const SECONDS_DAY = 86400;
	public const SECONDS_WEEK = 604800;
	public const SECONDS_YEAR = 31449600;
	public const MBYTE = 1048576;
	private const REG_DISALLOWEDCHARS = [
		'~&szlig;~',
		'~ ~',
		'~-*&(.)dash;-*~',
		'~&(.)uml;~',
		'~&(.)(grave|acute|circ|tilde|ring|cedil|slash|caron);|&(..)(lig);|&#.*;~',
		'~&[^;]+;~',
		'~[^0-9a-zA-Z/._-]~',
		'~--+~',
		'~//+~',
		'~/$~',
	];
	private const REG_DISALLOWEDCHARS_REPLACE = [
		'ss', //ß
		'_', //space
		'-', //~
		'${1}e', //uml
		'${1}${3}', //grave
		'', //;;
		'', //^a-z
		'-', //--
		'/', // //
		'', // dangling slash
	];
	const SEO_FILTER_CHARS = ['?', '#', '[', ']', '@', '$', '&', '\'', '"', '(', ')', '*', '+', ',', ':', ';', '='];
	//FIXME: add more extensions
	//FIXME: change this to $finfo = finfo_open(FILEINFO_MIME_TYPE); 		  $mime = finfo_file($finfo, $filepath);		  finfo_close($finfo);
	//NOTICE: WE contenttypes differ strongly from the following mime types!
	public const MIMETYPES = [
		'ai' => ['application/postscript'],
		'aif' => ['audio/x-aiff'],
		'aifc' => ['audio/x-aiff'],
		'aiff' => ['audio/x-aiff'],
		'avi' => ['video/x-msvideo'],
		'bin' => ['application/macbinary'],
		'bmp' => ['image/bmp'],
		'class' => ['application/octet-stream'],
		'cpt' => ['application/mac-compactpro'],
		'css' => ['text/css'],
		'dcr' => ['application/x-director'],
		'dir' => ['application/x-director'],
		'dll' => ['application/octet-stream'],
		'dms' => ['application/octet-stream'],
		'doc' => ['application/msword'],
		'dvi' => ['application/x-dvi'],
		'dxr' => ['application/x-director'],
		'eml' => ['message/rfc822'],
		'eps' => ['application/postscript'],
		'exe' => ['application/octet-stream'],
		'gif' => ['image/gif'],
		'gtar' => ['application/x-gtar'],
		'hqx' => ['application/mac-binhex40'],
		'htm' => ['text/html'],
		'html' => ['text/html'],
		'jfif' => ['image/jpeg'],
		'jpe' => ['image/jpeg'],
		'jpeg' => ['image/jpeg'],
		'jpg' => ['image/jpeg'],
		'js' => ['application/x-javascript'],
		'lha' => ['application/octet-stream'],
		'log' => ['text/plain'],
		'lzh' => ['application/octet-stream'],
		'm4v' => ['video/mp4'],
		'mid' => ['audio/midi'],
		'midi' => ['audio/midi'],
		'mif' => ['application/vnd.mif'],
		'mov' => ['video/quicktime'],
		'movie' => ['video/x-sgi-movie'],
		'mp2' => ['audio/mpeg'],
		'mp3' => ['audio/mp3'],
		'mp4' => ['video/mp4'],
		'mpe' => ['video/mpeg'],
		'mpeg' => ['video/mpeg'],
		'mpg' => ['video/mpeg'],
		'mpga' => ['audio/mpeg'],
		'oda' => ['application/oda'],
		'ogg' => ['application/ogg', 'video/ogg', 'audio/ogg'],
		'pdf' => ['application/pdf'],
		'php' => ['application/x-httpd-php'], //in WE this is text/html?
		'php3' => ['application/x-httpd-php'],
		'php4' => ['application/x-httpd-php'],
		'phps' => ['application/x-httpd-php-source'],
		'phtml' => ['application/x-httpd-php'],
		'png' => ['image/png'],
		'ppt' => ['application/vnd.ms-powerpoint'],
		'ps' => ['application/postscript'],
		'psd' => ['application/octet-stream'],
		'qt' => ['video/quicktime'],
		'ra' => ['audio/x-realaudio'],
		'ram' => ['audio/x-pn-realaudio'],
		'rm' => ['audio/x-pn-realaudio'],
		'rpm' => ['audio/x-pn-realaudio-plugin'],
		'rtf' => ['text/rtf'],
		'rtx' => ['text/richtext'],
		'rv' => ['video/vnd.rn-realvideo'],
		'sea' => ['application/octet-stream'],
		'shtm' => ['text/html'],
		'shtml' => ['text/html'],
		'sit' => ['application/x-stuffit'],
		'smi' => ['application/smil'],
		'smil' => ['application/smil'],
		'so' => ['application/octet-stream'],
		'svg' => ['image/svg+xml'],
		'svgz' => ['image/svg+xml'],
		'swf' => ['application/x-shockwave-flash'],
		'tar' => ['application/x-tar'],
		'text' => ['text/plain'],
		'tgz' => ['application/x-tar'],
		'tif' => ['image/tiff'],
		'tiff' => ['image/tiff'],
		'txt' => ['text/plain'],
		'wav' => ['audio/wav'],
		'wbxml' => ['application/vnd.wap.wbxml'],
		'webp' => ['image/webp'],
		'wmlc' => ['application/vnd.wap.wmlc'],
		'word' => ['application/msword'],
		'xht' => ['application/xhtml+xml'],
		'xhtml' => ['application/xhtml+xml'],
		'xl' => ['application/excel'],
		'xls' => ['application/vnd.ms-excel'],
		'xml' => ['text/xml'],
		'xsl' => ['text/xml'],
		'zip' => ['application/zip'],
	];

	/**
	 * Formates a number with a country specific format into computer readable format.
	 * Returns the formated number.
	 *
	 *
	 * @param string $number
	 */
	static function std_numberformat(string $number): float{
		if(str_contains($number, 'E')){ //  when number is too big, it is shown with E+xx
			$number = number_format((float) ($number), 2, '.', '');
		}
		$match = [];
		if(preg_match('|([0-9]*\.?[0-9]*),([0-9]*)|', $number, $match)){ // deutsche schreibweise
			return (float) (str_replace('.', '', $match[1]) . '.' . $match[2]);
		}
		if(preg_match('|([0-9]*)\.([0-9]*)|', $number)){ // engl schreibweise
			return (float) ($number);
		}
		return (float) (str_replace([',', '.'], '', $number));
	}

	/**
	 * Returns a formatted string representation for the given float.
	 *
	 * @param float   $number     The float to format
	 * @param string  $format    The number format to use (default:english,
	 *                           available: german, deutsch, french, swiss, english)
	 * @param integer $precision The number of decimal points (default: 2)
	 * @return string
	 */
	static function formatNumber(float $number, string $format = '', int $precision = 2): string{
		switch($format){
			case 'german':
			case 'deutsch':
				return number_format($number, $precision, ',', '.');
			case 'french':
				return number_format($number, $precision, ',', ' ');
			case 'swiss':
				return number_format($number, $precision, '.', "'");
			case 'english':
			default:
				return number_format($number, $precision, '.', '');
		}
	}

	/**
	 * Converts all windows and mac newlines from string to unix newlines
	 * Returns the converted String.
	 *
	 */
	static function cleanNewLine(string $string): string{
		return strtr($string, ["\n\r" => "\n", "\r\n" => "\n", "\r" => "\n"]);
	}

	/**
	 * Removes from string all newlines and converts all <br/> to newlines
	 * Returns the converted String.
	 */
	static function br2nl(string $string): string{
		return preg_replace('|<br ?/?>|i', "\n", strtr($string, ["\n" => '', "\r" => '']));
	}

	public static function rmPhp(string $in): string{
		return strtr($in, ['<?php' => '', '<?=' => '', '<?' => '', '?>' => '']);
	}

	public static function rmHTML(string $val, bool $removePhp = false): string{
		if($removePhp){
			return strip_tags($val, '<br><br/>');
		}
		return strtr(preg_replace(['%<br ?/?>%i', '/<[^><]+>/'], ['###BR###', ''], strtr($val, [
			'<?' => '###?###', '?>' => '###/?###'])), [
			'###BR###' => '<br/>', '###?###' => '<?', '###/?###' => '?>'
		]);
	}

	static function getGlobalPath(): string{
		return (empty(we_base_WE::$mainDoc['doc']->Path) ? '' : we_base_WE::$mainDoc['doc']->Path);
	}

	/**
	 * Converts a given number in a via array specified system.
	 * as default a number is converted in the matching chars 0->^,1->a,2->b, ...
	 * other systems can simply set via the parameter $chars for example ->[0,1]
	 * for bin-system
	 *
	 * @return string
	 * @param int $value
	 * @param string[] $chars
	 * @param string $str
	 */
	public static function number2System(int $value, array $chars = [], string $str = '', int $run = 10): string{

		if(!$chars){ //	in case of error take default-array
			$chars = ['^', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'];
		}
		$base = count($chars);

//	get some information about the numbers:
		$rest = $value % $base;
		$result = (int) (($value - $rest) / $base);

//	1. Deal with the rest
		$str = $chars[$rest] . $str;

//	2. Deal with remaining result
		return ($result > 0 && --$run > 0 ? self::number2System($result, $chars, $str, $run) : $str);
	}

	/**
	 *
	 * @param string $name
	 * @return array<string,DateTime>
	 */
	public static function convertDateInRequest(string $name, array &$r = []): array{
		if($name){
			$req = &$_REQUEST[$name];
		} else if($r){
			$req = $r;
		} else {
			$req = &$_REQUEST;
		}

		if(!is_array($req)){
			return [];
		}
		/* $dates = */$regs = $ret = [];
		foreach($req as $key => $value){
			if(!empty($r) && is_array($value)/* document */){
				$ret[$key] = new DateTime(($value['date'] ?? '1970-01-01') . ' ' . ($value['time'] ?? '00:00:00'));
				unset($req[$key]);
			} elseif(is_array($value) && preg_match('/^we_date_([a-zA-Z0-9_]+)$/', (string) $key, $regs)/* document */){// we do need this for creating new customers
				$ret[$regs[1]] = new DateTime(($value['date'] ?? '1970-01-01') . ' ' . ($value['time'] ?? '00:00:00'));
				unset($req[$key]);
			} elseif(is_array($value) && preg_match('/^we_([a-zA-Z0-9_]+)_(' . we_contents_objectBase::TYPE_DATETIME . '|' . we_contents_objectBase::TYPE_TIME . '|' . we_contents_objectBase::TYPE_DATE . ')_([a-zA-Z0-9_]+)$/', (string) $key, $regs)/* object */){

				$ret[$regs[2] . '_' . $regs[3]] = new DateTime(($value['date'] ?? '1970-01-01') . ' ' . ($value['time'] ?? '00:00:00'));
				unset($req[$key]);
			}
		}
		return $ret;
	}

	/**
	 * get the mime type of a given file
	 * @param string $ext the extension of the file
	 * @param string $filepath path of the file
	 * @param int $method the method how to determine the type
	 * @param bool $handleCompressed if true compressed data type is not returned as "application/compressed", so the real type should be determined
	 * @return string
	 */
	public static function getMimeType(string $ext, string $filepath = '', int $method = self::MIME_BY_HEAD_THEN_EXTENSION, bool $handleCompressed = false): string{
		$isCompressed = ($filepath && $handleCompressed ? we_base_file::isCompressed($filepath) : false);
		switch($filepath ? $method : self::MIME_BY_EXTENSION){
			case self::MIME_BY_DATA:
				if(function_exists('finfo_open') && ($finfo = finfo_open(FILEINFO_MIME_TYPE))){
					$mime = finfo_buffer($finfo, we_base_file::loadPart($filepath, 0, 512, $isCompressed));
					finfo_close($finfo);
					if($mime){
						return $mime;
					}
				}
				break;
			case self::MIME_BY_HEAD:
			case self::MIME_BY_HEAD_THEN_EXTENSION:
				if(function_exists('finfo_open') && ($finfo = finfo_open(FILEINFO_MIME_TYPE))){
					$mime = finfo_buffer($finfo, we_base_file::loadPart($filepath, 0, 8192, $isCompressed));
					finfo_close($finfo);
					if($mime || $method == self::MIME_BY_HEAD){
						return $mime ?: '';
					}
				}
				if(!($handleCompressed && $isCompressed) && function_exists('mime_content_type')){
					$mime = mime_content_type($filepath);
					if($mime || $method == self::MIME_BY_HEAD){
						return $mime ?: '';
					}
				}
				if($method == self::MIME_BY_HEAD){
					return '';
				}
				break;
			case self::MIME_BY_EXTENSION:
				break;
			default:
				//false means: no info about MIME type can be determined
				return '';
		}
		//self::MIME_BY_EXTENSION
		$lext = strtolower($ext);
		$ret = (!array_key_exists($lext, self::MIMETYPES)) ?
			we_mail_mime::TYPE_OCTETSTREAM :
			current(self::MIMETYPES[$lext]);
		assert(is_string($ret));
		return $ret;
	}

	/**
	 *
	 * @param string $mime
	 * @return string[]
	 */
	public static function mime2extensions(string $mime): array{
		$mime = str_replace('/*', '/', trim($mime));
		$extensions = [];
		foreach(self::MIMETYPES as $k => $v){
			foreach($v as $cur){
				if(str_starts_with($cur, $mime)){
					$extensions[] = $k;
				}
			}
		}

		return $extensions;
	}

	/**
	 *
	 * @param string $mimegroup
	 * @return string[]
	 */
	public static function mimegroup2mimes(string $mimegroup): array{
		$mimegroup = str_replace('/*', '/', trim($mimegroup));
		$mimes = [];
		foreach(self::MIMETYPES as $v){
			foreach($v as $cur){
				if(str_starts_with($cur, $mimegroup) && !in_array($cur, $mimes)){
					$mimes[] = $cur;
				}
			}
		}

		return $mimes;
	}

	public static function isExtensionMime(string $ext, string $mime): bool{
		$mime = str_replace('/*', '/', trim($mime));
		$check = self::MIMETYPES[$ext];
		foreach($check as $cur){
			if(str_contains($cur, $mime)){
				return true;
			}
		}
		return false;
	}

	public static function convertUnits(string $string, int $base = 16): string{
		$regs = [];
		if(!preg_match('/(\d+\.?\d*) ?(em|ex|pt|px|in|mm|cm|pc|ch|rem|vw|vh|vmin|vmax|%)?/', $string, $regs)){
			$regs[1] = intval($string);
			$regs[2] = 'px';
		}

		switch($regs[2] ?? 'px'){
			case 'ch':
				$regs[1] *= 1.2;
			case 'ex':
				$regs[1] *= 2;
			case 'rem':
			case 'em':
				return (string) ($regs[1] * $base);
			case 'pt':
				return (string) (round($regs[1] * 96 / 72));
			case 'pc':
				return (string) (round($regs[1] * 96 / 6));
			case 'in':
				return (string) (round($regs[1] * 96));
			case 'mm':
				return (string) (round($regs[1] * 96 / 254));
			case 'cm':
				return (string) (round($regs[1] * 96 / 2.54));
			case '%':
				//don't convert %
				return $string;
			default:
			case 'px':
				return (string) ($regs[1]);
		}
	}

	/**
	 *
	 * @param int|string $in
	 * @return int
	 */
	public static function convertIniSizes(/* mixed */$in): int{
		if(is_int($in)){
			return $in;
		}
		$regs = [];
		if(preg_match('#^([0-9]+)M$#i', $in, $regs)){
			return 1024 * 1024 * intval($regs[1]);
		}
		if(preg_match('#^([0-9]+)K$#i', $in, $regs)){
			return 1024 * intval($regs[1]);
		}
		return intval($in);
	}

	/**
	 * Returns a shortened string representation of a path (e.g. '/path/.../file.php')
	 *
	 * @param string $path  The path to be shortened.
	 * @param integer $len  Length (lower bound), when to start shortening (minimum = 10).
	 * @return string
	 */
	public static function shortenPath(string $path, int $len): string{
		if(strlen($path) <= $len || strlen($path) < 10){
			return $path;
		}
		$l = (int) (($len / 2) - 2);
		return substr($path, 0, $l) . '...' . substr($path, $l * -1);
	}

	/**
	 * Splits up the given path every n-th character and adds a space separator in between
	 *
	 * Example for $len = 10:
	 *   input:  "file(filename)"
	 *   output: "/segment-1 /segment-2 /segment-3 /file"
	 *
	 * @param string $path  The path to be split up.
	 * @param integer $len  Length, when to start the next segment (minimum = 10).
	 * @return string
	 */
	static function shortenPathSpace(string $path, int $len): string{
		if(strlen($path) <= $len || strlen($path) < 10){
			return $path;
		}
		$l = $len;
		return substr($path, 0, $l) . ' ' . self::shortenPathSpace(substr($path, $l) ?: '', $len);
	}

	public static function getPercent(int $total, float $value, int $precision = 0): float{
		return ($total ? round(($value * 100) / $total, $precision) : 0);
	}

	public static function setLangString(array $lang): string{
		return json_encode($lang, JSON_UNESCAPED_UNICODE) ?: '';
	}

	public static function updateAvailable(): array{
		try{
			$versionInfo = we_cache_file::load('newwe_version');
			if($versionInfo && (version_compare($versionInfo['dotted'], WE_VERSION) > 0 /* ||
				  //in branched mode, we compare revisions
				  ( WE_VERSION_BRANCH != "" && intval(WE_SVNREV) < intval($versionInfo['svnrevision'])
				  ) */
				)){
				return $versionInfo;
			}
		} catch (we_exception_cache $e){

		}
		return [];
	}

	public static function convertDisallowedChars(string $text, string $charset = ''): string{
		return preg_replace(self::REG_DISALLOWEDCHARS, self::REG_DISALLOWEDCHARS_REPLACE, htmlentities(strip_tags($text), ENT_COMPAT, ($charset ?: 'UTF-8')));
	}

	/**
	 *
	 * @param string $str
	 * @return array{string,string[]}
	 */
	public static function getParamStr(string $str): array{
		$param = [];
		$anchor = '';
		if($str){
			$regs = [];
			if(preg_match('|(#[^?&]*)|', $str, $regs)){
				$anchor = $regs[1];
				$str = strtr($str, [$anchor => '']);
			}
			parse_str(preg_replace('|^\?|', '', $str), $param);
		}
		return [$anchor, $param];
	}

	public static function isPHPSerialize(string $string): bool{
		return !empty($string) && preg_match('/^[asO]:\d+:|^b:[01];/', $string);
	}

	public static function isJSONSerialize(string $string): bool{
		return !empty($string) && in_array(substr($string, 0, 1), ['[', '{']) && in_array(substr($string, -1), [']', '}']); //regex does not work??!
	}

	public static function isIgBinarySerialize(string $string): bool{
		static $ser = '';
		$ser = $ser ?: (function_exists('igbinary_serialize') ? (string) substr((string) igbinary_serialize(null), 0, 4) : '______');
		return function_exists('igbinary_unserialize') && substr($string, 0, 4) === $ser;
	}

	public static function isSerialized(string $string): bool{
		return !empty($string) && (self::isJSONSerialize($string) || self::isPHPSerialize($string) || self::isIgBinarySerialize($string));
	}

	public static function getMaxLenCompare(string $str1, string $str2): int{
		$max = min(strlen($str1), strlen($str2));
		for($len = 0; $len < $max; $len++){
			if($str1[$len] !== $str2[$len]){
				return $len;
			}
		}
		return $max;
	}

	private static function getDomainFromMail(string $email): string{
		$out = [];
		if(preg_match_all('/[[:space:]\<_\.0-9A-Za-z-]+@([0-9a-zA-Z][0-9a-zA-Z-\.]+)(\>)?/', $email, $out, PREG_PATTERN_ORDER)){
			return $out[1][0];
		}
		return '';
	}

	/**
	 * check domain
	 */
	public static function we_checkMailDomain(string $email, string &$domain): bool{
		$domain = self::getDomainFromMail($email);
		if(!$domain){
			return false;
		}
		$mxhosts = [];
		return (getmxrr(trim($domain), $mxhosts));
	}

	/**
	 *
	 * @param int[] $arr
	 * @param string $table
	 * @param bool $isFolder
	 * @return int[]
	 */
	public static function pushChildsFromArr(array $arr, string $table = FILE_TABLE, bool $isFolder = false): array{
		$tmpArr = [];
		foreach($arr as $id){
			self::pushChilds($tmpArr, $id, $table, $isFolder);
		}
		return array_unique($tmpArr);
	}

	/**
	 *
	 * @param int[] $arr
	 * @param int $id
	 * @param string $table
	 * @param bool $isFolder
	 * @return void
	 */
	public static function pushChilds(array &$arr, int $id, string $table = FILE_TABLE, bool $isFolder = false): void{
		$arr[] = $id;
		$all = DB_WE::$inst->getAllq('SELECT ID FROM ' . DB_WE::$inst->escape($table) . ' WHERE ParentID=' . $id . ($isFolder ? ' AND IsFolder=1' : ''), true);
		foreach($all as $id){
			self::pushChilds($arr, (int) $id, $table, $isFolder);
		}
	}

	public static function readParents(int $id, array &$parentlist, string $tab, string $match = 'ContentType', array $matchvalue = we_base_ContentTypes::FOLDER_TYPES): void{
		if(($pid = fI('SELECT ParentID FROM ' . DB_WE::$inst->escape($tab) . ' WHERE ID=' . $id, -1)) !== -1){
			if($pid === 0){
				$parentlist[] = $pid;
			} elseif(fB('SELECT 1 FROM ' . DB_WE::$inst->escape($tab) . ' WHERE ID=' . $pid . ' AND ' . DB_WE::$inst->escape($match) . ' IN("' . implode('","', $matchvalue) . '") LIMIT 1')){
				$parentlist[] = $pid;
				self::readParents($pid, $parentlist, $tab, $match, $matchvalue);
			}
		}
	}

	/**
	 *
	 * @param int[] $pid
	 * @param int[] $childlist
	 * @param string $tab
	 * @param bool $folderOnly
	 * @param string $where
	 * @param string $match
	 * @param string[] $matchvalue
	 * @return void
	 */
	public static function readChilds(array $pid, array &$childlist, string $tab, bool $folderOnly = true, string $where = '', string $match = 'ContentType', array $matchvalue = we_base_ContentTypes::FOLDER_TYPES): void{
		if(empty($pid)){
			return;
		}
		$data = DB_WE::$inst->getAllFirstq('SELECT
 ID,' . DB_WE::$inst->escape($match) . '
 FROM ' . DB_WE::$inst->escape($tab) . '
WHERE ' . ($folderOnly ? ' IsFolder=1 AND ' : '') . 'ParentID IN (' . implode(',', $pid) . ') ' . $where, false);
		$todo = [];
		foreach($data as $id => $val){
			'@phan-var int $id';
			if(in_array($val, $matchvalue)){
				$todo[] = $id;
			}
			$childlist[] = $id;
		}
		if($todo){
			self::readChilds($todo, $childlist, $tab, $folderOnly, $where, $match, $matchvalue);
		}
	}

	/**
	 *
	 * @param string $table
	 * @param int $id
	 * @return int[]
	 */
	public static function getParentIDs(string $table, int $id): array{
		$ret = [];
		while(($pid = fI('SELECT ParentID FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id)) > 0){
			$id = $pid; // #5836
			$ret[] = $id;
		}
		return $ret;
	}

}
