<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 32d4b6906932 $
 * $Author: Lukas Imhof $
 * $Date: 2021-01-25 15:40 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_contents_collection extends we_contents_root{
	/** @var string */
	public $remTable = 'tblFile'; //TODO: set getters for all public props

	/** @var string[] */
	public $remCT = [];

	/** @var int */
	public $remClass = 0;

	/** @var string */
	public $remField = '';

	/** @var int */
	public $DefaultDir = IMAGESTARTID_DEFAULT;

	/** @var string */
	protected $DefaultPath = '';

	/** @var bool */
	public $IsDuplicates = false;

	/** @var bool */
	public $InsertRecursive = false;

	/** @var int[] */
	protected $fileCollection = [];

	/** @var int[] */
	protected $objectCollection = [];

	/** @var string */
	protected $view = 'grid';

	/** @var string */
	protected $viewSub = 'minimal';

	/** @var DateTime */
	public $Publish_Date = null;

	private const GRID_ITEM_DIMENSIONS = [
		2 => ['item' => 400, 'icon' => 56, 'smallicon' => 32, 'font' => 30, 'btnFontsize' => 30, 'btnHeight' => 60],
		3 => ['item' => 264, 'icon' => 42, 'smallicon' => 24, 'font' => 24, 'btnFontsize' => 22, 'btnHeight' => 44],
		4 => ['item' => 200, 'icon' => 32, 'smallicon' => 16, 'font' => 18, 'btnFontsize' => 16, 'btnHeight' => 31],
		5 => ['item' => 159, 'icon' => 24, 'smallicon' => 12, 'font' => 14, 'btnFontsize' => 13, 'btnHeight' => 25],
		6 => ['item' => 134, 'icon' => 20, 'smallicon' => 12, 'font' => 11, 'btnFontsize' => 13, 'btnHeight' => 25]
	];

	/** @var int */
	protected $itemsPerRow = 4;

	/** @var string database table in which the object is stored */
	public $Table = VFILE_TABLE;

	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::COLLECTION;

	/** Constructor
	 * @desc Constructor
	 */
	function __construct(){
		parent::__construct();
		$this->Mod_Date = $this->Mod_Date ?: $this->Creation_Date;
		$this->Publish_Date = $this->Mod_Date;
	}

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'Creation_Date' => we_base_request::DATE_TIME,
			'Mod_Date' => we_base_request::DATE_TIME,
			'fileCollection' => we_base_request::INTLISTA,
			'objectCollection' => we_base_request::INTLISTA,
			'remTable' => we_base_request::STRING,
			'remCT' => we_base_request::STRING_LIST,
			'remClass' => we_base_request::INT,
			'remField' => we_base_request::STRING,
			'DefaultDir' => we_base_request::INT,
			//'insertPrefs',
			'IsDuplicates' => we_base_request::BOOL,
			'InsertRecursive' => we_base_request::BOOL,
			'ContentType' => we_base_request::STRING,
			'view' => we_base_request::STRING,
			'viewSub' => we_base_request::STRING,
			'itemsPerRow' => we_base_request::INT,
		]);
		if(isWE()){
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_CONTENT, we_base_constants::WE_EDITPAGE_INFO]);
			if(we_modules_info::isActive(we_modules_info::CUSTOMER) && (we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER, we_base_perm::CUSTOMER_CAN_CHANGE_DOCS]))){
				$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_WEBUSER;
			}
		}
	}

	private function getRemTable(): string{
		return $this->remTable;
	}

	/**
	 *
	 * @return string[]
	 */
	private function getRemCT(): array{
		//FIXME: do not alloud to write ',' to db!
		return !$this->remCT ? we_base_ContentTypes::getContentTypes(FILE_TABLE, true) : $this->remCT;
	}

	/**
	 *
	 * @return string[]
	 */
	private function getRealRemCT(): array{
		$remCT = !$this->remCT ? we_base_ContentTypes::getContentTypes(FILE_TABLE, true) : $this->remCT;
		foreach($remCT as $ct){
			$remCT = array_merge($remCT, we_base_ContentTypes::getRealContentTypes((string) $ct));
		}

		return $remCT;
	}

	/**
	 * @return int[]
	 */
	public function getRemClass(): int{
		return $this->remClass;
	}

	/**
	 *
	 * @return int[]
	 */
	public function getCollection(): array{
		return $this->getRemTable() === stripTblPrefix(FILE_TABLE) ? $this->fileCollection : $this->objectCollection;
	}

	/**
	 *
	 * @param bool $skipEmpty
	 * @param bool $updateCollection
	 * @return int[]
	 */
	public function getValidCollection(bool $skipEmpty = true, bool $updateCollection = false): array{
		if($this->getRemTable() === stripTblPrefix(FILE_TABLE)){
			$collection = &$this->fileCollection;
			$table = FILE_TABLE;
			$where = $this->remCT ? 'AND ContentType IN("' . implode('","', $this->remCT) . '")' : '';
		} else {
			$collection = &$this->objectCollection;
			$table = OBJECT_FILES_TABLE;
			$where = 'AND TableID=' . $this->remClass;
		}
		if(empty($collection)){
			return [];
		}

		$v = $this->DB_WE->getAllq('SELECT ID FROM ' . $table . ' WHERE ID IN(' . implode(',', $collection) . ') ' . $where . '  AND IsFolder=0', true);
		'@phan-var int[] $v';
		$verifiedItems = array_combine($v, $v);
		$collection = ($this->IsDuplicates ? $collection : array_unique($collection));

		$ret = $tempCollection = [];

		foreach($collection as $id){
			$id = (int) $id;
			if(isset($verifiedItems[$id])){
				$ret[] = $verifiedItems[$id];
			}
			if(!$skipEmpty && $id === -1){
				$ret[] = -1;
			}
			$tempCollection[] = $id;
		}

		if($updateCollection){
			$collection = $tempCollection;
		}

		return $ret;
	}

	/**
	 *
	 * @param bool $skipEmpty
	 * @param bool $updateCollection
	 * @return we_collection_item[]
	 */
	public function getValidCollectionFull(bool $skipEmpty = true, bool $updateCollection = false): array{
		$validIDs = $this->getValidCollection($skipEmpty, $updateCollection);
		$fullItems = $this->getFullItems(array_filter($validIDs, function($v){
				return $v > 0;
			}));

		$ret = [];
		foreach($validIDs as $id){
			$ret[] = ((int) $id === -1) ? new we_collection_item() : ($fullItems[$id] ?? null);
		}

		return array_filter($ret);
	}

	/**
	 *
	 * @param int[] $coll
	 * @return void
	 */
	public function setCollection(array $coll): void{
		if($this->getRemTable() === stripTblPrefix(FILE_TABLE)){
			$this->fileCollection = $coll;
		} else {
			$this->objectCollection = $coll;
		}
	}

	/**
	 *
	 * @param int $editor
	 * @return array|array{ondragenter:string,ondragleave:string}
	 */
	public function getEditorBodyAttributes(int $editor = 0): array{
		switch($editor){
			case self::EDITOR_HEADER:
				return [
					'ondragenter' => "we_editor_header.dragEnter();",
					'ondragleave' => "we_editor_header.dragLeave();"
				];
			case self::EDITOR_FOOTER:
				return [
					'ondragenter' => "we_editor_footer.dragEnter();",
					'ondragleave' => "we_editor_footer.dragLeave();"
				];
			default:
				return [];
		}
	}

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			default:
				$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_CONTENT:
				return new we_editor_content_collection($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				return new we_editor_info($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_WEBUSER:
				return new we_editor_weDocumentCustomerFilter($this, $jsCmd);
		}
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd): string{
		return (new we_html_multiIconBox(
				(new we_html_multiIconBoxContent())
					->add(new we_html_multiIconBoxContentLine(g_l('weClass', '[path]'), $this->formPath(!we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_MOVE])), we_html_multiIconBox::PROP_PATH))
					->add(new we_html_multiIconBoxContentLine(g_l('global', '[content]'), $this->formContent(), we_html_multiIconBox::PROP_CONTENT))
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[owners]'), $this->formCreatorOwners($jsCmd), we_html_multiIconBox::PROP_USER))->fold())
				, 'PropertyPage'))->marginLeft()->get();
	}

	function formContent(bool $fixedRemTable = false): string{
		$fixedRemTable = $fixedRemTable || !we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_NEW]);

		$valsRemTable = [
			'tblFile' => g_l('navigation', '[documents]')
		];
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$valsRemTable['tblObjectFiles'] = g_l('navigation', '[objects]');
		}

		$allMime = we_base_ContentTypes::getContentTypes(FILE_TABLE, true);
		$remCtArr = array_filter($this->remCT);
		$tmpRemCT = [];
		$selectedMime = /* $unselectedMime = */ [];
		$mimes = [];
		foreach($allMime as $mime){
			if(in_array($mime, $remCtArr, true)){
				$selectedMime[$mime] = g_l('contentTypes', '[' . $mime . ']');
				$tmpRemCT[] = $mime;
			} else {
				//$unselectedMime[$mime] = g_l('contentTypes', '[' . $mime . ']');
			}
			$mimes[$mime] = g_l('contentTypes', '[' . $mime . ']');
		}
		$this->remCT = $tmpRemCT;
		$attribsFrom = !we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_NEW]) ? ['disabled' => 'disabled'] : [];
		$selRemTable = ($fixedRemTable && $this->getRemTable() ? we_html_element::hidden('we_' . $this->Name . '_remTable', $this->getRemTable()) . we_html_element::input([
				'disabled' => 'disabled', 'name' => 'disabledField', 'value' => $valsRemTable[$this->getRemTable()], 'width' => '356']) :
			we_html_tools::select('we_' . $this->Name . '_remTable', $valsRemTable, 1, $this->getRemTable(), false, ['onchange' => "we_cmd('collection_changeRemTable',this.value);",
				'style' => 'margin-top: 5px;']));

		$dublettes = we_html_forms::checkboxWithHidden($this->IsDuplicates, 'we_' . $this->Name . '_IsDuplicates', g_l('weClass', '[collection][allowDuplicates]'), false, '', '', !we_base_permission::hasPerm([
					we_base_perm::WORK_COLLECTION_NEW]));

		$this->DefaultDir = (int) ($this->DefaultDir ?: (IMAGESTARTID_DEFAULT ?: 0));
		$this->DefaultPath = $this->DefaultDir ? id2path($this->DefaultDir, FILE_TABLE) : '';
		$defDir = $this->formDirChooser(0, FILE_TABLE, 'DefaultPath', 'DefaultDir', '', g_l('weClass', '[collection][label_defaultDir]'), !we_base_permission::hasPerm([
				we_base_perm::WORK_COLLECTION_NEW]));

		$classes = $this->DB_WE->getAllFirstq('SELECT ID,Text FROM ' . OBJECT_TABLE . ' ORDER BY Text', false);
		$class = new we_contents_object((int) (!empty($this->remClass) ? $this->remClass : (array_keys($classes))[0]));
		$inputOpts = $imgOpts = [];

		foreach(array_keys($class->getAllVariantFields()) as $field){
			[$type, $name] = explode('_', (string) $field, 2);
			switch($type){
				case 'input':
					$inputOpts[$field] = $name;
					break;
				case 'img':
					$imgOpts[$field] = $name;
					break;
			}
		}

		$selField = (new we_html_select(['name' => 'we_' . $this->Name . '_remField', 'class' => 'weSelect']))
			->addOptions([0 => ''])
			->addOptionGroup(['label' => g_l('weClass', '[collection][fields_images]')])
			->addOptions($imgOpts ?: [0 => g_l('weClass', '[collection][fields_no_images]')])
			->addOptionGroup(['label' => g_l('weClass', '[collection][fields_textinputs]')])
			->addOptions($inputOpts ?: [0 => g_l('weClass', '[collection][fields_no_textinputs]')])
			->selectOption($this->remField ?: 0);

		$html = $selRemTable . we_html_element::div(['id' => 'collection_props-mime_class'], we_html_element::div(['id' => 'mimetype',
					'class' => 'collection_props-mime',
					'style' => 'display:' . ($this->getRemTable() === stripTblPrefix(OBJECT_FILES_TABLE) ? 'none' : 'block') . ';'
					], '<br/>' . g_l('weClass', '[collection][filter_contenttype]') . ':<br/>' .
					we_html_element::hidden('processRemCT', ($this->getRemTable() === stripTblPrefix(OBJECT_FILES_TABLE) ? '0' : '1')) .
					we_html_tools::select('we_' . $this->Name . '_remCT[]', $mimes, 1, array_keys($selectedMime), true, array_merge($attribsFrom, ['class' => 'newSelect', 'onchange' => "we_cmd('setHot');"]), true)
				) .
				we_html_element::div(['id' => 'classname',
					'class' => 'collection_props-classes',
					'style' => 'display:' . ($this->getRemTable() === stripTblPrefix(OBJECT_FILES_TABLE) ? 'block' : 'none') . ';'
					], g_l('weClass', '[collection][objectElems_class]') . '<br/>' .
					we_html_tools::select('we_' . $this->Name . '_remClass', $classes, 1, $this->remClass, false, array_merge($attribsFrom, ['class' => 'weSelect ', 'onchange' => "we_cmd('setHot');we_cmd('reload_editpage');"]), true) .
					'<br/><br/>' . g_l('weClass', '[collection][objectElems_previewField]') . '<br/>' .
					$selField->getHtml()
				)
			) .
			we_html_element::div(['class' => 'collection_props-dublettes'], $dublettes) .
			we_html_element::div(['id' => 'uploadDefault', 'style' => 'display:' . ($this->getRemTable() === stripTblPrefix(OBJECT_FILES_TABLE) ? 'none' : 'block')], $defDir);

		return $html;
	}

	function formCollection(): string{
		$recursive = we_html_forms::checkboxWithHidden($this->InsertRecursive, 'we_' . $this->Name . '_InsertRecursive', g_l('weClass', '[collection][insertRecursive]')) .
			we_html_element::hidden('check_we_' . $this->Name . '_IsDuplicates', $this->IsDuplicates);
		$slider = we_html_element::div(['id' => "sliderDiv", 'style' => 'display:' . ($this->view === 'grid' ? 'block' : 'none')], '<input type="range" id="collection_slider" class="collection-Slider" name="zoom" min="1" step="1" max="5" value="' . (7 - $this->itemsPerRow) . '"/>');

		//TODO: use tables and some padding
		$pos = 0;
		$toolbar = (new we_html_table([], 1, 8))
			->setCol(0, $pos++, $recursive)
			->setCol(0, $pos++, we_html_element::span(['id' => 'viewButtonIcon', 'style' => 'display:none;'], we_html_button::create_button(we_html_button::VIEW_ICON, '', 'grid', "document.getElementById('viewButtonIcon').style.display='none';document.getElementById('viewButtonList').style.display='inline';document.getElementById('viewButtonListMin').style.display='inline';", false, true, '', false, '', 'collection_btnView')) .
				we_html_element::span(['id' => 'viewButtonList'], we_html_button::create_button(we_html_button::VIEW_LIST, '', 'list', "document.getElementById('viewButtonIcon').style.display='inline';document.getElementById('viewButtonList').style.display='none';document.getElementById('viewButtonListMin').style.display='inline';", false, true, '', false, '', 'collection_btnView')) .
				we_html_element::span(['id' => 'viewButtonListMin'], we_html_button::create_button('fa:listview_minimal,fa-lg fa-align-justify', '', 'minimal', "document.getElementById('viewButtonIcon').style.display='inline';document.getElementById('viewButtonList').style.display='inline';document.getElementById('viewButtonListMin').style.display='none';", false, true, '', false, '', 'collection_btnView')))
			->setCol(0, $pos++, $slider)
			->setCol(0, $pos++, we_html_button::create_button('fa:btn_select_files' . ($this->ID ? '' : '_disabled') . ', fa-lg fa-sitemap, fa-lg fa-angle-right, fa-lg fa-copy', '', 'collection_btnAddFromTree', '', !$this->ID, true, '', false, '', 'collectionItem_btnAddFromTree'), [
				'class' => 'toolbarAdd'])
			->setCol(0, $pos++, ($this->getRemTable() === stripTblPrefix(FILE_TABLE) ? we_fileupload_ui_importer::getBtnImportFiles($this->DefaultDir, false, 'collection_insertFiles,' . $this->ID . ',-1,-1,0,' . we_base_WE::$transaction, 'btn_import_files_and_insert') : ''), [
				'class' => 'toolbarImport'])
			->setCol(0, $pos++, g_l('weClass', '[collection][number]') . ': ' . we_html_element::span(['id' => "numSpan"], '<i class="fa fa-2x fa-spinner fa-pulse"></i>'), [
			'class' => 'toolbarNum weMultiIconBoxHeadline']);

		return we_html_element::hiddens([
				'we_' . $this->Name . '_view' => $this->view,
				'we_' . $this->Name . '_viewSub' => $this->viewSub,
				'we_' . $this->Name . '_itemsPerRow' => (string) $this->itemsPerRow,
				'we_' . $this->Name . '_fileCollection' => implode(',', $this->fileCollection),
				'we_' . $this->Name . '_objectCollection' => implode(',', $this->objectCollection),
			]) .
			we_html_element::div(['class' => 'weMultiIconBoxHeadline collection-head'], g_l('weClass', '[collection][collectionTitle]')) .
			we_html_element::div(['class' => 'collection-head'], we_html_tools::alertAttentionBox(g_l('weClass', '[collection][long_description]', false, true), we_html_tools::TYPE_INFO, 850, false, 29)) .
			we_html_element::div(['class' => 'collection-toolbar'], $toolbar->getHtml()) .
			we_html_element::div(['id' => 'content_div_list', 'class' => 'collection-content', 'style' => 'display:' . ($this->view === 'grid' ? 'none' : 'block')]) .
			we_html_element::div(['id' => 'content_div_grid', 'class' => 'collection-content', 'style' => 'display:' . ($this->view === 'grid' ? 'inline-block' : 'none')]);
	}

	public static function getJSConsts(): string{
		$placeholders = [
			'index' => '##INDEX##',
			'id' => '##ID##',
			'path' => '##PATH##',
			'name' => '##NAME##',
			'type' => '##CT##',
			'class' => '##CLASS##',
			'icon' => ['url' => '##ICONURL##', 'sizeX' => 200, 'sizeY' => 200],
			'remTable' => '##REMTABLE##',
			'remCT' => '##REMCT##',
			'defaultDir' => '##DEFAULTDIR##',
			'elements' => [
				'attrib_title' => ['Dat' => '##ATTRIB_TITLE##', 'state' => '##S_ATTRIB_TITLE##', 'write' => '##W_ATTRIB_TITLE##'],
				'attrib_alt' => ['Dat' => '##ATTRIB_ALT##', 'state' => '##S_ATTRIB_ALT##', 'write' => '##W_ATTRIB_ALT##'],
				'meta_title' => ['Dat' => '##META_TITLE##', 'state' => '##S_META_TITLE##', 'write' => '##W_META_TITLE##'],
				'meta_description' => ['Dat' => '##META_DESC##', 'state' => '##S_META_DESC##', 'write' => '##W_META_DESC##'],
				'custom' => ['Dat' => '##CUSTOM##']
			]
		];

		$weSuggest = we_gui_suggest::getInstance();

		return we_base_util::setLangString([
				'blankItem' => [
					'grid' => self::makeGridItem($placeholders),
					'list' => self::makeListItem($placeholders, $weSuggest),
					'listMinimal' => self::makeListItemMinimal($placeholders, $weSuggest),
		]]);
	}

	public function getJSDynamic(): array{
		$items = $this->getValidCollectionFull(false, true);
		$storage = ['item_-1' => new we_collection_item()];
		$itemIDs = [];
		foreach($items as $item){
			'@phan-var we_collection_item $item';
			$storage['item_' . $item->id] = $item;
			$itemIDs[] = $item->id;
		}

		return [
			'doc' => [
				'docClass' => __CLASS__,
				'docTransaction' => we_base_WE::$transaction,
				'docId' => $this->ID,
				'docPath' => $this->Path,
				'docText' => $this->Filename,
				'docTable' => $this->Table,
				'docName' => $this->Name,
				'docRemTable' => $this->getRemTable(),
				'docIsObjects' => $this->getRemTable() !== stripTblPrefix(FILE_TABLE),
				'docRemCT' => ',' . implode(',', $this->getRemCT()) . ',', //set to array?!
				'docRealRemCT' => ',' . implode(',', $this->getRealRemCT()) . ',', //set to array?!
				'docRemClass' => $this->getRemClass(),
				'docDefaultDir' => $this->DefaultDir,
				'docIsDuplicates' => (int) $this->IsDuplicates,
			],
			'content' => [
				'collectionArr' => $itemIDs,
				'storage' => $storage,
				'collectionCsv' => '',
				'collectionCount' => 0,
				'maxIndex' => 0,
				'collectionName' => ''
			],
			'gui' => [
				'lastView' => 'grid', // save last view to session!
				'isDragAndDrop' => self::isDragAndDrop() ? 1 : 0,
				'gridItemDimensions' => self::GRID_ITEM_DIMENSIONS,
				'gridItemDimension' => self::GRID_ITEM_DIMENSIONS[$this->itemsPerRow],
				'itemsPerRow' => $this->itemsPerRow,
				'view' => $this->view,
				'viewSub' => $this->viewSub === 'minimal' ? 'minimal' : 'broad',
				'elements' => [
					'container' => [
						'grid' => null,
						'list' => null
					]
				]
			]
		];
	}

	private static function makeListItem(array $item, we_gui_suggest &$weSuggest): string{
		$textname = 'we_' . $item['name'] . '_ItemName_' . $item['index'];
		$idname = 'we_' . $item['name'] . '_ItemID_' . $item['index'];

		$weSuggest->setTable($item['remTable'])
			->setContentType(array_merge(we_base_ContentTypes::FOLDER_TYPES, [$item['remCT']]))
			->setCheckFieldValue(false)
			->setSelector(we_gui_suggest::DocSelector)
			->setAcId('Item_' . $item['index'])
			->setNoAutoInit(true)
			->setInput($textname, $item['path'], ['title' => $item['path'] . ' (ID: ' . $item['id'] . ')'])
			->setResult($idname, (string) $item['id'])
			->setWidth(240)
			->setMaxResults(10)
			->setSelectButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, '', '', '', false, true, '', false, '', 'collectionItem_btnEdit'))
			->setAdditionalButton(we_html_button::create_button('fa:btn_select_files, fa-lg fa-sitemap, fa-lg fa-angle-right, fa-lg fa-copy', '', '', '', false, false, '', false, '', 'collectionItem_btnAddFromTree'));
		//$weSuggest->setOpenButton($editButton, 4);
		//FIXME: this function is not implemented: repaintAndRetrieveCsv
		//$weSuggest->setDoOnItemSelect("weCollectionEdit.repaintAndRetrieveCsv();");

		$btnSelect = we_html_button::create_button(we_html_button::SELECT, '', '', '', false, true, '', false, '', 'collectionItem_btnSelect');
		$btnAdd = we_html_button::create_button(we_html_button::LIST_ADD, '', '', '', false, true, '', false, '', 'collectionItem_btnAdd');
		$btnUp = we_html_button::create_button(we_html_button::DIRUP, '', '', '', false, true, '', false, '', 'btn_up collectionItem_btnUp');
		$btnDown = we_html_button::create_button(we_html_button::DIRDOWN, '', '', '', false, true, '', false, '', 'btn_down collectionItem_btnDown');
		$btnRemove = we_html_button::create_button('fa:btn_remove_from_collection,fa-lg fa-trash-o', '', '', '', false, true, '', false, '', 'collectionItem_btnTrash');

		$rowControlls = $btnAdd . $btnUp . $btnDown . $btnRemove;

		$imgDiv = we_html_element::div(['id' => 'previweDiv_' . $item['index'],
				'class' => 'previewDiv',
				'style' => "background-image:url('" . $item['icon']['url'] . "');",
				'title' => $item['path'] . ' (ID: ' . $item['id'] . ')'
				], we_html_element::div(['class' => 'divBtnSelect'], $btnSelect));

		$attrTitle = we_html_element::div(['class' => 'innerDiv' . ($item['elements']['attrib_title']['Dat'] ? ' div_' . $item['elements']['attrib_title']['state'] : ''),
				'title' => ($item['elements']['attrib_title']['Dat'])
				], '<i class="fa fa-lg fa-circle ' . $item['elements']['attrib_title']['state'] . '"></i> ' . $item['elements']['attrib_title']['write']);
		$attrAlt = we_html_element::div(['class' => 'innerDiv' . ($item['elements']['attrib_alt']['Dat'] ? ' div_' . $item['elements']['attrib_alt']['state'] : ''),
				'title' => ($item['elements']['attrib_alt']['Dat'])
				], '<i class="fa fa-lg fa-circle ' . $item['elements']['attrib_alt']['state'] . '"></i> ' . $item['elements']['attrib_alt']['write']);
		$metaTitle = we_html_element::div(['class' => 'innerDiv' . ($item['elements']['meta_title']['Dat'] ? ' div_' . $item['elements']['meta_title']['state'] : ''),
				'title' => ($item['elements']['meta_title']['Dat'])
				], '<i class="fa fa-lg fa-dot-circle-o ' . $item['elements']['meta_title']['state'] . '"></i> ' . $item['elements']['meta_title']['write']);
		$metaDesc = we_html_element::div(['class' => 'innerDiv' . ($item['elements']['meta_description']['Dat'] ? ' div_' . $item['elements']['meta_description']['state'] : ''),
				'title' => ($item['elements']['meta_description']['Dat'])
				], '<i class="fa fa-lg fa-dot-circle-o ' . $item['elements']['meta_description']['state'] . '"></i> ' . $item['elements']['meta_description']['write']);

		$rowInnerTable = (new we_html_table(['draggable' => 'false'], 2, 1))
			->setCol(0, 0, $weSuggest->getHTML())
			->setCol(1, 0, $attrTitle . $attrAlt . $metaTitle . $metaDesc)
			->setRowAttributes(1, ['class' => 'rowAttribsMeta']);

		$rowHtml = (new we_html_table(['class' => $item['class'], 'draggable' => 'false'], 1, 4))
			->setCol(0, 0, we_html_element::span(['class' => "list_label", 'id' => 'label_' . $item['index']], $item['index']), ['class' => 'colNum weMultiIconBoxHeadline'])
			->setCol(0, 1, $imgDiv, ['class' => 'colPreview'])
			->setCol(0, 2, $rowInnerTable->getHtml(), ['class' => 'colContent'])
			->setCol(0, 3, $rowControlls, ['class' => 'colControls weMultiIconBoxHeadline']);

		return we_html_element::div(['id' => 'list_item_' . $item['index'], 'class' => 'listItem', 'draggable' => 'false'], $rowHtml->getHtml() .
				we_html_element::div(['id' => 'collectionItem_index_list_' . $item['index'], 'class' => 'collectionItem_index', 'style' => 'display:none'])
		);
	}

	private static function makeListItemMinimal(array $item, we_gui_suggest &$weSuggest): string{
		$textname = 'we_' . $item['name'] . '_ItemName_' . $item['index'];
		$idname = 'we_' . $item['name'] . '_ItemID_' . $item['index'];
		$weSuggest->setTable(addTblPrefix($item['remTable']))
			->setContentType(array_merge(we_base_ContentTypes::FOLDER_TYPES, [$item['remCT']]))
			->setCheckFieldValue(false)
			->setSelector(we_gui_suggest::DocSelector)
			->setAcId('Item_' . $item['index'])
			->setNoAutoInit(true)
			->setInput($textname, $item['path'], ['title' => $item['path'] . ' (ID: ' . $item['id'] . ')'])
			->setResult($idname, (string) $item['id'])
			->setWidth(500)
			->setMaxResults(10)
			->setSelectButton('')
			->setAdditionalButton('');
		//FIXME: this function is not implemented: repaintAndRetrieveCsv
		//$weSuggest->setDoOnItemSelect("weCollectionEdit.repaintAndRetrieveCsv();");
		$divRowContent = we_html_element::div(['class' => 'divContent'], we_html_element::div(['class' => 'colContentInput'], $weSuggest->getHTML()) .
				we_html_element::div(['class' => 'colContentTextOnly'])
		);

		$btnSelect = we_html_button::create_button(we_html_button::SELECT, '', '', '', false, true, '', false, '', 'collectionItem_btnSelect');
		$btnEdit = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, '', '', '', false, true, '', false, '', 'collectionItem_btnEdit');
		$btnAdd = we_html_button::create_button(we_html_button::LIST_ADD, '', '', '', false, true, '', false, '', 'collectionItem_btnAdd');
		$btnRemove = we_html_button::create_button('fa:btn_remove_from_collection,fa-lg fa-trash-o', '', '', '', false, true, '', false, '', 'collectionItem_btnTrash');

		$rowControlls = we_html_element::div(['class' => 'divBtnEditTextOnly'], $btnEdit) . $btnAdd . $btnRemove;
		$imgDiv = we_html_element::div([
				'id' => 'previweDiv_' . $item['index'],
				'class' => 'previewDiv',
				'style' => "background-image:url('" . $item['icon']['url'] . "');",
				'title' => $item['path'] . ' (ID: ' . $item['id'] . ')'
				], we_html_element::div(['class' => 'divBtnSelect'], $btnSelect));
		$rowHtml = (new we_html_table(['class' => $item['class'], 'draggable' => 'false'], 1, 4))
			->setCol(0, 0, we_html_element::span(['class' => "list_label", 'id' => 'label_' . $item['index']], $item['index']), ['class' => 'colNum weMultiIconBoxHeadline'])
			->setCol(0, 1, $imgDiv, ['class' => 'colPreview'])
			->setCol(0, 2, $divRowContent, ['class' => 'colContent'])
			->setCol(0, 3, $rowControlls, ['class' => 'colControls weMultiIconBoxHeadline']);

		return we_html_element::div(['id' => 'list_item_' . $item['index'], 'class' => 'listItem', 'draggable' => 'false'], $rowHtml->getHtml() .
				we_html_element::div(['id' => 'collectionItem_index_list_' . $item['index'], 'class' => 'collectionItem_index', 'style' => 'display:none']));
	}

	private static function makeGridItem(array $item): string{
		$btnRemove = we_html_button::create_button('fa:btn_remove_from_collection,fa-lg fa-trash-o', '', '', '', false, true, '', false, '', 'collectionItem_btnTrash');
		$btnEdit = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, '', '', '', false, true, '', false, '', 'collectionItem_btnEdit');
		$btnSelect = we_html_button::create_button(we_html_button::SELECT, '', '', '', false, true, '', false, '', 'collectionItem_btnSelect');

		$toolbar = we_html_element::div(['class' => 'toolbarLeft weMultiIconBoxHeadline'], we_html_element::span(['class' => 'grid_label', 'id' => 'label_' . $item['index']], $item['index'])) .
			we_html_element::div([
				'class' => 'toolbarAttribs',
				'style' => 'display: block'
				], we_html_element::div([
					'class' => 'toolbarAttr',
					'title' => $item['elements']['attrib_title']['Dat']
					], '<i class="fa fa-lg fa-circle ' . $item['elements']['attrib_title']['state'] . '"></i>') .
				we_html_element::div([
					'class' => 'toolbarAttr',
					'title' => $item['elements']['attrib_alt']['Dat'],
					], '<i class="fa fa-lg fa-circle ' . $item['elements']['attrib_alt']['state'] . '"></i>') .
				we_html_element::div(['class' => 'toolbarAttr',
					'title' => $item['elements']['meta_title']['Dat'],
					], '<i class="fa fa-lg fa-dot-circle-o ' . $item['elements']['meta_title']['state'] . '"></i>') .
				we_html_element::div(['class' => 'toolbarAttr',
					'title' => $item['elements']['meta_description']['Dat']
					], '<i class="fa fa-lg fa-dot-circle-o ' . $item['elements']['meta_description']['state'] . '"></i>')
			) . we_html_element::div(['class' => 'toolbarBtns'], $btnEdit . $btnRemove);

		return we_html_element::div([//TODO: set dimensions by JS
				'style' => 'width:200px;height:200px;',
				'id' => 'grid_item_' . $item['index'],
				'class' => 'gridItem'
				], we_html_element::div(['class' => 'divSpace_left',
					'id' => 'grid_space_left_' . $item['index'],
					'title' => g_l('weClass', '[collection][dblClick_to_insert]')
					], '') .
				we_html_element::div(['title' => $item['path'] . ' (ID: ' . $item['id'] . ')',
					'class' => 'divContent',
					'style' => ($item['icon'] ? "background-image:url('" . $item['icon']['url'] . "');" : '') . (max($item['icon']['sizeX'], $item['icon']['sizeY']) < 200 ? 'background-size:auto;' : ''),
					'draggable' => 'false',
					], we_html_element::div(['class' => 'divInnerSmall', 'style' => 'display:none'], we_html_element::span()) .
					we_html_element::div(['class' => 'divInner', 'style' => 'display:##SHOWBTN##'], $btnSelect) .
					we_html_element::div(['class' => 'divToolbar', 'draggable' => ''], $toolbar)) .
				we_html_element::div(['class' => 'divSpace_right',
					'id' => 'grid_space_' . $item['index'],
					'title' => g_l('weClass', '[collection][dblClick_to_insert]')
					], '') . we_html_element::div(['id' => 'collectionItem_index_grid_' . $item['index'], 'class' => 'collectionItem_index', 'style' => 'display:none']) .
				we_html_element::hidden('collectionItem_we_id', $item['id']) .
				we_html_element::hidden('collectionItem_we_id_' . $item['index'], $item['id'])
		);
	}

	/**
	 *
	 * @param we_collection_item[] $items
	 * @return we_collection_item[]
	 */
	private function setItemElements(array $items): array{
		if(empty($items)){
			return [];
		}

		if($this->getRemTable() === stripTblPrefix(FILE_TABLE)){
			$records = $this->DB_WE->getAllq('SELECT
	c.DID,c.Name,c.type,c.Dat,c.BDID
FROM ' . CONTENT_TABLE . ' c
WHERE
	c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND
	c.DID IN (' . implode(',', array_keys($items)) . ') AND (
	(c.type="attrib" AND c.nHash IN (x\'' . md5('title') . '\',x\'' . md5('alt') . '\') ) OR
	(c.type="txt" AND c.nHash IN (x\'' . md5('Title') . '\',x\'' . md5('Description') . '\') )
)');
			'@phan-var array<int,array{DID:int,Name:string,type:string,Dat:string,BDID:int}> $records';
			foreach($records as $record){
				switch(($dbname = $record['Name'])){
					case 'title':
					case 'alt':
						$fieldname = 'attrib_' . $dbname;
						break;
					case self::TITLE_FIELD:
					case self::DESCRIPTION_FIELD:
						$fieldname = 'meta_' . strtolower($dbname);
						break;
					default:
						$fieldname = 'custom';
				}
				$items[$record['DID']]->elements[$fieldname] = ($fieldname === 'custom' ?
					['type' => $record['type'],
					'Dat' => $record['Dat'],
					'BDID' => $record['BDID'],
					] :
					['Dat' => $record['Dat'],
					'state' => we_collection_item::CLASS_NONE
				]);
			}

			// mark the first 2 set elements of each item in an ordered way
			$hasMeta = [we_base_ContentTypes::APPLICATION, we_base_ContentTypes::AUDIO, we_base_ContentTypes::IMAGE, we_base_ContentTypes::WEDOCUMENT,
				we_base_ContentTypes::VIDEO, we_base_ContentTypes::PDF];
			foreach($items as $v){
				$v->updateElements($hasMeta);
			}
		}

		return $items;
	}

	protected function isFilenameDouble(): bool{
		return fB('SELECT 1 FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ParentID=' . ((int) $this->ParentID) . ' AND Filename="' . $this->DB_WE->escape($this->Filename) . '" AND ID!=' . ((int) $this->ID));
	}

	protected function mimetypeisFilenameNotValid(): bool{
		return false;
	}

	protected function we_load(int $from = self::LOAD_MAIN_DB): bool{
		$ret = parent::we_load(self::LOAD_MAIN_DB);
		//FIXME: remove this switch after 6.6
		$this->ContentType = $this->ContentType ?: we_base_ContentTypes::COLLECTION;
		return $ret;
	}

	function userCanSave(bool $ctConditionOk = false): bool{
		return (we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_SAVE, we_base_perm::WORK_COLLECTION_NEW])) && parent::userCanSave(true);
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		$this->errMsg = '';

		if(!$skipHook){// TODO: integrate hooks?
		}

		$collection = $this->getValidCollection();

		$ret = parent::we_save($resave) && $this->writeCollectionToDB($collection) && !$this->errMsg;
		if($ret){
			$this->unregisterMediaLinks();
			$this->registerMediaLinks(false, $collection);
		}

		return $ret;
	}

	protected function getContentData(string $table): bool{
		$this->fileCollection = array_map('intval', $this->DB_WE->getAllq('SELECT remObj FROM ' . FILELINK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix(VFILE_TABLE) . '" AND type IN("collection","archive") AND remTable="' . stripTblPrefix(FILE_TABLE) . '" ORDER BY position ASC', true));
		$this->objectCollection = array_map('intval', $this->DB_WE->getAllq('SELECT remObj FROM ' . FILELINK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix(VFILE_TABLE) . '" AND type IN("collection","archive") AND remTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '" ORDER BY position ASC', true));

		return !(empty($this->fileCollection) && empty($this->objectCollection));
	}

	protected function setElementsFromHTTP(): void{
		if(we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_NEW])){
			// FIXME: is this obsolete?
			if(we_base_request::bool('processRemCT', false)){
				$_REQUEST['we_' . $this->Name . '_remCT'] = implode(',', we_base_request::stringList('we_' . $this->Name . '_remCT'));
				unset($_REQUEST['we_' . $this->Name . '_remClass']);
			} else {
				unset($_REQUEST['we_' . $this->Name . '_remCT']);
			}
		} else {
			// with no perm to create new collectoion user must not change any collection properties
			unset($_REQUEST['we_' . $this->Name . '_remTable'], $_REQUEST['we_' . $this->Name . '_remCT'], $_REQUEST['we_' . $this->Name . '_remClass'], $_REQUEST['we_' . $this->Name . '_IsDuplicates'], $_REQUEST['we_' . $this->Name . '_DefaultDir'], $_REQUEST['we_' . $this->Name . '_DefaultPath']);
		}

		if(!we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_MOVE])){
			unset($_REQUEST['we_' . $this->Name . '_ParentID'], $_REQUEST['we_' . $this->Name . '_ParentPath']);
		}

		parent::setElementsFromHTTP();
	}

	function writeCollectionToDB(array $collection): bool{// FIXME: is there a standard function called by some parent to save non-persistent data?
		$ret = $this->DB_WE->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix(VFILE_TABLE) . '" AND type IN("collection","archive")');

		$i = 0;
		foreach(($this->IsDuplicates ? $collection : array_unique($collection)) as $remObj){
			$ret = $ret && $this->DB_WE->query('INSERT INTO ' . FILELINK_TABLE . ' SET ' . we_database_base::arraySetter([
						'ID' => $this->ID,
						'DocumentTable' => stripTblPrefix(VFILE_TABLE),
						'type' => 'collection',
						'remObj' => $remObj,
						'remTable' => $this->getRemTable(),
						'position' => $i++,
			]));
		}

		return $ret;
	}

	function registerMediaLinks(bool $temp = false, array $collection = []): bool{
		$this->MediaLinks = $collection ?: $this->getValidCollection();

		return parent::registerMediaLinks($temp);
	}

	public function addItemsToCollection(array $items, int $pos = -1): array{ // FIXME: when pos=-1 do append at the end of collection not replacing last element!
		$coll = $this->getCollection();
		array_pop($coll); // FIXME: maybe abandon the ending -1 inserted on we_load()?

		if($pos === -1){
			$pos = count($coll);
			// find last collection item not empty
			for($i = 0; $i < count($coll); $i++){
				if($coll[$i] != -1){
					$pos = $i;
				}
			}
			$pos++;
		}

		$tmpColl = array_slice($coll, $pos);
		$newColl = array_slice($coll, 0, $pos);
		$result = [[], []];
		$isFirstSet = false;
		foreach($items as $item){
			if($this->IsDuplicates || !in_array($item, $coll)){
				$newColl[] = $result[0][] = (int) $item;
				if(!$isFirstSet || (isset($tmpColl[0]) && $tmpColl[0] == -1)){
					array_shift($tmpColl);
					$isFirstSet = true;
				}
			} else {
				$result[1][] = $item;
			}
		}
		$this->setCollection(array_merge($newColl, $tmpColl, [-1]));

		return $result;
	}

	/**
	 *
	 * @param int[] $IDs
	 * @param bool $recursive
	 * @param string $table
	 * @return int[]
	 */
	public function getValidItemsFromIDs(array $IDs = [], bool $recursive = true, string $table = FILE_TABLE): array{
		if(empty($IDs)){
			return [];
		}
		if(($table = stripTblPrefix($table ?: FILE_TABLE)) && $table !== $this->getRemTable()){
			//throw!!
			return [];
		}
		/*
		  if($checkWs && (empty($wspaces))){
		  if(($ws = we_users_util::get_ws($this->getRemTable()))){
		  $wsPathArray = array_map('escape_sql_query', id2pathA($ws, $this->getRemTable()));
		  if($wsPathArray){
		  $wspaces[] = 'Path REGEXP "^(' . implode('|', $wsPathArray) . ')/"';
		  }
		  } elseif($this->getRemTable() === stripTblPrefix(OBJECT_FILES_TABLE) && (!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]))){
		  $ac = we_users_util::getAllowedClasses();
		  $paths = array_map('escape_sql_query', id2pathA($ac, OBJECT_TABLE));
		  if($paths){
		  $wspaces[] = 'Path REGEXP "^(' . implode('|', $paths) . ')/"';
		  }
		  }
		  $wspaces = empty($wspaces) ? [false] : $wspaces;
		  }
		  $wsQuery = ($checkWs && $wspaces[0] !== false ? ' AND (' . implode(' OR ', $wspaces) . ') ' : ' OR RestrictOwners=0 ' );
		 *
		 */
		return $this->rgetValidIDs($IDs, $recursive);
	}

	/**
	 *
	 * @param int[] $IDs
	 * @param bool $recursive
	 * @param string $table
	 * @return we_collection_item[]
	 */
	public function getValidItemsFromIDsFull(array $IDs = [], bool $recursive = true, string $table = FILE_TABLE): array{
		return array_values($this->getFullItems($this->getValidItemsFromIDs($IDs, $recursive, $table)));
	}

	/**
	 *
	 * @param array $ids
	 * @return we_collection_item[]
	 */
	private function getFullItems(array $ids = []): array{
		if(!($ids = array_filter($ids))){
			return [];
		}

		if($this->getRemTable() === stripTblPrefix(FILE_TABLE)){
			$fields = 'f.ID,f.Path,f.ContentType,f.Extension,f.Filename';
			$from = FILE_TABLE . ' f ';
		} else {
			$isImgField = str_starts_with($this->remField, 'img');
			$joinedFields = $this->remField ? (($isImgField ? ',ft.ID as ImgID,ft.Path AS ImgPath,ft.ContentType as ImgCT,ft.Extension AS ImgExtension' : ',ob.' . $this->remField . ' AS Name')) : '';
			$fields = 'f.ID,f.Path,f.ContentType,f.TableID,f.Filename' . $joinedFields;
			$from = OBJECT_FILES_TABLE . ' f ' . ($joinedFields ? 'JOIN ' . OBJECT_X_TABLE . $this->remClass . ' ob ON f.ID=ob.OF_ID' . ($isImgField ? ' LEFT JOIN ' . FILE_TABLE . ' ft ON ob.' . $this->remField . '=ft.ID' : '') : '');
		}

		$fullItems = [];
		foreach($this->DB_WE->getAllq('SELECT ' . $fields . ' FROM ' . $from . ' WHERE f.ID IN (' . implode(',', $ids) . ') AND f.IsFolder=0') as $rec){
			$ct = $rec['ContentType'];
			$item = new we_collection_item($rec['ID'], $rec['Path'], $ct, $rec['TableID'] ?? 0, $rec['Filename'] ?? ($rec['Name'] ?? $rec['Text']), $rec['Extension'] ?? '');

			if($ct === 'image/*' || !empty($rec['ImgPath'])){
				switch($ct){
					case 'image/*':
						$img = [
							'docID' => $item->id,
							'Path' => $item->path,
							'ContentType' => $item->ct,
							'Extension' => $item->ext,
						];
						break;
					default:
						$img = [
							'docID' => $rec['ImgID'] ?: 0,
							'Path' => $rec['ImgPath'] ?: 0,
							'ContentType' => $rec['ImgCT'] ?: 0,
							'Extension' => $rec['ImgExtension'] ?: 0,
						];
				}
				$img['size'] = file_exists(WEBEDITION_PATH . '../' . $img['Path']) ? (filesize(WEBEDITION_PATH . '../' . $img['Path']) ?: 0) : 0;
				$item->icon = self::getHtmlIconThmubnail($img, 400);
			}
			$fullItems[$rec['ID']] = $item;
		}

		if($this->getRemTable() === stripTblPrefix(FILE_TABLE)){
			return $this->setItemElements($fullItems);
		}

		return $fullItems;
	}

	/**
	 *
	 * @param array $file
	 * @param int $size
	 * @return array{url:string,sizeX:int,sizeY:int}
	 */
	public static function getHtmlIconThmubnail(array $file, int $size = 64): array{ // FIXME: move this as static fn to some image utilities class (from we_search_view too)
		$imagesize = [0, 0];
		$url = '';

		if($file['ContentType'] == we_base_ContentTypes::IMAGE && $file['size'] > 0){
			$imagesize = getimagesize(WEBEDITION_PATH . '../' . $file['Path']) ?: [0, 0];
			$url = we_document_image::getPreview($file['docID'], $file['Path'], $file['Extension'], $size, $size, [], true);
		}

		return ['url' => $url, 'sizeX' => $imagesize[0], 'sizeY' => $imagesize[1]];
	}

	public function isDragAndDrop(): bool{
		return !((we_base_browserDetect::isIE() && we_base_browserDetect::getIEVersion() < 10) ||
			(we_base_browserDetect::isSafari() && intval(we_base_browserDetect::inst()->getBrowserVersion()) < 7) ||
			(we_base_browserDetect::isOpera()));
	}

	private function rgetValidIDs(array $ids, bool $recursive = true, array &$result = [], bool $isFolder = false): array{
		$where = $isFolder ? 'ParentID=' . ((int) $ids[0]) : 'ID IN(' . implode(',', array_map('intval', array_filter($ids))) . ')';
		if($this->getRemTable() === stripTblPrefix(OBJECT_FILES_TABLE)){
			$table = OBJECT_FILES_TABLE;
			$where .= $this->remClass ? ' AND TableID=' . $this->remClass : '';
		} else {
			$table = FILE_TABLE;
			$where .= $this->remCT ? ' AND ContentType IN ("' . implode('","', $this->remCT) . '","' . we_base_ContentTypes::FOLDER_DOCUMENT . '") ' : '';
		}

		$items = $this->DB_WE->getAllFirstq('SELECT ID,IsFolder FROM ' . $this->DB_WE->escape($table) . ' WHERE ' . $where, false);
		foreach($items as $id => $folder){
			if($folder){
				$result = (!$isFolder || $recursive) ? $this->rgetValidIDs([$id], $recursive, $result, true) : $result;
			} else {
				$result[] = $id;
			}
		}

		return array_unique($result);
	}

	function publInScheduleTable(): ?DateTime{
		return null;
	}

}
