<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f797d914d8f8 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-03-31 22:45 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* a class for handling templates */

class we_contents_template extends we_document_files{
	/** @var string */
	public const BACKUP_XML_TAG = 'we:template';

	/** @var string */
	public const FULLCODE = 'completeData';
	private const ESCAPE_CHARS = [
		'?>' => '__WE_?__WE__',
		'=>' => '__WE_=__WE__',
		'->' => '__WE_-__WE__'
	];

	/** @var int */
	var $MasterTemplateID = 0;

	/** @var string */
	var $TagWizardCode = ''; // bugfix 1502

	/** @var string */
	var $TagWizardSelection = ''; // bugfix 1502

	/** @var int[] */
	var $IncludedTemplates = [];

	/** @var bool */
	var $doUpdateCode = true; // will be protected in later WE Versions

	/** @var string */
	var $Display = '';

	/** @var array<int,array<string,string>> */
	public static $usedReplacements = [];

	public function __construct(int $id = 0){
		$this->Table = TEMPLATES_TABLE;
		$this->ContentType = we_base_ContentTypes::TEMPLATE;
		parent::__construct();
		$this->Publish_Date = new DateTime();
		$this->InWebEdition = true;

		unset($this->persistent_slots['Extension']);
		$this->initByID($id);
	}

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'MasterTemplateID' => we_base_request::INT,
			'IncludedTemplates' => we_base_request::INTLISTA,
			'TagWizardCode' => we_base_request::STRING,
			'TagWizardSelection' => we_base_request::STRING,
			'Display' => we_base_request::STRING
		]);
		unset($this->persistent_slots['Extension']);
		if(isWE()){
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_INFO, we_base_constants::WE_EDITPAGE_CONTENT,
				we_base_constants::WE_EDITPAGE_PREVIEW, we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE, we_base_constants::WE_EDITPAGE_VARIANTS, we_base_constants::WE_EDITPAGE_VERSIONS,
				we_base_constants::WE_EDITPAGE_TEMPLATE_UNUSEDELEMENTS]);
		}
	}

	function copyDoc(int $id): bool{
		if(!$id){
			return false;
		}
		$temp = new we_contents_template($id);
		//$parentIDMerk = $this->ParentID;
		if($this->ID == 0){
			foreach(array_keys($this->persistent_slots) as $name){
				if($name != 'elements'){
					$this->{$name} = $temp->{$name};
				}
			}
			$this->Creation_Date = new DateTime();
			$this->ID = 0;
			$this->OldPath = '';
			$this->Filename .= '_copy';
			$this->Path = $this->ParentPath . $this->Filename; // . $this->Extension;
			$this->OldPath = $this->Path;
		}
		foreach($temp->nextElement('txt') as $k){
			$this->setElementS((string) $k, $temp->getElementS((string) $k));
		}
		$this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
		return true;
	}

	/* must be called from the editor-script. Returns a filename which has to be included from the global-Script */

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			default:
				$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				return new we_editor_info($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_CONTENT:
				$GLOBALS['we_editmode'] = we_base_WE::$editmode = true;
				return new we_editor_srcTmpl($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				$GLOBALS['we_editmode'] = we_base_WE::$editmode = true;
				$doc = new we_document_webEdition();
				//copy temporary data on template to document
				$doc->elements = $this->elements;
				$doc->ID = $this->ID;
				$doc->Table = $this->Table;
				$doc->EditPageNr = $this->EditPageNr;
				$doc->InWebEdition = true;
				we_base_WE::setWeDoc($doc);
				return (new we_editor_showTempDocumentData($this, $jsCmd))->setContent($this->getDocumentData());
			case we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE:
				$GLOBALS['we_editmode'] = we_base_WE::$editmode = false;
				$doc = new we_document_webEdition();
				//copy temporary data on template to document
				$doc->elements = $this->elements;
				$doc->ID = $this->ID;
				$doc->Table = $this->Table;
				$doc->EditPageNr = $this->EditPageNr;
				$doc->InWebEdition = true;
				we_base_WE::setWeDoc($doc);
				return (new we_editor_showTempDocumentData($this, $jsCmd))->setContent($this->getDocumentData());
			case we_base_constants::WE_EDITPAGE_VARIANTS:
				$GLOBALS['we_editmode'] = we_base_WE::$editmode = true;
				return new we_editor_variants($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VERSIONS:
				return new we_editor_versions($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_TEMPLATE_UNUSEDELEMENTS:
				return new we_editor_unusedElements($this, $jsCmd);
		}
	}

	/* 	private static function checkEndtags(string $tagname, $eq, $tags):bool{
	  $start = 0;
	  $end = 0;
	  foreach($tags as $tag){
	  if(strpos($tag, 'ifNoJavaScript') === false){
	  if($eq){
	  if(preg_match('|<we:' . $tagname . '[> ]|', $tag))
	  $start++;
	  if(preg_match('|</we:' . $tagname . '[> ]|', $tag))
	  $end++;
	  }else{
	  if(strpos($tag, '<we:' . $tagname) !== false)
	  $start++;
	  if(strpos($tag, '</we:' . $tagname) !== false)
	  $end++;
	  }
	  }
	  }
	  if($start != $end){
	  return we_tag_tag::parseError(sprintf($this->getText() . ': ' . g_l('parser', '[start_endtag_missing]'), $tagname . ((!$eq) ? "..." : "")));
	  }
	  return '';
	  } */

	private static function findIfStart(array $tags, int $nr): int{
		if($nr == 0){
			return -1;
		}
		$foo = [];
		$regs = [];
		for($i = $nr; $i >= 0; $i--){
			if(preg_match('%<(/?)we:if([[:alpha:]]+)( *[[:alpha:]]+ *= *"[^"]*")* */?>?%i', $tags[$i], $regs)){
				if($regs[1] === '/'){
					$foo[$regs[2]] = isset($foo[$regs[2]]) ? $foo[$regs[2]] + 1 : 1;
				} else {
					if(empty($foo)){
						return $i;
					} else if(isset($foo[$regs[2]]) && intval($foo[$regs[2]])){
						$foo[$regs[2]] = intval($foo[$regs[2]]) - 1;
					} else {
						return $i;
					}
				}
			}
		}
		return -1;
	}

	private static function findIfEnd(array $tags, int $nr): int{
		if($nr == count($tags)){
			return -1;
		}
		$foo = [];
		$regs = [];
		for($i = $nr; $i < count($tags); $i++){
			if(preg_match('%<(/?)we:if([[:alpha:]]+)( *[[:alpha:]]+ *= *"[^"]*")* */?>?%i', $tags[$i], $regs)){
				if($regs[1] != '/'){
					$foo[$regs[2]] = isset($foo[$regs[2]]) ? $foo[$regs[2]] + 1 : 1;
				} else {
					if(empty($foo)){
						return $i;
					} else if(isset($foo[$regs[2]]) && intval($foo[$regs[2]])){
						$foo[$regs[2]] = intval($foo[$regs[2]]) - 1;
					} else {
						return $i;
					}
				}
			}
		}
		return -1;
	}

	private static function checkElsetags(array $tags): string{
		for($i = 0; $i < count($tags); $i++){
			if(str_contains($tags[$i], '<we:else')){
				$ifStart = self::findIfStart($tags, $i);
				if($ifStart == -1){
					return we_weTag_tag::parseError(g_l('parser', '[else_start]'));
				}
				if(self::findIfEnd($tags, $i) == -1){
					return we_weTag_tag::parseError(g_l('parser', '[else_end]'));
				}
			}
		}
		return '';
	}

	public static function handleShutdown(string $code): void{
		if(we_base_errorHandler::getHandling('shutdown')){
			$error = error_get_last();
			if(!$error){
				return;
			}
			$tmp = explode("\n", $code);
			$errCode = "\n";
			for($ln = max(0, $error['line'] - 2); $ln <= $error['line'] + 2; $ln++){
				$errCode .= $ln . ': ' . $tmp[$ln] . "\n";
			}

			//FIXME: this->Path ist bei rebuild nicht gesetzt
			t_e('error', 'Error in template:' /* . $this->Path */, $error, 'Code: ' . $errCode);
		}
	}

	public static function checkPhpCode(string $code): void{
		static $call = 0;
		we_base_errorHandler::setHandling('shutdown', true);
		register_shutdown_function([__CLASS__, 'handleShutdown'], $code);
		$replCode = preg_replace('_(use|namespace) (\s*/\*.*\*/[\s*]*)*[\w,\s\\\\]*;_s', '', $code);
		//remove "use" since this is not allowed inside functions
		try{
			$file = we_base_file::saveTemp('<?php function __tmp' . ($call++) . '__(){?>' . $replCode . '<?php } return 1;', '', 5);
			$var = include($file);
		} catch (Throwable $err){
			$error = [
				'line' => $err->getLine(),
				'message' => $err->getMessage(),
			];
		}
		if(empty($var) && ( $error = empty($error) ? error_get_last() : $error )){
			$tmp = explode("\n", $replCode);
			if(!is_array($tmp)){
				$tmp = explode("\r", $code);
			}
			$errCode = "\n";
			$errHtml = '';
			for($ln = max(0, $error['line'] - 2); $ln <= $error['line'] + 2 && isset($tmp[$ln]); $ln++){
				$errCode .= $ln . ': ' . $tmp[$ln] . "\n";
				$errHtml .= '<tr><td>' . $ln . '</td><td>' . htmlspecialchars($tmp[$ln]) . '</td></tr>';
			}

			$errMsg = '<table>
					<tr><td>' . we_html_element::b('Error') . '</td><td>' . $error['message'] . '</td></tr>' .
				'<tr><td>' . we_html_element::b('Line') . '</td><td>' . $error['line'] . '</td></tr>' .
				$errHtml .
				'</table>';
			//type error will stop we
			t_e('warning', 'Error in template: ' . we_weTag_parser::$curFile, $error['message'], 'Line: ' . $error['line'], 'Code: ' . $errCode);
			we_base_errorHandler::setHandling('shutdown', false);
			throw new we_exception_parser('Error in template', $errMsg);
		}
		we_base_errorHandler::setHandling('shutdown', false);
	}

	/**
	 *
	 * @return string
	 * @throws we_exception_parser in case of update unsuccessfull
	 */
	private function parseTemplate(): string{
		we_base_WE::$currentParsedTemplate = $this->ID;
		$code = $this->getTemplateCode(true);
		$tp = new we_weTag_parser($code, $this->getPath());
		$tags = $tp->getAllTags();
		if(($foo = self::checkElsetags($tags))){
			$this->errMsg = $foo;
			return $foo;
		}
		/* if(($foo = self::checkEndtags('if',0,$tags))){
		  return $foo;
		  } */

		try{
			$tp->parseTags($code);
		} catch (we_exception_parser $e){
			$this->errMsg = htmlspecialchars($e->getMessage());
			return $e->getMessage();
		}
		//fix direct variable assignments where an echo is applied
		$code = strtr($code, ['<?php echo __WEVAR__$' => '<?php $', '<?= __WEVAR__$' => '<?php $']);
		if($this->doUpdateCode){
			try{
				self::checkPhpCode($code);
			} catch (we_exception_parser $ex){
				$this->errMsg = $ex->errMsg;
				throw new we_exception_file('error in template');
			}
		}

		$hasHtml = self::hasStartAndEndTag('html', $code);
		$hasHead = self::hasStartAndEndTag('head', $code);
		$hasBody = self::hasStartAndEndTag('body', $code);
		$fullHead = ($hasHtml && $hasHead && $hasBody);
		// echoed in templates with CacheType = document
		$pre_code = '<?php /* Generated template (ID: ' . $this->ID . ') by WE ' . WE_VERSION . ', REV ' . WE_SVNREV . ', ' . date('Y-m-d, H:i') . ' */
we_contents_template::frontEndInit(' . ($fullHead ? 'true' : 'false') . ');?>';

		if($fullHead){
			$code = strtr($code, self::ESCAPE_CHARS);
			//parse base href
			$code = preg_replace([
				'%(<body[^>]*)>%i',
				'%(<head(>| [^>]*>))%i',
				'%(</body[^>]*>)%i',
				'%(</html[^>]*>)%i',
				], [
				'${1}<?= (we_base_WE::$editmode ? \' onload="doScrollTo();" onunload="doUnload()">\':\'>\'); we_contents_template::frontEndPreContent(true);?>',
				'${1}<?php we_contents_template::frontEndHead();?>',
				'<?php we_contents_template::frontEndPostContent(true);?>${1}',
				'${1}<?php we_contents_template::frontEndPost();?>'
				], $code);

			$code = strtr($code, array_flip(self::ESCAPE_CHARS));
		} else if(!$hasHtml && !$hasHead && !$hasBody){
			$code = '<?php we_contents_template::frontEndHead(true);?>' . $code . '<?php we_contents_template::frontEndPostContent(false,true);we_contents_template::frontEndPost();?>';
		} else {
			return we_weTag_tag::parseError(g_l('parser', '[html_tags]')) . '<?php exit();?><!-- current parsed template code for debugging -->' . $code;
		}

		return $pre_code .
			strtr($code, [
				'exit(' => 'we_contents_template::frontEndExit(',
				'die(' => 'we_contents_template::frontEndExit(',
				'exit;' => 'we_contents_template::frontEndExit();'
		]);
	}

	private static function hasStartAndEndTag(string $tagname, string $code): bool{
		return preg_match('%< ?/ ?' . $tagname . '[^>]*>%i', $code) && preg_match('%< ?' . $tagname . '[^>]*>%i', $code) && preg_match('%< ?' . $tagname . '[ >]%i', $code) && preg_match('%< ?/ ?' . $tagname . '[ >]%i', $code);
	}

	protected function isElement(string $Name): bool{
		switch($Name){
			case 'data':
			case self::FULLCODE:
			case we_base_variants::ALLVARIANT_FIELD:
				return true;
			default:
				return str_starts_with($Name, 'variant_');
		}
	}

	protected function setElementsFromHTTP(): void{
		parent::setElementsFromHTTP();
		//get clean variants
		$regs = [];
		foreach($_REQUEST as $n => $v){
			if(is_array($v) && preg_match('|^we_' . preg_quote($this->Name, '|') . '_variant|', (string) $n, $regs)){
				foreach($v as $n2 => $v2){
					if($this->getElementS((string) $n2, 'type') === 'variant' && empty($v2)){
						$this->delElement((string) $n2);
					}
				}
			}
		}
	}

	function getDocumentData(string $includepath = ''): string{
		$this->_updateCompleteCode();
		/* remove unwanted/-needed start/stop parser tags (?><php) */
		return preg_replace(
			["/(:|;|{|})(\r|\n| |\t)*\?>(\r|\n|\t)*<\?=( |\t)*/si", "/(:|;|{|})(\r|\n| |\t)*\?>(\r|\n|\t)*<\?php( |\t)*/si"],
	 ['${1}' . "\n" . '${2} echo ', '${1}' . "\n" . '${2}'], $this->parseTemplate());
	}

	protected function writeSiteDir(string $content): bool{//@phan-suppress-current-line PhanUnusedProtectedMethodParameter
		return true;
	}

	protected function isFilenameNotAllowed(): bool{
		return false;
	}

	/**
	 * returns if this template contains fields required for a shop-document.
	 *
	 * if paramter checkField is true, this function checks also, if there are
	 * already fields selected for the variants.
	 *
	 * @param boolean $checkFields
	 * @return boolean
	 */
	function canHaveVariants(bool $checkFields = false): bool{
		return true;
	}

	/**
	 * the function returns the array with selected variant field names and field attributes/types
	 * @return array<string,array{type:string,content:string,attributes:array}> with the selected field names and attributes
	 */
	function getVariantFields(): array{
		$ret = [];
		$fields = $this->getAllVariantFields();
		if(!$fields){
			return [];
		}
		foreach(array_keys($fields)as $name){
			if($this->getElementS('variant_' . $name)){
				$ret[$name] = $fields[$name];
			}
		}

		return $ret;
	}

	/**
	 * @desc 	the function returns the array with all variant field names
	 * @return string[] with the varinat filed names
	 */
	function getVariantFieldNames(): array{
		$fields = $this->getAllVariantFields();
		return array_keys($fields);
	}

	/**
	 * @desc 	the function returns the array with all template field names and field attributes/types;
	 * 			if there is no fields in the elements, the template code will be parsed
	 * @return	array<string,array> with the filed names and attributes
	 */
	function getAllVariantFields(): array{
		return $this->getElementA(we_base_variants::ALLVARIANT_FIELD);
	}

	/**
	 * @desc 	the function parses the template code and returns all template field names and field attributes/types
	 * @return	array with the filed names and attributes
	 */
	function readAllVariantFields(bool $includedatefield = false): array{
		$templateCode = $this->getTemplateCode();
		$tp = new we_weTag_parser($templateCode, $this->getPath());
		$blocks = $out = [];
		$allTags = $tp->getTagsWithAttributes(false, true);
		'@phan-var array<int,array{name:string,attribs:array<string,string|string[]>,close:bool,weTag:string}> $allTags';
		foreach($allTags as $tag){
			$tagname = $tag['name'];
			if($tag['close']){ // endtag found
				switch($tagname){
					case 'block':
						if(!empty($blocks)){
							array_pop($blocks);
						}
						break;
				}
				continue;
			}


			if(empty($tag['attribs']['name'])){
				continue;
			}
			$name = (string) $tag['attribs']['name'];

			switch($tag['name']){
				case 'var':
				case 'field':
					continue 2;
				default:
					$name = (string) $tag['attribs']['name'];

					if(!empty($blocks)){
						$foo = end($blocks);
						$blockname = (string) $foo['name'];
						switch($foo['type']){
							case 'block':
								$name = self::makeBlockName($blockname, $name);
								break;
						}
					}


					if(in_array($tagname, we_base_variants::VARIANT_TAGS, true)){
						if($tagname === 'input' && !empty($tag['attribs']['type']) && $tag['attribs']['type'] === 'date' && !$includedatefield){
							// do nothing
						} else {
							$out[$name] = [
								'type' => $tagname,
								'attributes' => $tag['attribs'],
							];
						}
					}
					//additional parsing for selects
					switch($tagname){
						case 'select'://additional parsing for selects
							$selregs = [];
							if(preg_match('-' . preg_quote((string) $tag['weTag'], '-') . '(.*)<\s*/\s*we:select\s*>-i', $templateCode, $selregs)){
								$out[$name]['content'] = $selregs[1];
							}
							break;
						case 'block':
							$blocks[] = [
								'name' => $name,
								'type' => $tagname
							];
							break;
					}
			}
		}
		ksort($out);
		return $out;
	}

	function formMasterTemplate(): string{
		$weSuggest = we_gui_suggest::getInstance();
		$table = TEMPLATES_TABLE;
		$textname = 'MasterTemplateNameDummy';
		$idname = 'we_' . $this->Name . '_MasterTemplateID';
		$myid = $this->MasterTemplateID ?: 0;
		$path = fS('SELECT Path FROM ' . $this->DB_WE->escape($table) . ' WHERE ID=' . intval($myid));

		$weSuggest->setAcId('MasterTemplate')
			->setContentType([we_base_ContentTypes::FOLDER_TEMPLATE, we_base_ContentTypes::TEMPLATE])
			->setInput($textname, $path)
			->setLabel('')
			->setResult($idname, (string) $myid)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable($table)
			->setWidth(0)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:'" . $table . "',targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'checkSameMaster," . $this->ID . "',filter:WE().consts.contentTypes.TEMPLATE,open_doc:true})"))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $textname . "'].value='';WE().layout.weSuggest.checkRequired(window,'yuiAcInputMasterTemplate');_EditorFrame.setEditorIsHot(true);"))
			->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:goTemplate(document.we_form.elements.we_" . $this->Name . "_MasterTemplateID.value);"));
		return $weSuggest->getHTML();
	}

	/**
	 *
	 * @return array{int,array<int,array{string,string,string}>}
	 */
	private function isUsedByDocuments(): array{
		if($this->ID == 0){
			return [0, []];
		}
		$where = ' WHERE TemplateID=' . intval($this->ID);
		$cnt = fI('SELECT COUNT(1) FROM ' . FILE_TABLE . $where);
		return [$cnt, $this->DB_WE->getAllFirstq('SELECT ID,SUBSTRING_INDEX(Path,Text,1),CONCAT(Text," (ID: ",ID,")"),IF(IsPublished=0,"notpublished",IF(Mod_Date>Publish_Date,"changed","published")) FROM ' . FILE_TABLE . $where . ' ORDER BY Path LIMIT 100', true)];
	}

	/**
	 *
	 * @return array{int,string}
	 */
	function formTemplateDocuments(): array{
		if($this->ID == 0){
			return [0, g_l('weClass', '[no_documents]')];
		}
		[$count, $elems] = $this->isUsedByDocuments();
		if(!$count){
			return [0, g_l('weClass', '[no_documents]')];
		}
		$path = $elemAttribs = [];
		$oldpath = '';
		foreach($elems as $id => $data){
			if($oldpath != $data[0]){
				$path[$data[0]] = we_html_tools::OPTGROUP;
				$oldpath = $data[0];
			}
			$path[$id] = $data[1];
			$elemAttribs[$id] = $data[2];
		}

		return [$count, we_html_tools::formElementTable('', we_html_tools::select('TemplateDocuments', $path, 1, '', false, ['style' => 'margin-right: 20px;'], true, 0, '', true, $elemAttribs), we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.FILE_TABLE, document.we_form.elements.TemplateDocuments.value, WE().consts.contentTypes.WEDOCUMENT);") .
				we_html_button::create_button(we_html_button::VIEW, "javascript:WE().layout.openBrowser(document.we_form.elements.TemplateDocuments.value);")
		)];
	}

	/**
	 *
	 * @return array{int,string}
	 */
	function formTemplatesUsed(): array{
		if($this->ID == 0 || empty($this->IncludedTemplates)){
			return [0, g_l('weClass', '[no_documents]')];
		}
		$elems = $this->DB_WE->getAllFirstq('SELECT ID,SUBSTRING_INDEX(Path,Text,1),CONCAT(Text," (ID: ",ID,")") FROM ' . TEMPLATES_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND ID IN (' . implode(',', $this->IncludedTemplates) . ') ORDER BY Path', true);

		if(empty($elems)){
			return [0, g_l('weClass', '[no_documents]')];
		}
		$path = [];
		$oldpath = '';
		foreach($elems as $id => $data){
			if($oldpath != $data[0]){
				$path[$data[0]] = we_html_tools::OPTGROUP;
				$oldpath = $data[0];
			}
			$path[$id] = $data[1];
		}

		return [count($elems), we_html_tools::formElementTable('', we_html_tools::select('TemplateUsedTemplates', $path, 1, '', false, ['style' => 'margin-right: 20px;']), we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.TEMPLATES_TABLE, document.we_form.elements.TemplateUsedTemplates.value, WE().consts.contentTypes.TEMPLATE);")
		)];
	}

	/**
	 *
	 * @return array{int,string}
	 */
	function formTemplateUsedByTemplate(): array{
		if($this->ID == 0){
			return [0, g_l('weClass', '[no_documents]')];
		}
		$elems = $this->DB_WE->getAllFirstq('SELECT ID,SUBSTRING_INDEX(Path,Text,1),CONCAT(Text," (ID: ",ID,")") FROM ' . TEMPLATES_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND (FIND_IN_SET(' . $this->ID . ',IncludedTemplates) OR MasterTemplateID=' . $this->ID . ') ORDER BY Path', true);

		if(empty($elems)){
			return [0, g_l('weClass', '[no_documents]')];
		}
		$path = [];
		$oldpath = '';
		foreach($elems as $id => $data){
			if($oldpath != $data[0]){
				$path[$data[0]] = we_html_tools::OPTGROUP;
				$oldpath = $data[0];
			}
			$path[$id] = $data[1];
		}

		return [count($elems), we_html_tools::formElementTable('', we_html_tools::select('TemplateUsedByTemplates', $path, 1, '', false, ['style' => 'margin-right: 20px;']), we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.TEMPLATES_TABLE, document.we_form.elements.TemplateUsedByTemplates.value, WE().consts.contentTypes.TEMPLATE);")
		)];
	}

	/**
	 * @desc 	this function returns the code of the unparsed template
	 * @return	string with the filed names and attributes
	 * @param	boolean $completeCode if true then the function returns the code of the complete template (with master template)
	 */
	function getTemplateCode(bool $completeCode = true/* , bool $fillIncluded = false */): string{
		return $completeCode ? $this->getElementS(self::FULLCODE) : $this->getElementS('data');
	}

	/* setter for runtime variable doUpdateCode which allows save a class without rebuilding everything -> for later rebuild
	  do not access this variable directly, in later WE Versions, it will be protected */

	public function setDoUpdateCode(bool $doUpdateCode = true): void{
		$this->doUpdateCode = $doUpdateCode;
	}

	/* getter for runtime variable doUpdateCode which allows save a class without rebuilding everything -> for later rebuild
	  do not access this variable directly, in later WE Versions, it will be protected */

	public function getDoUpdateCode(): bool{
		return $this->doUpdateCode;
	}

	static function getUsedTemplatesOfTemplate(int $id, array &$arr): void{
		$hash = getHash('SELECT IncludedTemplates, MasterTemplateID FROM ' . TEMPLATES_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::TEMPLATE . '" AND ID=' . $id);
		$tmplCSV = ($hash ? $hash['IncludedTemplates'] : '');
		$tmpArr = array_map('intval', array_filter(explode(',', $tmplCSV)));
		$tmpArr[] = (int) ($hash ? $hash['MasterTemplateID'] : 0);

		$newTemplates = [];

		foreach($tmpArr as $tid){
			if($tid && $tid != $id && !in_array($tid, $arr)){
				$arr[] = $tid;
				$newTemplates[] = $tid;
			}
		}
		foreach($newTemplates as $tid){
			if($id != $tid){
				self::getUsedTemplatesOfTemplate($tid, $arr);
			}
		}
	}

	function _updateCompleteCode(): void{
		if(!$this->doUpdateCode || defined('IMPORT_RUNNING')){
			return;
		}
		static $cnt = 0;
		static $recursiveTemplates = [];
		if($cnt == 0){
			self::$usedReplacements = [];
			$recursiveTemplates = [$this->ID];
		}
		++$cnt;
		$code = $this->getTemplateCode(false);

		// find all we:master Tags
		$masterTags = $regs = [];

		if(preg_match_all('|(<we:master([^>+]*)>)\n?([\\s\\S]*?)</we:master>\n?|', $code, $regs, PREG_SET_ORDER)){
			foreach($regs as $reg){
				$attribs = we_weTag_parser::parseAttribs($reg[2] ?? '');
				if(!empty($attribs['name'])){
					$masterTags[$attribs['name']] = $reg[3] ?? '';
				}
			}
			$code = str_replace(array_column($regs, 0), '', $code);
		}

		if($this->MasterTemplateID){
			$templates = [];
			self::getUsedTemplatesOfTemplate($this->MasterTemplateID, $templates);
			if($this->ID == $this->MasterTemplateID ||
				in_array($this->ID, $templates) ||
				in_array($this->MasterTemplateID, $recursiveTemplates)){
				t_e(g_l('parser', '[template_recursion_error]'), 'Template ' . $this->ID, 'Mastertemplate: ' . $this->MasterTemplateID, 'Templates of Master: ' . implode(',', $templates), 'already processed: ' . implode(',', $recursiveTemplates));
				$this->setElementS(self::FULLCODE, g_l('parser', '[template_recursion_error]'));
				--$cnt;
				return;
			}
			// we have a master template. => surround current template with it
			// first get template code
			$recursiveTemplates[] = $this->MasterTemplateID;
			$templObj = new we_contents_template($this->MasterTemplateID);
			$masterTemplateCode = preg_replace('_(<we:(ifContent|ifNotContent))(( ?(?!template).)*>)_', '${1} template="' . $this->MasterTemplateID . '"${3}', $templObj->getTemplateCode(true));

			array_pop($recursiveTemplates);
			$contentTags = [];
			if(preg_match_all('|<we:content ?([^>+]*)/?>\n?|', $masterTemplateCode, $contentTags, PREG_SET_ORDER)){
				foreach($contentTags as $reg){
					$attribs = we_weTag_parser::parseAttribs($reg[1]);
					$name = $attribs['name'] ?? '';
					$val = ($name ? ( $masterTags[$name] ?? '') : $code);
					$masterTemplateCode = str_replace($reg[0], $val, $masterTemplateCode);
					self::$usedReplacements[$this->MasterTemplateID][$name] = trim($val);
				}
			}
			$code = $masterTemplateCode;
		}
		$this->IncludedTemplates = [];
		// look for included templates (<we:include type="template" id="99">)
		$tp = new we_weTag_parser($code, $this->getPath());
		// go through all tags
		$regs = [];
		foreach($tp->getTagsWithAttributes() as $tag){
			'@phan-var array{name:string,attribs:array<string,string|string[]>,close:bool,weTag:string} $tag';
			// search for include tag
			if($tag['name'] !== 'include' || empty($tag['attribs']['type']) || $tag['attribs']['type'] !== 'template'){
				continue;
			}

			// if id attribute is set and greater 0
			if(empty($tag['attribs']['id']) || !($tmplID = (int) $tag['attribs']['id'])){
				if(empty($tag['attribs']['path']) || !($tmplID = path_to_id((string) $tag['attribs']['path'], TEMPLATES_TABLE))){
					continue;
				}
			}

			$templates = [];
			self::getUsedTemplatesOfTemplate($tmplID, $templates);
			if(in_array($this->ID, $templates) || $tmplID === $this->ID || in_array($tmplID, $recursiveTemplates)){
				t_e(g_l('parser', '[template_recursion_error]'), 'Template ' . $this->ID, 'Included Template: ' . $tmplID, 'Templates of include: ' . implode(',', $templates), 'already processed: ' . implode(',', $recursiveTemplates));
				$this->setElementS(self::FULLCODE, /* str_replace($tag['weTag'], */ g_l('parser', '[template_recursion_error]'), $code/* ) */);
				--$cnt;
				return;
			}
			// get code of template
			$recursiveTemplates[] = $tmplID;
			array_pop($recursiveTemplates);
			// replace include tag with template code
			$this->IncludedTemplates[] = $tmplID;
		}

		$this->setElementS(self::FULLCODE, $code);
		--$cnt;
		if($cnt === 0){
			$keys = array_reverse(array_keys(self::$usedReplacements));
			$lastKey = array_shift($keys);
			foreach($keys as $curKey){
				foreach(self::$usedReplacements[$curKey] as &$cur){
					'@phan-var string $cur';
					foreach(self::$usedReplacements[$lastKey] as $name => $last){
						'@phan-var string $last';
						$cur = trim((string) preg_replace('|<we:content ?([^>+]*' . ($name ? '[\'"]' . $name . '[\'"]' : '(?!name=)*') . '[^>+]*)/?>|', $last, $cur));
					}
				}
				$lastKey = $curKey;
			}
		}
	}

	public function we_save(bool $resave = false, bool $updateCode = true): bool{
		//$this->Extension = count(we_base_ContentTypes::getExtension(we_base_ContentTypes::TEMPLATE)) ? we_base_ContentTypes::getExtension(we_base_ContentTypes::TEMPLATE)[0] : '';
		if($updateCode){
			$this->_updateCompleteCode();
			$this->setElementS(we_base_variants::ALLVARIANT_FIELD, we_serialize($this->readAllVariantFields(), SERIALIZE_JSON), 'variants');
		} else {
			$this->doUpdateCode = false;
		}
		$ret = parent::we_save($resave);
		if($ret){
			$tmplPathWithTmplExt = parent::getRealPath();
			we_base_file::deleteLocalFile($tmplPathWithTmplExt);
			$this->unregisterMediaLinks();
			$ret = $this->registerMediaLinks();
		} else {
			t_e('save template failed', $this->Path);
		}

		$this->setElementA(we_base_variants::ALLVARIANT_FIELD, we_unserialize($this->getElementS(we_base_variants::ALLVARIANT_FIELD)), 'variants');
		return $ret;
	}

	public function we_publish(): bool{
		if(VERSIONS_CREATE_TMPL){
			we_versions_version::save($this, 'published');
		}
		return true;
	}

	protected function we_load(int $from = self::LOAD_MAIN_DB): bool{
		$ret = parent::we_load(self::LOAD_MAIN_DB);
		//$this->Extension = count(we_base_ContentTypes::getExtension(we_base_ContentTypes::TEMPLATE)) ? we_base_ContentTypes::getExtension(we_base_ContentTypes::TEMPLATE)[0] : '';
		$this->_updateCompleteCode();
		$tmp = $this->getElementS(we_base_variants::ALLVARIANT_FIELD);
		$this->setElementA(we_base_variants::ALLVARIANT_FIELD, ($tmp ?
				we_unserialize($tmp) :
				$this->readAllVariantFields((bool) ($this->getElementS(self::FULLCODE)))
			), 'variants');

		return $ret;
	}

	function registerMediaLinks(bool $temp = false, bool $linksReady = false): bool{
		$tp = new we_weTag_parser($this->getTemplateCode());
		$c = 0;
		foreach($tp->getTagsWithAttributes() as $tag){
			$element = $tag['name'] . '[name=' . ($tag['attribs']['name'] ?? 'NN' . ++$c) . ']';
			$id = !empty($tag['attribs']['id']) ? intval($tag['attribs']['id']) : 0;
			switch($tag['name']){
				case 'icon':
				case 'img':
				case 'video':
					if($id){
						$this->MediaLinks[$element] = $id;
					}
					break;
				case 'url':
					if((empty($tag['attribs']['type']) || $tag['attribs']['type'] === 'document') && $id){
						$this->MediaLinks[$element] = $id;
					}
					break;
				case 'include': //nur type=document: id, path
					if(isset($tag['attribs']['type']) && $tag['attribs']['type'] === 'document'){
						if($id){
							$this->MediaLinks[$element] = $id; // selector: text/webEdition only
						}
						if(!empty($tag['attribs']['path']) && ($id = path_to_id((string) $tag['attribs']['path']))){
							$this->MediaLinks[$element] = $id; // selector: text/webEdition only
						}
					}
					break;
				case 'link':
					if($id){
						$this->MediaLinks[$element] = $id;
					}
					if(!empty($tag['attribs']['imageid']) && is_numeric($tag['attribs']['imageid'])){
						$this->MediaLinks[$element] = intval($tag['attribs']['imageid']);
					}
					break;
				case 'sessionfield':
					if(isset($tag['attribs']['type']) && $tag['attribs']['type'] === 'img' && $id){
						$this->MediaLinks[$element] = $id;
					}
					break;

				// the following cases are not meant to link media files: we check them anyway
				case 'a': // selector: text/webEdition only
				case 'linkToSeeMode':// selector: text/webEdition only
				case 'metadata':// selector: text/webEdition only
					if($id){
						$this->MediaLinks[$element] = $id;
					}
					break;
				case 'listview':
					if(isset($tag['attribs']['type']) && $tag['attribs']['type'] === 'document' && $id){
						foreach(we_base_request::filterIntList($tag['attribs']['id']) as $id){
							$this->MediaLinks[$element] = $id;
						}
					}
					break;
				default:
				//
			}
		}
		return (empty($this->MediaLinks) ?
			true :
			parent::registerMediaLinks(false, true));
	}

	// .tmpl mod

	function getRealPath(bool $old = false): string{
		return preg_replace('/.tmpl$/i', '.php', parent::getRealPath($old));
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd): string{
		return (new we_html_multiIconBox((new we_html_multiIconBoxContent())
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[path]'), $this->formPath(), we_html_multiIconBox::PROP_PATH)))
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[master_template]'), $this->formMasterTemplate(), we_html_multiIconBox::PROP_MASTER)))
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[copyTemplate]'), $this->formCopyDocument(), we_html_multiIconBox::PROP_COPY)))
					, 'PropertyPage'))
				->marginLeft()
				->get();
	}

	public function formPath(bool $disablePath = false, bool $notSetHot = false, string $extra = ''): string{
		$extra = we_html_element::br() .
			$this->formInput('', 'Display', g_l('navigation', '[display]'), 0, 255, ['onchange' => ($notSetHot ? '' : "we_cmd('setHot');")]);
		return parent::formPath($disablePath, $notSetHot, $extra);
	}

	/**
	 *
	 * @param bool $css
	 * @param bool[] $setting
	 * @return string
	 */
	public static function we_getCodeMirror2Tags(bool $css, array $setting): string{
		$cacheName = we_cache_file::DOMAIN_TW . 'cmTags_' . ($css ? '1' : '0') . '_' . md5(we_serialize($setting, SERIALIZE_PHP));
		try{
			$cache = we_cache_file::load($cacheName);
			return ($css ?
				implode('', $cache) :
				'WE().layout.editors.CodeMirror={weHints:{' . implode(',', $cache) . '}};');
		} catch (we_exception_cache $e){

		}
		$ret = $allTags = [];
		if($css || !empty($setting['WE'])){
			$allWeTags = we_wizard_tag::getExistingWeTags($css); //only load deprecated tags if css is requested
			foreach($allWeTags as $tagName){
				try{
					$weTag = we_weTag_twbase::getTagData((string) $tagName);
					if($css){
						$ret[] = '.cm-weTag_' . $tagName . ':hover:after {content: "' . strtr(html_entity_decode($weTag->getDescription(), ENT_COMPAT, 'UTF-8'), [
								'"' => '\'',
								"\n" => ' ']) . '";}';
					} else {
						$allTags['we:' . $tagName] = ['we' => $weTag->getAttributesForCM()];
					}
				} catch (Throwable $e){

				}
			}
		}

		if($css){
			we_cache_file::save($cacheName, $ret);
			return implode('', $ret);
		}

		$allTags = (empty($setting['htmlTag']) ? $allTags : array_merge($allTags, we_base_htmlTags::HTML));
		if(!$allTags){
			return '';
		}
		//keep we tags in front of ordinal html tags
		$ret[] = '"<":["' . implode('","', array_keys($allTags)) . '"]';

		ksort($allTags);
		foreach($allTags as $tagName => $cur){
			$attribs = [];
			foreach($cur as $type => $attribList){
				switch($type){
					case 'we':
						$ok = true;
						break;
					case 'default':
						$ok = (!empty($setting['htmlDefAttr']));
						break;
					case 'js':
						$ok = (!empty($setting['htmlJSAttr']));
						break;
					case 'norm':
						$ok = (!empty($setting['htmlAttr']));
						break;
					default:
						$ok = false;
				}
				if($ok){
					foreach($attribList as $attr){
						$attribs[] = '\'' . $attr . (strstr($attr ?? '', '"') === false ? '=""' : '') . '\'';
					}
				}
			}
			if($attribs){
				$attribs = array_unique($attribs);
				sort($attribs);
				$ret[] = '"<' . $tagName . ' ":[' . implode(',', $attribs) . ']';
			}
		}
		we_cache_file::save($cacheName, $ret);
		return 'WE().layout.editors.CodeMirror={weHints:{' . implode(',', $ret) . '}};';
	}

	public static function we_getCSSIds(int $tid): string{
		$data = fS('SELECT Dat FROM ' . CONTENT_TABLE . ' c WHERE DocumentTable="tblTemplates" AND DID=' . $tid . ' AND nHash=x\'' . md5('data') . '\'');
		if(!$data){
			return '';
		}
		$tp = new we_weTag_parser($data);
		$query = ['document' => [], 'template' => [], 'object' => [], 'class' => [], 'user' => []];

		foreach($tp->getTagsWithAttributes() as $tag){
			if(isset($tag['attribs']['id']) && intval($tag['attribs']['id'])){
				switch($tag['name']){
					case 'object':
						$type = 'object';
						break;
					case 'ifTemplate':
						$type = 'template';
						break;
					default:
						$type = ($tag['attribs']['type'] ?? 'document');
				}
				$query[$type][] = intval($tag['attribs']['id']);
			}
			foreach([
			'classid' => 'class',
			'tid' => 'template', //we:write
			'workflowuserid' => 'user',
			'userid' => 'user',
			'triggerid' => 'document',
			'startid' => 'document',
			'parentid' => 'document',
			'documentid' => 'document',
			'docid' => 'document',
			'objectid' => 'object',
			'thumbnailid' => 'object',
			] as $attr => $type){
				if(isset($tag['attribs'][$attr]) && intval($tag['attribs'][$attr])){
					$query[$type][] = intval($tag['attribs'][$attr]);
				}
			}
		}
		foreach($query as $type => &$ids){
			if(!$ids){
				continue;
			}
			$table = FILE_TABLE;
			switch($type){
				default:
				case 'document':
					$table = FILE_TABLE;
					break;
				case 'template':
					$table = TEMPLATES_TABLE;
					break;
				case 'user':
					$table = USER_TABLE;
					break;
				case 'object':
					if(!we_modules_info::isActive(we_modules_info::OBJECT)){
						$ids = [];
						continue 2;
					}
					$table = OBJECT_FILES_TABLE;
					break;
				case 'class':
					if(!we_modules_info::isActive(we_modules_info::OBJECT)){
						$ids = [];
						continue 2;
					}
					$table = OBJECT_TABLE;
					break;
			}
			$ids = DB_WE::$inst->getAllFirstq('SELECT ID,Path FROM ' . $table . ' WHERE ID IN (' . implode(',', array_unique($ids, SORT_NUMERIC)) . ')', false);
		}

		$ret = '';
		foreach($query as $type => $docs){
			foreach($docs as $id => $path){
				$ret .= '.cm-we' . $type . 'ID-' . $id . ':hover:after {content: "' . $path . '";}';
			}
		}
		return $ret;
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'fill_required_fields' => g_l('taged', '[fill_required_fields]'),
				'no_type_selected' => g_l('taged', '[no_type_selected]'),
		]);
	}

	public static function getJSTWConsts(): string{
		// Code Wizard
		$tagGroups = we_wizard_tag::getWeTagGroups();
		$groupJs = [];
		foreach($tagGroups as $tagGroupName => $tags){
			switch($tagGroupName){
				case 'custom_tags':
					$tagGroupName = 'custom';
			}
			$groupJs[$tagGroupName] = $tags;
		}

		return we_base_util::setLangString([
				'groups' => $groupJs
		]);
	}

	public function getLockedpageNr(): int{
		return we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE;
	}

	public function getDocumentCode(): string{
		return $this->getElementS('data');
	}

	public static function frontEndInit(bool $fullHead): void{
		if(we_base_WE::$template['first']){
			we_base_WE::$template['first'] = false;
			we_base_WE::$template['head'] = we_base_WE::$template['head'] || $fullHead;
		} else {
			++we_base_WE::$template['init'];
			we_base_WE::$template['head'] = we_base_WE::$template['head'] || $fullHead;

			if(!isset($GLOBALS['DB_WE'])){
				$GLOBALS['DB_WE'] = DB_WE::getStatic();
			}
//check for Trigger
			if(!we_base_WE::$schedulerActive &&
				!we_base_WE::$versioningActive &&
				we_modules_info::isActive(we_modules_info::SCHEDULER) &&
				(!we_base_WE::$we_doc->InWebEdition) &&
				(SCHEDULER_TRIGGER === we_schedule_scheduler::TRIGGER_PREDOC) &&
				(!we_tag_include::isIncludeActive()) //on first call this variable is unset, so we're not inside an include
			){
				$tmp = we_base_WE::save();
				we_schedule_scheduler::trigger_schedule(1);
				we_base_WE::load($tmp);
			}
			/* if(we_base_WE::$we_doc->InWebEdition){
			  header('X-XSS-Protection: 0');
			  } */
		}
		if(we_base_WE::$we_doc && (!we_base_WE::$mainDoc['id'] || we_base_WE::$template['currentDocID'] != we_base_WE::$we_doc->ID)){
			we_base_WE::$template['currentDocID'] = we_base_WE::$we_doc->ID;
			if(!we_base_WE::$mainDoc['id']){//check for id, cause scheduler sets we_editmode
				//check if CHARSET is valid
				we_html_tools::headerCtCharset('text/html', true);

				we_base_WE::$mainDoc = [
					'id' => we_base_WE::$we_doc->ID, //this is always the id of an document!
					'doc' => (we_base_WE::$we_obj ?? we_base_WE::$we_doc), //note this cannot be a refence, since this is changed dynamically so an included file would overwrite the main class
					'editmode' => we_base_WE::$editmode,
				];
				//FIXME:remove in Cardarra, just for compatibility
				$GLOBALS['WE_MAIN_ID'] = we_base_WE::$mainDoc['id'];
				$GLOBALS['WE_MAIN_DOC'] = we_base_WE::$mainDoc['doc'];
			}
		}
	}

	public static function frontEndHead(bool $fullHeader = false): void{
		if(!we_base_WE::$mainDoc['doc']->InWebEdition ||
			((!we_base_WE::$editmode) && isset($_SESSION['weS']) && we_editor_functions::isNormalMode())){
			if(we_base_WE::$mainDoc['id'] === we_base_WE::$we_doc->ID){
				if(!$fullHeader && we_base_WE::$template['head'] && we_base_WE::$template['preContent'] == 0){
					if(we_base_WE::$mainDoc['doc']->InWebEdition){
						echo we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
						we_html_element::jsScript(JS_DIR . 'pushTopCmd.js');
					} elseif(WE_LICENSE === 'GPL'){
						echo '<!--
This website is powered by webEdition Content Management System.
For more information see: https://www.webEdition.org
-->
<meta name="generator" content="webEdition CMS"/>';
					}
				}
				return;
			}
		}
		if($fullHeader && we_base_WE::$template['head']){
			self::frontEndPreContent(); //to increment we_templatePreContent-var
			return;
		}
		if(!we_base_WE::$editmode){
			return;
		}
		echo ($fullHeader ? we_html_element::docType() . '<html><head><title>WE</title>' . we_html_tools::metaCtCharset() : '') .
		we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();parent.openedWithWE=true;') .
		we_html_tools::getStyleSheet(true) .
		we_html_tools::getJquery() .
		we_editor_script::get(we_base_WE::getWeDoc());
		if($fullHeader){
			echo '</head><body onload="doScrollTo();" onunload="doUnload()">';
			self::frontEndPreContent();
			we_base_WE::$template['head'] = true;
		}
	}

	/**
	 * show form tag in case we are in WE-Editmode
	 * @param bool $force
	 */
	public static function frontEndPreContent(bool $force = false): void{//force is used by templates with a full html/body.
		if(MAINTENANCE_MODE && in_array(MAINTENANCE_MODE, [we_base_constants::MAINTENANCE_FRONTEND, we_base_constants::MAINTENANCE_ALL], true)){
			echo we_html_element::h(1, 'Wartungsmodus / Maintenance mode', ['style' => 'background-color:red;color:white;text-align:center;', 'class' => 'maintenance']);
		}
		if(!we_base_WE::$editmode){
			return;
		}
		if($force || (!we_base_WE::$template['head'] && we_base_WE::$template['preContent'] == 0)){
			echo '<form name="' . we_base_constants::WE_FORM . '" method="post" onsubmit="return false;">' .
			we_contents_base::hiddenTrans();
		}
		we_base_WE::$template['preContent'] = we_base_WE::$template['preContent'] ? we_base_WE::$template['preContent'] + 1 : we_base_WE::$template['init'];
	}

	/**
	 * close existing form tag in case we are in WE-Editmode
	 * @param bool $force
	 * @param bool $fullPoster if the template does not have
	 */
	public static function frontEndPostContent(bool $force = false, bool $fullPoster = false): void{//force on </body tag
		if($force || (we_base_WE::$editmode && we_base_WE::$template['preContent'] != 0 && --we_base_WE::$template['preContent'] == 0) && we_base_WE::$mainDoc['id'] === we_base_WE::$we_doc->ID){
			$out = (we_base_WE::$template['head'] && WE_LICENSE === 'GPL' ? '<div style="z-index: 1000;text-align:right;font-size:10px;color:grey;display:block !important;"><a style="display:inline !important;" title="This page was created with webEdition CMS" href="https://www.webEdition.org" target="_blank" rel="noopener">powered by webEdition CMS</a></div>' : '');
			if(we_base_WE::$editmode){
				if($force){//never do this again
					we_base_WE::$template['preContent'] = -10000;
				}
				//FIXME: check this new field to determine if all data has been transmitted
				$out .= we_html_element::hidden('we_complete_request', 1) .
					'</form>' .
					(we_wysiwyg_editor::isWysiwygInstances() ? we_wysiwyg_editor::getHTMLConfigurationsTag() : '') .
					($fullPoster ? '</body></html>' : '');
			}
			echo $out;
		}
	}

	public static function frontEndPost(): void{
		if(--we_base_WE::$template['init'] == 0 && !isWE()){
			echo we_wysiwyg_editor::isWysiwygInstances() ? we_wysiwyg_editor::getHTMLConfigurationsTag() : '';

			if(!empty($_SESSION['webuser']) && isset($_SESSION['webuser']['loginfailed'])){
				unset($_SESSION['webuser']['loginfailed']);
			}
			if(defined('DEBUG_MEM')){
				weMemDebug();
				p_r(get_included_files());
			}
			$handler = ob_get_level() ? (ob_list_handlers() ?: []) : [];

			if(!empty(array_diff($handler, ['zlib output compression'])) && (end($handler) == 'default output handler')){//if still document active, we have to do url replacements
				$urlReplace = we_folder_base::getUrlReplacements(DB_WE::$inst);
// --> Glossary Replacement
				$useGlossary = ((we_modules_info::isActive(we_modules_info::GLOSSARY) && (!we_base_WE::$mainDoc['id'] || we_base_WE::$mainDoc['id'] == we_base_WE::$we_doc->ID)) && (isset(we_base_WE::$we_doc->InGlossar) && we_base_WE::$we_doc->InGlossar == false) && we_glossary_replace::useAutomatic());
				$content = ob_get_clean() ?: '';
				if($useGlossary){
					$content = we_glossary_replace::replace($content, we_base_WE::$we_doc->Language);
				}
				if($urlReplace){
					$content = preg_replace($urlReplace, array_keys($urlReplace), $content);
				}
				if(we_base_WE::$schedulerActive || we_base_WE::$versioningActive){
					//restart, since we need the content
					ob_start();
				}
				echo $content;
			}
			//check for Trigger
			if(!we_base_WE::$schedulerActive && !we_base_WE::$versioningActive && we_modules_info::isActive(we_modules_info::SCHEDULER) && (!we_base_WE::$mainDoc['doc']->InWebEdition) &&
				(SCHEDULER_TRIGGER == we_schedule_scheduler::TRIGGER_POSTDOC) &&
				(!we_tag_include::isIncludeActive())//not inside an included Doc
			){ //is set to Post or not set (new default)
				we_base_request::finishRequest();
				$tmp = we_base_WE::save();
				we_schedule_scheduler::trigger_schedule(4);
				we_base_WE::load($tmp);
			}
		}
	}

	public static function frontEndExit($param = 0): void{
		if(defined('FROM_WE_SHOW_DOC')){
			exit($param);
		}
//we are inside we, we don't terminate here
		if($param){
			echo $param;
		}
//FIXME: use g_l
		t_e('template forces document to exit, see Backtrace for template name. Message of statement', $param);
	}

	public function getDocumentState(): int{
		return self::STATE_PUBLISHED;
	}

	/**
	 * @return bool
	 * @desc	checks if a document is restricted to several users and if
	  the user is one of the restricted users
	 */
	function userHasPerms(): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR, we_base_perm::WORK_TEMPLATES_CAN_SEE])){
			return true;
		}
		return false;
	}

}
