<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: db41555d8f2c $
 * $Author: Lukas Imhof $
 * $Date: 2021-03-18 13:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_linkWysiwyg extends we_dialog_link{
	/** @var string */
	protected $wysiwygInternalLink = '';

	/** @var array<string,string> */
	protected $dialogToWedoc = [];

	public function __construct(bool $noInternals = true){
		$this->isTinyDialog = true;

		parent::__construct($noInternals);
	}

	protected function initByHttp(): void{
		$this->wysiwygInternalLink = we_base_request::string('wysiwygInternalLink', '');
		parent::initByHttp();
	}

	protected function getJs(): string{
		return we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_plugin.js') .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'plugins/welink/js/welink.js') .
			parent::getJs();
	}

	/**
	 *
	 * @return array{editname:string,classNames:string,is_frontend:bool}
	 */
	protected function getJsDynamic_(): array{
		return [
			'editname' => '',
			'classNames' => ($this->cssClasses ? implode(',', $this->cssClasses) : 'getFromTiny'),
			'is_frontend' => $this->noInternals
		];
	}

	function initLinkFromSource(): void{
		$this->type = $this->noInternals ? we_base_link::TYPE_EXT : we_base_link::TYPE_INT;
		$ref = '';
		//$href =  $_REQUEST['we_dialog_args']['href'];// we_base_request::_(we_base_request::STRING, 'we_dialog_args', '', 'href');
		if(($tmpLink = $this->wysiwygInternalLink)){
			if(str_contains($tmpLink, '?') || str_contains($tmpLink, "#")){
				$urlparts = parse_url($tmpLink);
				$this->params = $urlparts['query'] ?? '';
				$this->anchor = $urlparts['fragment'] ?? '';
				[$tmpLink] = explode('?', $tmpLink, 2);
				[$tmpLink] = explode('#', $tmpLink, 2);
				$tmpLink = trim($tmpLink, '/?#');
			}

			// FIXME: refactor
			if(!$tmpLink && $this->anchor){
				$prefix = we_base_link::TYPE_ANCHOR_PREFIX;
			} else {
				$prefix = $ref = '';
				$hrefParts = explode(':', $tmpLink);
				if(count($hrefParts) === 2){
					[$prefix, $ref] = $hrefParts;
					$prefix .= ':';
				}

				// Object Links and internal links are not possible when outside webEdition
				if(($this->noInternals /* || (isset($this->args['outsideWE']) && $this->args['outsideWE'] == 1) */) &&
					(in_array($prefix, [we_base_link::TYPE_OBJ_PREFIX, we_base_link::TYPE_INT_PREFIX], true))
				){
					$this->extHref = $prefix = '';
				}
			}

			// IMPORTANT: parts of this can be moved to parent::initByHttp
			$this->params = str_replace('&amp;', '&', $this->params);
			$prefix = $this->noInternals && in_array($prefix, [we_base_link::TYPE_INT_PREFIX, we_base_link::TYPE_OBJ_PREFIX]) ? '' : $prefix;
			switch($prefix ){
				case we_base_link::TYPE_OBJ_PREFIX:
					$this->type = we_base_link::TYPE_OBJ;
					$this->objID = (int) $ref;
					break;
				case we_base_link::TYPE_INT_PREFIX:
					$this->type = we_base_link::TYPE_INT;
					$this->intID = (int) $ref;
					break;
				case we_base_link::TYPE_MAIL_PREFIX:
					$this->type = we_base_link::TYPE_MAIL;
					$match = [];
					//preg_match('|^([^\?#]+).*$|', $ref, $match);
					preg_match('|(subject=([^&]*)&?)?(cc=([^&]*)&?)?(bcc=([^&]*)&?)?|', (string) $this->params, $match);
					$this->subject = isset($match[2]) ? rawurldecode($match[2]) : '';
					$this->cc = isset($match[4]) ? rawurldecode($match[4]) : '';
					$this->bcc = isset($match[6]) ? rawurldecode($match[6]) : '';
					$this->emailHref = (string) $ref;
					$this->params = '';
					break;
				case we_base_link::TYPE_ANCHOR_PREFIX:
					$this->type = $this->noInternals ? we_base_link::TYPE_EXT : we_base_link::TYPE_INT;
					$this->intID = 0;
					$this->params = '';
					break;
				default:
					$this->type = we_base_link::TYPE_EXT;
					$this->extHref = $tmpLink ? preg_replace(['|^' . WEBEDITION_DIR . 'we_cmd.php[^"\'#]+(#.*)$|',
							'|^' . WEBEDITION_DIR . '|',
							'|^([^\?#]+).*$|'
							], ['${1}',
							'',
							'${1}'
							], rawurldecode($tmpLink)) : we_base_link::EMPTY_EXT;
			}
		}
	}

	protected function cmdFunction(array $args): string{
		$link = $this->makeLink();
		//'@phan-var array{type:string,params:string,anchor:string,emaillink:string,intID:int,intHref:string,extHref:string,objHref:string,objID:int} $link';
		switch($link['type'] ?? ''){
			case we_base_link::TYPE_INT:
				$link['internalLink'] = !empty($link['intID']) ? we_base_link::TYPE_INT_PREFIX . $link['intID'] : '';
				break;
			case we_base_link::TYPE_EXT:
				$link['internalLink'] = !empty($link['extHref']) && (empty($args['href']) || $args['href'] !== we_base_link::EMPTY_EXT) ? $link['extHref'] : '';
				break;
			case we_base_link::TYPE_OBJ:
				$link['internalLink'] = !empty($link['objID']) ? we_base_link::TYPE_OBJ_PREFIX . $link['objID'] : '';
				break;
			case we_base_link::TYPE_MAIL:
				$query = [];
				if(!empty($link['subject'])){
					$query['subject'] = $link['subject'];
				}
				if(!empty($link['cc'])){
					$query['cc'] = $link['cc'];
				}
				if(!empty($link['bcc'])){
					$query['bcc'] = $link['bcc'];
				}
				$link['internalLink'] = '';
				if(!empty($link['emailHref'])){
					$link['internalLink'] = we_base_link::TYPE_MAIL_PREFIX . $link['emailHref'] .
						(empty($query) ? '' : '?' . http_build_query($query, '', '&', PHP_QUERY_RFC3986) );
				}
				break;
		}

		if(!empty($link['type']) && $link['type'] !== we_base_link::TYPE_MAIL){
			$tmp = $link['internalLink'];
			$anchor = '';
			$params = '';

			if(strpos($tmp, '#') !== false){
				[$tmp, $anchor] = explode('#', $tmp, 2);
			}
			if(strpos($tmp, '?') !== false){
				[$tmp, $params] = explode('?', $tmp, 2);
			}

			$params = trim($link['params'] ?? $params, '?& ');
			if(!empty($params)){
				$tmp2 = [];
				parse_str($params, $tmp2);
				$params = '?' . http_build_query($tmp2, '', '&', PHP_QUERY_RFC3986);
			}
			$anchor = trim($link['anchor'] ?? $anchor, '# ');

			$link['internalLink'] = $tmp . $params . ($anchor ? '#' . $anchor : '');
		}
		$payload = ['attributes' => $link, 'isWeDialog' => $this->isTinyDialog];

		return we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_cmdFrame.js', "we_cmd('link_writeback')", [
					'id' => 'loadVarDialog_cmdFrame',
					'data-payload' => setDynamicVar($payload)
				]), we_html_element::body());
	}

}
