<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07c8118a2d3d $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-15 13:44 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_editor_thumbnails extends we_editor_base{

	/**
	 * @var null|we_document_image
	 */
	protected $we_doc = null;

	public function show(): string{
		$parts = new we_html_multiIconBoxContent();
		if(we_base_imageEdit::gd_version()){
			$doc = $this->we_doc->getDocument();
			$fname = $this->we_doc->getFilename();
			$imgType = we_base_imageEdit::detect_image_type($fname);

			if(!$fname){
				$parts->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('thumbnails', '[no_image_uploaded]'), we_html_tools::TYPE_INFO, 700)));
			} else if(we_base_imageEdit::is_imagetype_read_supported($imgType)){
				// look if the fields origwidth & origheight exixts. If not get and set the values
				if((!$this->we_doc->issetElement('origwidth')) || (!$this->we_doc->issetElement('origheight'))){
					$arr = $this->we_doc->getOrigSize();
					$this->we_doc->setElementI('origwidth', $arr[0], 'attrib');
					$this->we_doc->setElementI('origheight', $arr[1], 'attrib');
					unset($arr);
				}

				$thumbs = $this->we_doc->getThumbs();
				foreach($thumbs as $thumbid){
					'@phan-var int $thumbid';
					$thumbObj = new we_contents_thumbnail();
					$thumbObj->initByThumbID($thumbid, $this->we_doc->ID, $this->we_doc->Filename, $this->we_doc->Path, $this->we_doc->Extension, $this->we_doc->getElementI('origwidth'), $this->we_doc->getElementI('origheight'), $doc);


					$useOrig = $thumbObj->isOriginal();

					if((!$useOrig) && $this->we_doc->ID && (!$this->we_doc->DocChanged) && file_exists($thumbObj->getOutputPath(true))){
						$src = $thumbObj->getOutputPath(false, true);
					} else {
						$src = WEBEDITION_DIR . 'we_cmd.php?' . http_build_query([
									'we_cmd' => [
										0 => 'we_base_showBinaryDoc',
										1 => $this->we_doc->ContentType,
										2 => we_base_WE::$transaction,
										3 => ($useOrig ? '' : (int) $thumbid),
									],
									'm' => filemtime(WEBEDITION_PATH . '..' . $this->we_doc->Path)
								])
						;
					}

					$delbut = we_html_button::create_button(we_html_button::TRASH, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('del_thumb','" . $thumbid . "');");

					$thumbnail = '<table class="default" style="width:570px;"><tr><td style="width:538px;"><img src="' . $src . '" style="width:' . $thumbObj->getOutputWidth() . 'px;height:' . $thumbObj->getOutputHeight() . 'px;" /></td><td>' . $delbut . '</td></tr></table>';

					$parts->add((new we_html_multiIconBoxContentLine($thumbObj->getThumbName(), ''))
									->space(we_html_multiIconBox::SPACE_BIG)
									->noline())
							->add(new we_html_multiIconBoxContentLine('', $thumbnail));
				}
				$parts->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('thumbnails', '[add_descriptiontext]'), we_html_tools::TYPE_INFO, 700) . we_html_element::br(2) . we_html_button::create_button('fa:btn_add_thumbnail,fa-plus,fa-lg fa-picture-o', "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('add_thumbnail','" . we_base_WE::$transaction . "');"))
				);
			} else {
				$parts->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('thumbnails', '[format_not_supported]'), we_html_tools::TYPE_INFO, 700)));
			}
		} else {
			$parts->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('thumbnails', '[add_description_nogdlib]'), we_html_tools::TYPE_INFO, 700)));
		}

		return $this->getPage((new we_html_multiIconBox($parts))->marginLeft()->get(), '', ['onload' => "doScrollTo();"]);
	}

}
