<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 44f79311fc0c $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-23 19:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
we_modules_info::isActive(we_modules_info::EXPORT);

class we_export_frames extends we_modules_frame{
	/** @var we_export_tree */
	protected $SelectionTree;

	/** @var 	we_export_view */
	protected $View;

	/** @var we_export_tree */
	protected $Tree;

	private const TAB_PROPERTIES = 1;
	private const TAB_OPTIONS = 2;
	private const TAB_LOG = 3;

	public function __construct(){
		$this->module = we_modules_info::EXPORT;
		parent::__construct();
		$this->Tree = new we_export_treeMain($this->jsCmd);
		$this->SelectionTree = new we_export_tree($this->jsCmd);
		$this->View = new we_export_view();
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case 'load':
				return $this->getHTMLCmd();
			case 'frameset':
				$this->View->Model->clearSessionVars();
				return parent::getHTML($what, $mode, $step);
			case 'treeheader':
			case 'treefooter':
				return '';
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorHeader();
		}

		$we_tabs = new we_gui_tabs();
		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, false, (string) self::TAB_PROPERTIES, ['id' => 'tab_1', 'title' => g_l('export', '[property]')]);
		if($this->View->Model->IsFolder == 0){
			$we_tabs->addTab(g_l('export', '[options]'), '', false, (string) self::TAB_OPTIONS, ['id' => 'tab_2']);
			$we_tabs->addTab(g_l('export', '[log]'), '', false, (string) self::TAB_LOG, ['id' => 'tab_3']);
		}

		$tabsHead = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/export/export_top.js');

		if(!$this->View->Model->ID || $this->View->Model->IsFolder){
			$this->jsCmd->addCmd('setTab', 1);
		}

		$text = !empty($this->View->Model->Path) ? $this->View->Model->Path : '/' . $this->View->Model->Text;

		//TODO: we have the following body in several modules!
		$body = we_html_element::body(['onresize' => "weTabs.setFrameSize();", 'onload' => "loaded();", 'id' => 'eHeaderBody'], we_html_element::div([
					'id' => 'main'], we_html_element::div(['id' => 'headrow'], we_html_element::b(str_replace(' ', '&nbsp;', g_l('export', '[export]')) . ':&nbsp;') .
						we_html_element::span(['id' => 'h_path', 'class' => 'header_small'], '<b id="titlePath">' . str_replace(' ', '&nbsp;', $text) . '</b>'
						)
					) .
					$we_tabs->getHTML()
				)
		);

		return $this->getHTMLDocument($body, $tabsHead);
	}

	protected function getHTMLEditorBody(): string{
		$hiddens = ['cmd' => 'we_export_edit', 'pnt' => 'edbody'];

		if(we_base_request::bool("home")){
			return $this->View->getHomeScreen();
		}

		//$weSuggest = we_gui_suggest::getInstance();
		//FIXME: folder don't have a tree to start.
		$body = we_html_element::body(['class' => 'weEditorBody', 'onload' => "doOnload()", 'onunload' => "doUnload();"], we_html_element::form([
					'name' => we_base_constants::WE_FORM], $this->View->getCommonHiddens($hiddens) . $this->getHTMLProperties())
		);
		return $this->getHTMLDocument($body, $this->View->getJSProperty());
	}

	protected function getHTMLEditorFooter(array $btn_cmd = []): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorFooter([]);
		}

		$main = we_html_button::create_button(we_html_button::FOOTER_SAVE, "javascript:top.content.we_cmd('we_export_save');") .
			($this->View->Model->IsFolder == 0 ?
			we_html_button::create_button(we_html_button::EXPORT, "javascript:top.content.we_cmd('start_export')", '', '', !we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_MAKE])) : '');

		$text = we_base_request::string("current_description", g_l('export', '[working]'));
		$progress = we_base_request::int("percent", 0);

		$progressbar = new we_gui_progressBar($progress, 200);
		$progressbar->addText($text, we_gui_progressBar::TOP, "current_description");

		$main .= we_html_element::span(['id' => 'progress'], $progressbar->getHtml('', 'display: none'));

		$optional = we_html_element::div(['class' => 'optional'], we_html_button::create_button(we_html_button::FOOTER_TRASH, "javascript:top.content.we_cmd('we_export_delete')", '', '', !we_base_permission::hasPerm([
						we_base_perm::EXIM_EXPORT_DELETE])));

		return $this->getHTMLDocument(
				we_html_element::body(['id' => 'footerBody'], we_html_element::form([], we_html_element::div(['class' => 'footertable'], we_html_element::div(['class' => 'main'], $main) . $optional))
				), we_gui_progressBar::getJSCode()
		);
	}

	function getHTMLProperties(): string{
		$tabNr = we_base_request::int("tabnr", 1);

		return we_html_element::div(['id' => 'tab1', 'style' => ($tabNr == self::TAB_PROPERTIES ? '' : 'display: none')], (new we_html_multiIconBox($this->getHTMLTab1()))->marginLeft()->get()) .
			we_html_element::div(['id' => 'tab2', 'style' => ($tabNr == self::TAB_OPTIONS ? '' : 'display: none')], $this->getHTMLTab2()) .
			we_html_element::div(['id' => 'tab3', 'style' => ($tabNr == self::TAB_LOG ? '' : 'display: none')], (new we_html_multiIconBox($this->getHTMLTab3()))->marginLeft()->get());
	}

	private function getHTMLTab1(): we_html_multiIconBoxContent{
		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('export', '[property]'), we_html_tools::formElementTable(g_l('export', '[name]'), we_html_tools::textInput('Text_visible', $this->View->Model->Text, 0, [
						'style' => 'width: 490px;',
						'id' => 'yuiAcInputPathName',
						'oninput' => "we_cmd('setHot');",
						'onblur' => "we_cmd('setHeaderTitlePath', this.value);"
				])) .
				we_html_element::hiddens([
					'Text' => $this->View->Model->Text
				]) .
				'<br/>' . $this->getHTMLDirChooser()))
			->space(we_html_multiIconBox::SPACE_MED)
		);

		if($this->View->Model->IsFolder){
			return $parts;
		}

		$permittedExportTypes = self::getPermittedExportTypes();
		$exportTypes = array_filter([
			we_exim_ExIm::TYPE_WE => (in_array(we_exim_ExIm::TYPE_WE, $permittedExportTypes, true) ? g_l('export', '[wxml_export]') : false),
			we_exim_ExIm::TYPE_XML => (in_array(we_exim_ExIm::TYPE_XML, $permittedExportTypes, true) ? g_l('export', '[gxml_export]') : false),
			we_exim_ExIm::TYPE_CSV => (in_array(we_exim_ExIm::TYPE_CSV, $permittedExportTypes, true) ? g_l('export', '[csv_export]') : false),
		]);

		$parts->add((new we_html_multiIconBoxContentLine(g_l('export', '[export_to]'), we_html_tools::formElementTable(we_html_tools::select('ExportType', $exportTypes, 1, $this->View->Model->ExportType, false, [
							'onchange' => "we_cmd('switch_type', this);"], true, 520))))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::formElementTable(g_l('export', '[filename]'), we_html_tools::textInput('Filename', $this->View->Model->Filename, 0, [
							'style' => 'width: 490px;',
							'oninput' => "we_cmd('setHot');"
					]))))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline()
		);

		$table = (new we_html_table(['class' => 'default withSpace'], 3, 1))
			->setCol(0, 0, we_html_tools::formElementTable(g_l('export', '[file_format]'), we_html_tools::select('ExportTo', ['local' => g_l('export', '[export_to_local]'),
						'server' => g_l('export', '[export_to_server]')], 1, $this->View->Model->ExportTo, false, [
						'onchange' => "we_cmd('toggle', 'save_to');"], true, 520)))
			->setCol(1, 0, we_html_tools::formElementTable(g_l('export', '[save_to]'), $this->formFileChooser('ServerPath', $this->View->Model->ServerPath, '', we_base_ContentTypes::FOLDER_DOCUMENT)), [
			'id' => 'save_to', 'style' => ($this->View->Model->ExportTo === 'server' ? 'display:block' : 'display: none')]);

		$parts->add((new we_html_multiIconBoxContentLine('', $table->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		$dtq = we_doctype_docTypes::getDoctypeQuery();
		$docTypes = $this->db->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'], false);


		$FolderPath = $this->View->Model->Folder ? fS('SELECT Path FROM ' . FILE_TABLE . ' WHERE ID=' . intval($this->View->Model->Folder)) : '/';

		$table = new we_html_table(['class' => 'default'], 5, 1);

		$this->View->Model->SelectionType = $this->View->Model->ExportType === we_exim_ExIm::TYPE_CSV ? we_export_export::TYPE_OBJECT : $this->View->Model->SelectionType;

		$selectorAttribs = [
			'name' => 'SelectionType',
			'onchange' => "toggleSelectionType(this.value);",
			'style' => 'width:520px;',
		];

		$selectionType = new we_html_select(array_filter($selectorAttribs));
		$selectionType->addOptions([
			we_export_export::TYPE_DOCUMENT => g_l('export', '[documents]'),
			we_export_export::TYPE_DOCTYPE => g_l('export', '[doctypename]')
		]);
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$selectionType->addOption(we_export_export::TYPE_OBJECT, g_l('export', '[classname]'));
		}
		$selectionType->selectOption($this->View->Model->SelectionType);
		$table
			->setCol(0, 0, $selectionType->getHtml(), ['style' => 'padding-bottom:5px;'])
			->setCol(1, 0, we_html_tools::select('DocType', $docTypes, 1, $this->View->Model->DocType, false, [
					'onchange' => "we_cmd('setHot');"], true, 520), ['class' => 'selectionTypes doctype', 'style' => ($this->View->Model->SelectionType === we_export_export::TYPE_DOCTYPE ? 'display:block' : 'display: none')]
			)
			->setCol(2, 0, we_html_tools::formElementTable(g_l('export', '[dir]'), $this->formWeChooser(FILE_TABLE, 400, 0, 'Folder', $this->View->Model->Folder, 'FolderPath', $FolderPath)), [
				'class' => 'selectionTypes doctype document', 'style' => ($this->View->Model->SelectionType !== we_export_export::TYPE_OBJECT ? 'display:block' : 'display: none')]
		);

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$classNames = $this->db->getAllFirstq('SELECT ID,Text FROM ' . OBJECT_TABLE . ' ORDER BY Text', false);
			$table->setCol(3, 0, we_html_tools::select('TableID', $classNames, 1, $this->View->Model->TableID, false, [
					'onchange' => "we_cmd('setHot');"], true, 520), ['class' => 'selectionTypes classname', 'style' => ($this->View->Model->SelectionType === we_export_export::TYPE_OBJECT ? 'display:block' : 'display: none')]
			);
		}

		if(we_modules_info::isActive(we_modules_info::CATEGORY)){
			$table->setCol(4, 0, $this->getHTMLCategory());
		}
		$selectionTypeHtml = $table->getHTML();

		$table = (new we_html_table(['class' => 'default'], 3, 1))
			->setCol(0, 0, we_html_tools::select('Selection', ['auto' => g_l('export', '[auto_selection]'), "manual" => g_l('export', '[manual_selection]')], 1, $this->View->Model->Selection, false, [
					'onchange' => "we_cmd('toggle_selection', this.value);"], true, 520), ['style' => 'padding-bottom:5px;'])
			->setCol(1, 0, we_html_tools::alertAttentionBox(g_l('export', '[txt_auto_selection]'), we_html_tools::TYPE_INFO, 520) .
			$selectionTypeHtml, ['id' => 'auto', 'style' => ($this->View->Model->Selection === 'auto' ? 'display:block' : 'display: none')]
		);

		switch($this->View->Model->ExportType){
			case we_exim_ExIm::TYPE_CSV:
				$selected = addTblPrefix($this->View->Model->XMLTable);
				break;
			case we_exim_ExIm::TYPE_XML:
				$selected = addTblPrefix($this->View->Model->XMLTable);
				break;
			default:
				$selected = FILE_TABLE;
		}

		$table->setCol(2, 0, we_html_tools::alertAttentionBox(g_l('export', '[txt_manual_selection]') . " " . g_l('export', '[select_export]'), we_html_tools::TYPE_INFO, 520) .
			$this->SelectionTree->getHTMLMultiExplorer(520, 200, true, $selected, $this->View->Model->ExportType) . we_html_element::hidden('XMLTable', $this->View->Model->XMLTable ?: stripTblPrefix(FILE_TABLE)), [
			'id' => 'manual', 'style' => ($this->View->Model->Selection === 'manual' ? "display:block" : "display: none")]
		);

		$parts->add((new we_html_multiIconBoxContentLine(g_l('export', '[selection]'), $table->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		return $parts;
	}

	private function getHTMLTab2(): string{
		$optionsWE = (new we_html_multiIconBox($this->getHTMLOptionsWE()))->marginLeft()->get();
		$optionsXML = (new we_html_multiIconBox($this->getHTMLOptionsXML()))->marginLeft()->get();
		$optionsCSV = (new we_html_multiIconBox($this->getHTMLOptionsCSV()))->marginLeft()->get();

		return we_html_element::div(['id' => 'optionsWXML', 'class' => 'exportOptions', 'style' => 'display: ' . ($this->View->Model->ExportType === we_exim_ExIm::TYPE_WE ? 'block' : 'none') . ';'], $optionsWE) .
			we_html_element::div(['id' => 'optionsGXML', 'class' => 'exportOptions', 'style' => 'display: ' . ($this->View->Model->ExportType === we_exim_ExIm::TYPE_XML ? 'block' : 'none') . ';'], $optionsXML) .
			we_html_element::div(['id' => 'optionsCSV', 'class' => 'exportOptions', 'style' => 'display: ' . ($this->View->Model->ExportType === we_exim_ExIm::TYPE_CSV ? 'block' : 'none') . ';'], $optionsCSV);
	}

	private function getHTMLOptionsWE(): we_html_multiIconBoxContent{
		$doctable = (new we_html_table([], 5, 1))
			->setCol(0, 0, we_html_forms::checkboxWithHidden((bool) ($this->View->Model->HandleDefTemplates), "HandleDefTemplates", g_l('export', '[handle_def_templates]'), false, '', "we_cmd('setHot');"))
			->setCol(1, 0, we_html_forms::checkboxWithHidden((bool) ($this->View->Model->HandleDocIncludes), "HandleDocIncludes", g_l('export', '[handle_document_includes]'), false, '', "we_cmd('setHot');"));
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$doctable->setCol(2, 0, we_html_forms::checkboxWithHidden((bool) ($this->View->Model->HandleObjIncludes), "HandleObjIncludes", g_l('export', '[handle_object_includes]'), false, '', "we_cmd('setHot');"));
		}
		$doctable
			->setCol(3, 0, we_html_forms::checkboxWithHidden((bool) ($this->View->Model->HandleDocLinked), "HandleDocLinked", g_l('export', '[handle_document_linked]'), false, '', "we_cmd('setHot');"))
			->setCol(4, 0, we_html_forms::checkboxWithHidden(($this->View->Model->HandleThumbnails ? true : false), 'HandleThumbnails', g_l('export', '[handle_thumbnails]'), false, '', "we_cmd('setHot');"));

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('export', '[handle_document_options]') . we_html_element::br() . g_l('export', '[handle_template_options]'), we_html_tools::alertAttentionBox(g_l('export', '[txt_document_options]'), we_html_tools::TYPE_INFO, 520, true, 60) . $doctable->getHtml()))
			->space(we_html_multiIconBox::SPACE_MED)
		);

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$objtable = (new we_html_table([], 3, 1))
				->setCol(0, 0, we_html_forms::checkboxWithHidden((bool) ($this->View->Model->HandleDefClasses), 'HandleDefClasses', g_l('export', '[handle_def_classes]'), false, '', "we_cmd('setHot');"), [
					'colspan' => '2'])
				->setCol(1, 0, we_html_forms::checkboxWithHidden((bool) ($this->View->Model->HandleObjEmbeds), 'HandleObjEmbeds', g_l('export', '[handle_object_embeds]'), false, '', "we_cmd('setHot');"));

			$parts->add((new we_html_multiIconBoxContentLine(g_l('export', '[handle_object_options]') . we_html_element::br() . g_l('export', '[handle_classes_options]'), we_html_tools::alertAttentionBox(g_l('export', '[txt_object_options]'), we_html_tools::TYPE_INFO, 520, true, 60) . $objtable->getHtml()))
					->space(we_html_multiIconBox::SPACE_MED)
			);
		}

		$formattable = (new we_html_table([], 3, 1))
			->setCol(0, 0, we_html_forms::checkboxWithHidden((bool) ($this->View->Model->HandleDoctypes), 'HandleDoctypes', g_l('export', '[handle_doctypes]'), false, '', "we_cmd('setHot');"))
			->setCol(1, 0, we_html_forms::checkboxWithHidden((bool) ($this->View->Model->HandleCategorys), "HandleCategorys", g_l('export', '[handle_categorys]'), false, '', "we_cmd('setHot');"))
			->setCol(2, 0, we_html_forms::checkboxWithHidden((bool) ($this->View->Model->HandleNavigation), "HandleNavigation", g_l('export', '[handle_navigation]'), false, '', "we_cmd('setHot');", false, g_l('export', '[navigation_hint]'), we_html_tools::TYPE_HELP));

		$ownertable = we_html_forms::checkboxWithHidden((bool) ($this->View->Model->HandleOwners), "HandleOwners", g_l('export', '[handle_owners]'), false, 'defaultfont', "we_cmd('setHot');");

		$parts->add((new we_html_multiIconBoxContentLine(g_l('export', '[handle_doctype_options]', false, true), $formattable->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('export', '[export_depth]'), we_html_tools::alertAttentionBox(g_l('export', '[txt_exportdeep_options]'), we_html_tools::TYPE_INFO, 520) . '<br/>' . we_html_element::label([
						'style' => 'padding-right:5px;'], g_l('export', '[to_level]')) . we_html_tools::textInput('ExportDepth', (string) $this->View->Model->ExportDepth, 0, ['onblur' => "we_cmd('setExportDepth', this.value, " . intval($this->View->Model->ExportDepth) . ")"], 'text', '50px')))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('export', '[handle_owners_option]'), we_html_tools::alertAttentionBox(g_l('export', '[txt_owners]'), we_html_tools::TYPE_INFO, 520) . $ownertable))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		return $parts;
	}

	private function getHTMLOptionsXML(): we_html_multiIconBoxContent{
		$table = (new we_html_table(['class' => 'default withSpace'], 2, 1))
			->setCol(0, 0, we_html_forms::radiobutton('1', $this->View->Model->XMLCdata, 'XMLCdata', g_l('export', '[export_xml_cdata]'), true, 'defaultfont'))
			->setCol(1, 0, we_html_forms::radiobutton('0', !$this->View->Model->XMLCdata, 'XMLCdata', g_l('export', '[export_xml_entities]'), true, 'defaultfont'));

		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('export', '[cdata]'), $table->getHtml()))
					->space(we_html_multiIconBox::SPACE_MED)
		);
	}

	private function getHTMLOptionsCSV(): we_html_multiIconBoxContent{
		$file_encoding = (new we_html_select(['name' => 'CSVLineend', 'class' => 'weSelect', 'style' => 'width: 254px;']))
			->addOptions(we_exim_ExportCSV::getCSVLineEnds())
			->selectOption($this->View->Model->CSVLineend);

		$fileformattable = (new we_html_table([], 4, 1))
			->setCol(0, 0, g_l('export', '[csv_lineend]') . '<br/>' . $file_encoding->getHtml())
			->setCol(1, 0, $this->getHTMLChooser('CSVDelimiter', ($this->View->Model->CSVDelimiter ?: 'comma'), we_exim_ExportCSV::getCSVDelimeters(), g_l('export', '[csv_delimiter]')))
			->setCol(2, 0, $this->getHTMLChooser("CSVEnclose", $this->View->Model->CSVEnclose, [
					'doublequote' => g_l('export', '[double_quote]'),
					'singlequoteF' => g_l('export', '[single_quote]')
					], g_l('export', '[csv_enclose]')))
			->setCol(3, 0, we_html_forms::checkboxWithHidden($this->View->Model->CSVFieldnames, 'CSVFieldnames', g_l('export', '[csv_fieldnames]'), false, 'defaultfont'));

		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('export', '[csv_params]'), $fileformattable->getHtml()))
					->space(we_html_multiIconBox::SPACE_MED)
		);
	}

	private function getHTMLTab3(): we_html_multiIconBoxContent{
		return (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', we_html_element::div(['class' => 'blockWrapper', 'style' => 'padding:12px 0 0 12px;width: 650px; height: 400px; border:1px #dce6f2 solid;',
							'id' => 'log'], '')));
	}

	private function getHTMLDirChooser(): string{
		$path = id2path($this->View->Model->ParentID, EXPORT_TABLE);

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('PathGroup')
			->setContentType([we_base_ContentTypes::FOLDER_COMMON])
			->setInput('ParentPath', $path, [], false, true)
			->setLabel(g_l('export', '[group]'))
			->setMaxResults(10)
			->setResult('ParentID', (string) $this->View->Model->ParentID)
			->setSelector(we_gui_suggest::DirSelector)
			->setTable(EXPORT_TABLE)
			->setWidth(400)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_export_dirSelector',{id:document.we_form.elements.ParentID.value,targetID:'ParentID',targetName:'ParentPath',jsCmd:'setHot'})"));

		return $weSuggest->getHTML();
	}

	/**
	 *
	 * @param string $name
	 * @param string $value
	 * @param array<string,string> $values
	 * @param string $title
	 * @return string
	 */
	private function getHTMLChooser(string $name, string $value, array $values, string $title): string{
		$select = new we_html_select([
			'name' => $name . '_select',
			'class' => 'weSelect',
			'onchange' => "we_cmd('inputChooser_syncChoice', this, '" . $name . "')",
			'style' => 'width:200;'
		]);
		$select->addOption('', '');
		foreach($values as $k => $v){
			$select->addOption(htmlspecialchars($k), htmlspecialchars($v));
		}

		$table = (new we_html_table(['class' => 'default', 'width' => '250'], 1, 2))
			->setCol(0, 0, we_html_tools::textInput($name, $value) . '  ')
			->setCol(0, 1, $select->getHtml());

		return we_html_tools::formElementTable($title, $table->getHtml());
	}

	private function getLoadCode(): string{
		if(($pid = we_base_request::int('pid', -1)) > -1){
			$table = $this->View->Model->ExportType === we_exim_ExIm::TYPE_CSV ? OBJECT_FILES_TABLE : we_base_request::_(we_base_request::TABLE, 'tab');
			$this->jsCmd->addCmd('location', ['doc' => 'document', 'loc' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=loadTree&we_cmd[1]=' . $table . '&we_cmd[2]=' . $pid . '&we_cmd[3]=' . implode(',', we_base_request::intList('openFolders', [
			]))]);

			return $this->getHTMLDocument(we_html_element::body());
		}
		return '';
	}

	private function getMainLoadCode(): string{
		if(($pid = we_base_request::int('pid', -1)) > -1){
			$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_export_treeMain::getItems($pid), 'sorted' => true]);

			return $this->getHTMLDocument(we_html_element::body());
		}
		return '';
	}

	public static function getPermittedExportTypes(): array{
		return array_filter([
			(we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_NEW, we_base_perm::EXIM_EXPORT_DELETE, we_base_perm::EXIM_EXPORT_EDIT, we_base_perm::EXIM_EXPORT_MAKE]) ? we_exim_ExIm::TYPE_WE : false),
			(we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_GENERICXML]) ? we_exim_ExIm::TYPE_XML : false),
			(we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_CSV]) ? we_exim_ExIm::TYPE_CSV : false)
		]);
	}

	private function getDoExportCode(): string{
		if(!in_array($this->View->Model->ExportType, self::getPermittedExportTypes(), true)){
			$jsCmd = new we_base_jsCmd();
			$jsCmd->addMsg(g_l('export', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
			return we_html_tools::getHtmlTop('', $jsCmd->getCmds(), we_html_element::body());
		}

		$progress_update = [
			'log' => [],
			'text' => '',
			'percent' => 0
		];

		$bodyContent = we_html_element::form(['name' => we_base_constants::WE_FORM,
				'method' => 'post',
				'action' => $this->frameset], we_html_element::hiddens(['pnt' => 'cmd', 'cmd' => 'do_export'])
		);

		switch($this->View->Model->ExportType){
			case we_exim_ExIm::TYPE_CSV:
				$exporter = new we_exim_ExportCSV();
				break;
			case we_exim_ExIm::TYPE_XML:
				$exporter = new we_exim_ExportXML();
				break;
			default:
				$exporter = new we_exim_ExportWE();
		}
		$exporter->loadPreserves();

		switch($exporter->getNextTask()){
			case we_exim_Export::EXPORT_PRE_PROCESS:
				$exporter->exportPreprocess($this->View->Model);

				$progress_update['log'][] = we_html_element::b(g_l('export', '[start_export]') . ' - ' . date(g_l('weEditorInfo', '[date_format_sec]')));
				$progress_update['log'][] = we_html_element::b(g_l('export', '[prepare]') . '<br/>');
				$progress_update['log'][] = we_html_element::b(g_l('export', '[export]'));
				$progress_update['percent'] = 0;
				$progress_update['text'] = g_l('export', '[working]');

				$this->jsCmd->addCmd('updateLog', $progress_update);
				$this->jsCmd->addCmd('submitCmdForm');
				break;
			case we_exim_Export::EXPORT_PROCESS_NEXT:
				$lastExported = $exporter->exportNext();
				$path = '';
				if($lastExported['success']){
					switch($lastExported['contentType']){
						case we_base_ContentTypes::BACKUP_BINARY:
							$progress_update['log'][] = we_html_element::b(g_l('export', '[weBinary]')) . '&nbsp;&nbsp;' . $lastExported['id'];
							$proceed = false;
							break;
						case we_base_ContentTypes::DOCTYPE:
							$path = fS('SELECT Text FROM ' . $lastExported['table'] . ' WHERE ID=' . intval($lastExported['id']));
							$proceed = true;
							break;
						case we_base_ContentTypes::NAVIGATIONRULE:
							$path = fS('SELECT NavigationName FROM ' . $lastExported['table'] . ' WHERE ID=' . intval($lastExported['id']));
							$proceed = true;
							break;
						case we_base_ContentTypes::THUMBNAIL:
							$path = fS('SELECT Name FROM ' . $lastExported['table'] . ' WHERE ID=' . intval($lastExported['id']));
							$proceed = true;
							break;
						default:
							$path = id2path($lastExported['id'], $lastExported['table']);
							$proceed = true;
							break;
					}

					if($proceed){
						try{
							$progress_text = we_html_element::b(g_l('contentTypes', '[' . $lastExported['contentType'] . ']', true));
						} catch (Exception $ex){
							$progress_text = we_html_element::b(g_l('export', '[' . $lastExported['contentType'] . ']', true));
						}
						$progress_text .= '&nbsp;&nbsp;' . $path;
						if(strlen($path) > 75){
							$progress_text = addslashes(substr($progress_text, 0, 65) . '<abbr title="' . $path . '">...</abbr>' . substr($progress_text, -10));
						}
						$progress_update['log'][] = $progress_text;
					}

					$percent = round(max(min(($lastExported['total'] ? intval(($lastExported['current'] / $lastExported['total']) * 100) : 0), 100), 0), 2);
					$progress_update['percent'] = $percent;

					$this->jsCmd->addCmd('updateLog', $progress_update);
					$this->jsCmd->addCmd('submitCmdForm');
				}
				break;
			case we_exim_Export::EXPORT_POST_PROCESS:
				$exporter->exportPostprocess();

				if($this->View->Model->ExportTo === 'local'){
					$progress_update['log'][] = we_html_element::b(we_html_element::span(['class' => 'defaultfont'], we_html_element::br() . g_l('export', '[backup_finished]') . ' - ' . date(g_l('weEditorInfo', '[date_format_sec]')) . we_html_element::br() .
								g_l('export', '[download_starting2]', false, true) . we_html_element::br(2) .
								g_l('export', '[download_starting3]', false, true) . we_html_element::br() .
								we_html_element::b(we_html_element::a(['href' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=export&pnt=cmd&cmd=upload&exportfile=' . urlencode($this->View->Model->ExportFilename),
										'download' => $this->View->Model->ExportFilename], g_l('export', '[download]'))) . we_html_element::br(2)
					));
				} else {
					$progress_update['log'][] = we_html_element::b(we_html_element::br() . g_l('export', '[end_export]') . ' - ' . date(g_l('weEditorInfo', '[date_format_sec]')));
				}
				$progress_update['percent'] = 100;

				$this->jsCmd->addCmd('updateLog', $progress_update);
				if($this->View->Model->ExportTo === 'local'){
					$this->jsCmd->addCmd('startDownload', $this->View->Model->ExportFilename);
				}
				$this->jsCmd->addCmd('setStatusEnd');

				$bodyContent = '';
				break;
		}

		return we_html_tools::getHtmlTop('', $this->jsCmd->getCmds(), we_html_element::body([], $bodyContent));
	}

	private function getUploadCode(): void{
		if(($filename = we_base_request::_(we_base_request::FILE, "exportfile"))){
			$filename = basename(urldecode($filename));

			if(file_exists(TEMP_PATH . $filename) // Does file exist?
				&& !we_export_export::exportToFilenameInvalid($filename)){ // Security check
				session_write_close();
				$size = filesize(TEMP_PATH . $filename);

				header('Pragma: public');
				header('Expires: 0');
				header('Cache-control: private, max-age=0, must-revalidate');

				header('Content-Type: application/octet-stream');
				header('Content-Disposition: attachment; filename="' . trim(htmlentities($filename)) . '"');
				header('Content-Description: ' . trim(htmlentities($filename)));
				header('Content-Length: ' . $size);

				readfile(TEMP_PATH . $filename);
			} else {
				header('Location: ' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=export&pnt=cmd&cmd=upload_failed');
			}
		} else {
			header('Location: ' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=export&pnt=cmd&cmd=error=upload_failed');
		}
		exit();
	}

	protected function getHTMLCmd(): string{
		switch(we_base_request::string('cmd')){
			case 'load':
				return $this->getLoadCode();
			case 'mainload':
				return $this->getMainLoadCode();
			case 'do_export':
				return $this->getDoExportCode();
			case 'upload':
				$this->getUploadCode();
				exit();
			case 'upload_failed':
				$jsCmd = new we_base_jsCmd();
				$jsCmd->addMsg(g_l('export', '[error_download_failed]'), we_base_util::WE_MESSAGE_ERROR);
				return we_html_tools::getHtmlTop('', $jsCmd->getCmds(), we_html_element::body());
			default:
				return '';
		}
	}

	private function formWeChooser(string $table = FILE_TABLE, int $width = 0, int $rootDirID = 0, string $IDName = 'ID', int $IDValue = 0, string $Pathname = 'Path', string $Pathvalue = '/'): string{
		$Pathvalue = ($Pathvalue ?: fS('SELECT Path FROM ' . $this->db->escape($table) . ' WHERE ID=' . $IDValue));

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('SelPath')
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT])
			->setInput($Pathname, $Pathvalue, [], false, true)
			->setMaxResults(10)
			->setResult($IDName, (string) $IDValue)
			->setSelector(we_gui_suggest::DirSelector)
			->setTable(FILE_TABLE)
			->setWidth($width)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements['" . $IDName . "'].value,table:'" . $table . "',targetID:'" . $IDName . "',targetName:'" . $Pathname . "',rootDirID:'" . $rootDirID . "'})"));

		return $weSuggest->getHTML();
	}

	private function getHTMLCategory(): string{
		switch(we_base_request::string('cmd')){
			case 'add_cat':
				$arr = array_filter($this->View->Model->Categorys);
				if(($cat = we_base_request::intList('cat'))){
					foreach($cat as $id){
						if($id && (!in_array($id, $arr))){
							$arr[] = $id;
						}
					}
					$this->View->Model->Categorys = $arr;
				}
				break;
			case 'del_cat':
				$arr = array_filter($this->View->Model->Categorys);
				if(($cat = we_base_request::int('cat'))){
					foreach($arr as $k => $v){
						if($v == $cat){
							unset($arr[$k]);
						}
					}
					$this->View->Model->Categorys = $arr;
				}
				break;
			case 'del_all_cats':
				$this->View->Model->Categorys = [];
				break;
			default:
		}

		$hiddens = we_html_element::hiddens([
				'Categorys' => implode(',', $this->View->Model->Categorys),
				'cat' => we_base_request::string('cat', '')
		]);


		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('del_all_cats')", '', '', (empty($this->View->Model->Categorys)));
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{id:0,table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'add_cat'})");

		$cats = new we_chooser_multiDir($this->View->Model->Categorys, 'del_cat', $delallbut . $addbut, [], '"' . we_base_ContentTypes::CATEGORY . '"', CATEGORY_TABLE);

		if(!we_base_permission::hasPerm([we_base_perm::WORK_CATEGORY_EDIT])){
			$cats->setEditable(false);
		}
		return $hiddens . we_html_tools::formElementTable(g_l('export', '[categories]'), $cats->get($this->jsCmd));
	}

}
