<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 47da61573169 $
 * $Author: Lukas Imhof $
 * $Date: 2021-03-25 11:12 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_import_siteFrag extends we_fragment_base{

	protected function doTask(): void{
		switch($this->data['contentType']){
			case 'post/process':
				we_import_site::postprocessFile($this->data['path'], $this->data['sourceDir'], $this->data['destDirID']);
				return;
			default:
				$ret = we_import_site::importFile($this->data['path'], $this->data['contentType'], $this->data['sourceDir'], (int) $this->data['destDirID'], $this->data['sameName'], $this->data['thumbs'], $this->data['width'], $this->data['height'], $this->data['widthSelect'], $this->data['heightSelect'], $this->data['keepRatio'], $this->data['quality'], $this->data['degrees'], $this->data['importMetadata'], $this->data['isSearchable']);
				if(!empty($ret)){
					t_e('import error:', $ret);
				}
		}
	}

	protected function updateProgressBar(): void{
		$path = substr($this->data['path'], strlen(WE_DOCUMENT_ROOT)) ?: '';
		$this->jsCmd->addCmd('setProgress', [
			'progress' => ((int) ((100 / $this->numberOfTasks) * $this->currentTask)),
			'name' => 'progressTxt',
			'text' => we_base_util::shortenPath($path, 30),
			'win' => 'wizbusy'
		]);
		$this->jsCmd->addCmd('disableBackNext', 'wizbusy');
	}

	protected function finish(): void{
		$this->jsCmd->addCmd('setProgress', [
			'progress' => 100,
			'name' => 'progressTxt',
			'text' => '',
			'win' => 'wizbusy'
		]);
		$this->jsCmd->addMsg(g_l('siteimport', '[importFinished]'), we_base_util::WE_MESSAGE_NOTICE);
		$this->jsCmd->addCmd('we_cmd', ['load', FILE_TABLE]);
		$this->jsCmd->addCmd('close_delayed', 2000);
	}

}
