<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 785c2a488c30 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-02-25 17:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_version extends we_rpc_cmd{

	function execute(): we_rpc_response{
		switch($this->CmdShell->getCmd()){
			case 'versionDeleteWizard':
				return $this->versionDeleteWizard();
			case 'versionDelete':
				return $this->versionDelete();
			case 'versionGetParameters':
				return $this->versionGetParameters();
			case 'versionGetSearchResult':
				return $this->versionGetSearchResult();
			case 'versionLogGetDetails':
				return $this->versionLogGetDetails();
			case 'versionReset':
				return $this->versionReset();
		}

		return $this->resp;
	}

	private function versionReset(): we_rpc_response{
		$ids = we_base_request::stringList('id');

		$specVersion = we_base_request::int('version', 0);
		$resetIds = [];
		foreach($ids as $documents){
			'@phan-var string $documents';
			$parts = explode('___', $documents);
			$id = (int)$parts[0];
			$publish = (bool) ($parts[1] ?? false);
			$version = ($specVersion ?: fI('SELECT version FROM ' . VERSIONS_TABLE . ' WHERE ID=' . $id . ' ORDER BY version DESC LIMIT 1'));
			try{
				$resetIds = array_merge($resetIds, we_versions_version::resetVersion($id, $version, $publish));
			} catch (we_exception_version $ex){
				t_e('warning', $ex->getMessage());
			}
		}

		if($resetIds){
			$versionslog = new we_versions_log();
			$versionslog->saveVersionsLog($resetIds, we_versions_log::VERSIONS_RESET);
		}

		return $this->resp;
	}

	private function versionLogGetDetails(): we_rpc_response{
		$id = we_base_request::int('id', 0);
		$code = we_versions_logView::handleData($id, 1, 1);
		$this->resp->setData('data', $code);
		return $this->resp;
	}

	private function versionGetSearchResult(): we_rpc_response{
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', 0);
		$document = we_contents_root::getSessionDocument($we_transaction);
		assert($document instanceof we_document_base);
		$versionsView = new we_versions_view($document->versionsModel);

		$code = $versionsView->tabListContent();

		$this->resp->setData('data', $code);
		return $this->resp;
	}

	private function versionGetParameters(): we_rpc_response{
		$pos = we_base_request::string('position', '');
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', 0);
		$document = we_contents_root::getSessionDocument($we_transaction);
		assert($document instanceof we_document_base);
		$versionsView = new we_versions_view($document->versionsModel);

		$code = '';
		switch($pos){
			case 'top':
				$code = $versionsView->getParameterTop();
				break;
			case 'bottom':
				$code = $versionsView->getParameterBottom(true);
				break;
		}

		$this->resp->setData('data', $code);
		return $this->resp;
	}

	private function versionDelete(): we_rpc_response{
		$ids = we_base_request::intList('we_cmd', [], 'deleteVersion');
		if($ids){
			$dels = we_versions_version::deleteVersion($ids);
			if($dels){
				$versionslog = new we_versions_log();
				$versionslog->saveVersionsLog($dels, we_versions_log::VERSIONS_DELETE);
			}
		}

		return $this->resp;
	}

	private function versionDeleteWizard(): we_rpc_response{
		$db = DB_WE::$inst;

		$db->query('DELETE FROM ' . VERSIONS_TABLE . ' WHERE ' . $_SESSION['weS']['versions']['deleteWizardWhere']);

		unset($_SESSION['weS']['versions']['deleteWizardWhere']);

//		while($db->next_record()) {
//			weVersions::deleteVersion($db->f("ID"));
//		}
//		foreach($_SESSION['weS']['versions']['IDs'] as $k=>$v) {
//			weVersions::deleteVersion($v);
//		}
		if(!empty($_SESSION['weS']['versions']['deleteWizardbinaryPath']) && is_array($_SESSION['weS']['versions']['deleteWizardbinaryPath'])){
			foreach($_SESSION['weS']['versions']['deleteWizardbinaryPath'] as $v){
				$binaryPath = WE_VERSION_PATH . $v;
				$binaryPathUsed = fB('SELECT 1 FROM ' . VERSIONS_TABLE . ' WHERE binaryPath="' . $db->escape($v) . '" LIMIT 1');

				if(!$binaryPathUsed){
					we_base_file::deleteLocalFile($binaryPath);
				}
			}
			unset($_SESSION['weS']['versions']['deleteWizardbinaryPath']);
		}

		if(!empty($_SESSION['weS']['versions']['logDeleteIds'])){
			$versionslog = new we_versions_log();
			$versionslog->saveVersionsLog((array) $_SESSION['weS']['versions']['logDeleteIds'], we_versions_log::VERSIONS_DELETE);
		}
		unset($_SESSION['weS']['versions']['logDeleteIds']);


		$WE_PB = new we_gui_progressBar(100, 200);

		$WE_PB->addText(g_l('versions', '[deleteDateVersionsOK]'), we_gui_progressBar::TOP, "pb1");
		//$js = $WE_PB->getJSCode();
		$pb = $WE_PB->getHTML();


		$this->resp->setData('data', $pb);
		return $this->resp;
	}

}
