<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3644b5810887 $
 * $Author: Lukas Imhof $
 * $Date: 2020-05-11 22:05 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class representing the model data of the search
 */
abstract class we_search_modelBase extends we_base_model{
	/** @var int */
	public $mode = 0;
	// in these variables we hold data of actual search (= whichSearch) after initByHttp();

	/** @var int */
	protected $currentSearchstart = 0;

	/** @var string[] */
	protected $currentSearch = [];

	/** @var string[] */
	protected $currentLocation = [];

	/** @var string[] */
	protected $currentSearchTables = [];

	/** @var string[] */
	protected $currentSearchFields = [];

	/** @var string */
	protected $currentOrder = 'Text';

	/** @var int */
	protected $currentAnzahl = 10;

	/** @var int */
	protected $currentAnzahlMedialinks = 0;

	/** @var string */
	protected $currentSetView = we_search_view::VIEW_LIST;

	/** @var int */
	protected $currentFolderID = 0;

	/** @var array<string,bool> */
	protected $currentSearchForField = [
		'text' => false,
		'title' => false,
		'meta' => false,
	];

	/** @var array<string,bool> */
	protected $currentSearchForContentType = [
		'image' => false,
		'audio' => false,
		'video' => false,
		'other' => false,
	];

	/** @var int */
	protected $currentWorkspace = 0;

	/** @var bool */
	protected $currentGreenOnly = false;

	/** @var int */
	protected $TableID = 0;


	//FIXME: copied from tool_model, check what is needed

	/** @var int */
	var $ID = 0;

	/** @var string */
	var $Text = '';

	/** @var int */
	var $ParentID = 0;

	/** @var string */
	var $Path = '';

	/** @var bool */
	var $IsFolder = false;

	/** @var bool flag if the search ist predefined or not */
	public $predefined = false;

	/** @var int */
	public $activTab = 1;

	abstract public function initByHttp(string $whichSearch = '', bool $isWeCmd = true): void;

	abstract public function prepareModelForSearch(): void;

	/**
	 * @param string $property
	 * @return mixed
	 */
	public function getProperty(string $property = ''){
		switch($property){
			case 'currentSearch':
				return $this->currentSearch;
			case 'currentLocation':
				return $this->currentLocation;
			case 'currentSearchTables':
				return $this->currentSearchTables;
			case 'currentSearchFields':
				return $this->currentSearchFields;
			case 'currentOrder':
				return $this->currentOrder;
			case 'currentSetView':
				return $this->currentSetView;
			case 'currentSearchForField':
				return $this->currentSearchForField;
			case 'currentSearchForContentType':
				return $this->currentSearchForContentType;
		}
		return '';
	}

	/**
	 * @param string $property
	 * @return int
	 */
	public function getPropertyInt(string $property = ''): int{
		switch($property){
			case 'currentSearchstart':
				return $this->currentSearchstart;
			case 'currentAnzahl':
				return $this->currentAnzahl;
			case 'currentAnzahlMedialinks':
				return $this->currentAnzahlMedialinks;
			case 'currentFolderID':
				return $this->currentFolderID;
			case 'currentWorkspace':
				return $this->currentWorkspace;
			case 'TableID':
				return $this->TableID;
			case 'activTab':
				return $this->activTab;
		}

		return 0;
	}

	/**
	 * @param string $property
	 * @return bool
	 */
	public function getPropertyB(string $property = ''): bool{
		switch($property){
			case 'currentGreenOnly':
				return $this->currentGreenOnly;
		}

		return false;
	}

	public function setIsFolder(bool $value): void{
		$this->IsFolder = $value;
	}

	public function filenameNotValid(string $text = ''): bool{
		return preg_match('|[^a-z0-9._-]|i', $text) ? true : false;
	}

	public static function getLangText(string $path, string $text): string{
		switch($path){
			case '/_PREDEF_':
				return g_l('searchtool', '[vordefinierteSuchanfragen]');
			case '/_PREDEF_/document':
				return g_l('searchtool', '[dokumente]');
			case '/_PREDEF_/document/unpublished':
				return g_l('searchtool', '[unveroeffentlicheDokumente]');
			case '/_PREDEF_/document/static':
				return g_l('searchtool', '[statischeDokumente]');
			case '/_PREDEF_/document/dynamic':
				return g_l('searchtool', '[dynamischeDokumente]');

			case '/_PREDEF_/object':
				return g_l('searchtool', '[objekte]');
			case '/_PREDEF_/object/unpublished':
				return g_l('searchtool', '[unveroeffentlicheObjekte]');
			case '/_CUSTOM_':
				return g_l('searchtool', '[eigeneSuchanfragen]');
			case '/_VERSION_':
				return g_l('searchtool', '[versionen]');
			case '/_VERSION_/document':
				return g_l('searchtool', '[dokumente]');
			case '/_VERSION_/document/deleted':
				return g_l('searchtool', '[geloeschteDokumente]');

			case '/_VERSION_/object':
				return g_l('searchtool', '[objekte]');
			case '/_VERSION_/object/deleted':
				return g_l('searchtool', '[geloeschteObjekte]');
			default:
				return $text;
		}
	}

	function clearSessionVars(): void{
		if(isset($_SESSION['weS']['module_sessions']['search'])){
			unset($_SESSION['weS']['module_sessions']['search']);
		}
	}

	function isSelf(): bool{
		if($this->ID){
			$count = 0;
			$parentid = $this->ParentID;
			while($parentid != 0){
				if($parentid == $this->ID){
					return true;
				}
				$parentid = fI('SELECT ParentID FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($parentid));
				$count++;
				if($count == 9999){
					return false;
				}
			}
			return false;
		}
		return false;
	}

	function isAllowedForUser(): bool{
		return true;
	}

	function updateChildPaths(string $oldpath): bool{
		$success = true;
		if($this->IsFolder && $oldpath != '' && $oldpath != '/' && $oldpath != $this->Path){
			$all = $this->db->getAllq('SELECT ID FROM ' . $this->db->escape($this->table) . ' WHERE Path LIKE \'' . $this->db->escape($oldpath) . '%\' AND ID!=' . intval($this->ID), true);
			foreach($all as $id){
				$success = $success && $this->db->query('UPDATE ' . $this->db->escape($this->table) . ' SET Path=\'' . $this->db->escape($this->evalPath((int) $id, $this->table)) . '\' WHERE ID=' . $id);
			}
		}

		return $success;
	}

	function delete(): bool{
		if(!$this->ID){
			return false;
		}
		if($this->IsFolder){
			$this->deleteChilds();
		}
		parent::delete();

		return true;
	}

}
