<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 7809b1598ad0 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-15 19:15 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_shop_frames extends we_modules_frame{
	const TAB_OVERVIEW = 1;
	const TAB_ORDERLIST = 2;
	const TAB_ADMIN1 = 10;
	const TAB_ADMIN2 = 11;
	const TAB_ADMIN3 = 12;

	/** @var we_shop_view */
	protected $View;

	/** @var we_tree_shop */
	protected $Tree;

	public function __construct(){
		$this->module = we_modules_info::SHOP;
		parent::__construct();
		$this->hasIconbar = true;
		$this->Tree = new we_tree_shop($this->jsCmd);
		$this->View = new we_shop_view();
	}

	function getHTMLIconbar(): string{ //TODO: move this to weShopView::getHTMLIconbar();
//	$bid = we_base_request::int('bid', 0);
		$data = getHash("SELECT ID,DATE_FORMAT(DateOrder,'" . g_l('date', '[format][mysqlDate]') . "') AS orddate FROM " . SHOP_ORDER_TABLE . ' ORDER BY ID DESC LIMIT 1');

		$headline = $data ? '<a style="text-decoration: none;" href="javascript:we_cmd(\'openOrder\', ' . $data['ID'] . ',\'shop\',\'' . SHOP_ORDER_TABLE . '\');">' . sprintf(g_l('modules_shop', '[lastOrder]'), $data['ID'], $data['orddate']) . '</a>' : '';

/// config
		$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		for($i = 0; $i <= 3; $i++){
			$feldnamen[$i] = $feldnamen[$i] ?? '';
		}
		$fe = explode(',', $feldnamen[3]);

		$classid = $fe[0];


		/* TODO: we have this or similar code at least four times!! */

		$resultO = array_shift($fe);

// wether the resultset ist empty?
		$resultD = fB('SELECT 1 FROM ' . CONTENT_TABLE . ' WHERE nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\' LIMIT 1');

		$c = 0;
		$iconBarTable = (new we_html_table(['class' => 'iconBar'], 1, 4))
			->setCol(0, $c++, we_html_button::create_button('fa:btn_shop_extArt,fa-lg fa-cart-plus', "javascript:top.opener.top.we_cmd('new_article')", '', "", !we_base_permission::hasPerm([
						we_base_perm::USER_NEW])))
			->setCol(0, $c++, we_html_button::create_button('fa:btn_shop_delOrd,fa-lg fa-shopping-cart,fa-lg fa-trash-o', "javascript:top.opener.top.we_cmd('delete_shop')", '', "", !we_base_permission::hasPerm([
					we_base_perm::USER_NEW])));

		if($resultD){
			$iconBarTable->addCol()
				->setCol(0, $c++, we_html_button::create_button('fa:btn_shop_sum,fa-lg fa-line-chart', "javascript:top.content.editor.location=WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=editor&top=1&typ=document '"));
		} elseif($resultO){
			$iconBarTable->addCol()
				->setCol(0, $c++, we_html_button::create_button('fa:btn_shop_sum,fa-lg fa-line-chart', "javascript:top.content.editor.location=WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=editor&top=1&typ=object&ViewClass=" . $classid));
		}

		$iconBarTable->setCol(0, $c++, we_html_button::create_button('fa:btn_shop_pref,fa-lg fa-pencil,fa-lg fa-list-alt', "javascript:top.content.we_cmd('settings')", '', "", !we_base_permission::hasPerm([
					we_base_perm::USER_NEW])));
		$iconBarTable->setCol(0, $c++, we_html_button::create_button('fa:btn_payment_val,fa-lg fa-long-arrow-right,fa-lg fa-money', "javascript:top.content.we_cmd('payment_val')", '', "", !we_base_permission::hasPerm([
					we_base_perm::USER_NEW])));

		if($headline){
			$iconBarTable->addCol()
				->setCol(0, $c++, '<span style="margin-left:15px">' . $headline . '</span>', ['style' => 'text-align:right', 'class' => 'header_shop']);
		}

		return $this->getHTMLDocument(we_html_element::body(['id' => 'iconBar'], $iconBarTable->getHTML()), we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/shop_frames.js'));
	}

	protected function getHTMLCmd(): string{
		return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
	}

	protected function getHTMLEditor(string $extraUrlParams = '', string $extraHead = ''): string{//TODO: maybe abandon the split between former Top- and other editor files
		if(we_base_request::bool('top')){//doing what have been done in edit_shop_editorFramesetTop before
			return $this->getHTMLEditorTop();
		}

//do what have been done in edit_shop_editorFrameset before

		$bid = we_base_request::int('bid', 0);
		$mid = we_base_request::string('mid');
		$yearView = we_base_request::int('ViewYear', 0);
		$home = we_base_request::bool('home');

		if($home){
			$bodyURL = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&home=1&pnt=edbody';
		} elseif($mid){
			$year = substr($mid, (strlen($mid) - 4)) ?: '';
			$month = str_replace($year, '', $mid);
			$bodyURL = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_revenueTop&ViewYear=' . $year . '&ViewMonth=' . $month;
		} elseif($yearView){
			$bodyURL = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_revenueTop&ViewYear=' . $yearView;
		} else {
			$bodyURL = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edbody&bid=' . $bid;
		}

		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'moduleEditor'], we_html_element::iframe('edheader', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edheader&home=' . ((string) $home ) . '&mid=' . $mid . $yearView . '&bid=' . $bid, '', '', '', 'editorHeader') .
					we_html_element::iframe('edbody', $bodyURL, 'bottom:0;', 'border:0;width:100%;height:100%;', '', 'editorBody')
				)
		);
	}

	function getHTMLEditorTop(): string{
		$home = we_base_request::bool('home');
		$mid = we_base_request::int('mid', 0);
		$bid = we_base_request::int('bid', 0);

// config
		$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		for($i = 0; $i <= 3; $i++){
			$feldnamen[$i] = $feldnamen[$i] ?? '';
		}
		$fe = explode(',', $feldnamen[3]);

		$classid = $fe[0];


		$resultO = array_shift($fe);

// wether the resultset ist empty?
		$resultD = fB('SELECT 1 FROM ' . CONTENT_TABLE . ' WHERE nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\' LIMIT 1');

		if($home){
			$bodyURL = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&home=1&pnt=edbody'; //same as in getHTMLRight()
		} elseif($resultD && !$resultO){ // docs but no objects
			$bodyURL = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_article_extend&typ=document';
		} elseif(!$resultD && $resultO){ // no docs but objects
			$bodyURL = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_article_extend&typ=object&ViewClass=' . $classid;
		} elseif($resultD && $resultO){
			$bodyURL = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_article_extend&typ=document';
		} else {
			$bodyURL = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_revenueTop';
		}

		$body = we_html_element::iframe('edheader', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edheader&top=1&home=' . ((string) $home) . '&mid=' . $mid . '&bid=' . $bid . '&typ=object&ViewClass=' . $classid, 'position:absolute;top:0;height:60px;left:0;right:0;') .
			we_html_element::iframe('edbody', $bodyURL, 'position:absolute;top:60px;bottom:0;left:0;right:0;');
		return $this->getHTMLDocument(we_html_element::body([], $body));
	}

	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorHeader();
		}

		if(we_base_request::bool('top')){
			return $this->getHTMLEditorHeaderTop();
		}

		$bid = we_base_request::int('bid', 0);

		$hash = getHash('SELECT customerID,DATE_FORMAT(DateOrder,"' . g_l('date', '[format][mysqlDate]') . '") AS d FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);
		if($hash){
			$cid = $hash['customerID'];
			$cdat = $hash['d'];
		} else {
			$cid = 0;
			$cdat = '';
		}
		$we_tabs = new we_gui_tabs();
		$mid = we_base_request::string('mid');
		if($mid && $mid != '00'){
			$we_tabs->addTab(g_l('tabs', '[module][overview]'), '', true, (string) (self::TAB_OVERVIEW));
		} else {
			$we_tabs->addTab(g_l('tabs', '[module][orderdata]'), '', true, (string) (self::TAB_OVERVIEW));
			$we_tabs->addTab(g_l('tabs', '[module][orderlist]'), '', false, (string) (self::TAB_ORDERLIST));
		}

		$textPre = g_l('modules_shop', $bid > 0 ? '[orderList][order]' : '[order_view]');

		$textPost = (string) ($mid > 0 ? (strlen($mid) > 5 ? g_l('modules_shop', '[month][' . substr($mid, 0, -5) . ']') . ' ' . substr($mid, -5, 4) : substr($mid, 1)) : ($bid ? sprintf(g_l('modules_shop', '[orderNo]'), $bid, $cdat) : ''));

		$tab_head = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/shop_frames.js', '', ['id' => 'loadVarShop', 'data-shop' => setDynamicVar([
					'bid' => $bid,
					'cid' => $cid
		])]);

		$tab_body_content = '<div id="main"><div id="headrow"><b>' .
			strtr($textPre, [' ' => '&nbsp;']) .
			':&nbsp;</b><span id="h_path" class="header_small"><b id="titlePath">' .
			strtr($textPost, [' ' => '&nbsp;']) . '</b></span></div>' .
			$we_tabs->getHTML() .
			'</div>';
		$tab_body = we_html_element::body(['onresize' => "weTabs.setFrameSize()", 'onload' => "weTabs.setFrameSize()", 'id' => "eHeaderBody"], $tab_body_content);

		return $this->getHTMLDocument($tab_body, $tab_head);
	}

	function getHTMLEditorHeaderTop(): string{
//$yid = we_base_request::int("ViewYear", date("Y"));
//$bid = we_base_request::int("bid", 0);
		$data = getHash("SELECT ID,DATE_FORMAT(DateOrder,'" . g_l('date', '[format][mysqlDate]') . "') AS orddate FROM " . SHOP_ORDER_TABLE . ' ORDER BY ID DESC LIMIT 1');
		$headline = ($data ? sprintf(g_l('modules_shop', '[lastOrder]'), $data['ID'], $data["orddate"]) : '');

/// config
		$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		$fe = isset($feldnamen[3]) ? explode(',', $feldnamen[3]) : [0];

		$classid = $fe[0];
		$resultO = array_shift($fe);

// wether the resultset ist empty?
		$resultD = fB('SELECT 1 FROM ' . CONTENT_TABLE . ' WHERE nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\' LIMIT 1');

// grep the last element from the year-set, wich is the current year
		$yearTrans = fI('SELECT DATE_FORMAT(MAX(DateOrder),"%Y") AS DateOrd FROM ' . SHOP_ORDER_TABLE);


		$we_tabs = new we_gui_tabs();
		if(!empty($_REQUEST['mid'])){
			$we_tabs->addTab(g_l('tabs', '[module][overview]'), '', true, (string) (self::TAB_ADMIN1));
		} else {
			switch(true){
				default:
				case ($resultD):
					$we_tabs->addTab(g_l('tabs', '[module][admin_1]'), '', true, (string) (self::TAB_ADMIN1));
				case ($resultO):
					$we_tabs->addTab(g_l('tabs', '[module][admin_2]'), '', (!$resultD), (string) (self::TAB_ADMIN2));
				case ($yearTrans != 0):
					$we_tabs->addTab(g_l('tabs', '[module][admin_3]'), '', false, (string) (self::TAB_ADMIN3));
					break;
			}
		}

		$tab_head = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/shop_frames.js', '', ['id' => 'loadVarShop', 'data-shop' => setDynamicVar([
					'yearTrans' => $yearTrans,
					'classid' => $classid
		])]);

		$tab_body_content = '<div id="main"><div id="headrow">&nbsp;' . we_html_element::b($headline) . '</div>' .
			$we_tabs->getHTML() .
			'</div>';
		$tab_body = we_html_element::body(['id' => 'eHeaderBody'], $tab_body_content);

		return $this->getHTMLDocument($tab_body, $tab_head);
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		$protect = we_modules_info::isActive(we_modules_info::SHOP) && we_users_util::canEditModule(we_modules_info::SHOP) ? [] : [false];
		switch($what){
			case 'iconbar':
				return $this->getHTMLIconbar();
			case 'frameset':
				$bid = we_base_request::int('bid');
				return $this->getHTMLFrameset($this->Tree->getJSTreeCode(), ($bid > 0 ? '&bid=' . ($bid === -1 ? fI('SELECT MAX(ID) FROM ' . SHOP_ORDER_TABLE) : $bid) : '&top=1&home=1'));
			case 'customerOrderList':
				we_html_tools::protect($protect);
				return self::showCustomerOrderList();
			case 'settings':
				return self::showPrefDialog();
			case 'savePrefDialog':
				return self::savePrefDialog();
			case 'payment_val':
				return self::showPaymentDialog();
			case 'savePaymentDialog':
				return self::savePaymentDialog($this->jsCmd);
			case 'edit_shop_status':
				return self::showStatusMailsDialog();
			case 'edit_shop_vats':
				return self::showVatsDialog($this->jsCmd);
			case 'edit_shop_vat_country':
				return self::showVatCountryDialog();
			case 'edit_shop_categories':
				return self::showCategoriesDialog($this->jsCmd);
			case 'edit_shop_shipping':
				return self::showDialogShipping($this->jsCmd);
			case 'edit_order_properties':
				we_html_tools::protect($protect);
				return $this->View->getProperties($this->jsCmd);
			case 'edit_shop_article_extend':
				we_html_tools::protect($protect);
				return we_shop_articleExtend::getDialog();
			case 'edit_shop_revenueTop':
				we_html_tools::protect($protect);
				return we_shop_revenueTop::getDialog();
			case 'edit_order_customer':
				return self::showDialogOrderCustomer();
			case 'add_new_article':
				return self::showDialogAddNewArcticle();
			case 'edit_shipping_cost':
				return self::showDialogShippingCost();
			case 'edit_shop_cart_custom_field':
				return self::showDialogCartCustomField();
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	protected function getHTMLEditorBody(): string{
		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen();
		}
		return $this->View->getProperties($this->jsCmd);
	}

	private static function showCustomerOrderList(): string{
		if(($cid = we_base_request::int('cid'))){
			$customer = fS('SELECT CONCAT(Forename," ",Surname) AS Name FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . $cid);
			$orderList = we_shop_shop::getCustomersOrderList($cid);
		} else {
			$customer = $orderList = '';
		}
		return we_html_tools::getHtmlTop('', '', we_html_element::body(['class' => "weEditorBody"], we_html_tools::dialogLayout($orderList, g_l('modules_shop', '[order_liste]') . "&nbsp;" . $customer)));
	}

	private static function showPrefDialog(): string{
		$protect = we_modules_info::isActive(we_modules_info::SHOP) && we_users_util::canEditModule(we_modules_info::SHOP) ? [] : [false];
		we_html_tools::protect($protect);

		$DB_WE = DB_WE::$inst;


		$customerTableFields = $DB_WE->metaFieldNames(CUSTOMER_TABLE);
		$selectFields = ['-' => '-'];
		foreach($customerTableFields as $tblField){
			if(!in_array($tblField, we_shop_shop::IGNORED_EDIT_FIELDS, true)){
				$selectFields[$tblField] = $tblField;
			}
		}

		$shoplocation = fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_location"');
		$categorymode = fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="category_mode"');

		$CLFields = we_unserialize(fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_CountryLanguage"'), ['stateField' => '-',
			'stateFieldIsISO' => 0,
			'languageField' => '-',
			'languageFieldIsISO' => false
		]);


//	generate html-output table
		$htmlTable = new we_html_table(['class' => 'default withBigSpace', 'width' => '410'], 10, 3);


//	NumberFormat - currency and taxes
		$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));

		//$fe = (isset($feldnamen[3]) ? explode(',', $feldnamen[3]) : []);

		if(!isset($feldnamen[4])){
			$feldnamen[4] = '-';
		}

		$row = 0;
//we_html_tools::htmlSelectCountry('weShopVatCountry', '', [], false, array('id' => 'weShopVatCountry'), 200)

		$yesno = [0 => 'false', 1 => 'true'];
		$list = ['german' => 'german', 'english' => 'english', 'french' => 'french', 'swiss' => 'swiss'];
		$pager = ['default' => '-', 5 => 5, 10 => 10, 15 => 15, 20 => 20, 25 => 25, 30 => 30, 35 => 35, 40 => 40, 45 => 45, 50 => 50];
		$htmlTable
			->setCol($row, 0, g_l('modules_shop', '[shopcats][use_shopCats]'))
			->setColAttributes($row, 1, ['style' => 'width:10px;'])
			->setCol($row++, 2, we_html_tools::select('categorymode', $yesno, 1, $categorymode, false, ['id' => "categorymode", 'onchange' => "document.getElementById('shop_holders_location').style.display = (this.value == 1 ? '' : 'none'); document.getElementById('shop_holders_location_br').style.display = (this.value == 1 ? '' : 'none');"]))
			->setRowAttributes($row, ['id' => 'shop_holders_location_br', 'style' => 'display:' . ($categorymode ? '' : 'none')])
			->setRowAttributes($row, ['id' => 'shop_holders_location', 'style' => 'display:' . ($categorymode ? '' : 'none')])
			->setCol($row, 0, g_l('modules_shop', '[shopcats][shopHolderCountry]'))
			->setCol($row++, 2, we_html_tools::selectCountry('shoplocation', '', [$shoplocation], false, ['id' => 'shoplocation'], 280)->getHtml())
			->setCol($row, 0, g_l('modules_shop', '[waehrung]'))
			->setCol($row++, 2, we_html_tools::textInput('waehr', $feldnamen[0]))
			->setCol($row, 0, g_l('modules_shop', '[mwst]'), ['class' => 'defaultfont', 'style' => 'vertical-align:top'])
			->setCol($row++, 2, we_html_tools::textInput('mwst', $feldnamen[1]) . '&nbsp;%', ['class' => 'defaultfont', 'style' => 'padding-bottom:5px;'])
			->setCol($row++, 0, we_html_tools::alertAttentionBox(g_l('modules_shop', '[mwst_expl]'), we_html_tools::TYPE_INFO, "400", false, 45), ['colspan' => '3', 'class' => 'small'])
			->setCol($row, 0, g_l('modules_shop', '[format]'))
			->setCol($row++, 2, we_html_tools::select('format', $list, 1, $feldnamen[2]))
			->setCol($row, 0, g_l('modules_shop', '[pageMod]'))
			->setCol($row++, 2, we_html_tools::select('pag', $pager, 1, $feldnamen[4]));


		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$htmlTable
				->setCol($row, 0, g_l('modules_shop', '[classID]'))
				->setCol($row++, 2, we_html_tools::textInput('classID', ($feldnamen[3] ?? ''), 0, [], 'text', '280px') . '<br/><span class="small">&nbsp;' . g_l('modules_shop', '[classIDext]') . ' </span>');
		}

// look for all available fields in tblCustomer
		$showFields = [];
		foreach(array_column($DB_WE->getAllq('SHOW FIELDS FROM ' . CUSTOMER_TABLE), 'Field') as $field){
			if(!in_array($field, we_shop_shop::IGNORED_EDIT_FIELDS, true) && !in_array($field, we_shop_shop::IGNORED_EXTRA_SHOW_FIELDS, true)){
				$showFields[$field] = $field;
			}
		}
		$showFields = we_html_tools::groupArray($showFields);

//	get the already selected fields ...
		$entry = fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="edit_shop_properties"');

// ...
		if(($fields = we_unserialize($entry))){
			// we have an array with following syntax:
			// array ( 'customerFields' => array('fieldname ...',...)
			//         'orderCustomerFields' => array('fieldname', ...) )
		} else {
			t_e('unsupported Shop-Settings found');
		}
		$languageSelect = we_html_tools::select('languageField', $selectFields, 1, $CLFields['languageField'], false, ['class' => 'searchSelectUp']);
		$languageSelectISO = we_html_forms::checkboxWithHidden((bool) $CLFields['languageFieldIsISO'], 'languageFieldIsISO', g_l('modules_shop', '[preferences][ISO-Kodiert]'), false, "defaultfont");

		$htmlTable
			->setCol($row, 0, g_l('modules_shop', '[preferences][customerFields]', false, true), ['class' => 'defaultfont', 'style' => 'vertical-align:top'])
			->setCol($row++, 2, we_html_tools::select('orderfields[]', $showFields, 1, implode(',', $fields['customerFields']), true, ['class' => 'searchSelectUp'], true, 280))
			->setCol($row, 0, g_l('modules_shop', '[preferences][orderCustomerFields]', false, true), ['class' => 'defaultfont', 'style' => 'vertical-align:top'])
			->setCol($row++, 2, we_html_tools::select('ordercustomerfields[]', $showFields, 1, implode(',', $fields['orderCustomerFields']), true, ['class' => 'searchSelectUp'], true, 280))
			->setCol($row, 0, g_l('modules_shop', '[preferences][CountryField]'), ['class' => 'defaultfont', 'style' => 'vertical-align:top'])

			/* 		$countrySelect = we_html_tools::htmlSelect('stateField', $selectFields, 1, $CLFields['stateField']);
			  $countrySelectISO = we_html_forms::checkboxWithHidden($CLFields['stateFieldIsISO'], 'stateFieldIsISO', g_l('modules_shop', '[preferences][ISO-Kodiert]'), false, "defaultfont");
			 */
			->setCol($row, 0, g_l('modules_shop', '[preferences][LanguageField]'), ['class' => 'defaultfont', 'style' => 'vertical-align:top'])
			->setCol($row++, 2, $languageSelect . '<br/>' . $languageSelectISO);


		$buttons = we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:if(document.getElementById('categorymode').value == 1 && document.getElementById('shoplocation').value === ''){WE().util.showMessage(WE().consts.g_l.shop.preferences_save_alert,WE().consts.message.WE_MESSAGE_ERROR, window);}else{document.we_form.submit();}"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"));

		return we_html_tools::getHtmlTop('', '', we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], '<form name="' . we_base_constants::WE_FORM . '" method="post" action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=savePrefDialog" style="margin-left:8px;">
	' . we_html_tools::dialogLayout($htmlTable->getHtml(), g_l('modules_shop', '[pref]'), $buttons) . '</form>'));
	}

	private static function savePrefDialog(): string{
		$protect = we_modules_info::isActive(we_modules_info::SHOP) && we_users_util::canEditModule(we_modules_info::SHOP) ? [] : [false];
		we_html_tools::protect($protect);
		$DB_WE = DB_WE::$inst;
		if(($format = we_base_request::_(we_base_request::RAW, "format"))){ //	save data in arrays ..
			$settings = ['shop_location' => we_base_request::string('shoplocation'),
				'category_mode' => we_base_request::int('categorymode'),
			];

			foreach($settings as $dbField => $value){
				$DB_WE->query('REPLACE INTO ' . SETTINGS_TABLE . ' SET ' . we_database_base::arraySetter(['tool' => "shop",
						'pref_name' => $dbField,
						'pref_value' => $value
				]));
			}

			$DB_WE->query('REPLACE ' . SETTINGS_TABLE . ' SET ' . we_database_base::arraySetter(['tool' => 'shop',
					'pref_name' => 'shop_pref',
					'pref_value' => we_base_request::string('waehr') . '|' . we_base_request::string('mwst') . '|' . $format . '|' . we_base_request::int('classID') . '|' . we_base_request::string('pag')
			]));

			$fields = [
				'customerFields' => we_base_request::stringList('orderfields'),
				'orderCustomerFields' => we_base_request::stringList('ordercustomerfields')
			];

			// check if field exists
			$DB_WE->query('REPLACE ' . SETTINGS_TABLE . ' SET ' . we_database_base::arraySetter(['tool' => 'shop',
					'pref_name' => 'edit_shop_properties',
					'pref_value' => we_serialize($fields, SERIALIZE_JSON)
			]));

			$CLFields = [
				'stateField' => we_base_request::_(we_base_request::RAW, 'stateField', '-'),
				'stateFieldIsISO' => we_base_request::string('stateFieldIsISO'),
				'languageField' => we_base_request::string('languageField', '-'),
				'languageFieldIsISO' => we_base_request::_(we_base_request::RAW, 'languageFieldIsISO', 0)
			];

			// check if field exists
			$DB_WE->query('REPLACE ' . SETTINGS_TABLE . ' SET tool="shop",pref_name="shop_CountryLanguage", pref_value="' . $DB_WE->escape(we_serialize($CLFields, SERIALIZE_JSON)) . '"');
			// Update Country Field in weShopVatRule
			$weShopVatRule = we_shop_vatRule::getShopVatRule();
			$weShopVatRule->stateField = $CLFields['stateField'];
			$weShopVatRule->stateFieldIsISO = $CLFields['stateFieldIsISO'];
			$weShopVatRule->save();
			// Update Language Field in weShopStatusMails
			$weShopStatusMails = we_shop_statusMails::getShopStatusMails();
			$weShopStatusMails->LanguageData['languageField'] = $CLFields['languageField'];
			$weShopStatusMails->LanguageData['languageFieldIsISO'] = $CLFields['languageFieldIsISO'];
			$weShopStatusMails->save();

			//	Close window when finished
			$jsCmd = new we_base_jsCmd();
			$jsCmd->addCmd('close_dialog');
			return we_html_tools::getHtmlTop('', $jsCmd->getCmds(), we_html_element::body());
		}
		return '';
	}

	private static function showPaymentDialog(): string{
		$protect = we_modules_info::isActive(we_modules_info::SHOP) && we_users_util::canEditModule(we_modules_info::SHOP) ? [] : [false];
		we_html_tools::protect($protect);

		$DB_WE = DB_WE::$inst;

//	NumberFormat - currency and taxes
		$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="payment_details"'));

		for($i = 0; $i <= 18; $i++){
			$feldnamen[$i] = $feldnamen[$i] ?? '';
		}

		$row = 0;

		$Parts = new we_html_multiIconBoxContent();

		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$tmp = array_column($DB_WE->getAllq('SHOW FIELDS FROM ' . CUSTOMER_TABLE), 'Field');
			$custfields = array_combine($tmp, $tmp);

			$htmlTable = (new we_html_table(['class' => 'default withSpace', 'width' => "100%"], 4, 3))
				->setCol($row++, 0, g_l('modules_shop', '[FormFieldsTxt]'), ['colspan' => '4', 'class' => 'defaultfont'])
				->setCol($row, 0, g_l('modules_shop', '[fieldForname]'))
				->setColAttributes($row, 1, ['style' => 'width:10px;'])
				->setCol($row++, 2, we_html_tools::select('fieldForname', $custfields, 1, $feldnamen[0]))
				->setCol($row, 0, g_l('modules_shop', '[fieldSurname]'))
				->setCol($row++, 2, we_html_tools::select('fieldSurname', $custfields, 1, $feldnamen[1]))
				->setCol($row, 0, g_l('modules_shop', '[fieldStreet]'))
				->setCol($row++, 2, we_html_tools::select('fieldStreet', $custfields, 1, $feldnamen[2]))
				->setCol($row, 0, g_l('modules_shop', '[fieldZip]'))
				->setCol($row++, 2, we_html_tools::select('fieldZip', $custfields, 1, $feldnamen[3]))
				->setCol($row, 0, g_l('modules_shop', '[fieldCity]'))
				->setCol($row++, 2, we_html_tools::select('fieldCity', $custfields, 1, $feldnamen[4]))
				->setCol($row, 0, g_l('modules_shop', '[fieldEmail]'))
				->setCol($row++, 2, we_html_tools::select('fieldEmail', array_merge(['' => ''], $custfields), 1, $feldnamen[18]));

			$Parts->add(new we_html_multiIconBoxContentLine('', $htmlTable->getHtml()));
		}

// PayPal
		$row = 0;
		$list = [
			"AI" => "Anguilla",
			"AR" => "Argentina",
			"AU" => "Australia",
			"AT" => "Austria",
			"BE" => "Belgium",
			"BR" => "Brazil",
			"CA" => "Canada",
			"CL" => "Chile",
			"CN" => "China",
			"CR" => "Costa Rica",
			"CY" => "Cyprus",
			"CZ" => "Czech Republic",
			"DK" => "Denmark",
			"DO" => "Dominican Rep.",
			"EC" => "Equador",
			"EE" => "Estonia",
			"FI" => "Finland",
			'fr' => "France",
			"DE" => "Deutschland",
			"GR" => "Greece",
			"HK" => "Hong Kong",
			"HU" => "Hungary",
			"IS" => "Iceland",
			"IN" => "India",
			"IE" => "Ireland",
			"IL" => "Israel",
			"IT" => "Italy",
			"JM" => "Jamaica",
			"JP" => "Japan",
			"LV" => "Latvia",
			"LT" => "Lithuania",
			"LU" => "Luxemburg",
			"MY" => "Malaysia",
			"MT" => "Malta",
			"MX" => "Mexico",
			"NL" => "Netherlands",
			"NZ" => "New Zealand",
			"NO" => "Norway",
			"PL" => "Poland",
			"PT" => "Portugal",
			"SG" => "Singapore",
			"SK" => "Slovakia",
			"ZA" => "South Afrika",
			"KR" => "South Korea",
			"ES" => "Spain",
			"SE" => "Sweden",
			"CH" => "Switzerland",
			"TW" => "Taiwan",
			"TH" => "Thailand",
			"TR" => "Turkey",
			"GB" => "United Kingdom",
			"US" => "United States",
			"UY" => "Uruguay",
			"VE" => "Venezuela"
		];
		$paypalPV = ["default" => "PayPal-Shop", "test" => "Sandbox (Test) "];
		$htmlTable = (new we_html_table(['class' => 'default withSpace', 'width' => "100%"], 4, 3))
			->setCol($row++, 0, g_l('modules_shop', '[paypal][name]'), ['class' => 'weDialogHeadline', 'colspan' => '4'])
			->setCol($row, 0, g_l('modules_shop', '[lc]'))
			->setCol($row++, 2, we_html_tools::select('lc', $list, 1, $feldnamen[5]) . '<span class="small">&nbsp;' . g_l('modules_shop', '[paypalLcTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[paypalbusiness]'))
			->setCol($row++, 2, we_html_tools::textInput("ppB", $feldnamen[6], 0, [], "text", '128px') . '<span class="small">&nbsp;' . g_l('modules_shop', '[paypalbTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[paypalSB]'))
			->setCol($row++, 2, we_html_tools::select('psb', $paypalPV, 1, $feldnamen[7]) . '<span class="small">&nbsp;' . g_l('modules_shop', '[paypalSBTxt]') . ' </span>');

		$Parts->add(new we_html_multiIconBoxContentLine('', $htmlTable->getHtml()));

// saferpay
		$row = 0;
		$saferPayLang = ["en" => "english", "de" => "deutsch", 'fr' => "francais", "it" => "italiano"];
		$htmlTable = (new we_html_table(['class' => 'default withSpace', 'width' => "100%"], 10, 3))
			->setCol($row++, 0, g_l('modules_shop', '[saferpay]'), ['class' => 'weDialogHeadline', 'colspan' => '4'])
			->setCol($row, 0, g_l('modules_shop', '[saferpayTermLang]'))
			->setCol($row++, 2, we_html_tools::select('lcS', $saferPayLang, 1, $feldnamen[8]) . '<span class="small">&nbsp;' . g_l('modules_shop', '[saferpayLcTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[saferpayID]'))
			->setCol($row++, 2, we_html_tools::textInput("spAID", $feldnamen[9], 0, [], 'text', '128px') . '<span class="small">&nbsp;' . g_l('modules_shop', '[saferpayIDTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[saferpaybusiness]'))
			->setCol($row++, 2, we_html_tools::textInput("spB", $feldnamen[10], 0, [], 'text', '128px') . '<span class="small">&nbsp;' . g_l('modules_shop', '[saferpaybTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[saferpayAllowCollect]'))
			->setCol($row++, 2, we_html_tools::select('spC', ["no" => g_l('modules_shop', '[saferpayNo]'), "yes" => g_l('modules_shop', '[saferpayYes]')], 1, $feldnamen[11]) . '<span class="small">&nbsp;' . g_l('modules_shop', '[saferpayAllowCollectTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[saferpayDelivery]'))
			->setCol($row++, 2, we_html_tools::select('spD', ["no" => g_l('modules_shop', '[saferpayNo]'), "yes" => g_l('modules_shop', '[saferpayYes]')], 1, $feldnamen[12]) . '<span class="small">&nbsp;' . g_l('modules_shop', '[saferpayDeliveryTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[saferpayUnotify]'))
			->setCol($row++, 2, we_html_tools::select('spCo', ["no" => g_l('modules_shop', '[saferpayNo]'), "yes" => g_l('modules_shop', '[saferpayYes]')], 1, $feldnamen[13]) . '<span class="small">&nbsp;' . g_l('modules_shop', '[saferpayUnotifyTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[saferpayProviderset]'))
			->setCol($row++, 2, we_html_tools::textInput("spPS", $feldnamen[14], 0, [], "text", '128px') . '<span class="small">&nbsp;' . g_l('modules_shop', '[saferpayProvidersetTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[saferpayCMDPath]'))
			->setCol($row++, 2, we_html_tools::textInput("spcmdP", $feldnamen[15], 0, [], "text", '128px') . '<span class="small">&nbsp;' . g_l('modules_shop', '[saferpayCMDPathTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[saferpayconfPath]'))
			->setCol($row++, 2, we_html_tools::textInput("spconfP", $feldnamen[16], 0, [], "text", '128px') . '<span class="small">&nbsp;' . g_l('modules_shop', '[saferpayconfPathTxt]') . ' </span>')
			->setCol($row, 0, g_l('modules_shop', '[saferpaydesc]'), ['class' => 'defaultfont', 'style' => 'padding-bottom:20px;'])
			->setCol($row++, 2, we_contents_base::htmlTextArea("spdesc", 2, 30, $feldnamen[17]) . '<span class="small">&nbsp;' . g_l('modules_shop', '[saferpaydescTxt]') . ' </span>');

		$Parts->add(new we_html_multiIconBoxContentLine('', $htmlTable->getHtml()));

		$buttons = we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:document.we_form.submit();"), "", we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
		);

		return we_html_tools::getHtmlTop('', '', we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], '<form name="' . we_base_constants::WE_FORM . '" method="post" action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=savePaymentDialog"">
' . (new we_html_multiIconBox($Parts))->marginLeft()->buttons($buttons)->headline(g_l('modules_shop', '[paymentP]'))->get() . '</form>'));
	}

	private static function savePaymentDialog(we_base_jsCmd $jsCmd): string{
		$protect = we_modules_info::isActive(we_modules_info::SHOP) && we_users_util::canEditModule(we_modules_info::SHOP) ? [] : [false];
		we_html_tools::protect($protect);
		$DB_WE = DB_WE::$inst;

		if(($fname = we_base_request::string("fieldForname"))){ //	save data in arrays ..
			$DB_WE->query('REPLACE INTO ' . SETTINGS_TABLE . ' SET ' . we_database_base::arraySetter(['pref_value' => $fname . "|" . we_base_request::string("fieldSurname") . "|" . we_base_request::string("fieldStreet") . "|" . we_base_request::string("fieldZip") . "|" . we_base_request::string("fieldCity") . "|" . we_base_request::string("lc") . "|" . we_base_request::string("ppB") . "|" . we_base_request::string("psb") . "|" . we_base_request::string("lcS") . "|" . we_base_request::string("spAID") . "|" . we_base_request::string("spB") . "|" . we_base_request::string("spC") . "|" . we_base_request::string("spD") . "|" . we_base_request::string("spCo") . "|" . we_base_request::string("spPS") . "|" . we_base_request::string("spcmdP") . "|" . we_base_request::string("spconfP") . "|" . we_base_request::string("spdesc") . "|" . we_base_request::string("fieldEmail"),
					'tool' => 'shop',
					'pref_name' => 'payment_details',
			]));
			$jsCmd->addCmd('close_dialog');

			//	Close window when finished
			return we_html_tools::getHtmlTop('', $jsCmd->getCmds(), we_html_element::body());
		}
		return '';
	}

	private static function showStatusMailsDialog(): string{
		$protect = we_modules_info::isActive(we_modules_info::SHOP) && we_users_util::canEditModule(we_modules_info::SHOP) ? [] : [false];
		we_html_tools::protect($protect);
		$DB_WE = DB_WE::$inst;

// initialise the shopStatusMails Object
		if(we_base_request::string('cmd', '') === 'saveShopStatusMails'){
			// initialise the vatRule by request
			$weShopStatusMails = we_shop_statusMails::initByRequest();
			$weShopStatusMails->save();
		} else {

			$weShopStatusMails = we_shop_statusMails::getShopStatusMails();
		}

// array with all rules

		$customerTableFields = $DB_WE->metaFieldNames(CUSTOMER_TABLE);
		$selectFields = ['-' => '-'];
		foreach($customerTableFields as $tblField){
			if(!in_array($tblField, we_shop_shop::IGNORED_EDIT_FIELDS, true)){
				$selectFields[$tblField] = $tblField;
			}
		}

		$frontendL = we_base_WE::$Langs['FrontendLanguages'];
		foreach($frontendL as &$lcvalue){
			$lccode = explode('_', $lcvalue);
			$lcvalue = $lccode[0];
		}
		unset($lcvalue);

		$tabStatus = (new we_html_table(['class' => 'withSpace'], 5, 6 + count($frontendL)))
			->setCol(0, 1, g_l('modules_shop', '[statusmails][AnzeigeDaten]'), ['colspan' => '4', 'class' => 'defaultfont bold', 'style' => 'text-align:center;background-color:yellow;'])
			->setCol(0, 5, g_l('modules_shop', '[statusmails][Dokumente]'), ['colspan' => (string) (1 + count($frontendL)), 'class' => 'defaultfont bold', 'style' => 'text-align:center;background-color:lightblue;'])
			->setCol(1, 0, g_l('modules_shop', '[statusmails][fieldname]', false, true), ['class' => 'defaultfont bold'])
			->setCol(1, 1, g_l('modules_shop', '[statusmails][hidefield]', false, true), ['class' => 'defaultfont bold'])
			->setCol(1, 2, g_l('modules_shop', '[statusmails][hidefieldCOV]', false, true), ['class' => 'defaultfont bold'])
			->setCol(1, 3, g_l('modules_shop', '[statusmails][fieldtext]', false, true), ['class' => 'defaultfont bold'])
			->setCol(1, 4, g_l('modules_shop', '[statusmails][EMailssenden]', false, true), ['class' => 'defaultfont bold'])
			->setCol(1, 5, g_l('modules_shop', '[statusmails][defaultDocs]', false, true), ['class' => 'defaultfont bold']);

		foreach($frontendL as $pos => $langkey){
			$tabStatus->setCol(1, (int) (6 + $pos), g_l('languages', '[' . $langkey . ']') . ' (' . $langkey . ')', ['class' => 'defaultfont bold']);
		}

		foreach(we_shop_statusMails::StatusFields as $fieldkey => $fieldname){
			$tabStatus->setCol($fieldkey + 2, 0, $fieldname, ['class' => 'defaultfont bold', 'width' => '120']);
		}
		foreach(we_shop_statusMails::StatusFields as $fieldkey => $fieldname){
			$tabStatus->setCol($fieldkey + 2, 1, we_html_forms::checkboxWithHidden($weShopStatusMails->FieldsHidden[$fieldname], 'FieldsHidden[' . $fieldname . ']', g_l('modules_shop', '[statusmails][hidefieldJa]'), false, "defaultfont"));
		}

		foreach(we_shop_statusMails::StatusFields as $fieldkey => $fieldname){
			$tabStatus->setCol($fieldkey + 2, 2, we_html_forms::checkboxWithHidden($weShopStatusMails->FieldsHiddenCOV[$fieldname], 'FieldsHiddenCOV[' . $fieldname . ']', g_l('modules_shop', '[statusmails][hidefieldJa]'), false, "defaultfont"));
		}

		foreach(we_shop_statusMails::StatusFields as $fieldkey => $fieldname){
			$tabStatus->setCol($fieldkey + 2, 3, '<input name="FieldsText[' . $fieldname . ']" size="15" type="text" value="' . $weShopStatusMails->FieldsText[$fieldname] . '" />');
		}

		foreach(we_shop_statusMails::StatusFields as $fieldkey => $fieldname){
			$tabStatus->setCol($fieldkey + 2, 4, we_html_forms::radioButtonV('0', (string) $weShopStatusMails->FieldsMails[$fieldname], 'FieldsMails[' . $fieldname . ']', g_l('modules_shop', '[statusmails][EMailssendenNein]')) .
				we_html_forms::radioButtonV('1', (string) $weShopStatusMails->FieldsMails[$fieldname], 'FieldsMails[' . $fieldname . ']', g_l('modules_shop', '[statusmails][EMailssendenHand]')) .
				we_html_forms::radioButtonV('2', (string) $weShopStatusMails->FieldsMails[$fieldname], 'FieldsMails[' . $fieldname . ']', g_l('modules_shop', '[statusmails][EMailssendenAuto]')));
		}

		foreach(we_shop_statusMails::StatusFields as $fieldkey => $fieldname){
			$tabStatus->setCol($fieldkey + 2, 5, we_html_tools::textInput("FieldsDocuments[default][" . $fieldname . "]", $weShopStatusMails->FieldsDocuments['default'][$fieldname]));
		}

		foreach($frontendL as $pos => $langkey){
			foreach(we_shop_statusMails::StatusFields as $fieldkey => $fieldname){
				$tabStatus->setCol($fieldkey + 2, (int) (6 + $pos), we_html_tools::textInput('FieldsDocuments[' . $langkey . '][' . $fieldname . ']', $weShopStatusMails->FieldsDocuments[$langkey][$fieldname]));
			}
		}

		$tabEMail = (new we_html_table(['class' => 'withSpace'], 4, 6))
			->setCol(0, 0, g_l('modules_shop', '[statusmails][AbsenderAdresse]') .
				'<br/>' . we_html_tools::textInput("EMailData[address]", $weShopStatusMails->EMailData['address']), ['class' => 'defaultfont', 'width' => '220'])
			->setCol(1, 0, g_l('modules_shop', '[statusmails][AbsenderName]') .
				'<br/>' . we_html_tools::textInput("EMailData[name]", $weShopStatusMails->EMailData['name']), ['class' => 'defaultfont', 'width' => '220'])
			->setCol(2, 0, g_l('modules_shop', '[statusmails][bcc]') .
				'<br/>' . we_html_tools::textInput("EMailData[bcc]", $weShopStatusMails->EMailData['bcc']), ['class' => 'defaultfont', 'width' => '220'])
			->setCol(0, 1, g_l('modules_shop', '[statusmails][EMailFeld]') .
				'<br/>' . we_html_tools::select('EMailData[emailField]', $selectFields, 1, $weShopStatusMails->EMailData['emailField']), ['class' => 'defaultfont', 'width' => '340'])
			->setCol(1, 1, g_l('modules_shop', '[statusmails][TitelFeld]') .
				'<br/>' . we_html_tools::select('EMailData[titleField]', $selectFields, 1, $weShopStatusMails->EMailData['titleField']), ['class' => 'defaultfont', 'width' => '340'])
			->setCol(2, 1, g_l('modules_shop', '[statusmails][DocumentSubjectField]') .
				'<br/>' . we_html_tools::textInput("EMailData[DocumentSubjectField]", $weShopStatusMails->EMailData['DocumentSubjectField']), ['class' => 'defaultfont', 'width' => '340'])
			->setCol(3, 0, g_l('modules_shop', '[statusmails][DocumentAttachmentFieldA]') . '<br/>' . we_html_tools::textInput("EMailData[DocumentAttachmentFieldA]", $weShopStatusMails->EMailData['DocumentAttachmentFieldA']), [
				'class' => 'defaultfont', 'width' => '340'])
			->setCol(3, 1, g_l('modules_shop', '[statusmails][DocumentAttachmentFieldB]') . '<br/>' . we_html_tools::textInput("EMailData[DocumentAttachmentFieldB]", $weShopStatusMails->EMailData['DocumentAttachmentFieldB']), [
			'class' => 'defaultfont', 'width' => '340']);

		$tabSprache = (new we_html_table(['class' => 'withSpace'], 2, 5))
			->setCol(0, 0, we_html_forms::checkboxWithHidden((bool) $weShopStatusMails->LanguageData['useLanguages'], 'LanguageData[useLanguages]', g_l('modules_shop', '[statusmails][useLanguages]', false, true), false), [
				'class' => 'defaultfont', 'width' => '220'])
			->setCol(0, 2, g_l('modules_shop', '[statusmails][SprachenFeld]') . we_html_tools::select('LanguageData[languageField]', $selectFields, 1, $weShopStatusMails->LanguageData['languageField']) . we_html_forms::checkboxWithHidden((bool) $weShopStatusMails->LanguageData['languageFieldIsISO'], 'LanguageData[languageFieldIsISO]', g_l('modules_shop', '[preferences][ISO-Kodiert]'), false), [
			'class' => 'defaultfont', 'width' => '220']);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[statusmails][AnzeigeDaten]'), we_html_tools::alertAttentionBox(g_l('modules_shop', '[statusmails][hintDokumente]', false, true), we_html_tools::TYPE_INFO, 650, false)))
				->noline()
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add(new we_html_multiIconBoxContentLine('', $tabStatus->getHtml()))
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[statusmails][EMailDaten]'), we_html_tools::alertAttentionBox(g_l('modules_shop', '[statusmails][hintEMailDaten]', false, true), we_html_tools::TYPE_INFO, 650, false)))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine('', $tabEMail->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED))
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[statusmails][Spracheinstellungen]'), we_html_tools::alertAttentionBox(g_l('modules_shop', '[statusmails][hintSprache]', false, true), we_html_tools::TYPE_INFO, 650, false)))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline())
			->add((new we_html_multiIconBoxContentLine('', $tabSprache->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline()
			)
			->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('modules_shop', '[statusmails][hintISO]', false, true), we_html_tools::TYPE_INFO, 650, false)));

		return we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/statusMails.js'), we_html_element::body(['class' => 'weDialogBody',
					'onload' => "window.focus();"], '<form name="' . we_base_constants::WE_FORM . '" method="post" action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_status">
		<input type="hidden" name="cmd" value="saveShopStatusMails" />' .
						(new we_html_multiIconBox($parts, 'weShopStatusMails'))->marginLeft()->buttons(we_html_button::position_yes_no_cancel(
								we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('we_shop_save');"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
						))
						->headline(g_l('modules_shop', '[statusmails][box_headline]'))
						->get() .
					'</form>'));
	}

	private static function showVatsDialog(we_base_jsCmd $jsCmd): string{
		$protect = we_modules_info::isActive(we_modules_info::SHOP) && we_users_util::canEditModule(we_modules_info::SHOP) ? [] : [false];
		we_html_tools::protect($protect);
		$shopVat = null;
		$saveSuccess = false;
		$onsaveClose = we_base_request::bool('onsaveclose', false);
		switch(we_base_request::string('cmd', '')){
			case 'saveVat':
				$province = we_base_request::string('weShopVatProvince');
				$territory = we_base_request::string('weShopVatCountry') . ($province ? '-' . $province : '');

				$shopVat = new we_shop_vat(we_base_request::int('weShopVatId'), we_base_request::string('weShopVatText'), we_base_request::float('weShopVatVat'), we_base_request::bool('weShopVatStandard'), $territory, we_base_request::string('weShopVatTextProvince'));

				if(($newId = we_shop_vats::saveWeShopVAT($shopVat))){
					$shopVat->id = $newId;
					unset($newId);
					$saveSuccess = true;
					$jsCmd->addMsg(g_l('modules_shop', '[vat][save_success]'), we_base_util::WE_MESSAGE_NOTICE);
				} else {
					$jsCmd->addMsg(g_l('modules_shop', '[vat][save_error]'), we_base_util::WE_MESSAGE_ERROR);
				}

				break;

			case 'deleteVat':
				if(we_shop_vats::deleteVatById(we_base_request::int('weShopVatId'))){
					$jsCmd->addMsg(g_l('modules_shop', '[vat][delete_success]'), we_base_util::WE_MESSAGE_NOTICE);
				} else {
					$jsCmd->addMsg(g_l('modules_shop', '[vat][delete_error]'), we_base_util::WE_MESSAGE_ERROR);
				}
				break;
		}

		$shopVat = $shopVat ?: new we_shop_vat(0, g_l('modules_shop', '[vat][new_vat_name]'), 19, false);

// at top of page show a table with all actual vats
		$allVats = we_shop_vats::getAllShopVATs();
		$vatJSON = [
			'vat_0' => [
				'id' => 0,
				'text' => g_l('modules_shop', '[vat][new_vat_name]'),
				'vat' => 19,
				'standard' => 0,
				'country' => 'DE',
				'province' => '',
				'textProvince' => ''
			]
		];

		if($allVats){
			$vatTable = '
	<div style="height:300px; width: 550px; padding-right: 40px; overflow:auto;">
		<table class="defaultfont">
		<tr>
			<td><strong>Id</strong></td>
			<td><strong>' . g_l('modules_shop', '[vat][vat_form_name]') . '</strong></td>
			<td><strong>' . g_l('modules_shop', '[vat][vat_form_vat]') . '</strong></td>
			<td><strong>' . g_l('modules_shop', '[vat][vat_form_StateRegion]') . '</strong></td>
			<td><strong>ISO</strong></td>
			<td><strong>' . g_l('modules_shop', '[vat][vat_form_standard]') . '</strong></td>
		</tr>';

			foreach($allVats as $Vat){
				$vatJSON['vat_' . $Vat->id] = [
					'id' => $Vat->id,
					'text' => $Vat->getNaturalizedText(),
					'vat' => $Vat->vat,
					'standard' => ($Vat->standard ? 1 : 0),
					'territory' => $Vat->territory,
					'country' => $Vat->country,
					'province' => $Vat->province,
					'textProvince' => $Vat->textProvince,
				];
				$vatTable .= '
		<tr>
			<td>' . $Vat->id . '</td>
			<td>' . htmlspecialchars($Vat->getNaturalizedText()) . '</td>
			<td>' . $Vat->vat . '%</td>
			<td>' . $Vat->textTerritory . '</td>
			<td>' . $Vat->territory . '</td>
			<td>' . g_l('global', ($Vat->standard ? '[yes]' : '[no]')) . '</td>
			<td>' . we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:we_cmd('edit','" . $Vat->id . "');") . '</td>
			<td>' . we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('delete','" . $Vat->id . "');") . '</td>
		</tr>';
				unset($Vat);
			}


			$vatTable .= '</table>
	</div>';
		} else {
			$vatTable = '';
		}

		$plusBut = we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('addVat')");

// formular to edit the vats
		$selPredefinedNames = we_html_tools::select(
				'sel_predefinedNames', array_merge(['---'], we_shop_vat::getPredefinedNames()), 1, 0, false, ['onchange' => "var elem=document.getElementById('weShopVatText');elem.value=this.options[this.selectedIndex].text;this.selectedIndex=0"]
		);

		$formVat = '
<form name="' . we_base_constants::WE_FORM . '" method="post" action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_vats">
<input type="hidden" name="weShopVatId" id="weShopVatId" value="' . $shopVat->id . '" />
<input type="hidden" name="onsaveclose" value="0" />
<input type="hidden" name="we_cmd[0]" value="we_showMod" />
<input type="hidden" name="cmd" value="saveVat" />
<table class="defaultfont" id="editShopVatForm" style="display:none;">
<tr>
	<td colspan="2"><strong>' . g_l('modules_shop', '[vat][vat_edit_form_headline]') . '</strong></td>
</tr>
<tr>
	<td style="width:100px">' . g_l('modules_shop', '[vat][vat_form_name]') . ':</td>
	<td><input class="wetextinput" type="text" id="weShopVatText" name="weShopVatText" value="' . $shopVat->text . '" />' . $selPredefinedNames . '</td>
</tr>
<tr>
	<td>' . g_l('modules_shop', '[vat][vat_form_vat]') . ':</td>
	<td><input class="wetextinput" type="number" id="weShopVatVat" name="weShopVatVat" value="' . $shopVat->vat . '" />%</td>
</tr>

<tr>
	<td>' . g_l('modules_shop', '[vat][vat_edit_form_state]') . ':</td>
	<td>' . we_html_tools::selectCountry('weShopVatCountry', '', [], false, ['id' => 'weShopVatCountry'], 200)->getHtml() . '</td>
</tr>
<tr>
	<td>' . g_l('modules_shop', '[vat][vat_edit_form_province]') . ':</td>
	<td>(-<input style="width:6em" class="wetextinput" type="text" id="weShopVatProvince" name="weShopVatProvince" value="" />)</td>
</tr>

<tr>
	<td>' . g_l('modules_shop', '[vat][vat_form_standard]') . ':</td>
	<td><select id="weShopVatStandard" name="weShopVatStandard">
			<option value="1"' . ($shopVat->standard ? ' selected="selected"' : '') . '>' . g_l('modules_shop', '[vat][vat_edit_form_yes]') . '</option>
			<option value="0"' . ($shopVat->standard ? '' : ' selected="selected"') . '>' . g_l('modules_shop', '[vat][vat_edit_form_no]') . '</option>
		</select>
	</td>
	<td></td>
</tr>
<tr>
	<td></td>
	<td>' . we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_notclose');") . ' ' . we_html_button::create_button(we_html_button::CANCEL, "javascript:we_cmd('cancel_notclose');") . '</td>
</tr>
</table>
</form>';

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', $vatTable))
			->add(new we_html_multiIconBoxContentLine('', $plusBut))
			->add(new we_html_multiIconBoxContentLine('', $formVat));

		if($saveSuccess && $onsaveClose){
			$jsCmd->addCmd('close_dialog');
		}

		return we_html_tools::getHtmlTop('', $jsCmd->getCmds() .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/edit_shop_vats.js', '', ['id' => 'loadVarEdit_shop_vats', 'data-allVats' => setDynamicVar($vatJSON)]), we_html_element::body([
					'class' => 'weDialogBody', 'onload' => "window.focus();addListeners();"],
																																																																																										 (new we_html_multiIconBox($parts, 'weShopVates'))->marginLeft()->buttons(we_html_button::formatButtons(we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")))->headline(g_l('modules_shop', '[vat][vat_edit_form_headline_box]'))
						->get()
		));
	}

	private static function showVatCountryDialog(): string{
		$protect = we_modules_info::isActive(we_modules_info::SHOP) && we_users_util::canEditModule(we_modules_info::SHOP) ? [] : [false];
		we_html_tools::protect($protect);
		$DB_WE = DB_WE::$inst;

// initialise the vatRuleObject
		if(we_base_request::string('cmd') === 'saveVatRule'){
			// initialise the vatRule by request
			$weShopVatRule = we_shop_vatRule::initByRequest();
			$weShopVatRule->save();
		} else {
			$weShopVatRule = we_shop_vatRule::getShopVatRule();
		}

// array with all rules

		$selectFields = [];
		$customerTableFields = $DB_WE->metaFieldNames(CUSTOMER_TABLE);
		foreach($customerTableFields as $tblField){
			$selectFields[$tblField] = $tblField;
		}

// default value f�r mwst
		$defaultInput = we_html_tools::select('defaultValue', ['true' => 'true', 'false' => 'false'], 1, $weShopVatRule->defaultValue);
// select field containing land
		$countrySelect = we_html_tools::select('stateField', $selectFields, 1, $weShopVatRule->stateField);
		$countrySelectISO = we_html_forms::checkboxWithHidden((bool) $weShopVatRule->stateFieldIsISO, 'stateFieldIsISO', g_l('modules_shop', '[preferences][ISO-Kodiert]'), false, "defaultfont");
// states which must always pay vat

		$textAreaLiableStates = we_contents_base::htmlTextArea('liableToVat', 3, 30, implode("\n", $weShopVatRule->liableToVat));
// states which must never pay vat

		$textAreaNotLiableStates = we_contents_base::htmlTextArea('notLiableToVat', 3, 30, implode("\n", $weShopVatRule->notLiableToVat));

// states which must only pay under certain circumstances
		// we_shop_vatRule::makeConditionalArrayFromRequest() is ready to process multiple rules
		// to make more rules possible - adjust dialog here
		$actCondition = $weShopVatRule->conditionalRules[0];
		$conditionTextarea = we_contents_base::htmlTextArea('conditionalStates[0]', 3, 30, implode("\n", $actCondition['states']));
		$conditionField = we_html_tools::select('conditionalCustomerField[0]', $selectFields, 1, $actCondition['customerField']);
		$conditionSelect = we_html_tools::select('conditionalCondition[0]', ['is_empty' => g_l('modules_shop', '[vat_country][condition_is_empty]'), 'is_set' => g_l('modules_shop', '[vat_country][condition_is_set]')], 1, $actCondition['condition']);
		$conditionReturn = we_html_tools::select('conditionalReturn[0]', ['false' => 'false', 'true' => 'true'], 1, $actCondition['returnValue']);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[vat_country][defaultReturn]'), $defaultInput))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('modules_shop', '[vat_country][defaultReturn_desc]'), we_html_tools::TYPE_INFO, 600)))
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[vat_country][stateField]') . ':', $countrySelect . $countrySelectISO))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('modules_shop', '[vat_country][stateField_desc]'), we_html_tools::TYPE_INFO, 600)))
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[vat_country][statesLiableToVat]') . ':', $textAreaLiableStates))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('modules_shop', '[vat_country][statesLiableToVat_desc]'), we_html_tools::TYPE_INFO, 600)))
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[vat_country][statesNotLiableToVat]') . ':', $textAreaNotLiableStates))
				->noline()
				->space(we_html_multiIconBox::SPACE_BIG)
			)
			->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('modules_shop', '[vat_country][statesNotLiableToVat_desc]'), we_html_tools::TYPE_INFO, 600)))
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[vat_country][statesSpecialRules]') . ':', $conditionTextarea))
				->noline()
				->space(we_html_multiIconBox::SPACE_BIG)
			)
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('modules_shop', '[vat_country][statesSpecialRules_desc]'), we_html_tools::TYPE_INFO, 600)))
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[vat_country][statesSpecialRules_condition]'), $conditionField . ' ' . $conditionSelect))
				->noline()
				->space(we_html_multiIconBox::SPACE_BIG)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[vat_country][statesSpecialRules_result]'), $conditionReturn))
			->space(we_html_multiIconBox::SPACE_BIG)
		);

		return we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/edit_shop_country.js'), we_html_element::body(['class' => 'weDialogBody',
					'onload' => "window.focus();"], '<form name="' . we_base_constants::WE_FORM . '" method="post" action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_vat_country">
		<input type="hidden" name="cmd" value="saveVatRule" />' .
						(new we_html_multiIconBox($parts, 'weShopCountryVat'))->marginLeft()
						->buttons(we_html_button::position_yes_no_cancel(
								we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('we_shop_save');"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
						))
						->headline(g_l('modules_shop', '[vat_country][box_headline]'))
						->get() . '</form>'));
	}

	private static function showCategoriesDialog(we_base_jsCmd $jsCmd): string{
		$protect = we_modules_info::isActive(we_modules_info::SHOP) && we_users_util::canEditModule(we_modules_info::SHOP) ? [] : [false];
		we_html_tools::protect($protect);

		$DB_WE = DB_WE::$inst;
//FIXME: mak sowme view class for this editor and use processVariables() and processCommands()?
//process request
		$shopCategoriesDir = ($val = we_base_request::int('weShopCatDir', -1)) > -1 ? $val : we_shop_category::getShopCatDirID();
		$relations = [];
		$saveSuccess = false;
		$onsaveClose = we_base_request::bool('onsaveclose', false);
		if($shopCategoriesDir && we_base_request::string('cmd') === 'saveShopCatRels'){
			$saveSuccess = we_shop_category::saveShopCatsDir($shopCategoriesDir);

			$destPrincipleIds = [];
			foreach(we_base_request::intList('weShopCatDestPrinciple') as $k => $v){
				if($v){
					$destPrincipleIds[] = intval($k);
				}
			}
			$saveSuccess = $saveSuccess && we_shop_category::saveSettingDestPrinciple($destPrincipleIds);

			//FIXME: get destPrinciple and isActive from db at once
			$isInactiveIds = [];
			foreach(we_base_request::intList('weShopCatIsActive') as $k => $v){
				if(!$v){
					$isInactiveIds[] = intval($k);
				}
			}
			$saveSuccess = $saveSuccess && we_shop_category::saveSettingIsInactive($isInactiveIds);

			$saveCatIds = [];
			$relations = we_base_request::stringList('weShopCatRels');
			'@phan-var array<string,array<string,string>> $relations';
			foreach($relations as $k => $v){
				foreach($v as $id){
					if(!isset($saveCatIds[$id])){
						$saveCatIds[$id] = [];
					}
					$saveCatIds[$id][] = intval($k);
				}
			}

			//reset all vat-category relations before saving the new set of relations
			$saveSuccess = $saveSuccess && $DB_WE->query('UPDATE ' . WE_SHOP_VAT_TABLE . ' SET categories=""');
			foreach($saveCatIds as $vatId => $catIds){
				$saveSuccess = $saveSuccess && $DB_WE->query('UPDATE ' . WE_SHOP_VAT_TABLE . ' SET categories="' . implode(',', $catIds) . '" WHERE id=' . intval($vatId));
			}

			if($saveSuccess){
				$jsCmd->addMsg(g_l('modules_shop', '[shopcats][save_success]'), we_base_util::WE_MESSAGE_NOTICE);
			} else {
				$jsCmd->addMsg(g_l('modules_shop', '[shopcats][save_error]'), we_base_util::WE_MESSAGE_ERROR);
			}
		} else {
			//please select category dir...
		}

//make category dirs select
		$allCategoryDirs = ['-1' => g_l('modules_shop', '[shopcats][select_shopCatDir]')] + $DB_WE->getAllFirstq('SELECT ID,Path FROM ' . CATEGORY_TABLE . ' ORDER BY Path', false);

		$selCategoryDirs = we_html_tools::select('weShopCatDir', $allCategoryDirs, 1, $shopCategoriesDir, false, ['id' => 'weShopCatDir', 'onchange' => "we_submitForm(WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_categories');"]);

		if($shopCategoriesDir && $shopCategoriesDir !== -1){
			$allVats = we_shop_vats::getAllShopVATs();
			$vatGroups = [];
			if(count($allVats) > 0){
				$doWriteRelations = empty($relations);
				foreach($allVats as $vatObj){
					if(!isset($vatGroups[$vatObj->territory])){
						$vatGroups[$vatObj->territory] = [];
						$vatGroups[$vatObj->territory]['selOptions'] = [0 => ' '];
					}
					$vatGroups[$vatObj->territory]['textTerritory'] = $vatObj->textTerritory;
					$vatGroups[$vatObj->territory]['selOptions'][$vatObj->id] = $vatObj->getNaturalizedText() . ': ' . $vatObj->vat . '%';

					if($doWriteRelations){
						foreach(explode(',', $vatObj->categories) as $cat){
							if($cat){
								if(!isset($relations[$cat])){
									$relations[$cat] = [];
								}
								$relations[$cat][$vatObj->territory] = $vatObj->id;
							}
						}
					}
				}
			}

			$shopCategories = we_shop_category::getShopCatsFromDir($shopCategoriesDir, false, true, '', 'Path');
			$catsTable = new we_html_table(['class' => 'withSpace'], (count($shopCategories) * 6), 3);
			$catsDirTable = new we_html_table(['class' => 'withSpace'], 7, 3);
			if(count($shopCategories) > 1){
				$i = $iTmp = 0;

				foreach($shopCategories as $cat){
					if($cat->ID === $shopCategoriesDir){
						$isShopCatsDir = true;
						$table = $catsDirTable;
						$iTmp = $i;
						$i = 0;
					} else {
						$table = $catsTable;
						$isShopCatsDir = false;
					}

					$table
						->setCol($i, 0, '<abbr title="ShopCatID: ' . $cat->ID . '">' . $cat->Text . '</abbr>', ['class' => 'defaultfont bold', 'width' => '140px'])
						->setCol($i++, 1, $cat->Path, ['class' => 'defaultfont bold', 'colspan' => '2']);
					if($cat->ID != $shopCategoriesDir){
						$table
							->setCol($i, 1, g_l('modules_shop', '[shopcats][active_shopCat]'))
							->setCol($i++, 2, we_html_forms::checkboxWithHidden(($cat->IsInactive == 0), 'weShopCatIsActive[' . $cat->ID . ']', '', false, '', 'we_switch_active_by_id(' . $cat->ID . ')'));
					}

					//set attribute $unique for radio button to 'true' for corret labels
					$taxPrinciple = we_html_forms::radioButton('0', !$cat->DestPrinciple, 'weShopCatDestPrinciple[' . $cat->ID . ']', g_l('modules_shop', '[shopcats][text_originPrinciple]'), true, 'defaultfont', 'we_switch_principle_by_id(' . $cat->ID . ', this, ' . ($isShopCatsDir ? 'true' : 'false') . ')') .
						we_html_forms::radioButton('1', $cat->DestPrinciple, 'weShopCatDestPrinciple[' . $cat->ID . ']', g_l('modules_shop', '[shopcats][text_destPrinciple]'), true, 'defaultfont', 'we_switch_principle_by_id(' . $cat->ID . ', this, ' . ($isShopCatsDir ? 'true' : 'false') . ')') .
						we_html_element::hidden('taxPrinciple_tmp[' . $cat->ID . ']', (int) $cat->DestPrinciple, 'taxPrinciple_tmp[' . $cat->ID . ']');

					$table
						->setRowAttributes($i, ['id' => 'destPrincipleRow_' . $cat->ID, 'style' => ($cat->IsInactive ? 'display: none;' : '')])
						->setCol($i, 1, g_l('modules_shop', '[shopcats][title_taxationMode]'), ['class' => 'defaultfont', 'style' => 'padding-bottom: 10px'])
						->setCol($i++, 2, $taxPrinciple, ['class' => 'defaultfont', 'style' => 'padding-bottom: 10px']);

					if(!count($allVats)){
						$table->setCol($i, 1, g_l('modules_shop', '[shopcats][warning_noVatsDefined]'));
					} else {
						$holderCountryTable = new we_html_table(['class' => 'default'], 1, 2);
						$countriesTable = new we_html_table(['class' => 'default'], max((count($allVats) - 1), 1), 2);

						$c = -1;
						foreach($vatGroups as $key => $v){
							if(we_shop_category::getDefaultCountry() == $key){
								$innerTable = $holderCountryTable;
								$num = 0;
								$isDefCountry = true;
							} else {
								$innerTable = $countriesTable;
								$c++;
								$num = $c;
								$isDefCountry = false;
							}

							$value = !empty($relations[$cat->ID][$key]) ? $relations[$cat->ID][$key] : 0;
							$selAttribs = ['id' => 'weShopCatRels[' . $cat->ID . '][' . $key . ']'];
							$sel = we_html_tools::select('weShopCatRels[' . $cat->ID . '][' . $key . ']', $v['selOptions'], 1, $value, false, $selAttribs, true, 220);

							$innerTable
								->setCol($num, 0, ($v['textTerritory'] ?: 'N.N.'), ['class' => 'defaultfont' . ($isDefCountry ? ' bold' : ''), 'style' => ($isDefCountry ? '' : 'padding-bottom: 8px;')])
								->setCol($num, 1, $sel);
						}

						$table
							->setRowAttributes($i, ['id' => 'defCountryRow_' . $cat->ID, 'style' => (!$cat->IsInactive ? '' : 'display: none;')])
							->setCol($i++, 1, $holderCountryTable->getHtml(), ['class' => 'defaultfont', 'colspan' => '2'])
							->setRowAttributes($i, ['id' => 'countriesRow_' . $cat->ID, 'style' => ($cat->IsInactive || !$cat->DestPrinciple ? 'display: none;' : '')])
							->setCol($i++, 1, $countriesTable->getHtml(), ['class' => 'defaultfont', 'colspan' => '2']);
					}
					$i = $cat->ID == $shopCategoriesDir ? $iTmp : $i;
				}
				$catsTableHtml = $catsTable->getHtml();
				$catsDirTableHtml = $catsDirTable->getHtml();
			} else {
				$catsTableHtml = g_l('modules_shop', '[shopcats][warning_shopCatDirEmpty]');
				$catsDirTableHtml = g_l('modules_shop', '[shopcats][warning_shopCatDirEmpty]');
			}
		} else {
			$catsTableHtml = $catsDirTableHtml = g_l('modules_shop', '[shopcats][warning_noShopCatDir]');
		}

		if($saveSuccess && $onsaveClose){
			$jsCmd->addCmd('close_dialog');
		}

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[shopcats][text_shopCatDir]'), $selCategoryDirs))
				->space(we_html_multiIconBox::SPACE_BIG)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[shopcats][text_editShopCatDir]'), we_html_tools::alertAttentionBox(g_l('modules_shop', '[shopcats][info_edit_shopCatDir]'), we_html_tools::TYPE_INFO, 400, false, 40)))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add(new we_html_multiIconBoxContentLine('', $catsDirTableHtml))
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[shopcats][text_editShopCats]'), we_html_tools::alertAttentionBox(g_l('modules_shop', '[shopcats][info_editShopCats]'), we_html_tools::TYPE_INFO, 400, false, 40)))
				->noline()
				->space(we_html_multiIconBox::SPACE_BIG)
			)
			->add((new we_html_multiIconBoxContentLine('', $catsTableHtml))
			->noline());

		return we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/showCategoriesDialog.js') . $jsCmd->getCmds(), we_html_element::body([
					'class' => 'weDialogBody', 'onload' => "window.focus(); addListeners();"], '<form name="' . we_base_constants::WE_FORM . '" method="post" >
	<input type="hidden" name="cmd" value="load" /><input type="hidden" name="onsaveclose" value="0" />' .
						(new we_html_multiIconBox($parts, 'weShopCategories'))
						->marginLeft()
						->buttons(we_html_button::position_yes_no_cancel(
								we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_notclose');"), '', we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")))
						->headline(g_l('modules_shop', '[shopcats][title_editorShopCats]'))
						->get() . '</form>'));
	}

	private static function showDialogShipping(we_base_jsCmd $jsCmd): string{
		$protect = we_modules_info::isActive(we_modules_info::SHOP) && we_users_util::canEditModule(we_modules_info::SHOP) ? [] : [false];
		we_html_tools::protect($protect);
		$DB_WE = DB_WE::$inst;
		$weShippingControl = we_shop_shippingControl::getShippingControl();
		$weShipping = null;

		switch(we_base_request::string('cmd', '')){
			case 'newShipping':
				$weShipping = we_shop_shipping::getNewEmptyShipping();
				break;

			case 'editShipping':
				$weShipping = $weShippingControl->getShippingById(we_base_request::string('weShippingId'));
				break;

			case 'deleteShipping':
				$weShippingControl->delete(we_base_request::string('weShippingId'));
				break;

			case 'saveShipping':
				$weShippingControl->setByRequest(); //FIXME: bad this is unchecked!!!
				if($weShippingControl->save()){
					$jsCmd->addMsg(g_l('modules_shop', '[shipping][save_success]'), we_base_util::WE_MESSAGE_NOTICE);
				} else {
					$jsCmd->addMsg(g_l('modules_shop', '[shipping][save_error]'), we_base_util::WE_MESSAGE_ERROR);
				}
				if(($sid = we_base_request::string('weShippingId'))){
					$weShipping = $weShippingControl->getShippingById($sid);
				}
				break;
		}

// show shippingControl
// first show fields: country, vat, isNet?

		$customerTableFields = $DB_WE->metaFieldNames(CUSTOMER_TABLE);
		$selectFieldsCtl = $selectFieldsVat = $selectFieldsTbl = [];
		foreach($customerTableFields as $tblField){
			$selectFieldsTbl[$tblField] = $tblField;
		}
		$shopVats = we_shop_vats::getAllShopVATs();
		foreach($shopVats as $shopVat){ //Fix #9625 use shopVat->Id as key instead of the sorted array $id!
			$selectFieldsVat[$shopVat->id] = $shopVat->vat . '% - ' . $shopVat->getNaturalizedText() . ' (' . $shopVat->territory . ')';
		}
// selectBox with all existing shippings
// select menu with all available shipping costs
		foreach($weShippingControl->shippings as $key => $shipping){
			$selectFieldsCtl[$key] = $shipping->text;
		}

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[vat_country][stateField]'), we_html_tools::select('stateField', $selectFieldsTbl, 1, $weShippingControl->stateField, false, [
						], true, 280)))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[mwst]'), we_html_tools::select('vatId', $selectFieldsVat, 1, $weShippingControl->vatId, false, [
						], true, 280)))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[shipping][prices_are_net]'), we_html_tools::select('isNet', [1 => g_l('global', '[true]'), 0 => g_l('global', '[false]')], 1, [
						$weShippingControl->isNet], false, [], true, 280)))
				->space(we_html_multiIconBox::SPACE_BIG)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[shipping][insert_packaging]'), '<table class="default defaultfont">
	<tr>
		<td>' . we_html_tools::select('editShipping', $selectFieldsCtl, 4, we_base_request::string('weShippingId'), false, ['onchange' => "document.location=WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_shipping&cmd=editShipping&weShippingId=' + this.options[this.selectedIndex].value;"], true, 280) . '</td>
		<td style="width:10px;"></td>
		<td style="vertical-align:top">'
				. we_html_button::create_button(we_html_button::NEW_ENTRY, "javascript:we_cmd('newEntry');") .
				'<div style="margin:5px;"></div>' .
				we_html_button::create_button(we_html_button::DELETE, "javascript:we_cmd('delete')") .
				'</td>
	</tr>
	</table>'))
			->space(we_html_multiIconBox::SPACE_BIG)
		);


// if a shipping should be edited, show it in a form

		if($weShipping){ // show the shipping which must be edited
			$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[shipping][name]'), we_html_tools::textInput('weShipping_text', $weShipping->text) . we_html_element::hidden('weShippingId', $weShipping->id)))
					->space(we_html_multiIconBox::SPACE_BIG)
					->noline()
				)
				->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[shipping][countries]'), we_contents_base::htmlTextArea('weShipping_countries', 4, 21, implode("\n", $weShipping->countries))))
					->noline()
					->space(we_html_multiIconBox::SPACE_BIG)
			);
			// foreach ...
			// form table with every value -> cost entry
			$tblPart = '';
			if($weShipping->shipping){
				foreach($weShipping->shipping as $pos => $ship){
					$tblPart .= '
<tr id="weShippingId_' . $pos . '">
	<td>' . we_html_tools::textInput('weShipping_cartValue[]', (string) $weShipping->cartValue[$pos], 0, ['step' => '0.01', 'min' => '0'], 'number') . '</td>
	<td>' . we_html_tools::textInput('weShipping_shipping[]', (string) $ship, 0, ['step' => '0.01', 'min' => '0'], 'number') . '</td>
	<td>' . we_html_button::create_button(we_html_button::TRASH, "we_cmd('deleteShippingCostTableRow','weShippingId_" . $pos . "');") . '</td>
			</tr>';
				}
			}

			$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[shipping][costs]'), '
<table style="width:100%" class="default defaultfont" id="shippingCostTable">
	<thead><tr>
		<th>' . g_l('modules_shop', '[shipping][order_value]') . '</th>
		<th>' . g_l('modules_shop', '[shipping][shipping_costs]') . '</th>
		<th></th>
	</tr></thead>
		<tbody id="shippingCostTableEntries">
	' . $tblPart . '
		</tbody>
	</table>' .
						we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('addShippingCostTableRow','12');")))
					->space(we_html_multiIconBox::SPACE_BIG)
					->noline())
				->add((new we_html_multiIconBoxContentLine('Standard', we_html_tools::select('weShipping_default', [1 => g_l('global', '[true]'), 0 => g_l('global', '[false]')], 1, $weShipping->default)))
					->noline()
					->space(we_html_multiIconBox::SPACE_BIG)
			);
		}

		return we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/edit_shop_shipping.js', '', ['id' => 'loadVarShopping', 'data-shopping' => setDynamicVar([
						'trashButton' => we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('deleteShippingCostTableRow', 'weShippingId_" . we_base_constants::PLACEHOLDER . "');")
				])]) .
				$jsCmd->getCmds(), we_html_element::body(['class' => 'weDialogBody', 'onload' => "window.focus();"], '<form name="' . we_base_constants::WE_FORM . '">
		<input type="hidden" id="we_cmd_field" name="cmd" value="saveShipping" />' .
						(new we_html_multiIconBox($parts, 'weShipping'))
						->marginLeft()
						->buttons(we_html_button::position_yes_no_cancel(
								we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('we_shop_save');"), '', we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")
						))
						->headline(g_l('modules_shop', '[shipping][shipping_package]'))
						->get() . '</form>'));
	}

	private static function showDialogOrderCustomer(): string{
		$orderId = we_base_request::int('bid');
		$saveBut = we_html_button::create_button(we_html_button::SAVE, "javascript:document.we_form.submit();WE().util.dialogTop(window).doClose();");
		$cancelBut = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");

		// 1st get the customer for this order
		$customerDb = getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID=(SELECT customerID FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $orderId . ')');
		$customer = array_merge($customerDb, we_unserialize(fS('SELECT customerData FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $orderId)));
		ksort($customer);

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('modules_shop', '[preferences][explanation_customer_odercustomer]'), we_html_tools::TYPE_INFO, 470))
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_customer', '[Forname]') . ': ', we_html_tools::textInput('weCustomerOrder[Forename]', $customer['Forename'])))
				->space(we_html_multiIconBox::SPACE_MED2)
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_customer', '[Surname]') . ': ', we_html_tools::textInput('weCustomerOrder[Surname]', $customer['Surname'])))
			->space(we_html_multiIconBox::SPACE_MED2)
			->noline()
		);
		//$editFields = ['Forename', 'Surname'];

		$CLFields = we_unserialize(fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_CountryLanguage"'), we_shop_view::CLFieldsDefault);
		foreach($customer as $k => $v){
			if(!in_array($k, we_shop_shop::IGNORED_EDIT_FIELDS, true) && !is_numeric($k)){
				assert(is_string($k));
				if(isset($CLFields['stateField']) && !empty($CLFields['stateFieldIsISO']) && $k == $CLFields['stateField']){
					$lang = explode('_', we_base_WE::$Language);
					$langcode = array_search($lang[0], we_base_country::getWELangs(), true) ?: 'de';
					$countrycode = array_search($langcode, we_base_country::getWECountries(), true) ?: 'DE';
					$countryselect = new we_html_select(['name' => 'weCustomerOrder[' . $k . ']', 'style' => 'width:280px;', 'class' => 'wetextinput']);

					$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
					foreach($topCountries as $countrykey => &$countryvalue){
						$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
					}
					unset($countryvalue);
					$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
					foreach($shownCountries as $countrykey => &$countryvalue){
						$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
					}
					unset($countryvalue);
					$oldLocale = setlocale(LC_ALL, NULL);
					setlocale(LC_ALL, $langcode . '_' . $countrycode . '.UTF-8');
					asort($topCountries, SORT_LOCALE_STRING);
					asort($shownCountries, SORT_LOCALE_STRING);
					if($oldLocale){
						setlocale(LC_ALL, $oldLocale);
					}
					if(WE_COUNTRIES_DEFAULT != ''){
						$countryselect->addOption('--', WE_COUNTRIES_DEFAULT);
					}
					foreach($topCountries as $countrykey => &$countryvalue){
						$countryselect->addOption((string) $countrykey, (string) $countryvalue);
					}
					unset($countryvalue);
					$countryselect->addOption('-', '----', ['disabled' => 'disabled']);
					foreach($shownCountries as $countrykey => &$countryvalue){
						$countryselect->addOption((string) $countrykey, (string) $countryvalue);
					}
					unset($countryvalue);
					$countryselect->selectOption((string) $v);

					$parts->add((new we_html_multiIconBoxContentLine($k . ': ', $countryselect->getHtml()))
							->space(we_html_multiIconBox::SPACE_MED2)
							->noline()
					);
				} elseif((isset($CLFields['languageField']) && !empty($CLFields['languageFieldIsISO']) && $k == $CLFields['languageField'])){
					$frontendL = we_base_WE::$Langs['FrontendLanguages'];
					foreach($frontendL as &$lcvalue){
						[$lcvalue] = explode('_', $lcvalue);
					}
					unset($lcvalue);
					$languageselect = new we_html_select(['name' => 'weCustomerOrder[' . $k . ']', 'style' => 'width:280px;', 'class' => 'wetextinput']);
					foreach(we_base_country::g_l_a('languages', '') as $languagekey => $languagevalue){
						if(in_array($languagekey, $frontendL, true)){
							$languageselect->addOption((string) $languagekey, (string) $languagevalue);
						}
					}
					$languageselect->selectOption($v);

					$parts->add((new we_html_multiIconBoxContentLine($k . ': ', $languageselect->getHtml()))
							->space(we_html_multiIconBox::SPACE_MED2)
							->noline()
					);
				} else {
					$parts->add((new we_html_multiIconBoxContentLine($k . ': ', we_html_tools::textInput('weCustomerOrder[' . $k . ']', $v)))
							->space(we_html_multiIconBox::SPACE_MED2)
							->noline()
					);
				}
				//$editFields[] = $k;
			}
		}

		return we_html_tools::getHtmlTop('', '', '
				<body class="weDialogBody">
				<form name="we_form" method="post" action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod" target="edbody">' .
				we_html_element::hiddens([
					'mod' => we_modules_info::SHOP,
					'pnt' => 'edbody',
					'bid' => (string) $orderId,
					'cmd' => 'save_order_customer'
				]) .
					(new we_html_multiIconBox($parts))
					->marginLeft()
					->buttons(we_html_button::position_yes_no_cancel($saveBut, '', $cancelBut))
					->headline(g_l('modules_shop', '[preferences][customerdata]'))
					->get() .
				'</form>
				</body>');
	}

	private static function showDialogAddNewArcticle(): string{
		$DB_WE = DB_WE::$inst;
		$orderId = we_base_request::int('bid');
		$saveBut = '';
		$cancelBut = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");
		$searchBut = we_html_button::create_button(we_html_button::SEARCH, "javascript:searchArticles();");
		$searchArticle = we_base_request::string('searchArticle');

		// first get all shop documents
		$shopArticles = $DB_WE->getAllFirstq('SELECT CONCAT(c.DID,"_' . we_shop_shop::DOCUMENT . '"), CONCAT(c.dat," [' . g_l('modules_shop', '[isDoc]') . ': ",c.DID,"]") AS shopTitle FROM ' . CONTENT_TABLE . ' c JOIN ' . FILE_TABLE . ' f ON f.ID=c.DID WHERE c.nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\' AND c.DocumentTable="tblFile" ' .
			($searchArticle ? ' AND c.Dat LIKE "%' . $DB_WE->escape($searchArticle) . '%"' : ''), false);

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			// now get all shop objects
			$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
			foreach(we_base_request::filterIntList($feldnamen[3] ?? '') as $classId){
				$shopArticles = array_merge($shopArticles, $DB_WE->getAllFirstq('SELECT CONCAT(`of`.ID,"_' . we_shop_shop::OBJECT . '"),CONCAT(obx.input_' . we_shop_shop::TITLE_FIELD_NAME . '" [' . g_l('modules_shop', '[isObj]') . ': ",`of`.ID,"]") AS shopTitle FROM ' . OBJECT_X_TABLE . intval($classId) . ' obx JOIN ' . OBJECT_FILES_TABLE . ' `of` ON (obx.OF_ID=`of`.ID AND OF_DocumentTable="tblObjectFiles") ' .
						($searchArticle ? ' WHERE obx.input_' . we_shop_shop::TITLE_FIELD_NAME . '  LIKE "%' . $DB_WE->escape($searchArticle) . '%"' : ''), false)
				);
			}
		}

		//  determine which articles should be shown ...

		asort($shopArticles);
		$max_per_page = 10;
		$amount_articles = count($shopArticles);

		$page = we_base_request::int('page', 0);

		$shopArticlesParts = array_chunk($shopArticles, $max_per_page, true);

		$start_entry = $page * $max_per_page + 1;
		$end_entry = (($page * $max_per_page + $max_per_page < $amount_articles) ? ($page * $max_per_page + $max_per_page) : $amount_articles );

		$backBut = ($start_entry - $max_per_page > 0 ?
			we_html_button::create_button(we_html_button::BACK, "javascript:switchEntriesPage(" . ($page - 1) . ");") :
			we_html_button::create_button(we_html_button::BACK, '', '', '', true));

		$nextBut = (($end_entry) < $amount_articles ?
			we_html_button::create_button(we_html_button::NEXT, "javascript:switchEntriesPage(" . ($page + 1) . ");") :
			we_html_button::create_button(we_html_button::NEXT, '', '', '', true));


		$shopArticlesSelect = empty($shopArticlesParts[$page]) ? [] : $shopArticlesParts[$page];
		asort($shopArticlesSelect);

		// determine which articles should be shown >>>

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[Artikel]'),
																									($amount_articles > 0 ? '
<table class="default">
<tr>
<td style="margin-right:10px">' . we_html_tools::select("add_article", $shopArticlesSelect, 15, we_base_request::_(we_base_request::RAW, 'add_article', ''), false, [
					'onchange' => "selectArticle(this.options[this.selectedIndex].value)"], true, 380) . '</td>
<td style="vertical-align:top">' . $backBut . '<div style="margin:5px 0"></div>' . $nextBut . '</td>
</tr>
<tr>
	<td class="small" colspan="2">' . sprintf(g_l('modules_shop', '[add_article][entry_x_to_y_from_z]'), $start_entry, $end_entry, $amount_articles) . '</td>
</tr>
</table>' : g_l('modules_shop', '[add_article][empty_articles]'))))
			->noline()
			->space(we_html_multiIconBox::SPACE_MED)
		);
		$search = we_base_request::string('searchArticle', '');
		if($amount_articles > 0 || $search){
			$parts->add((new we_html_multiIconBoxContentLine(g_l('global', '[search]'), '
	<table class="default">
		<tr><td>' . we_html_tools::textInput('searchArticle', $search, 0, ['id' => 'searchArticle'], 'text', '380px') . '</td>
			<td></td>
			<td>' . $searchBut . '</td>
		</tr>
	</table>'))
					->space(we_html_multiIconBox::SPACE_MED)
			);
		}
		$add_article = we_base_request::string('add_article');
		if($add_article){
			$saveBut = we_html_button::create_button(we_html_button::SAVE, "javascript:document.we_form.submit();WE().util.dialogTop(window).doClose();");
			[$id, $type] = explode('_', $add_article);

			$variantOptions = [
				'' => '-'//Fix #11087 we need an empty key for default: non variant
			];

			$model = ($type == we_shop_shop::OBJECT ? new we_contents_objectFile((int) $id) : new we_document_webEdition((int) $id));
			$variantData = we_base_variants::getVariantData($model, '-');

			if(count($variantData) > 1){
				foreach(array_keys($variantData) as $key){
					if($key != '-'){
						$variantOptions[$key] = $key;
					}
				}
			}

			$parts
				->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[Artikel]'), '<b>' . $model->getElementS(we_shop_shop::TITLE_FIELD_NAME) . '</b>'))
					->space(we_html_multiIconBox::SPACE_MED)
					->noline()
				)
				->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[anzahl]'), we_html_tools::textInput('anzahl', '', 0, ['min' => '1'], 'number', '380px')))
					->noline()
					->space(we_html_multiIconBox::SPACE_MED)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[variant]'), we_html_tools::select(we_base_variants::REQUEST, $variantOptions, 1, '', false, [
							], true, 380)))
					->space(we_html_multiIconBox::SPACE_MED)
					->noline()
				)
				->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[customField]'), we_html_tools::textInput('we_customField', '', 0, [], 'text', '380px') .
						'<br/><span class="small">Eingabe in der Form: <i>name1=wert1;name2=wert2</i></span></form>'))
					->space(we_html_multiIconBox::SPACE_MED)
					->noline()
			);
		}

		return we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view2.js', '', ['id' => 'loadVarShop', 'data-shop' => setDynamicVar([
						'cmd0' => we_base_request::string('we_cmd', '', 0),
						'bid' => (string) $orderId,
						'searchArticle' => we_base_request::string('searchArticle', ''),
						'page' => $page
				])])
				, '<body class="weDialogBody"><form name="we_form" method="post" action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod" target="edbody">' .
				we_html_element::hiddens([
					'mod' => we_modules_info::SHOP,
					'pnt' => 'edbody',
					'bid' => (string) $orderId,
					'we_cmd[0]' => 'add_article',
					'add_article' => we_base_request::string('add_article', '')
				]) .
					(new we_html_multiIconBox($parts))
					->marginLeft()
					->buttons(we_html_button::position_yes_no_cancel($saveBut, '', $cancelBut))
					->headline(g_l('modules_shop', '[add_article][title]'))
					->get() .
				'</form>');
	}

	private static function showDialogShippingCost(): string{
		$orderId = we_base_request::int('bid');
		$shopVats = we_shop_vats::getAllShopVATs();
		$shippingVats = [];

		foreach($shopVats as $shopVat){
			$shippingVats[$shopVat->vat] = $shopVat->vat . ' - ' . $shopVat->getNaturalizedText() . ' (' . $shopVat->territory . ')';
		}

		$saveBut = we_html_button::create_button(we_html_button::SAVE, "javascript:document.we_form.submit();WE().util.dialogTop(window).doClose();");
		$cancelBut = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");

		$shipping = getHash('SELECT shippingCost,shippingNet,shippingVat FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $orderId);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[edit_order][shipping_costs]'), we_html_tools::textInput('weShipping_costs', (string) $shipping['shippingCost'])))
				->space(we_html_multiIconBox::SPACE_MED2)
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[edit_shipping_cost][isNet]'), we_html_tools::select('weShipping_isNet', [1 => g_l('global', '[yes]'),
						0 => g_l('global', '[no]')], 1, $shipping['shippingNet'])))
				->space(we_html_multiIconBox::SPACE_MED2)
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[edit_shipping_cost][vatRate]'), we_html_tools::inputChoiceField('weShipping_vatRate', (string) $shipping['shippingVat'], $shippingVats, [
					], '', true)))
			->space(we_html_multiIconBox::SPACE_MED2)
			->noline()
		);

		return we_html_tools::getHtmlTop('', '', '
				<body class="weDialogBody">
				<form name="we_form" method="post" action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod" target="edbody">' .
				we_html_element::hiddens([
					'mod' => we_modules_info::SHOP,
					'pnt' => 'edbody',
					'bid' => (string) $orderId,
					'cmd' => 'save_shipping_cost'
				]) .
					(new we_html_multiIconBox($parts))->marginLeft()
					->buttons(we_html_button::position_yes_no_cancel($saveBut, '', $cancelBut))
					->headline(g_l('modules_shop', '[edit_shipping_cost][title]'))
					->get() .
				'</form>');
	}

	private static function showDialogCartCustomField(): string{
		$orderId = we_base_request::int('bid');
		$saveBut = we_html_button::create_button(we_html_button::SAVE, "javascript:we_submit();");
		$cancelBut = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");

		$cartfield = we_base_request::string('cartfieldname');
		if($cartfield){
			$customFields = we_unserialize(fS('SELECT customFields FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $orderId));
			$val = $customFields[$cartfield] ?: '';
			$fieldHtml = $cartfield . '<input type="hidden" name="cartfieldname" id="cartfieldname" value="' . $cartfield . '" />';
		} else {
			$fieldHtml = we_html_tools::textInput('cartfieldname', '', 0, ['id' => "cartfieldname"]);
			$val = '';
		}

		// make input field, for name or textfield
		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[field_name]'), $fieldHtml))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[field_value]'), '<textarea name="cartfieldvalue" style="width: 350px; height: 150px">' . $val . '</textarea>'))
			->space(we_html_multiIconBox::SPACE_MED)
		);

		return we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view2.js'), '
<body class="weDialogBody">
<form name="we_form" method="post" action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod">' . we_html_element::hiddens([
					'mod' => we_modules_info::SHOP,
					'pnt' => 'edbody',
					'bid' => (string) $orderId,
					'cmd' => 'save_shop_cart_custom_field',
				]) .
					(new we_html_multiIconBox($parts))
					->marginLeft()
					->buttons(we_html_button::position_yes_no_cancel($saveBut, '', $cancelBut))
					->headline(g_l('modules_shop', '[add_shop_field]'))
					->get() .
				'</form>');
	}

	public function processCommands(we_base_jsCmd $jscmd): void{
		$this->View->processCommands($jscmd);
	}

}
