<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 00fa50227a81 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-22 16:20 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_repeatShopItem extends we_weTag_tag{

	public static function parse(array $attribs, string $content): string{
		self::attributeMissing($attribs, ['shopname' => false], __CLASS__);

		return '<?php ' . we_weTag_parser::printTag('repeatShopItem', $attribs + ['_type' => 'start']) . '; while(we_tag_listview::$current->next_record()) {?>' . $content . '<?php } ' . we_weTag_parser::printTag('repeatShopItem', [
					'_type' => 'stop']) . ';?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): void{
		if(!we_modules_info::isActive(we_modules_info::SHOP)){
			echo self::modulMissingError('Shop', __CLASS__);
			return;
		}
		$shopname = self::getAttribute('shopname', $attribs, '', we_base_request::STRING);

		//internal Attribute
		$intType = self::getAttribute('_type', $attribs, '', we_base_request::STRING);
		switch($intType){
			case 'start':
				self::attributeMissing($attribs, ['shopname' => false], __CLASS__);
				we_modules_info::isActive(we_modules_info::SHOP);

				$_SESSION['we_shopname'] = $shopname;

				try{
					$basket = we_tag_createShop::getBasket($shopname);
				}catch(we_exception_base $e){
					echo self::parseError(sprintf(g_l('parser', '[missing_createShop]'), 'repeatShopItem'), __CLASS__);
					return;
				}
				we_tag_listview::pre(new we_listview_shopItems($basket));
				break;
			case 'stop':
				we_tag_listview::post();
				break;
		}
	}

}
