<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_saveRegisteredUser extends we_weTag_tag{
	/** @var int */
	public static $status = -1;

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		if(!(we_modules_info::isActive(we_modules_info::CUSTOMER) && isset($_REQUEST['s']))){
			return '';
		}

		$changesessiondata = self::getAttribute('changesessiondata', $attribs, true, we_base_request::BOOL);
		$registerallowed = self::getAttribute('register', $attribs, false, we_base_request::BOOL);
		$protected = self::getAttribute('protected', $attribs, [], we_base_request::STRING_LIST);
		$allowed = self::getAttribute('allowed', $attribs, [], we_base_request::STRING_LIST);
		$pwdRule = self::getAttribute('passwordRule', $attribs, '', we_base_request::RAW);
		$uniqueFields = self::getAttribute('uniqueFields', $attribs, [], we_base_request::STRING_LIST);

		if(isset($_REQUEST['s']['Password2'])){
			unset($_REQUEST['s']['Password2']);
		}
		foreach(we_base_util::convertDateInRequest('s') as $name => $date){
			$_REQUEST['s'][$name] = $date->format(we_database_base::DATE_TIME_FORMAT);
		}

		$uid = we_base_request::int('s', -1, 'ID');
		$username = trim(we_base_request::string('s', '', 'Username'));
		$password = we_base_request::_(we_base_request::RAW, 's', '', 'Password');
		//register new User
		try{
			if(($uid <= 0) && (!isset($_SESSION['webuser']['ID'])) && $registerallowed && (!isset($_SESSION['webuser']['registered']) || !$_SESSION['webuser']['registered'])){ // neuer User
				self::addNewUser($username, $password, $uniqueFields, $pwdRule, $protected, $allowed);
				$changesessiondata = (self::$status == we_customer_customer::PWD_ALL_OK);
			} else if(!empty($_SESSION['webuser']['registered']) && ($uid == $_SESSION['webuser']['ID'])){ // existing user
				self::changeCurrentUser($username, $uniqueFields, $pwdRule, $protected, $allowed, $changesessiondata);
			}
		} catch (we_exception_base $e){
			return '';
		}
		//die neuen daten in die session schreiben
		$oldReg = !empty($_SESSION['webuser']['registered']);
		$oldPwd = '';
		if($changesessiondata && $oldReg && self::$status === we_customer_customer::PWD_ALL_OK){
			//keep Password if known
			if(SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_PASSWORD){
				//FIXME: on register password is in $_REQUEST['s']['Password']
				$oldPwd = $_SESSION['webuser']['_Password'] ?? $password;
			}
			$_SESSION['webuser'] = array_merge(getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . $_SESSION['webuser']['ID']), we_customer_customer::getEncryptedFields());
			if((SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_DBPASSWORD) == 0){
				unset($_SESSION['webuser']['Password']);
			}
			if(SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_PASSWORD){
				$_SESSION['webuser']['_Password'] = $oldPwd;
			}
		}
		//don't set anything that wasn't set before
		$_SESSION['webuser']['registered'] = $oldReg;
		return '';
	}

	protected static function changeCurrentUser(string $username, array $uniqueFields, string $pwdRule, array $protected, array $allowed, bool $changesessiondata): void{
		// existierender User (Daten werden von User geaendert)!!
		$weUsername = $username ?: $_SESSION['webuser']['Username'];

		if(fB('SELECT 1 FROM ' . CUSTOMER_TABLE . ' WHERE Username="' . DB_WE::$inst->escape($weUsername) . '" AND ID!=' . intval($_SESSION['webuser']['ID']))){
			self::$status = we_customer_customer::PWD_USER_EXISTS;
			throw new we_exception_base('err');
		}
		if(!isset($_REQUEST['s'])){
			return;
		}
		foreach($uniqueFields as $unique){
			if(fB('SELECT 1 FROM ' . CUSTOMER_TABLE . ' WHERE `' . $unique . '`="' . DB_WE::$inst->escape($_REQUEST['s'][$unique]) . '" AND ID!=' . intval($_SESSION['webuser']['ID']))){
				self::$status = we_customer_customer::PWD_USER_NOT_UNIQUE;
				throw new we_exception_base('err');
			}
		}
		// es existiert kein anderer User mit den neuen Username oder username hat sich nicht geaendert
		$hook = new we_hook_handler('customer_preSave', ['customer' => &$_REQUEST['s'], 'from' => 'tag', 'type' => 'modify', 'tagname' => 'saveRegisteredUser']);
		if(!$hook->executeHook()){
			self::$status = we_customer_customer::PWD_UNKNOWN_ERROR;
			throw new we_exception_base('err');
		}

		self::saveCustomerImages();
		$set_a = self::processRequest($protected, $allowed);
		$password = we_base_request::_(we_base_request::RAW, 's', we_customer_customer::NOPWD_CHANGE, 'Password');
		if($password !== we_customer_customer::NOPWD_CHANGE){
			if(!$password){
				if($changesessiondata){
					self::keepInput(true);
				}
				self::$status = we_customer_customer::PWD_FIELD_NOT_SET;
				throw new we_exception_base('err');
			}
			if($pwdRule && !preg_match('/' . addcslashes($pwdRule, '/') . '/', $password)){
				if($changesessiondata){
					self::keepInput(true);
				}
				self::$status = we_customer_customer::PWD_NOT_SUFFICIENT;
				throw new we_exception_base('err');
			}

			if(!we_customer_customer::comparePassword(fS('SELECT Password FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . $_SESSION['webuser']['ID']), $password, $_SESSION['webuser']['ID'])){//bei Passwordaenderungen muessen die Autologins des Users geloescht werden
				DB_WE::$inst->query('DELETE FROM ' . CUSTOMER_AUTOLOGIN_TABLE . ' WHERE WebUserID=' . intval($_SESSION['webuser']['ID']));
			}
		}
		if($set_a){
			$set_a['Mod_Date'] = sql_function('NOW()');
			$set_a['ModifiedBy'] = 'frontend';
			DB_WE::$inst->query('UPDATE ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter($set_a) . ' WHERE ID=' . intval($_SESSION['webuser']['ID']));
			self::$status = DB_WE::$inst->affected_rows() ?
				we_customer_customer::PWD_ALL_OK :
				we_customer_customer::PWD_UNKNOWN_ERROR;
		}
	}

	protected static function addNewUser(string $username, string $password, array $uniqueFields, string $pwdRule, array $protected, array $allowed): void{
		if(!$username){
			self::keepInput();
			self::$status = we_customer_customer::PWD_USER_EMPTY;
			throw new we_exception_base('err');
		}
		if(we_customer_customer::customerNameExist($username)){
			// Eingabe in Session schreiben, damit die eingegebenen Werte erhalten bleiben!
			self::keepInput();
			self::$status = we_customer_customer::PWD_USER_EXISTS;
			throw new we_exception_base('err');
		}
		if(!$password){
			self::keepInput();
			self::$status = we_customer_customer::PWD_FIELD_NOT_SET;
			throw new we_exception_base('err');
		}
		if($pwdRule && !preg_match('/' . addcslashes($pwdRule, '/') . '/', $password)){
			self::keepInput();
			self::$status = we_customer_customer::PWD_NOT_SUFFICIENT;
			throw new we_exception_base('err');
		}

		foreach($uniqueFields as $unique){
			if(fB('SELECT 1 FROM ' . CUSTOMER_TABLE . ' WHERE `' . $unique . '`="' . DB_WE::$inst->escape($_REQUEST['s'][$unique]) . '"')){
				self::$status = we_customer_customer::PWD_USER_NOT_UNIQUE;
				throw new we_exception_base('err');
			}
		}


		// username existiert noch nicht!
		$hook = new we_hook_handler('customer_preSave', ['customer' => &$_REQUEST['s'], 'from' => 'tag', 'type' => 'new', 'tagname' => 'saveRegisteredUser']);
		if(!$hook->executeHook()){
			self::$status = we_customer_customer::PWD_UNKNOWN_ERROR;
			throw new we_exception_base('err');
		}

		self::saveCustomerImages();
		$set = self::processRequest($protected, $allowed);

		if($set){
			// User in DB speichern
			$set['Mod_Date'] = sql_function('NOW()');
			$set['MemberSince_Date'] = sql_function('NOW()');
			$set['LastAccess_Date'] = sql_function('NOW()');
			$set['LastLogin_Date'] = sql_function('NOW()');
			$set['ModifiedBy'] = 'frontend';

			DB_WE::$inst->query('INSERT INTO ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter($set));
			$id = DB_WE::$inst->getInsertId();
			if($id){
				// User in session speichern
				$_SESSION['webuser'] = [
					'ID' => $id,
					'registered' => true, //needed for reload
				];
				$GLOBALS['we_customer_write_ID'] = $_SESSION['webuser']['ID'];
				//make sure to always load session data
				self::$status = we_customer_customer::PWD_ALL_OK;
			} else {
				self::$status = we_customer_customer::PWD_UNKNOWN_ERROR;
			}
		}
	}

	protected static function saveCustomerImages(): void{
		$images = $_FILES['WE_SF_IMG_DATA'] ?? '';
		'@phan-var array{name:array<string,string>,tmp_name:array<string,string>,size:array<string,int>,type:array<string,string>} $images';
		if(empty($images) || empty($images['name']) || !is_array($images['name'])){
			return;
		}
		$webuserId = $_SESSION['webuser']['ID'] ?? 0;
		foreach($images['name'] as $imgName => $filename){
			'@phan-var string $filename';
			$imgId = $_SESSION['webuser'][$imgName] ?? 0;
			if(!id2path($imgId)){
				$imgId = 0;
			}

			if(we_base_request::bool('WE_SF_DEL_CHECKBOX_' . $imgName)){
				if($imgId){
					$imgDocument = new we_document_image($imgId);
					if($imgDocument->WebUserID == $webuserId){
						//everything ok, now delete

						we_base_delete::deleteEntry($imgId, FILE_TABLE);
						// reset image field
						$_SESSION['webuser'][$imgName] = 0;
						$_REQUEST['s'][$imgName] = '0';
					}
				}
			} else if($filename){
				// file is selected, check to see if it is an image
				if(we_base_ContentTypes::getFromFilename($filename) === we_base_ContentTypes::IMAGE){

					$serverPath = TEMP_PATH . we_base_file::getUniqueId();
					we_base_file::insertIntoCleanUp($serverPath);
					move_uploaded_file((string) $images['tmp_name'][$imgName], $serverPath);

					$we_size = we_contents_thumbnail::getimagesize($serverPath);

					if(empty($we_size)){
						continue;
					}

					$tmp_Filename = $imgName . '_' . we_base_file::getUniqueId() . '_' . preg_replace('/[^A-Za-z0-9._-]/', '', (string) $images['name'][$imgName]);
					$tmp = explode('.', $tmp_Filename);
					$extension = '.' . $tmp[count($tmp) - 1];
					array_pop($tmp);
					$fileName = implode('.', $tmp);

					//image needs to be scaled
					if((!empty($_SESSION['webuser']['imgtmp'][$imgName]['width'])) ||
						(!empty($_SESSION['webuser']['imgtmp'][$imgName]['height']))){
						$imageData = we_base_file::load($serverPath);
						$thumb = new we_contents_thumbnail();
						$thumb->init(0, $_SESSION['webuser']['imgtmp'][$imgName]['width'], $_SESSION['webuser']['imgtmp'][$imgName]['height'], [$_SESSION['webuser']['imgtmp'][$imgName]['keepratio'] ? we_contents_thumbnail::OPTION_RATIO : 0,
							$_SESSION['webuser']['imgtmp'][$imgName]['maximize'] ? we_contents_thumbnail::OPTION_MAXSIZE : 0], '', 'dummy', 0, '', '', $extension, $we_size[0], $we_size[1], $imageData, null, $_SESSION['webuser']['imgtmp'][$imgName]['quality'], true);

						try{
							we_base_file::save($serverPath, $thumb->getThumb());
							$we_size = we_contents_thumbnail::getimagesize($serverPath);
						} catch (we_exception_base $e){
							t_e($e);
						}
					}

					$imgwidth = $we_size[0];
					$imgheight = $we_size[1];
					//$type = $_FILES['WE_SF_IMG_DATA']['type'][$imgName];
					$size = (int) $images['size'][$imgName];

					$imgDocument = new we_document_image();

					if($imgId){// document has already an image, so change binary data
						$imgDocument->initByID($imgId);
					} else {// set path for new image
						$imgDocument->setParentID($_SESSION['webuser']['imgtmp'][$imgName]['parentid']);
					}

					$imgDocument->Filename = $fileName;
					$imgDocument->Extension = $extension;
					$imgDocument->Path = $imgDocument->getPath();

					$imgDocument->setElementI('width', $imgwidth, 'attrib');
					$imgDocument->setElementI('height', $imgheight, 'attrib');
					$imgDocument->setElementI('origwidth', $imgwidth, 'attrib');
					$imgDocument->setElementI('origheight', $imgheight, 'attrib');
					//$imgDocument->setElement('type', we_base_ContentTypes::IMAGE, 'attrib');

					$imgDocument->setElementS('data', $serverPath, 'image');

					$imgDocument->setElementI('filesize', $size, 'attrib');

					$imgDocument->Table = FILE_TABLE;
					$imgDocument->Publish_Date = new DateTime();
					$imgDocument->WebUserID = $webuserId;
					$imgDocument->we_save();
					$newId = $imgDocument->ID;

					$_SESSION['webuser'][$imgName] = $newId;
					$_REQUEST['s'][$imgName] = (string) $newId;
				}
			}
		}
	}

	protected static function keepInput(bool $merge = false): void{
		if(!isset($_REQUEST['s'])){
			return;
		}
		$registered = $_SESSION['webuser']['registered'];
		$_SESSION['webuser'] = $merge ? array_merge($_SESSION['webuser'], we_base_request::_(we_base_request::STRING, 's')) : we_base_request::_(we_base_request::STRING, 's');
		//never set ID + Password
		if(!$registered && isset($_SESSION['webuser']['ID'])){
			unset($_SESSION['webuser']['ID']);
		}
		if(isset($_SESSION['webuser']['Password'])){
			unset($_SESSION['webuser']['Password']);
		}
		//make sure we stay unregistered!
		$_SESSION['webuser']['registered'] = $registered;
	}

	protected static function processRequest(array $protected, array $allowed): array{
		$set = [];
		$allEncryptedFields = we_customer_customer::getEncryptedFields();
		$fnames = DB_WE::$inst->metaFieldNames(CUSTOMER_TABLE);
		$usedFields = array_fill_keys(array_unique(we_base_request::_(we_base_request::STRING, we_tag_sessionField::USED_FIELDS_NAME, [])), '');
		//set missing fields to empty string, since they are not transfered by the request
		$request = $_REQUEST['s'];
		'@phan-var array<string,string> $request';

		foreach(array_merge($usedFields, $request) as $name => $val){
			switch($name){
				case 'Username': //QUICKFIX !!!
					$set['Username'] = strip_tags($val);
					break;
				case 'Text':
				case 'ID':
					break;
				case 'Password':
					if($val == we_customer_customer::NOPWD_CHANGE || !$val){
						continue 2;
					}
					$val = we_customer_customer::cryptPassword($val);
				//no break
				default:
					if(($protected && in_array($name, $protected, true)) ||
						(in_array($name, we_customer_customer::PROTECTED_FIELDS, true)) ||
						($allowed && !in_array($name, $allowed, true)) ||
						!in_array($name, $fnames, true)
					){
						continue 2;
					}
					$set[$name] = (isset($allEncryptedFields[$name]) && $val != we_customer_customer::ENCRYPTED_DATA ?
						we_customer_customer::cryptData(we_base_util::rmPhp((string) $val), SECURITY_ENCRYPTION_KEY, false) :
						$val
						);
					break;
			}
		}
		return $set;
	}

}
