<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 6a3c6209a897 $
 * $Author: nilsole <nils.reibold@aldente-it.de> $
 * $Date: 2021-04-03 02:05 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_sessionField extends we_weTag_tag{

	public const USED_FIELDS_NAME = '__usedField';

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);
		self::$doc = we_base_WE::getWeDoc();

		$name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$removeFirstParagraph = self::getAttribute('removefirstparagraph', $attribs, defined('REMOVEFIRSTPARAGRAPH_DEFAULT') ? REMOVEFIRSTPARAGRAPH_DEFAULT : true, we_base_request::BOOL);
		//$autobrAttr = self::getAttribute('autobr', $attribs, false, we_base_request::BOOL);
		$checked = self::getAttribute('checked', $attribs, false, we_base_request::BOOL);
		$values = self::getAttribute('values', $attribs, [], we_base_request::STRING_LIST);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$size = self::getAttribute('size', $attribs, '', we_base_request::UNIT);
		$dateformat = self::getAttribute('dateformat', $attribs, '', we_base_request::RAW);
		$value = self::getAttribute('value', $attribs, '', we_base_request::RAW);
		$orgVal = (isset($_SESSION['webuser'][$name]) && $_SESSION['webuser'][$name]) ? $_SESSION['webuser'][$name] : (($type === 'radio') ? '' : $value);


		$autofill = self::getAttribute('autofill', $attribs, false, we_base_request::BOOL);
		if($autofill){
			$condition = ($name === 'Username' ?
					['caps' => 4, 'small' => 4, 'nums' => 4, 'specs' => 0] :
					['caps' => 3, 'small' => 4, 'nums' => 3, 'specs' => 2]);

			$pass = new we_customer_generatePassword(8, $condition);
			$orgVal = $pass->PassGen();
			//echo $tmppass;
		}
		$newAtts = self::removeAttribs($attribs, [
					'checked',
					'choice',
					'cols',
					'doc',
					'htmlspecialchars',
					'languageautofill',
					'maxlength',
					'mode',
					'name',
					'onChange',
					'onClick',
					'onchange',
					'onclick',
					'options',
					'pure',
					'rows',
					'selected',
					'type',
					'usevalue',
					'value',
					'values',
					'wysiwyg',
					'xml',
		]);
		$ret = we_html_element::hidden(self::USED_FIELDS_NAME . '[]', $name);
		switch($type){
			case 'date' :
				return $ret . self::getDate($attribs, $name, $orgVal/* , $dateformat */);
			case 'country':
				return $ret . self::getCountryField($attribs, $name, $orgVal);
			case 'language':
				return $ret . self::getLangField($attribs, $name, $orgVal);
			case 'select':
				if(!$values){
					$regs = [];
					preg_match_all("/'(.*?)'/", DB_WE::$inst->getColTyp(CUSTOMER_TABLE, $name), $regs);
					$values = $regs[1];
				}
				return $ret . self::selectField('s[' . $name . ']', $orgVal, $values, $newAtts, true);
			case 'choice':
				if(!$values){
					$regs = [];
					preg_match_all("/'(.*?)'/", $GLOBALS['DB_WE']->getColTyp(CUSTOMER_TABLE, $name), $regs);
					$values = $regs[1];
				}
				return $ret . we_html_tools::inputChoiceField('s_' . $name, $orgVal, $values, $newAtts, self::getAttribute('mode', $attribs, '', we_base_request::STRING));

			case 'textinput':
				$choice = self::getAttribute('choice', $attribs, false, we_base_request::BOOL);
				//FIXME: can't this be done by calling the 'choice' switch?
				if($choice){ // because of backwards compatibility
					$newAtts = self::removeAttribs($newAtts, ['maxlength']);
					$newAtts['name'] = 's[' . $name . ']';

					$optionsAr = self::getAttribute('options', $attribs, [], we_base_request::STRING_LIST);
					$isin = false;
					$options = '';
					foreach($optionsAr as $cur){
						if($cur === $orgVal){
							$options .= self::htmlTag('option', ['value' => htmlspecialchars($cur), 'selected' => 'selected'], $cur, true);
							$isin = true;
						} else {
							$options .= self::htmlTag('option', ['value' => htmlspecialchars($cur)], $cur, true);
						}
					}
					if(!$isin){
						$options .= self::htmlTag('option', ['value' => htmlspecialchars($orgVal), 'selected' => 'selected'], htmlspecialchars($orgVal), true);
					}
					return $ret . self::htmlTag('select', $newAtts, $options, true);
				}
				return $ret . self::inputTextInputField('s[' . $name . ']', ((string) $orgVal), $newAtts);

			case 'textarea':
				//old Attribute
				$pure = self::getAttribute('pure', $attribs, false, we_base_request::BOOL);
				if($pure){
					$attribs = self::removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick',
								'mode',
								'choice', 'pure', 'size', 'wysiwyg']);
					return $ret . self::textareaField('s[' . $name . ']', $orgVal, $attribs);
				}
				return $ret . (!defined('WE_TEXTAREA_JS') || define('WE_TEXTAREA_JS', 1) ? //FIXME: check if we double this....
						we_html_element::jsScript(JS_DIR . 'weFrontendEdit_header.js') .
						we_html_element::jsScript(JS_DIR . 'we/layout/jsWindow.js') : '') .
						we_html_forms::weTextarea('s[' . $name . ']', $orgVal, $attribs, false, false, $removeFirstParagraph, false, '', self::$doc->getDocumentCss());

			case 'radio':
				if((!isset($_SESSION['webuser'][$name])) && $checked){
					$orgVal = $value;
				}

				return $ret . self::inputRadioField('s[' . $name . ']', $orgVal, $value, $newAtts);
			case 'checkbox':
				if((!isset($_SESSION['webuser'][$name])) && $checked){
					$orgVal = '1';
				}
				return $ret . self::inputCheckboxField('s[' . $name . ']', (string) $orgVal, $newAtts);
			case 'password':

				$newAtts['name'] = 's[' . $name . ']';
				$newAtts['maxlength'] = $newAtts['maxlength'] ?? (string) we_users_user::MAX_PWD_LEN;
				$newAtts['value'] = !empty($_SESSION['webuser']['registered']) ? we_customer_customer::NOPWD_CHANGE : '';
				$newAtts['type'] = 'password';

				return $ret . self::htmlTag('input', $newAtts);
			case 'print':
				$ascountry = self::getAttribute('ascountry', $attribs, false, we_base_request::BOOL);
				$aslanguage = self::getAttribute('aslanguage', $attribs, false, we_base_request::BOOL);
				['code' => $langcode] = self::getOutLang($attribs);

				if($ascountry || $aslanguage){
					return ($ascountry && $orgVal === '--' ? '' : we_base_country::getTranslation($orgVal, ($ascountry ? we_base_country::TERRITORY : we_base_country::LANGUAGE), $langcode));
				}
				if($dateformat){//FIXME: use document settings for dateformat => get locale - note DateTime doesn't use locale settings
					try{
						$date = new DateTime((is_numeric($orgVal) ? '@' : '') . $orgVal);
						//we need to set it explicitly
						$date->setTimezone(new DateTimeZone(date_default_timezone_get()));
						return we_base_country::dateformat($langcode, $date, $dateformat);
					}catch(Exception $e){
						//fallback to default return
					}
				}
				return self::getAttribute('htmlspecialchars', $attribs, false, we_base_request::BOOL) && is_string($orgVal) ? htmlspecialchars($orgVal) : (string) $orgVal;
			case 'hidden':
				$usevalue = self::getAttribute('usevalue', $attribs, false, we_base_request::BOOL);
				$languageautofill = self::getAttribute('languageautofill', $attribs, false, we_base_request::BOOL);
				$v = ($usevalue ? $value : $orgVal);
				$hidden = [
					'type' => 'hidden',
					'name' => 's[' . $name . ']',
					'value' => self::getAttribute('htmlspecialchars', $attribs, false, we_base_request::BOOL) ? htmlspecialchars($v) : $v,
				];
				if($languageautofill){
					['code' => $langcode] = self::getOutLang($attribs);
					$hidden['value'] = $langcode;
				}
				return self::htmlTag('input', $hidden + $newAtts);
			case 'img':
				if(!isset($_SESSION['webuser']['imgtmp'])){
					$_SESSION['webuser']['imgtmp'] = [];
				}
				$_SESSION['webuser']['imgtmp'][$name] = [
					'parentid' => self::getAttribute('parentid', $attribs, 0, we_base_request::INT),
					'width' => self::getAttribute('width', $attribs, 0, we_base_request::INT),
					'height' => self::getAttribute('height', $attribs, 0, we_base_request::INT),
					'quality' => self::getAttribute('quality', $attribs, 8, we_base_request::INT),
					'keepratio' => self::getAttribute('keepratio', $attribs, true, we_base_request::BOOL),
					'maximize' => self::getAttribute('maximize', $attribs, false, we_base_request::BOOL),
					'id' => (int) $orgVal ?: 0
				];

				$idExists = id2path($_SESSION['webuser']['imgtmp'][$name]['id'] ?? 0);
				$imgId = ($idExists ? $_SESSION['webuser']['imgtmp'][$name]['id'] : 0);
				$_SESSION['webuser']['imgtmp'][$name]['id'] = $imgId;

				$bordercolor = self::getAttribute('bordercolor', $attribs, '#006DB8', we_base_request::STRING);
				$checkboxstyle = self::getAttribute('checkboxstyle', $attribs, '', we_base_request::STRING);
				$inputstyle = self::getAttribute('inputstyle', $attribs, '', we_base_request::STRING);
				$checkboxclass = self::getAttribute('checkboxclass', $attribs, '', we_base_request::STRING);
				$inputclass = self::getAttribute('inputclass', $attribs, '', we_base_request::STRING);
				$checkboxtext = self::getAttribute('checkboxtext', $attribs, g_l('parser', '[delete]'), we_base_request::STRING);

				if($imgId){
					$attribs['id'] = $imgId;
				}

				unset($attribs['width']);
				unset($attribs['height']);

				$showcontrol = self::getAttribute('showcontrol', $attribs, true, we_base_request::BOOL);
				if($showcontrol){

					self::attributeMissing($attribs, ['parentid' => false], __CLASS__);
				}


				$thumbnail = self::getAttribute('thumbnail', $attribs, '', we_base_request::STRING);
				$thumbid = self::getAttribute('thumbnailid', $attribs, 0, we_base_request::INT);
				if($thumbnail != ''){
					$imgTag = (string) self::$doc->getFieldByVal((string) $imgId, 'img', ['thumbnail' => $thumbnail]);
				} elseif($thumbid){
					$imgTag = (string) self::$doc->getFieldByVal((string) $imgId, 'img', ['thumbnailid' => $thumbid]);
				} else {
					$imgTag = (string) self::$doc->getFieldByVal((string) $imgId, 'img');
				}

				if($showcontrol){
					$checked = '';
					$inputstyle = ($size ? 'width:' . $size . 'em;' . $inputstyle : $inputstyle);
					return '<table class="we we_sessionfield weEditTable" style="border-color:' . $bordercolor . ';">
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:center">' .
							$imgTag . '
			<input type="hidden" name="s[' . $name . ']" value="' . $_SESSION['webuser']['imgtmp'][$name]['id'] . '" /></td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<input name="WE_SF_IMG_DATA[' . $name . ']" type="file" accept="' . implode(',', we_base_ContentTypes::getRealContentTypes(we_base_ContentTypes::IMAGE)) . '"' . ($inputstyle ? (' style="' . $inputstyle . '"') : '') . ($inputclass ? (' class="' . $inputclass . '"') : '') . ' />
		</td>
	</tr>
	<tr>
		<td class="weEditmodeStyle" colspan="2" style="text-align:left">
			<table class="weEditTable">
				<tr>
					<td style="padding-right: 5px;">
						<input style="border:0 solid black;" type="checkbox" id="WE_SF_DEL_CHECKBOX_' . $name . '" name="WE_SF_DEL_CHECKBOX_' . $name . '" value="1" ' . $checked . '/>
					</td>
					<td>
						<label for="WE_SF_DEL_CHECKBOX_' . $name . '"' . ($checkboxstyle ? (' style="' . $checkboxstyle . '"') : '') . ($checkboxclass ? (' class="' . $checkboxclass . '"') : '') . '>' . $checkboxtext . '</label>
					</td>
				</tr>
			</table>
		</td>
	</tr>
</table>';
				}
				return ($imgId ? $imgTag : '');
		}
		return '';
	}

	protected static function getDate(array $attribs, string $name, string $orgVal/* , string $dateformat */): string{
		$currentdate = self::getAttribute('currentdate', $attribs, false, we_base_request::BOOL);
		$minyear = self::getAttribute('minyear', $attribs, 0, we_base_request::INT);
		$maxyear = self::getAttribute('maxyear', $attribs, 0, we_base_request::INT);
		$newAtts = self::removeAttribs($attribs, [
					'checked',
					'choice',
					'cols',
					'doc',
					'htmlspecialchars',
					'languageautofill',
					'maxlength',
					'mode',
					'name',
					'onChange',
					'onClick',
					'onchange',
					'onclick',
					'options',
					'pure',
					'rows',
					'selected',
					'type',
					'usevalue',
					'value',
					'values',
					'wysiwyg',
					'xml',
					'currentdate',
					'minyear',
					'maxyear',
					'_name_orig',
		]);
		if($currentdate){
			$orgVal = time();
		}
		try{
			$date = ($orgVal ? new DateTime((is_numeric($orgVal) ? date('Y-m-d H:i:s', (int) $orgVal) : $orgVal)) : new DateTime('now'));
		}catch(Exception $e){
			$date = new DateTime('now');
		}
		//list('code' => $langcode) = self::getOutLang($attribs);
		// do we need to add the name as 2nd array index here?
		return we_html_tools::getNewDateInput('s[we_date_' . $name . '][%s]', $date, we_html_tools::FORMAT_DATE_TIME, false, false, $minyear ? new DateTime($minyear . '-01-01') : null, $maxyear ? new DateTime($maxyear . '-12-31') : null, '', $newAtts);
	}

	protected static function getCountryField(array $attribs, string $name, string $orgVal): string{
		$newAtts = self::removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'name', 'value', 'values', 'onclick', 'onClick', 'mode', 'choice', 'pure',
					'rows',
					'cols', 'maxlength', 'wysiwyg']);
		$newAtts['name'] = 's[' . $name . ']';
		['lang' => $lang, 'code' => $langcode] = self::getOutLang($attribs);

		$topCountries = WE_COUNTRIES_TOP ? array_flip(explode(',', WE_COUNTRIES_TOP)) : [];
		foreach($topCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
		}

		$shownCountries = WE_COUNTRIES_SHOWN ? array_flip(explode(',', WE_COUNTRIES_SHOWN)) : [];
		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $lang . '.UTF-8');
		asort($topCountries, SORT_LOCALE_STRING);
		asort($shownCountries, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}
		$content = '';
		if(WE_COUNTRIES_DEFAULT != ''){
			$content .= '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>';
		}
		foreach($topCountries as $countrykey => $countryvalue){
			$content .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . $countryvalue . '</option>';
		}

		if($topCountries && $shownCountries){
			$content .= '<option value="-" disabled="disabled">----</option>';
		}
		foreach($shownCountries as $countrykey => $countryvalue){
			$content .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . $countryvalue . '</option>';
		}

		return self::htmlTag('select', $newAtts, $content, true);
	}

	protected static function getLangField(array $attribs, string $name, string $orgVal): string{
		$newAtts = self::removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
					'choice', 'pure', 'rows', 'cols', 'maxlength', 'wysiwyg']);
		$newAtts['name'] = 's[' . $name . ']';
		['lang' => $lang, 'code' => $langcode] = self::getOutLang($attribs);

		$frontendL = we_base_WE::$Langs['FrontendLanguages'];
		foreach($frontendL as &$lcvalue){
			$lccode = explode('_', $lcvalue);
			$lcvalue = $lccode[0];
		}
		$frontendLL = [];
		foreach($frontendL as &$lcvalue){
			$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
		}

		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $lang . '.UTF-8');
		asort($frontendLL, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}
		$content = '';
		foreach($frontendLL as $langkey => &$langvalue){
			$content .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . $langvalue . '</option>' . "\n";
		}
		unset($langvalue);
		return self::htmlTag('select', $newAtts, $content, true);
	}

}
