<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 81a61498f914 $
 * $Author: Lukas Imhof $
 * $Date: 2021-03-09 14:07 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_textarea extends we_weTag_tag{

	public static function parse(array $attribs, string $content): string{
		return $content ?
			'<?php ob_start();?>' . $content . '<?= ' . we_weTag_parser::printTag('textarea', $attribs, 'ob_get_clean()?:\'\'', false, true) . '; ?>' :
			parent::parse($attribs, $content);
	}

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);
		self::$doc = we_base_WE::getWeDoc();
		$autobr = self::getAttribute('autobr', $attribs, false, we_base_request::BOOL);

		if(we_base_WE::$editmode){
			$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);

			$removeFirstParagraph = self::getAttribute('removefirstparagraph', $attribs, defined('REMOVEFIRSTPARAGRAPH_DEFAULT') ? REMOVEFIRSTPARAGRAPH_DEFAULT : true, we_base_request::BOOL);
			$docVal = self::$doc->getElementS($name);
			$value = (string) ($docVal ?: $content);
			if(!$docVal && $content){ // when not inlineedit, we need to save the content in the object, if the field is empty
				self::$doc->setElementS($name, $content);
				self::$doc->saveInSession(we_base_WE::$transaction);
			}

			if(!we_weTag_tag::getAttribute('wysiwyg', $attribs, false, we_base_request::BOOL)){
				$attribs['onchange'] = "we_cmd('setHot');";
			}

			return we_html_element::span(['class' => 'we we_textarea'], we_html_forms::weTextarea('we_' . self::$doc->Name . '_txt[' . $name . ']', $value, $attribs, false, false, $removeFirstParagraph, false, $name, self::$doc->getDocumentCss()));
		}

		$txt = we_document_base::parseInternalLinks((string) self::$doc->getField($attribs), 0);
		$txt = ($autobr ? strtr($txt, [
				"\r" => '',
				"<br>\n" => "\n",
				"<br/>\n" => "\n",
				"<br />\n" => "\n",
				"<BR>\n" => "\n",
				"<BR/>\n" => "\n",
				"<BR />\n" => "\n",
				"\n" => we_html_element::br()]) :
			$txt);
		return we_wysiwyg_editor::parseGalleryTags($txt);
	}

}
