<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: dfc1b1b26b9f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-12-19 14:26 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * General Definition of WebEdition Voting
 *
 */
class we_voting_voting extends we_base_model{
//voting status codes

	const SUCCESS = 1;
	const ERROR = 2;
	const ERROR_REVOTE = 3;
	const ERROR_ACTIVE = 4;
	const ERROR_BLACKIP = 5;
	const ERROR_REQUIRED = 6;

	//properties
	/** @var int */
	var $ID;

	/** @var int */
	var $ParentID = 0;

	/** @var string */
	var $Path = '';

	/** @var bool */
	var $IsFolder = false;

	/** @var string */
	var $Text = '';

	/** @var DateTime */
	var $PublishDate = null;

	/** @var array<int,array{question:string,answers:string[]}> */
	var $QASet = [];

	/** @var array<int,array{imageID:array<int,int>,mediaID:array<int,int>,successorID:array<int,int>}> */
	var $QASetAdditions = [];

	/** @var bool */
	var $IsRequired = false;

	/** @var bool */
	var $AllowFreeText = false;

	/** @var bool */
	var $AllowImages = false;

	/** @var bool */
	var $AllowMedia = false;

	/** @var bool */
	var $AllowSuccessor = false;

	/** @var bool */
	var $AllowSuccessors = false;

	/** @var int */
	var $Successor = 0;

	/** @var float[] */
	var $Scores = [];

	/** @var int */
	var $RevoteControl = 0;

	/** @var int */
	var $RevoteTime = 630720000;

	/** @var int[] */
	var $Owners = [];

	/** @var bool */
	var $RestrictOwners = false;

	/** @var array<string,int> */
	var $Revote = [];

	/** @var array<string,array> */
	var $RevoteUserAgent = [];

	/** @var DateTime */
	var $Valid = null;

	/** @var bool */
	var $Active = true;

	/** @var bool */
	var $ActiveTime = false;

	/** @var bool */
	var $FallbackIp = false;

	/** @var bool */
	var $UserAgent = false;

	/** @var bool */
	var $FallbackUserID = false;

	/** @var bool */
	var $Log = false;

	/** @var bool */
	var $RestrictIP = false;

	/** @var string[] */
	var $BlackList = [];

	/** @var int */
	var $answerCount = -1;

	/** @var int */
	var $defVersion = 0;

	/** @var bool */
	var $FallbackActive = false; //IsRadio

	/** @var bool */
	var $IsRadio = false;

	/** @var bool */
	var $IsCheckbox = false;

	/**
	 * Default Constructor
	 * Can load or create new Newsletter depends of parameter
	 */
	function __construct(int $votingID = 0){
		parent::__construct(VOTING_TABLE);
		$this->persistent_slots = [
			'ID' => we_base_request::INT,
			'ParentID' => we_base_request::INT,
			'Path' => we_base_request::FILE,
			'IsFolder' => we_base_request::BOOL,
			'Text' => we_base_request::FILE,
			'PublishDate' => we_base_request::DATE_TIME,
			'QASet' => we_base_request::SERIALIZED_JSON,
			'QASetAdditions' => we_base_request::SERIALIZED_JSON,
			'IsRequired' => we_base_request::BOOL,
			'AllowFreeText' => we_base_request::BOOL,
			'AllowImages' => we_base_request::BOOL,
			'AllowMedia' => we_base_request::BOOL,
			'AllowSuccessor' => we_base_request::BOOL,
			'AllowSuccessors' => we_base_request::BOOL,
			'Successor' => we_base_request::INT,
			'Scores' => we_base_request::SERIALIZED_JSON,
			'RevoteControl' => we_base_request::INT,
			'RevoteTime' => we_base_request::INT,
			'Owners' => we_base_request::INTLISTA,
			'RestrictOwners' => we_base_request::BOOL,
			'Revote' => we_base_request::SERIALIZED_JSON,
			'RevoteUserAgent' => we_base_request::SERIALIZED_JSON,
			'Valid' => we_base_request::DATE_TIME,
			'Active' => we_base_request::BOOL,
			'ActiveTime' => we_base_request::BOOL,
			'FallbackIp' => we_base_request::BOOL,
			'UserAgent' => we_base_request::BOOL,
			'FallbackUserID' => we_base_request::BOOL,
			'Log' => we_base_request::BOOL,
			'RestrictIP' => we_base_request::BOOL,
			'BlackList' => we_base_request::STRING_LIST,
		];

		if($votingID){
			$this->ID = $votingID;
			$this->load($votingID);
		}

		$this->QASet = $this->QASet ?: [
			0 => [
				'question' => '',
				'answers' => [
					0 => '',
					1 => ''
				]
			]
		];

		$this->QASetAdditions = $this->QASetAdditions ?: [0 => ['imageID' => [], 'mediaID' => [], 'successorID' => []]];

		$this->Valid = ($this->Valid ?: time() + 31536000); //365 days
		$this->Active = ($this->Valid < time() && $this->ActiveTime ? false : $this->Active);
		$this->Scores = ($this->Scores ?: []);
		$this->PublishDate = ($this->PublishDate ?: time());
	}

	function save(bool $with_scores = true, bool $rebuild = false, array $skipWrite = []): bool{
		if($this->ActiveTime && $this->Valid < time()){
			$this->Active = false;
		}

		if(!$this->IsFolder && ($with_scores || $this->ID == 0)){
			$qaset_count = count($this->QASet[$this->defVersion]['answers']);
			$scores_count = count($this->Scores);

			if($qaset_count != $scores_count){
				$diff = $qaset_count - $scores_count;
				if($diff > 0){
					for($i = 0; $i < $diff; $i++){
						$this->Scores[] = 0;
					}
				}
				if($diff < 0){
					$diff = abs($diff);
					for($i = 0; $i < $diff; $i++){
						array_pop($this->Scores);
					}
				}
			}
		}

		$this->ParentID = $this->ParentID ?: 0;
		if(we_users_user::getID() && ($this->RestrictOwners && empty($this->Owners) || !in_array(we_users_user::getID(), $this->Owners))){
			$this->Owners[] = we_users_user::getID();
		}

		if($with_scores || $this->ID == 0){

		} elseif(we_base_request::bool('updateScores')){
			$ic = we_base_request::int('item_count');
			$temp = [];
			for($xcount = 0; $xcount < $ic; $xcount++){
				if(($tmp = we_base_request::float('scores_' . $xcount))){
					$temp[$xcount] = $tmp;
				}
			}
			$this->Scores = $temp;
		} else {
			unset($this->persistent_slots['Scores']);
		}

		$ret = parent::save(false);

		return $ret;
	}

	function delete(): bool{
		if(!$this->ID){
			return false;
		}
		if($this->IsFolder){
			$this->deleteChilds();
		}
		parent::delete();
		return true;
	}

	function initByName(string $name): bool{
		return $this->load(fI('SELECT ID FROM ' . VOTING_TABLE . ' WHERE Text="' . $this->db->escape($name) . '"'));
	}

	function clearSessionVars(): void{
		if(isset($_SESSION['weS']['module_sessions']['voting'])){
			unset($_SESSION['weS']['module_sessions']['voting']);
		}
	}

	static function filenameNotValid(string $text): bool{
		return preg_match('%[^a-z0-9äüöß\._\@\ \-]%i', $text) ? true : false;
	}

	function getNext(): bool{
		$this->answerCount++;
//	if($this->AllowFreeText){
		$addOne = 0;
		return ($this->answerCount < count($this->QASet[$this->defVersion]['answers']) + $addOne);
	}

	function resetSets(): void{
		$this->answerCount = -1;
	}

	function getAnswer(): string{
		return $this->QASet[$this->defVersion]['answers'][$this->answerCount];
	}

	function getResult(string $type = 'count', string $num_format = '', int $precision = 2): string{
		switch($type){
			case 'percent':
				$total = (int) $this->getResult('total');
				if($total <= 0){
					return '0';
				}
				$scores = $this->Scores[$this->answerCount] ?? 0;
				$result = ($total > 0 && $this->answerCount >= 0 ? round((($scores / $total) * 100), $precision) : 100);
				break;
			case 'total':
				$result = array_sum($this->Scores);
				break;
			case 'count':
			default:
				$result = ($this->answerCount >= 0 && isset($this->Scores[$this->answerCount]) ?
					$this->Scores[$this->answerCount] :
					0);
				break;
		}

		return ($num_format ?
			we_base_util::formatNumber($result, $num_format, $precision) :
			(string) ($result));
	}

	function isLastSet(): bool{
		return ($this->answerCount >= count($this->QASet[$this->defVersion]['answers']) - 1);
	}

	function setDefVersion(int $version): void{
		if($version < 0){
			$this->defVersion = 0;
		}
		$this->defVersion = ($version < count($this->QASet) ? $version : count($this->QASet) - 1);
	}

	function isAllowedForUser(): bool{
		if($this->RestrictOwners == 0 || we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) || in_array(we_users_user::getID(), $this->Owners)){
			return true;
		}

		$userids = [];
		we_base_util::readParents(we_users_user::getID(), $userids, USER_TABLE, 'Type', ['group']);

		return array_intersect($userids, $this->Owners) ? true : false;
	}

	public static function getOwnersSql(): string{
		$owners_sql = '';
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			$userids = [we_users_user::getID()];
			we_base_util::readParents(we_users_user::getID(), $userids, USER_TABLE, 'Type', ['group']);

			$owners_sql = ' AND (RestrictOwners=0 OR CONCAT(",",Owners,",") REGEXP ",(' . implode('|', $userids) . '),") ';
		}

		return $owners_sql;
	}

	function getSuccessor(array $answers): int{
		$answerID = (int) $answers[0];
		return (int) $this->QASetAdditions[$this->defVersion]['successorID'][$answerID];
	}

	function setSuccessor(array $answers): int{
		foreach($answers as &$answer){
			if($this->AllowSuccessors &&
				($mySuccessorID = (int) (stripslashes((string) $this->QASetAdditions[$this->defVersion]['successorID'][$answer])))){
				we_tag_writeVoting::$successorID = $mySuccessorID;
			}
		}
		if($this->AllowSuccessor && ( $mySuccessorID = (int) ($this->Successor))){
			we_tag_writeVoting::$successorID = $mySuccessorID;
		}
		return self::SUCCESS;
	}

	function vote(array $answers, array $addfields = []): int{
		$votingsession = ($_SESSION['_we_voting_sessionID'] ?? 0);
		sort($answers);
		$answerID = implode(',', $answers);
		$answertext = '';
		$successor = 0;
		if(!(count($answers) > 0)){
			if($this->Log){
				$this->logVoting(self::ERROR, $votingsession, $answerID, $answertext, $successor);
			}
			return self::ERROR;
		}

		$ret = $this->canVote();

		if($ret != self::SUCCESS){
			if($this->Log){
				$this->logVoting($ret, $votingsession, $answerID, $answertext, $successor);
			}
			return $ret;
		}

		$countanswers = count($this->QASet[$this->defVersion]['answers']);
		$mySuccessorID = -1;
		foreach($answers as &$answer){
			if(is_numeric($answer) && $answer < $countanswers){
				if($answer > -1 && $answer < count($this->Scores)){
					$this->Scores[$answer]++;
				}
			} else {
				$answertext = $answer;
				$answer = $countanswers - 1;
				$this->Scores[$answer]++;
			}
			if($this->AllowSuccessors &&
				( $mySuccessorID = (int) (stripslashes((string) $this->QASetAdditions[$this->defVersion]['successorID'][$answer])))){
				we_tag_writeVoting::$successorID = $mySuccessorID;
			}
		}
		if($this->AllowSuccessor && ($mySuccessorID = intval($this->Successor))){
			we_tag_writeVoting::$successorID = $mySuccessorID;
		}
		$answerID = implode(',', $answers);
		$this->saveField('Scores');
		if($this->RevoteTime){
			if($this->RevoteControl == 1){
				$revotetime = $this->RevoteTime < 0 ? 630720000 : $this->RevoteTime;
				if(version_compare(PHP_VERSION, '7.3.0') >= 0){
					setcookie(md5('_we_voting_' . $this->ID), (string) (time()), ['expires' => time() + $revotetime, 'path' => '/', 'httponly' => true, 'samesite' => 'strict']);
				} else {
					setcookie(md5('_we_voting_' . $this->ID), (string) (time()), time() + $revotetime);
				}
			} else {
				$this->Revote[$_SERVER['REMOTE_ADDR']] = time();
				$this->saveField('Revote');
				if($this->UserAgent){
					if(!isset($this->RevoteUserAgent[$_SERVER['REMOTE_ADDR']]) || !is_array($this->RevoteUserAgent[$_SERVER['REMOTE_ADDR']])){
						$this->RevoteUserAgent[$_SERVER['REMOTE_ADDR']] = [];
					}
					$this->RevoteUserAgent[$_SERVER['REMOTE_ADDR']][] = $_SERVER['HTTP_USER_AGENT'];
					$this->saveField('RevoteUserAgent');
				}
			}
		}
		$mySuccessorID = ($mySuccessorID <= 0 ? 0 : $mySuccessorID);

		$addfieldsdata = ($addfields ? we_serialize($addfields) : '');

		if($this->Log){
			$this->logVoting($ret, $votingsession, $answerID, (string) $answertext, $mySuccessorID, $addfieldsdata);
		}

		return $ret;
	}

	function canVote(): int{
		if(!$this->isActive()){
			return self::ERROR_ACTIVE;
		}
		if($this->isBlackIP()){
			return self::ERROR_BLACKIP;
		}
		if(!$this->RevoteTime){
			return self::SUCCESS;
		}

		switch($this->RevoteControl){
			case 2:
				return $this->canVoteUserID();
			case 1:
				return $this->canVoteCookie();
			default:
				return $this->canVoteIP();
		}
	}

	function canVoteCookie(): int{
		if($this->cookieDisabled() && $this->FallbackIp){
			$this->RevoteControl = 0;
			$this->FallbackActive = true;
			return $this->canVoteIP();
		}

		if(isset($_COOKIE[md5('_we_voting_' . $this->ID)])){
			return self::ERROR_REVOTE;
		}
		return ($this->FallbackUserID ? $this->canVoteUserID() : self::SUCCESS);
	}

	function canVoteIP(): int{
		$revotetime = ($this->RevoteTime < 0 ? time() + 5 : $this->RevoteTime);

		if(in_array($_SERVER['REMOTE_ADDR'], array_keys($this->Revote))){
			if(($revotetime + $this->Revote[$_SERVER['REMOTE_ADDR']]) > time()){
				if($this->UserAgent){
					if(isset($this->RevoteUserAgent[$_SERVER['REMOTE_ADDR']]) && is_array($this->RevoteUserAgent[$_SERVER['REMOTE_ADDR']])){
						if(in_array($_SERVER['HTTP_USER_AGENT'], $this->RevoteUserAgent[$_SERVER['REMOTE_ADDR']])){
							return self::ERROR_REVOTE;
						}
						return ($this->FallbackUserID ? $this->canVoteUserID() : self::SUCCESS);
					}
				}

				return self::ERROR_REVOTE;
			}
			return ($this->FallbackUserID ? $this->canVoteUserID() : self::SUCCESS);
		}
		return ($this->FallbackUserID ? $this->canVoteUserID() : self::SUCCESS);
	}

	function canVoteUserID(): int{
		$userid = (we_modules_info::isActive(we_modules_info::CUSTOMER) && !empty($_SESSION['webuser']['registered']) && !empty($_SESSION['webuser']['ID'])) ? $_SESSION['webuser']['ID'] : -1;

		if(($userid <= 0)){
			return self::SUCCESS;
		}
		$testtime = time() - $this->RevoteTime;

		if(fB('SELECT 1 FROM `' . VOTING_LOG_TABLE . '` WHERE voting=' . intval($this->ID) . ' AND userid=' . intval($userid) . ' AND time>' . intval($testtime) . ' LIMIT 1')){
			return self::ERROR_REVOTE;
		}
		return self::SUCCESS;
	}

	function isActive(): bool{
		if(!$this->Active){
			return false;
		}
		if($this->ActiveTime == 0){
			return true;
		}
		if(time() > $this->Valid){
			return false;
		}
		return true;
	}

	function cookieDisabled(): bool{
		return !(isset($_SESSION['_we_cookie_']));
	}

	function isBlackIP(): bool{
		if(!$this->RestrictIP){
			return false;
		}
		$ip = $_SERVER['REMOTE_ADDR'];
		foreach($this->BlackList as $fip){
			$reg = str_replace('*', '[0-9]+', $fip);
			if(preg_match('/' . $reg . '/', $ip)){
				return true;
			}
		}
		return false;
	}

	function resetIpData(): void{
		$this->Revote = [];
		$this->RevoteUserAgent = [];
		$this->saveField('Revote');
		$this->saveField('RevoteUserAgent');
	}

	/**
	 * writes a single entry to the voting log table VOTING_LOG_TABLE
	 * @return boolean success or failure of this operation
	 * @author Alexander Lindenstruth
	 * @since 5.1.1.2 - 02.05.2008
	 */
	function logVoting(int $status, int $votingsession, string $answer, string $answertext, int $successor, string $additionalfields = ''): bool{
		$cookieStatus = $this->cookieDisabled() ? 0 : 1;
		$userid = (we_modules_info::isActive(we_modules_info::CUSTOMER) && !empty($_SESSION['webuser']['registered']) && !empty($_SESSION['webuser']['ID']) ?
			$_SESSION['webuser']['ID'] : 0);
		$this->db->query('INSERT INTO `' . VOTING_LOG_TABLE . '` SET ' . we_database_base::arraySetter([
				'votingsession' => $votingsession,
				'voting' => $this->ID,
				'time' => sql_function('UNIX_TIMESTAMP()'),
				'ip' => inet_pton(strstr($_SERVER['REMOTE_ADDR'], ':') ? $_SERVER['REMOTE_ADDR'] : '::ffff:' . $_SERVER['REMOTE_ADDR']),
				'agent' => $_SERVER['HTTP_USER_AGENT'],
				'userid' => $userid,
				'cookie' => $cookieStatus,
				'fallback' => $this->FallbackActive,
				'answer' => $answer,
				'answertext' => $answertext,
				'successor' => $successor,
				'additionalfields' => $additionalfields,
				'status' => $status,
		]));

		return true;
	}

	function deleteLogData(): bool{
		if($this->IsFolder){
			$this->deleteGroupLogData();
			return true;
		}
		$this->deleteLogDataDB();
		return true;
	}

	function deleteGroupLogData(): bool{
		$entries = $this->db->getAllq('SELECT ID FROM ' . VOTING_TABLE . ' WHERE Path LIKE "' . $this->db->escape($this->Path) . '%"', true);
		foreach($entries as $id){
			$child = new we_voting_voting(intval($id));
			$child->deleteLogDataDB();
		}
		return true;
	}

	/* ================================= NEW FUNCTIONS FOR LOGGING TO VOTING_LOG_TABLE ================================= */
	/**
	 * new functions for logging votings to a separate database table
	 * new logging mode logs votings to a new table called "tblvotinglog" instead of saving them
	 * 			to the field "LogData" as a serialized array. So $this->LogData contains an array if the new mode
	 * 			is used (for newly created votings). For existing votings using the old logging style nothing has changed
	 * 			and the variable will still contain a string with the serialized logging data array from tblvoting.LogData
	 * 			In the first case the variable $this->LogDB will be set to true, else it will stay false
	 * @author Alexander Lindenstruth
	 * @since 5.1.1.2 - 05.05.2008
	 */

	/**
	 * fetches all log entries for current voting from database
	 * @return array<int,array{votingsession:string,voting:int,time:int,ip:string,agent:string,userid:int,cookie:int,fallback:int,status:int,answer:string,answertext:string,successor:int,additionalfields:string}>
	 */
	function loadDB(int $id = 0): array{
		$this->db->query(($this->IsFolder ?
				'SELECT v.*,l.* FROM ' . VOTING_TABLE . ' v JOIN ' . VOTING_LOG_TABLE . ' l ON v.ID=l.voting WHERE v.Path LIKE "' . $this->db->escape($this->Path) . '%" AND v.IsFolder=0 ORDER BY l.time' :
				'SELECT * FROM ' . VOTING_LOG_TABLE . ' WHERE voting=' . $id . ' ORDER BY time'
		));

		$ret = [];
		while($this->db->next_record()){
			$ret[] = [
				'votingsession' => $this->db->f('votingsession'),
				'voting' => $this->db->f('voting'),
				'time' => (int) $this->db->f('time'),
				'ip' => inet_ntop($this->db->f('ip')),
				'agent' => $this->db->f('agent'),
				'userid' => (int) $this->db->f('userid'),
				'cookie' => (int) $this->db->f('cookie'),
				'fallback' => (int) $this->db->f('fallback'),
				'status' => (int) $this->db->f('status'),
				'answer' => $this->db->f('answer'),
				'answertext' => $this->db->f('answertext'),
				'successor' => (int) $this->db->f('successor'),
				'additionalfields' => $this->db->f('additionalfields'),
			];
		}
		return $ret;
	}

	/**
	 * deletes all log entries for a given voting
	 * @return boolean success or failure of this operation
	 * @author Alexander Lindenstruth
	 * @since 5.1.1.2 - 02.05.2008
	 */
	function deleteLogDataDB(): bool{
		$this->db->query('DELETE FROM ' . VOTING_LOG_TABLE . ' WHERE voting=' . intval($this->ID));
		return true;
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'answer_limit' => g_l('modules_voting', '[answer_limit]'),
				'delete_alert' => g_l('modules_voting', '[delete_alert]'),
				'delete_ipdata_question' => g_l('modules_voting', '[delete_ipdata_question]'),
				'delete_log_question' => g_l('modules_voting', '[delete_log_question]'),
				'imageID_text' => g_l('modules_voting', '[imageID_text]'),
				'mediaID_text' => g_l('modules_voting', '[mediaID_text]'),
				'new_ip_add' => g_l('modules_voting', '[new_ip_add]'),
				'no_perms' => (g_l('modules_voting', '[no_perms]')),
				'not_valid_ip' => g_l('modules_voting', '[not_valid_ip]'),
				'nothing_to_delete' => (g_l('modules_voting', '[nothing_to_delete]')),
				'nothing_to_save' => (g_l('modules_voting', '[nothing_to_save]')),
				'result_delete_alert' => g_l('modules_voting', '[result_delete_alert]'),
				'save_changed_voting' => g_l('modules_voting', '[save_changed_voting]'),
				'successorID_text' => g_l('modules_voting', '[successorID_text]'),
				'variant_limit' => g_l('modules_voting', '[variant_limit]'),
		]);
	}

}
