<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d938a246a5b9 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-29 12:22 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_widget_mdc extends we_widget_base{//Meine Dokumente

	/** @var string */

	private $mdc = '';

	/** @var string */
	private $binary = '';

	/** @var array{int,string,string} */
	private $splitMdc = [0, '', ''];

	//FIXME: $aProps
	public function __construct(string $curID = '', array $aProps = []){
		$DB_WE = DB_WE::$inst;
		$this->mdc = '';
		if(!$curID){//preview requested
			$this->splitMdc = [
				0, //unused - compatibility
				we_base_request::string('we_cmd', '', 0),
				implode(',', we_base_request::intList('we_cmd', [], 1))
			];
		} else {
			$this->splitMdc = $aProps ? explode(';', $aProps[3]) : [];
		}

		if(!$this->splitMdc || count($this->splitMdc) !== 3){
			return;
		}
		$this->binary = $this->splitMdc[1];
		$table = ($this->binary[1]) ? OBJECT_FILES_TABLE : FILE_TABLE;
		$ids = we_base_request::filterIntList($this->splitMdc[2]);
		$wsArr = we_users_util::get_ws($table);
		//get workspace query
		if($wsArr){
			$childs = array_values($wsArr);
			we_base_util::readChilds($wsArr, $childs, $table, true, '', 'ContentType', we_base_ContentTypes::FOLDER_TYPES);
			$wsQuery = ' t.ParentID IN(' . implode(',', array_unique($childs)) . ') ';
		} else {
			$wsQuery = '';
		}

		if($this->binary[0]){
			$folders = getAllFoldersRecursive($ids);
			$query = ($folders ? 'SELECT
ID,Path,Text,ContentType
FROM ' .
					$table . ' t
WHERE
	t.ParentID IN(' . implode(',', $folders) . ') AND
	t.IsFolder=0' .
					($wsQuery ? ' AND ' . $wsQuery : '') : false);
		} else {
			$categories = [];
			if($this->splitMdc[4]){
				$cats = explode(',', (string) $this->splitMdc[4]);
				foreach($cats as $myCat){
					$id = fI('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE Path="' . DB_WE::$inst->escape(base64_decode($myCat) ?: '') . '"');
					$categories[] = 'FIND_IN_SET("' . $id . '",Category)';
				}
			}
			[, $folderPath] = $ids;
			$where = [
				'Path LIKE "' . $folderPath . '%"',
				'IsFolder=0',
				(($this->splitMdc[3] != 0) ? (!$this->binary[1] ? 'DocType' : 'TableID') . '="' . $this->splitMdc[3] . '"' : ''),
				($categories ? '(' . implode(' OR ', $categories) . ')' : ''),
				($wsQuery ?: '')
			];

			$query = 'SELECT ID,Path,Text,ContentType
FROM ' .
					$table . ' t
WHERE ' .
					implode(' AND ', array_filter($where));
		}

		if($query && ($all = $DB_WE->getAllq($query . ' LIMIT 100'))){
			$this->mdc .= '<table class="default">';
			foreach($all as $rec){
				$this->mdc .= '<tr><td class="mdcIcon" data-contenttype="' . $rec['ContentType'] . '"></td><td style="vertical-align:middle" class="middlefont">' .
						we_html_element::a([
							'href' => "javascript:WE().layout.weEditorFrameController.openDocument('" . $table . "','" . $rec['ID'] . "','" . $rec['ContentType'] . "');",
							'title' => $rec['Path'],
							'style' => 'color:#000000;text-decoration:none;'
								], $rec['Path']) . '</td></tr>';
			}
			$this->mdc .= '</table>';
		}
	}

	/**
	 * @return array{string,array{string,string}}
	 */
	public function getInsertDiv(int $iCurrId, array $aProps, we_base_jsCmd $jsCmd): array{
		$cfg = self::getDefaultConfig();
		$oTblDiv = we_html_element::div(['id' => 'm_' . $iCurrId . '_inline',
					'style' => 'width:100%;height:' . ($cfg["height"] - 25) . 'px;overflow:auto;'
						], $this->mdc);
		$aLang = [($this->splitMdc[0]) ? (base64_decode((string) $this->splitMdc[0]) ?: '') : g_l('cockpit', (empty($this->splitMdc[1][1]) ? '[my_documents]' : '[my_objects]')),
			""];
		$jsCmd->addCmd('setIconOfDocClass', 'mdcIcon');

		return [$oTblDiv, $aLang];
	}

	/**
	 * @return array<string,mixed>
	 */
	public static function getDefaultConfig(): array{
		return [
			'width' => self::WIDTH_SMALL,
			'expanded' => 0,
			'height' => 307,
			'res' => 0,
			'cls' => 'white',
			'csv' => ';10;',
			'dlgHeight' => 450,
			'isResizable' => true
		];
	}

	private static function getHTMLDirSelector(bool $selType): string{
		$rootDirID = 0;
		$folderID = 0;
		$button_doc = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements.FolderID.value,table:WE().consts.tables.FILE_TABLE,targetID:'FolderID',targetName:'FolderPath',rootDirID:'" . $rootDirID . "'})");
		$button_obj = we_modules_info::isActive(we_modules_info::OBJECT) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements.FolderID.value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'FolderID',targetName:'FolderPath',rootDirID:'" . $rootDirID . "'})") : '';

		$buttons = '<div id="docFolder" style="display: ' . (!$selType ? "inline" : "none") . '">' . $button_doc . "</div>" . '<div id="objFolder" style="display: ' . ($selType ? "inline" : "none") . '">' . $button_obj . "</div>";
		$path = id2path($folderID, (!$selType ? FILE_TABLE : OBJECT_FILES_TABLE));

		//FIXME: autocompleter?!

		return we_html_element::div(['style' => "margin-top:10px;"
						], we_html_tools::formElementTable(g_l('cockpit', '[dir]'), we_html_tools::textInput('FolderPath', $path, 0, ['id' => "yuiAcInputDoc"], 'text', (420 - 120) . 'px'), we_html_element::hidden("FolderID", $folderID, "yuiAcIdDoc"), $buttons));
	}

	private static function getHTMLCategory(array &$widgetData): string{
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{id:0,table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'add_cat'})", '', "", (!we_base_permission::hasPerm([
							we_base_perm::WORK_CATEGORY_EDIT])));
		$del_but = we_html_button::create_button(we_html_button::TRASH, "javascript:" . we_base_constants::PLACEHOLDER . ";top.setHot();");
		$widgetData['cats'] = [
			'del' => $del_but,
			'items' => []
		];

		$table = new we_html_table(['id' => 'CategoriesBlock',
			'style' => 'display: block;margin-top: 5px;',
			'class' => 'default'
				], 5, 1);

		$table->setCol(1, 0, "Kategorien");
		$table->setCol(2, 0, we_html_element::div(['id' => 'categories',
					'class' => 'blockWrapper',
					'style' => 'width:420px;height:60px;border:#AAAAAA solid 1px;margin-bottom:5px;'
		]));

		$table->setCol(4, 0, we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:removeAllCats();") . $addbut, ['colspan' => '2', 'style' => 'text-align:right']);

		return $table->getHtml();
	}

	public static function showDialog(): void{
		$DB_WE = DB_WE::$inst;
		[$jsFile, $oSelCls] = self::getDialogPrefs();
		$jsCmd = new we_base_jsCmd();
		$widgetData = [];

		[$sTitle, $selBinary, $sCsv] = explode(';', we_base_request::string('we_cmd', ';;', 1));
		$title = base64_decode($sTitle) ?: '';
		$selection = (bool) $selBinary[0];
		$selType = (bool) $selBinary[1];

		if($selection){
			$_SESSION['weS']['module_sessions']['exportVars'][($selType ? "selObjs" : "selDocs")] = $sCsv;
		}

		$selTable = ($selType && we_modules_info::isActive(we_modules_info::OBJECT)) ? OBJECT_FILES_TABLE : FILE_TABLE;
		$dtq = we_doctype_docTypes::getDoctypeQuery();
		$docTypes = [0 => g_l('cockpit', '[no_entry]')] + $DB_WE->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'], false);

		$doctypeElement = we_html_tools::formElementTable(g_l('cockpit', '[doctype]'), we_html_tools::select('DocTypeID', $docTypes, 1, 0, false, ['style' => "width:420px; border: #AAAAAA solid 1px;"]));

		$cls = (new we_html_select(['name' => "classID",
					'class' => 'defaultfont',
					'style' => "width:420px; border: #AAAAAA solid 1px"
						]))
				->addOption('0', g_l('cockpit', '[no_entry]'));

		if(($ac = we_users_util::getAllowedClasses())){
			$options = $DB_WE->getAllFirstq('SELECT ID,Text FROM ' . OBJECT_TABLE . ' WHERE ID IN(' . implode(',', $ac) . ') ORDER BY Text', false);
			'@phan-var array<int,string> $options';
			$cls->addOptions($options);
		}

		/* $seltype = [
		  'doctype' => g_l('cockpit', '[documents]')

		  ];
		  if(we_modules_info::isActive(we_modules_info::OBJECT)){
		  $seltype['classname'] = g_l('cockpit', '[objects]');
		  } */

		$tree = new we_export_tree($jsCmd);

		$captions = [];
		if(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){
			$captions[FILE_TABLE] = g_l('export', '[documents]');
		}
		if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
			$captions[OBJECT_FILES_TABLE] = g_l('export', '[objects]');
		}


		$divContent = we_html_element::div(['style' => "display:block;"
						], we_html_tools::select('Selection', ["dynamic" => g_l('cockpit', '[dyn_selection]'), "static" => g_l('cockpit', '[stat_selection]')
								], 1, ($selection ? "static" : "dynamic"), false, ['style' => "width:420px;border:#AAAAAA solid 1px;", 'onchange' => "closeAllSelection();we_submit();"]) .
						we_html_element::br() .
						we_html_tools::select('headerSwitch', $captions, 1, (!$selType ? FILE_TABLE : OBJECT_FILES_TABLE), false, ['style' => "width:420px;border:#AAAAAA solid 1px;margin-top:10px;",
							'onchange' => "setHead(this.value);we_submit();"], true, 420) .
						we_html_element::div(['id' => "static", 'style' => ($selection ? "display:block;" : "display:none;")], we_html_element::div(['id' => "treeContainer"], $tree->getHTMLMultiExplorer(420, 180, false)) . '<iframe name="cmd" src="about:blank" style="visibility:hidden; width: 0; height: 0;"></iframe>') .
						we_html_element::div(['id' => "dynamic", 'style' => (!$selection ? 'display:block;' : 'display:none;')
								], self::getHTMLDirSelector($selType) . we_html_element::br() . ((!$selType) ? $doctypeElement : we_html_tools::formElementTable(g_l('cockpit', '[class]'), $cls->getHTML())) . we_html_element::br() . self::getHTMLCategory($widgetData)) .
						we_html_element::br() .
						we_html_tools::formElementTable(g_l('cockpit', '[title]'), we_html_tools::textInput("title", $title, 255, [], 'text', '420px')));

		$parts = (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', $divContent))
				->add(new we_html_multiIconBoxContentLine('', $oSelCls->getHTML()));

		$save_button = we_html_button::create_button(we_html_button::SAVE, "javascript:save();");
		$preview_button = we_html_button::create_button(we_html_button::PREVIEW, "javascript:preview();");
		$cancel_button = we_html_button::create_button(we_html_button::CLOSE, "javascript:exit_close();");
		$buttons = we_html_button::position_yes_no_cancel($save_button, $preview_button, $cancel_button);

		$sTblWidget = (new we_html_multiIconBox($parts, 'mdcProps'))
				->marginLeft()
				->buttons($buttons)
				->headline(g_l('cockpit', '[my_documents]'))
				->get();

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[my_documents]'), $jsFile .
				we_html_element::jsScript(JS_DIR . 'widgets/mdc.js', '', ['id' => 'loadVarWidget', 'data-widget' => setDynamicVar($widgetData)]) .
				$jsCmd->getCmds(), we_html_element::body(
						['class' => 'weDialogBody', 'onload' => "init('" . $selTable . "','" . $sTitle . "','" . $selBinary . "','" . $sCsv . "');"
						], we_html_element::form([], we_html_element::hiddens([
									'table' => '',
									'FolderID' => '0',
									'CategoriesControl' => (string) (we_base_request::int('CategoriesControl', 0))
								]) . $sTblWidget)));
	}

	public function showPreview(): void{

		$cmd4 = we_base_request::string('we_cmd', '', 4);
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('initPreview', [
			'id' => we_base_request::string('we_cmd', '', 5),
			'type' => 'mdc',
			'tb' => ($cmd4 ?: g_l('cockpit', (($this->binary[1] ? '[my_objects]' : '[my_documents]')))),
		]);
		$jsCmd->addCmd('setIconOfDocClass', 'mdcIcon');
		echo we_html_tools::getHtmlTop(g_l('cockpit', '[my_documents]'), $jsCmd->getCmds(), we_html_element::body(
						[
							'style' => 'margin:10px 15px;',
						], we_html_element::div([
							'id' => "mdc"
								], $this->mdc)));
	}

}
