<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d938a246a5b9 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-29 12:22 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_widget_upb extends we_widget_base{ //unpublished documents/objects
	/** @var string */

	private $ct = '';

	/** @var bool */
	private $bTypeDoc = false;

	/** @var bool */
	private $bTypeObj = false;

	public function __construct(string $curID = '', array $aProps = []){
		$aProps = $aProps ?: [
			0,
			0,
			0,
			we_base_request::string('we_cmd', '', 0)
		];

// widget UNPUBLISHED
		$this->bTypeDoc = (bool) $aProps[3][0];
		$this->bTypeObj = (bool) $aProps[3][1];
		$numRows = 25;

		$tbls = [];
		if($this->bTypeDoc && $this->bTypeObj){
			$tbls[] = FILE_TABLE;
			if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
				$tbls[] = OBJECT_FILES_TABLE;
			}
		} else {
			if($this->bTypeDoc){
				$tbls[] = FILE_TABLE;
			}
			if($this->bTypeObj && we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
				$tbls[] = OBJECT_FILES_TABLE;
			}
		}

		$cont = [];
		$db = DB_WE::$inst;
		foreach($tbls as $table){
			'@phan-var string $table';
			$wsArr = we_users_util::get_ws($table);
			if(we_modules_info::isActive(we_modules_info::WORKFLOW)){
				$myWfDocsArray = we_workflow_utility::getWorkflowDocsForUser(we_users_user::getID(), $table, we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]), we_base_permission::hasPerm([
									we_base_perm::WORK_CAN_PUBLISH]), $wsArr);
				$wfDocsArray = we_workflow_utility::getAllWorkflowDocs($table);
			} else {
				$wfDocsArray = $myWfDocsArray = [];
			}

			$offset = we_base_request::int('offset', 0);
			$order = we_base_request::string('order', 'Mod_Date DESC');

			//get workspace query
			if($wsArr){
				$childs = array_values($wsArr);
				we_base_util::readChilds($wsArr, $childs, $table, true, '', 'ContentType', we_base_ContentTypes::FOLDER_TYPES);
				$wsQuery = ' t.ParentID IN(' . implode(',', $childs) . ') ';
			} else {
				$wsQuery = '';
			}

			$all = $db->getAllq('SELECT ' .
					($wfDocsArray ? '(t.ID IN(' . implode(',', $wfDocsArray) . ')) AS wforder,' : '') . '
	t.ContentType,
	t.ID,
	t.Text,
	t.ParentID,
	t.Path,
	t.Mod_Date,
	t.IsPublished,
	u2.username AS Modifier,
	u1.username AS Creator
FROM ' .
					$table . ' t LEFT JOIN ' .
					USER_TABLE . ' u1 ON u1.ID=t.CreatorID LEFT JOIN ' .
					USER_TABLE . ' u2 ON u2.ID=t.ModifierID
WHERE
	(
		(
			(t.IsPublished=0 OR t.Publish_Date<t.Mod_Date) AND
			t.ContentType IN ("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::HTML . '","' . we_base_ContentTypes::OBJECT_FILE . '")
		)' .
					($myWfDocsArray ? ' OR t.ID IN(' . implode(',', $myWfDocsArray) . ') ' : '') . '
	)' .
					($wsQuery ? ' AND (' . $wsQuery . ') ' : '') . '
ORDER BY ' .
					($myWfDocsArray ? '(t.ID IN(' . implode(',', $myWfDocsArray) . ')) DESC,' : '') . $order . '
LIMIT ' . $offset . ',' . $numRows);
			$haswf = we_modules_info::isActive(we_modules_info::WORKFLOW);

			foreach($all as $cur){
				$wfDat = '';
				if($haswf){
					if(!empty($cur['wforder'])){
						try{
							$step = we_workflow_utility::findLastActiveStep($cur['ID'], $table) + 1;
							$steps = we_workflow_utility::getNumberOfSteps($cur['ID'], $table);
							$wfDat = $step . '&nbsp;' . g_l('exchange', '[step_of]') . '&nbsp;' . $steps . '&nbsp;<i class="fa fa-lg fa-circle" style="color:#' . ($cur['wforder'] ? '006DB8' : 'E7E7E7') . ';"></i>';
						}catch(we_exception_base $e){

						}
					}
				}
				$cont[$cur['Mod_Date']] = '<tr><td class="upbIcon" data-contenttype="' . $cur['ContentType'] . '"></td><td class="upbEntry middlefont ' . ($cur['IsPublished'] ? 'changed' : 'notpublished') . '" onclick="WE().layout.weEditorFrameController.openDocument(\'' . $table . '\',' . $cur['ID'] . ',\'' . $cur['ContentType'] . '\')" title="' . $cur['Path'] . '">' . $cur['Path'] . $wfDat . '</td></tr>';
			}
		}

		asort($cont);
		$this->ct = '<table class="default">' . implode('', $cont) . '</table>';
	}

	/**
	 * @return array{string,array{string,string}}
	 */
	public function getInsertDiv(int $iCurrId, array $aProps, we_base_jsCmd $jsCmd): array{
		$cfg = self::getDefaultConfig();
		$oTblDiv = we_html_element::div(['id' => 'm_' . $iCurrId . '_inline',
					'style' => 'height:' . ($cfg["height"] - 25) . 'px;overflow:auto;'
						], $this->ct);

		$sTb = g_l('cockpit', ($this->bTypeDoc && $this->bTypeObj ? '[upb_docs_and_objs]' : ($this->bTypeDoc ? '[upb_docs]' : ($this->bTypeObj ? '[upb_objs]' : '[upb_docs_and_objs]'))));

		$jsCmd->addCmd('setIconOfDocClass', 'upbIcon');

		return [$oTblDiv, [$sTb, ""]];
	}

	/**
	 * @return array<string,mixed>
	 */
	public static function getDefaultConfig(): array{
		$shortCuts = (we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE]) ? '1' : '0') .
				(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE]) ? '1' : '0');

		return [
			'width' => self::WIDTH_SMALL,
			'expanded' => 0,
			'height' => 210,
			'res' => 0,
			'cls' => 'lightCyan',
			'csv' => $shortCuts,
			'dlgHeight' => 190,
			'isResizable' => true
		];
	}

	public static function showDialog(): void{
		[$jsFile, $oSelCls] = self::getDialogPrefs();

		$oChbxDocs = we_html_forms::checkbox('0', true, "chbx_type", g_l('cockpit', '[documents]'), true, "defaultfont", "", false, "", 0, 0);
		$oChbxObjs = we_html_forms::checkbox('0', true, "chbx_type", g_l('cockpit', '[objects]'), true, "defaultfont", "", false, "", 0, 0);

		$dbTableType = '<table><tr>' .
				'<td>' . $oChbxDocs . ' </td>' .
				(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE]) ? "<td>" . $oChbxObjs . '</td>' : '') .
				"</tr></table>";

		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[type]'), $dbTableType))
						->space(we_html_multiIconBox::SPACE_MED)
				)
				->add(new we_html_multiIconBoxContentLine('', $oSelCls->getHTML()));

		$save_button = we_html_button::create_button(we_html_button::SAVE, "javascript:save();");
		$preview_button = we_html_button::create_button(we_html_button::PREVIEW, "javascript:preview();");
		$cancel_button = we_html_button::create_button(we_html_button::CLOSE, "javascript:exit_close();");
		$buttons = we_html_button::position_yes_no_cancel($save_button, $preview_button, $cancel_button);

		$sTblWidget = (new we_html_multiIconBox($parts, 'mfdProps'))
				->marginLeft()
				->buttons($buttons)
				->headline(g_l('cockpit', '[unpublished]'))
				->get();

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[unpublished]'), $jsFile .
				we_html_element::jsScript(JS_DIR . 'widgets/upb.js')
				, we_html_element::body(
						['class' => 'weDialogBody', 'onload' => "init();"], we_html_element::form([], $sTblWidget)));
	}

	public function showPreview(): void{
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('initPreview', [
			'id' => we_base_request::string('we_cmd', '', 5),
			'type' => 'upb',
			'tb' => g_l('cockpit', ($this->bTypeDoc && $this->bTypeObj ? '[upb_docs_and_objs]' : ($this->bTypeDoc ? '[upb_docs]' : ($this->bTypeObj ? '[upb_objs]' : '[upb_docs_and_objs]')))),
		]);
		$jsCmd->addCmd('setIconOfDocClass', 'upbIcon');

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[unpublished]'), $jsCmd->getCmds(), we_html_element::body(['style' => 'margin:10px 15px;',
						], we_html_element::div(['id' => "upb"
								], $this->ct)));
	}

}
