/* global WE, weFocusedField, top */

/**
 * webEdition CMS
 *
 * $Rev: 06e5fa498a3b $
 * $Author: Lukas Imhof $
 * $Date: 2021-03-09 10:00 +0100 $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';
var
	vars = WE().util.getDynamicVar(document, 'loadVarLink', 'data-link');
var
	editname = vars.editname ? vars.editname : '',
	classNames = vars.classNames && vars.classNames === 'getFromTiny' ? WE().util.dialogTop(window).opener.weclassNames_tinyMce : vars.classNames;

if (!vars.is_frontend) {
	$(function () {
		document.we_form.onsubmit = function () {
			return false;
		};
	});
}

var weAcCheckLoop = 0;

function closeOnEscape() {
	return true;
}

function applyOnEnter(evt) { // is this obsolete?
	var _elemName = "target";
	if (evt.srcElement !== undefined) { // IE
		_elemName = "srcElement";
	}

	if (!(evt[_elemName].tagName === "SELECT" ||
		(evt[_elemName].tagName === "INPUT" && (evt[_elemName].name === "href_int" || evt[_elemName].name === "href_obj" || evt[_elemName].name === "src_int"))
		)) {
		document.we_form.submit();
		return true;

	}
}

function checkAnchor(el) {
	if (el.value && !new RegExp('#?[a-z]+[a-z0-9_:.-=]*$', 'i').test(el.value)) {
		WE().util.showMessage(WE().consts.g_l.alert.anchor_invalid, WE().consts.message.WE_MESSAGE_ERROR);
		window.setTimeout(function () {
			el.focus();
		}, 10);
		return false;
	}
}

function weCheckAcFields() {
	if (weFocusedField !== undefined) {
		weFocusedField.blur();
	}
	if (document.getElementById("weDialogType").value === WE().consts.linkPrefix.TYPE_INT) {
		window.setTimeout(weDoCheckAcFields, 100);
	} else {
		document.we_form.submit();
	}
}

function changeTypeSelect(s) {
	var elem = document.getElementsByClassName('changetype');
	for (var i = 0; i < elem.length; i++) {
		if (elem[i].classList.contains(s + '_row')) {
			elem[i].classList.remove('hidden');
		} else {
			elem[i].classList.add('hidden');
		}
	}
}

function changeCTypeSelect(s) {
	var elems = document.getElementsByClassName('changecontent');
	for (var i = 0; i < elems.length; i++) {
		if (elems[i].classList.contains(s + '_row')) {
			elems[i].classList.remove('hidden');
		} else {
			elems[i].classList.add('hidden');
		}
	}
}

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
		args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
		url = WE().util.getWe_cmdArgsUrl(args);

	switch (args[0]) {
		case "we_selector_image":
		case "we_selector_document":
			(new (WE().layout.overlay)(caller, 'top', url, "we_docselector", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "selector_callback":
			if (args[1].currentID) {
				caller.document.getElementById(args[2]).disabled = false;
				if (args[2] === 'btn_edit_int') {
					caller.document.we_form.yuiAcResultCT.value = args[1].currentType;
				}
			}
			break;
		default :
			window.we_cmd_dialogBase.apply(caller, Array.prototype.slice.call(arguments));
			//window.opener.parent.we_cmd.apply(caller, Array.prototype.slice.call(arguments)); // from linklist.js
			break;
	}
}

function extHref_doOnchange(input) {
	if (input.value === '' || input.value === WE().consts.linkPrefix.EMPTY_EXT) {
		WE().layout.button.switch_button_state(document, 'btn_edit_ext', 'disabled');
		checkMakeEmptyHrefExt();
	} else {
		WE().layout.button.switch_button_state(document, 'btn_edit_ext', 'enabled');
		var x = input.value.match(/((.*:\/)?\/[^#?]*)(\?([^?#]*))?(#([^?#]*))?/);
		//top.console.log('x', x);
		input.value = x[1];
		if (x[4] !== undefined) {
			document.getElementsByName('we_dialog_args[params]')[0].value = x[4];
		}
		if (x[6] !== undefined) {
			document.getElementsByName('we_dialog_args[anchor]')[0].value = x[6];
		}
		input.value = input.value.replace(WE().consts.linkPrefix.EMPTY_EXT + WE().consts.linkPrefix.EMPTY_EXT, WE().consts.linkPrefix.EMPTY_EXT).
			replace(WE().consts.linkPrefix.EMPTY_EXT + 'https://', 'https://');
	}
}

function checkMakeEmptyHrefExt() {
	var f = document.we_form;
	var hrefField = f.elements['we_dialog_args[extHref]'],
		anchor = f.elements['we_dialog_args[anchor]'].value,
		params = f.elements['we_dialog_args[params]'].value;

	if ((anchor || params) && hrefField.value === WE().consts.linkPrefix.EMPTY_EXT) {
		if (anchor && !new RegExp('^#?[a-z]+[a-z0-9_:.-=]*$', 'i').test(anchor)) {
			WE().util.showMessage(WE().consts.g_l.alert.anchor_invalid, WE().consts.message.WE_MESSAGE_ERROR);
			//this.focus();
		} else {
			hrefField.value = '';
		}

		hrefField.value = '';
	} else if (!(anchor || params) && !hrefField.value) {
		hrefField.value = WE().consts.linkPrefix.EMPTY_EXT;
	}

}

function extHref_doOnFocus(input) {
	input.value = (input.value === '' ? WE().consts.linkPrefix.EMPTY_EXT : input.value);
}

function openToEdit(id, ct, table) {
	if (!id || !ct) {
		return;
	}

	table = table ? table : WE().consts.tables.FILE_TABLE;
	WE().layout.weEditorFrameController.openDocument(table, id, ct);
}

//FIXME: this function is not used anymore, why?
/*function showanchors(name, val, onCh) {
 var pageAnchors = WE().util.dialogTop(window).opener.document.getElementsByTagName("A");
 var objAnchors = WE().util.dialogTop(window).opener["weWysiwygObject_" + editname].eDocument.getElementsByTagName("A");
 var allAnchors = [];
 var i;

 for (i = 0; i < pageAnchors.length; i++) {
 if (!pageAnchors[i].href && pageAnchors[i].name !== "") {
 allAnchors.push(pageAnchors[i].name);
 }
 }

 for (i = 0; i < objAnchors.length; i++) {
 if (!objAnchors[i].href && objAnchors[i].name !== "") {
 allAnchors.push(objAnchors[i].name);
 }
 }
 if (allAnchors.length) {
 document.writeln('<select class="defaultfont" style="width:100px" name="' + name + '" id="' + name + '"' + (onCh ? ' onchange="' + onCh + '"' : '') + '>');
 document.writeln('<option value="">');

 for (i = 0; i < allAnchors.length; i++) {
 document.writeln('<option value="' + allAnchors[i] + '"' + ((val === allAnchors[i]) ? ' selected' : '') + '>' + allAnchors[i]);
 }

 document.writeln('</select>');
 }
 }*/

function weDoCheckAcFields() {
	if (!WE().layout.weSuggest.checkRequired(window)) {
		WE().util.showMessage(WE().consts.g_l.main.save_error_fields_value_not_valid, WE().consts.message.WE_MESSAGE_ERROR, window);
	} else {
		document.we_form.submit();
	}
}