/* global WE, top */

/**
 webEdition CMS
 *
 * $Rev: 353737934962 $
 * $Author: Lukas Imhof $
 * $Date: 2021-04-12 10:20 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

function Fileupload_imageEdit_abstract(uploader) {
	this.uploader = uploader;

	this.imageFilesToProcess = [];
	this.isImageEditActive = false;

	this.PICA_CONFIGURATION = {
		quality: 3, // [0,3]
		unsharpAmount: 0, // [0, 200]
		unsharpRadius: 0.5, // [0.5, 2]
		unsharpThreshold: 0, // [0, 255]
		alpha: true
	};

	this.IMG_NEXT = 0;
	this.IMG_START = 10;
	this.IMG_LOAD_CANVAS = 1;
	this.IMG_FORCE_LOAD_CANVAS = 11;
	this.IMG_EXTRACT_METADATA = 2;
	this.IMG_SCALE = 3;
	this.IMG_ROTATE = 4;
	this.IMG_APPLY_FILTERS = 5;
	this.IMG_WRITE_IMAGE = 6;
	this.IMG_INSERT_METADATA = 7;
	this.IMG_MAKE_PREVIEW = 8;
	this.IMG_POSTPROCESS = 9;

	this.MAXSIZE_AUTO_PREVIEW = 10485760;
	this.OPTS_QUALITY_NEUTRAL_VAL = 90;
	this.PRESERVE_IMG_DATAURL = true;
	this.EDITABLE_CONTENTTYPES = ['image/jpeg', 'image/gif', 'image/png','image/webp'];
	this.PREVIEWABLE_CONTENTTYPES = this.EDITABLE_CONTENTTYPES.concat(['image/svg+xml', 'image/svg-xml']);
	this.IMPORTMETA_CONTENTTYPES = ['image/jpeg', 'image/gif', 'image/png', 'application/pdf'];
	this.MAX_LONGEST = -1;

	this.IS_MEMORY_MANAGMENT = false;
	this.PROCESS_PREVIEWS_ONLY = false;
	//self.MEMORY_LIMIT = 31457280;
	this.MEMORY_LIMIT = 83886080; // 80 MB

	this.imageEditOptions = {
		doEdit: false,
		from: 'general',
		scaleWhat: 'pixel_l',
		scale: 0,
		rotate: 0,
		quality: this.OPTS_QUALITY_NEUTRAL_VAL
	};

	this.currentEdit = {
		mode: 'global',
		index: 0,
		task: 'edit'
	};

	this.processimageRepeatLoadCanvas = 0;
	this.memoryManagement = {
		registeredSum: 0,
		registeredValues: {},
		queueEdited: [],
		queueNotEdited: []
	};

	this.init = function (conf) {
		this.controller = this.uploader.controller; // on init all components are initialized
		this.imageEdit = this.uploader.imageEdit;
		this.sender = this.uploader.sender;
		this.view = this.uploader.view;
		this.utils = this.uploader.utils;
		this.MAX_LONGEST = conf.imageeditMaxLongest ? parseInt(conf.imageeditMaxLongest) : -1;

		this.init_sub(conf);
	};

	this.init_sub = function () {
		// to be overridden
	};

	this.processImages = function () {
		this.PROCESS_PREVIEWS_ONLY = false;
		if (this.imageFilesToProcess && this.imageFilesToProcess.length) {
			this.setImageEditOptionsGeneral();
			this.view.setImageEditMessage();
			this.processNextImage();
		} else {
			this.view.unsetImageEditMessage();
		}
	};

	this.processNextImage = function () {
		if (this.imageFilesToProcess.length) {
			var fileobj = this.imageFilesToProcess.shift();
			this.utils.logTimeFromStart('start edit image', true, fileobj);
			this.setImageEditOptionsFile(fileobj);
			this.processImage(fileobj, this.IMG_START);
		} else {
			this.processImages();
		}
	};

	this.processSingleImage = function (fileobj, finishProcess) {
		this.PROCESS_PREVIEWS_ONLY = false;
		if (finishProcess) {
			fileobj.isProcessSingleImage = false;
			this.view.unsetImageEditMessage();
			switch(fileobj.img.callback){
				case 'prepareImageAndSend':
					window.setTimeout(this.sender.prepareImageAndSend.bind(this.sender, fileobj), 100);
					break;
				case 'sendNextFile':
					if(this.uploader.fileuploadType === 'editFull'){
						this.sender.prepareUpload();
					}
					window.setTimeout(this.sender.sendNextFile.bind(this.sender), 100);
					break;
			}
			return;
		}

		if (fileobj) {
			fileobj.isProcessSingleImage = true;
			if (fileobj.img.callback === 'prepareImageAndSend') {
				this.view.setShowElemProcess(fileobj, true);
			} else {
				this.view.setImageEditMessage();
			}
			this.utils.logTimeFromStart('start edit image', true);
			this.setImageEditOptionsFile(fileobj);
			this.processImage(fileobj, this.IMG_START);
		}
	};

	/*
	 * reedit a single image or all images using general options applying opts from GUI
	 *
	 */
	this.reeditImage = function (index, general) {
		var indices = this.getImageEditIndices(index, general, false);
		for (var i = 0; i < indices.length; i++) {
			this.imageFilesToProcess.push(this.sender.preparedFiles[indices[i]]);
		}
		this.processImages();
	};

	/*
	 * set property isEdited of a single image or all (= general) to false
	 * -> empty props like dataArray, dataUrl
	 * -> do not change GUI (= edit opts) // Why?
	 *
	 * we use this to free disk space when edited images are not valid anymore after changing options
	 *
	 */
	this.uneditImage = function (index, general) {
		var indices = this.getImageEditIndices(index, general, false);
		var fileobj;

		for (var i = 0; i < indices.length; i++) {
			fileobj = this.sender.preparedFiles[indices[i]];
			fileobj.dataArray = null;
			fileobj.dataUrl = null;
			fileobj.size = fileobj.img.originalSize;
			fileobj.img.previewImg = null;
			fileobj.img.fullPrev = null;
			fileobj.img.actualRotation = 0;
			this.setImageEditOptionsFile(fileobj); // write actually valid editoptions

			fileobj.img.processedOptions = {// reset last edited options
				doEdit: false,
				from: 'general',
				scaleWhat: 'pixel_l',
				scale: 0,
				rotate: 0,
				quality: this.OPTS_QUALITY_NEUTRAL_VAL
			};
			fileobj.isEdited = false;
			this.memorymanagerRegister(fileobj);
		}


	};

	this.openImageEditor = function (pos) {
		// to be overridden
	};

	this.processImage = function (fileobj, task) {
		if (!fileobj) {
			this.processNextImage();
			return;
		}

		switch (task) {
			case this.IMG_START:
				if (!fileobj.isEditable) {
					this.processimageMakePreviewNoneditable(fileobj, this.IMG_NEXT);
					return;
				}
				if (this.PROCESS_PREVIEWS_ONLY && fileobj.img.previewCanvas) {
					this.processImage(fileobj, this.IMG_NEXT);
					return;
				}
				this.processimageExtractLandscape(fileobj, this.IMG_LOAD_CANVAS);
				break;
			case this.IMG_LOAD_CANVAS: // TODO: make IMG_START
				if (!fileobj.img.editOptions.doEdit && !fileobj.img.landscape.correction && fileobj.size > this.MAXSIZE_AUTO_PREVIEW) { // nothing to edit and image is too big for preview (>10MB)
					this.view.disableBtnsRotate(fileobj, true);
					this.processImage(fileobj, this.IMG_NEXT);
					return;
				} else {
					this.view.disableBtnsRotate(fileobj, false);
				}
				/* fall through */
			case this.IMG_FORCE_LOAD_CANVAS: // TODO: make IMG_START
				var loadCanvasToGetDimensions = this.imageEditOptions.doEdit && (!fileobj.img.origWidth || !fileobj.img.origHeight); // load new images with general edit opts set!
				this.processimageLoadCanvas(fileobj, (fileobj.img.editOptions.doEdit || fileobj.img.landscape.correction || loadCanvasToGetDimensions) ? this.IMG_EXTRACT_METADATA : this.IMG_MAKE_PREVIEW);
				break;
			case this.IMG_EXTRACT_METADATA:
				this.processimageExtractMetadata(fileobj, this.IMG_SCALE);
				break;
			case this.IMG_SCALE:
				this.processimageScale(fileobj, this.IMG_ROTATE);
				break;
			case this.IMG_ROTATE:
				this.processimageRotate(fileobj, this.IMG_APPLY_FILTERS);
				break;
			case this.IMG_APPLY_FILTERS:
				this.imageEdit.processimageApplyFilters(fileobj, this.IMG_WRITE_IMAGE);
				break;
			case this.IMG_WRITE_IMAGE:
				// skip re-insert meta to be sure, we do not insert twice
				//this.processimageWriteImage(fileobj, this.IMG_INSERT_METADATA);
				this.processimageWriteImage(fileobj, this.IMG_MAKE_PREVIEW);
				break;
			case this.IMG_INSERT_METADATA:
				this.processimagInsertMetadata(fileobj, this.IMG_MAKE_PREVIEW);
				break;
			case this.IMG_MAKE_PREVIEW:
				this.processimageMakePreview(fileobj, this.IMG_POSTPROCESS);
				break;
			case this.IMG_POSTPROCESS:
				this.processimagePostProcess(fileobj, this.IMG_NEXT);
				break;
			case this.IMG_NEXT:
			/*falls through*/
			default:
				this.processNextImage();
		}
	};

	this.resetImageEdit = function (fileobj) {
		fileobj.dataArray = null;
		fileobj.dataUrl = null;
		if (this.IS_MEMORY_MANAGMENT) {
			this.memorymanagerRegister(fileobj);
		}
	};

	this.abstractSetImageEditOptionsGeneral = function () {
		var form = this.uploader.doc.forms.we_form;
		var scale = form.elements.fuOpts_scale.value;
		var quality = parseInt(form.elements.fuOpts_quality.value);
		var opts = this.imageEditOptions;

		scale = parseInt(scale ? scale : 0);

		if (parseInt(form.elements.fuOpts_doEdit.value) === 1 && (scale || quality !== this.OPTS_QUALITY_NEUTRAL_VAL)) {
			opts.doEdit = true;
			opts.scaleWhat = form.elements.fuOpts_scaleWhat.value;
			opts.scale = scale;
			opts.rotate = 0;
			opts.quality = quality;
		} else {
			opts.doEdit = false;
			opts.scaleWhat = 'pixel_l';
			opts.scale = 0;
			opts.rotate = 0;
			opts.quality = this.OPTS_QUALITY_NEUTRAL_VAL;
		}
	};

	this.setImageEditOptionsGeneral = function () {
		this.abstractSetImageEditOptionsGeneral();
	};

	this.setImageEditOptionsFile = function (fileobj) {
		// read out global editOptiona and init filobj editOptions by it
		this.setImageEditOptionsGeneral();
		fileobj.img.editOptions = JSON.parse(JSON.stringify(this.imageEditOptions));

		// reset quality to neutral for all images but jpeg
		fileobj.img.editOptions.quality = fileobj.type === 'image/jpeg' ? fileobj.img.editOptions.quality : this.OPTS_QUALITY_NEUTRAL_VAL;

		// read out rotation from item
		fileobj.img.editOptions.rotate = fileobj.entry ? parseInt(fileobj.entry.getElementsByClassName('fuOpts_rotate')[0].value) : 0;

		// change scaleWhat if rotation = 90 or 270
		if (fileobj.img.editOptions.rotate % 180 === 90 && fileobj.img.editOptions.scaleWhat !== 'pixel_l') {
			fileobj.img.editOptions.scaleWhat = fileobj.img.editOptions.scaleWhat === 'pixel_w' ? 'pixel_h' : 'pixel_w';
		}

		// check for tooSmallToScale
		var scaleReference = fileobj.img.editOptions.scaleWhat === 'pixel_w' ? fileobj.img.origWidth : (
						fileobj.img.editOptions.scaleWhat === 'pixel_h' ? fileobj.img.origHeight : Math.max(fileobj.img.origHeight, fileobj.img.origWidth));

		if (scaleReference < fileobj.img.editOptions.scale) {
			fileobj.img.editOptions.scale = '';
			fileobj.img.tooSmallToScale = true;
		} else {
			fileobj.img.tooSmallToScale = false;
		}
		fileobj.img.editOptions.doEdit = (parseInt(fileobj.img.editOptions.scale) !== 0) || (parseInt(fileobj.img.editOptions.rotate) !== 0) || (parseInt(fileobj.img.editOptions.quality) !== this.OPTS_QUALITY_NEUTRAL_VAL);
	};

	this.processimageExtractLandscape = function (fileobj, nexttask, isReaderOnload, e) {
		var task = isReaderOnload ? 'readerOnload' : 'main';

		// FIXME: REDESIGN LANDCSAPE CORRECTION
		// temporarily skip this task
		/*
			fileobj.img.landscape.checked = true;
			fileobj.img.landscape.correction = 0;
			this.processImage(fileobj, nexttask);
		*/

		switch (task) {
			case 'main':
				if (fileobj.type === 'image/jpeg') {
					if(!fileobj.img.landscape.checked){
						fileobj.img.landscape.checked = true;
						var reader = new FileReader();
						reader.onloadend = this.processimageExtractLandscape.bind(this, fileobj, nexttask, true); // the function calls itself to process reader onload
						reader.readAsArrayBuffer(fileobj.file.slice(0, 128 * 1024));
					} else {
						this.processImage(fileobj, nexttask);
					}
				} else {
					fileobj.img.landscape.checked = true;
					fileobj.img.landscape.correction = 0;
					this.processImage(fileobj, nexttask);
				}
				break;
			case 'readerOnload':
				try {
					var exif = new this.uploader.win.ExifReader();
					exif.load(e.target.result);
					var tags = exif.getAllTags();

					if (tags.Orientation) {
						switch (tags.Orientation.value) {
							case 3:
								fileobj.img.landscape.correction = 180;
								break;
							case 6:
								fileobj.img.landscape.correction = 90;
								break;
							case 8:
								fileobj.img.landscape.correction = -90;
								break;
						}
					}
				} catch (error) {
				}

				this.processImage(fileobj, nexttask);
				break;
		}
	};

	this.processimageLoadCanvas = function (fileobj, nexttask, isReaderOnload, isImageOnload, e) {
		var task = isReaderOnload ? 'readerOnload' : (isImageOnload ? 'imageOnload' : 'main');

		switch (task) {
			case 'main':
				var reader = new FileReader();
				reader.onload = this.processimageLoadCanvas.bind(this, fileobj, nexttask, true, false);
				reader.readAsDataURL(fileobj.file);
				break;
			case 'readerOnload':
				if (!fileobj.img.editOptions.doEdit) { // we will not edit and rewrite image so we hold original dataUrl in fileobj to be used by fullpreview
					fileobj.dataUrl = e.target.result;

					if (fileobj.img.previewCanvas) { // preview done during earlier editing: directly jump top postprocess
						this.processimagePostProcess(fileobj, this.IMG_NEXT);
						return;
					}
				}
				this.view.repaintImageEditMessage();
				fileobj.img.image = new Image();
				fileobj.img.image.onload = this.processimageLoadCanvas.bind(this, fileobj, nexttask, false, true);
				fileobj.img.image.src = e.target.result;
				break;
			case 'imageOnload':
				fileobj.img.workingCanvas = this.uploader.doc.createElement('canvas');
				if (!fileobj.img.image && this.processimageRepeatLoadCanvas < 5) {
					this.processimageRepeatLoadCanvas++;
					this.processimageLoadCanvas(fileobj, nexttask);
				}
				this.processimageRepeatLoadCanvas = 0;
				fileobj.img.workingCanvas.width = fileobj.img.image.width;
				fileobj.img.workingCanvas.height = fileobj.img.image.height;
				if (!fileobj.img.origWidth || !fileobj.img.origHeight) {
					fileobj.img.origWidth = fileobj.img.image.width;
					fileobj.img.origHeight = fileobj.img.image.height;
					this.setImageEditOptionsFile(fileobj); // set editOptions again after orig dimensions are extracted
				}
				fileobj.img.workingCanvas.getContext("2d").drawImage(fileobj.img.image, 0, 0);
				fileobj.img.image = null;
				this.utils.logTimeFromStart('canvas loaded');
				nexttask = this.PROCESS_PREVIEWS_ONLY ? this.IMG_MAKE_PREVIEW : nexttask;

				// abort imageEdit steps when too small to scale and no other edit options active
				// FIXME: if all works correct, condition !fileobj.img.editOptions.doEdit should suffice
				if (!fileobj.img.landscape.correction && !this.imageEditOptions.rotate &&
								this.imageEditOptions.quality === this.OPTS_QUALITY_NEUTRAL_VAL &&
								(!this.imageEditOptions.scale || fileobj.img.tooSmallToScale) &&
								!fileobj.img.editOptions.doEdit &&
								nexttask === this.IMG_EXTRACT_METADATA
								) {
					nexttask = this.IMG_MAKE_PREVIEW;
				}

				this.processImage(fileobj, nexttask);
				break;
		}
	};

	this.processimageExtractMetadata = function (fileobj, nexttask) {
		switch (fileobj.type) {
			case 'image/jpeg':
				if (fileobj.img.jpgCustomSegments && fileobj.img.jpgCustomSegments.length) {
					this.processImage(fileobj, nexttask);
					return;
				}
				this.processimageExtractMetadataJPG(fileobj, nexttask);
				break;
			case 'image/png':
				this.processimageExtractMetadataPNG(fileobj, nexttask);
				break;
			default:
				this.processImage(fileobj, nexttask);
		}
	};

	this.processimageExtractMetadataJPG = function (fileobj, nexttask, complete, isReaderOnload, e) {
		var task = isReaderOnload ? 'readerOnload' : 'main';

		switch (task) {
			case 'main':
				var reader = new FileReader();
				reader.onloadend = this.processimageExtractMetadataJPG.bind(this, fileobj, nexttask, complete, true);
				reader.readAsArrayBuffer((complete ? fileobj.file : fileobj.file.slice(0, 128 * 1024)));
				break;
			case 'readerOnload':
				var extract;
				this.utils.logTimeFromStart('file read to extract meta');

				extract = this.jpgGetSegmentsIfExist(new Uint8Array(e.target.result), [225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239], false);
				if(!extract.success){
					if(!fileobj.img.extractMetaSecondTry) {
						// 128 KB was not enough to reach SOS: we try again reading the whole file
						fileobj.img.extractMetaSecondTry = true;
						this.processimageExtractMetadataJPG(fileobj, nexttask, true);
						return;
					}
				} else {
					fileobj.img.jpgCustomSegments = extract.segments;
					this.utils.logTimeFromStart('meta jpg extracted');
					this.view.repaintImageEditMessage(true);
				}
				if (typeof nexttask !== 'undefined') {
					this.processImage(fileobj, nexttask);
				}
				break;
		}
	};

	this.processimageExtractMetadataPNG = function (fileobj, nexttask, isReaderOnload, e) {
		var task = isReaderOnload ? 'readerOnload' : 'main';

		switch (task) {
			case 'main':
				var reader = new FileReader();
				reader.onloadend = this.processimageExtractMetadataPNG.bind(this, fileobj, nexttask, true);
				reader.readAsArrayBuffer(fileobj.file);
				break;
			case 'readerOnload':
				fileobj.img.pngTextChunks = [];
				try {
					var chunks = this.uploader.win.extractChunks(new Uint8Array(e.target.result));
					for (var i = 0; i < chunks.length; i++) {
						if (chunks[i].name === 'iTXt' || chunks[i].name === 'tEXt' || chunks[i].name === 'zTXt') {
							fileobj.img.pngTextChunks.push(chunks[i]);
							/*
							 var decodedChunk = chunks[i].name !== 'iTXt' ? WE().layout.fileupload.decodeChunk(chunks[i]) :
							 decodeURIComponent(String.fromCharCode.apply(null, chunks[i].data));
							 */
						}
					}
				} catch (e) {
					WE().t_e('extracting png metadata failed');
				}

				this.utils.logTimeFromStart('meta png exttracted');
				this.view.repaintImageEditMessage(true);
				this.processImage(fileobj, nexttask);
				break;
		}
	};

	this.processimageScale = function (fileobj, nexttask, isPicaCallback, targetCanvas, err) {
		var task = isPicaCallback ? 'picaCallback' : 'main';

		switch (task) {
			case 'main':
				if (!fileobj.img.editOptions.scale) {
					this.utils.logTimeFromStart('scaling skipped');
					this.processImage(fileobj, nexttask);
					return; // IMPORTANT!
				}

				var scaleWhat = fileobj.img.editOptions.scaleWhat !== 'pixel_l' ? fileobj.img.editOptions.scaleWhat :
								(fileobj.img.workingCanvas.width >= fileobj.img.workingCanvas.height ? 'pixel_w' : 'pixel_h');
				var ratio = (scaleWhat === 'pixel_w' ? fileobj.img.editOptions.scale / fileobj.img.workingCanvas.width :
								fileobj.img.editOptions.scale / fileobj.img.workingCanvas.height);

				if(this.MAX_LONGEST !== -1 && fileobj.img.editOptions.scaleWhat !== 'pixel_l'){
					var newScaleWhat = scaleWhat;
					switch(scaleWhat){
						case 'pixel_w':
							newScaleWhat = fileobj.img.workingCanvas.height * ratio > this.MAX_LONGEST ? 'pixel_h' : scaleWhat;
							break;
						case 'pixel_h':
							newScaleWhat = fileobj.img.workingCanvas.width * ratio > this.MAX_LONGEST ? 'pixel_w' : scaleWhat;
							break;
					}
					if(newScaleWhat !== scaleWhat){
						scaleWhat = newScaleWhat;
						ratio = (scaleWhat === 'pixel_w' ? fileobj.img.editOptions.scale / fileobj.img.workingCanvas.width :
							fileobj.img.editOptions.scale / fileobj.img.workingCanvas.height);
						fileobj.img.editOptions.forceMaxLongest = true;
					} else {
						fileobj.img.editOptions.forceMaxLongest = false;
					}
				}

				if (ratio >= 1) {
					this.utils.logTimeFromStart('scaling: image smaller than targetsize');
					this.processImage(fileobj, nexttask); // we do not upscale!
					return; // IMPORTANT!
				}

				targetCanvas = this.uploader.doc.createElement('canvas');
				targetCanvas.width = fileobj.img.workingCanvas.width * ratio;
				targetCanvas.height = fileobj.img.workingCanvas.height * ratio;

				//this.uploader.win.pica.prototype.debug = console.log.bind(console);
				var opts = ['js', 'ww']; // ['js', 'wasm', 'cib', 'ww']
				var resizer = this.uploader.win.pica({features: opts});
				var imageEdit = this;
				try {
					resizer.resize(fileobj.img.workingCanvas, targetCanvas, this.PICA_CONFIGURATION).then(function () {
						imageEdit.processimageScale(fileobj, nexttask, true, targetCanvas);
					}).catch(function (err) {
						imageEdit.processimageScale(fileobj, nexttask, true, targetCanvas, err);
					});
				} catch (err) {
					this.processimageScale(fileobj, nexttask, true, targetCanvas, err);
				}
				break;
			case 'picaCallback':
				if (err) {
					WE().t_e('scaling image: pica returned error');
					fileobj.img.editedWidth = 'n.n.';
					fileobj.img.editedHeight = 'n.n.';
				} else {
					fileobj.img.workingCanvas = targetCanvas;
					fileobj.img.editedWidth = targetCanvas.width;
					fileobj.img.editedHeight = targetCanvas.height;
					targetCanvas = null;
				}

				this.utils.logTimeFromStart('scaling done');
				fileobj.isEdited = true;

				this.view.repaintImageEditMessage(true);
				this.processImage(fileobj, nexttask);
				break;
		}
	};

	this.processimageRotate = function (fileobj, nexttask, preview, degrees, correctPreviewOrientation) {
		var deg = (preview && degrees !== undefined) ? degrees : fileobj.img.editOptions.rotate;
		var target = preview ? 'previewCanvas' : 'workingCanvas';

		/*
		 * IMPORTANT: NEVER automatically correct orientation of the preview!
		 * => we correct preview when creating it and then take it for not rotated!!
		 */
		if (target === 'workingCanvas') {
			// FIXME: REDESIGN LANDCSAPE CORRECTION
			deg = (deg/* + fileobj.img.landscape.correction*/) % 360;
		}

		// FIXME: REDESIGN LANDCSAPE CORRECTION
		/*
		 if (correctPreviewOrientation) {
		 deg = -fileobj.img.landscape.correction;
		 target = 'previewCanvas';
		 nexttask = undefined;
		 preview = true;
		 }
		 */

		var cw = fileobj.img[target].width, ch = fileobj.img[target].height, cx = 0, cy = 0;
		switch (deg) {
			case 90 :
			case - 270 :
				cw = fileobj.img[target].height;
				ch = fileobj.img[target].width;
				cy = -fileobj.img[target].height;
				break;
			case 180 :
			case - 180 :
				cx = -fileobj.img[target].width;
				cy = -fileobj.img[target].height;
				break;
			case 270 :
			case - 90 :
				cw = fileobj.img[target].height;
				ch = fileobj.img[target].width;
				cx = -fileobj.img[target].width;
				break;
			default:
				this.utils.logTimeFromStart('rotation skipped');
				this.processImage(fileobj, nexttask);
				return;
		}

		var targetCanvas = this.uploader.doc.createElement('canvas');
		var ctxTargetCanvas = targetCanvas.getContext("2d");
		targetCanvas.width = cw;
		targetCanvas.height = ch;

		ctxTargetCanvas.rotate(deg * Math.PI / 180);
		ctxTargetCanvas.drawImage(fileobj.img[target], cx, cy);

		if (!preview) {
			fileobj.img.editedWidth = targetCanvas.width;
			fileobj.img.editedHeight = targetCanvas.height;
		}

		fileobj.img[target] = targetCanvas;
		targetCanvas = null;

		if (correctPreviewOrientation) {
			return;
		}

		if (target === 'workingCanvas') {
			fileobj.isEdited = true;
			fileobj.img.actualRotation = deg;
			this.utils.logTimeFromStart('rotation done');
			this.view.repaintImageEditMessage(true);
			this.processImage(fileobj, nexttask);
		}
	};

	this.processimageApplyFilters = function (fileobj, nexttask) {
		this.processImage(fileobj, nexttask);
	};

	this.processimageWriteImage_2 = function (fileobj, nexttask, isReaderOnload, e) {
		var task = isReaderOnload ? 'readerOnload' : 'main';

		switch (task) {
			case 'main':
				fileobj.img.workingCanvas.toBlob(function (blob) { // DO WE NEED toBlob TO GET UINT8ARRAY? => THIS FN CAUASES PROBLEMS WITH PNG!
					var reader = new FileReader();
					reader.onload = this.processimageWriteImage_2.bind(this, fileobj, nexttask, true);
					reader.readAsArrayBuffer(blob);
				}, fileobj.type, (fileobj.img.editOptions.quality / 100));
				break;
			case 'readerOnload':
				fileobj.dataArray = new Uint8Array(e.target.result);
				this.utils.logTimeFromStart('image written');
				this.processImage(fileobj, nexttask);
				break;
		}
	};

	this.processimageWriteImage = function (fileobj, nexttask) {
		/*
		 if(fileobj.type !== 'image/png'){
		 self.processimageWriteImage_2(fileobj, nexttask);
		 }
		 */
		fileobj.dataUrl = fileobj.externallyEditedData ? fileobj.externallyEditedData : fileobj.img.workingCanvas.toDataURL(fileobj.type, (fileobj.img.editOptions.quality / 90));
		fileobj.externallyEditedData = '';
		fileobj.dataArray = this.utils.dataURLToUInt8Array(fileobj.dataUrl);

		if (!this.PRESERVE_IMG_DATAURL) {
			fileobj.dataUrl = null;
		}
		fileobj.isEdited = fileobj.img.editOptions.quality < this.OPTS_QUALITY_NEUTRAL_VAL ? true : fileobj.isEdited;
		fileobj.isLandscapeCorrected = fileobj.img.landscape.correction;

		this.utils.logTimeFromStart('image written fn 2');
		this.processImage(fileobj, nexttask);
	};

	this.processimagInsertMetadata = function (fileobj, nexttask) {
		switch (fileobj.type) {
			case 'image/jpeg':
				this.processimagInsertMetadataJPG(fileobj, nexttask);
				break;
			case 'image/png':
				this.processimagInsertMetadataPNG(fileobj, nexttask);
				break;
			default:
				this.utils.logTimeFromStart('no metadata to reinsert');
				this.processImage(fileobj, nexttask);
		}
	};

	this.processimagInsertMetadataJPG = function (fileobj, nexttask) {
		if (fileobj.img.jpgCustomSegments && fileobj.img.jpgCustomSegments.length) {
			fileobj.dataArray = this.jpgInsertSegments(fileobj.dataArray, null, null, fileobj.img.jpgCustomSegments, true);
			fileobj.size = fileobj.dataArray.length;
			this.view.repaintImageEditMessage(true);
		}

		this.utils.logTimeFromStart('metadata reinserted');
		this.processImage(fileobj, nexttask);
	};

	this.processimagInsertMetadataPNG = function (fileobj, nexttask) {
		this.utils.logTimeFromStart('metadata reinsert skipped');
		//self.processImage(fileobj, nexttask);

		if (fileobj.img.pngTextChunks && fileobj.img.pngTextChunks.length) {
			fileobj.dataArray = this.pngReinsertTextchunks(fileobj.dataArray, fileobj.img.pngTextChunks);

			/*
			 var combinedChuks = [];
			 try{
			 var chunks = this.uploader.win.extractChunks(fileobj.dataArray),
			 combinedChuks = [];

			 combinedChuks.push(chunks.shift()); // new IHDR
			 while(fileobj.pngTextChunks.length){
			 combinedChuks.push(fileobj.pngTextChunks.shift()); // all extracted text chunks
			 }
			 while(chunks.length){
			 combinedChuks.push(chunks.shift()); // all extracted text chunks
			 }
			 } catch(e){
			 combinedChuks = false;
			 }

			 var newUInt8Array = false;
			 if(combinedChuks){
			 try{
			 newUInt8Array = WE().layout.fileupload.encodeChunks(combinedChuks);
			 } catch (e) {
			 newUInt8Array = false;
			 }
			 }

			 fileobj.dataArray = newUInt8Array ? newUInt8Array : fileobj.dataArray;
			 */

			this.view.repaintImageEditMessage(true);
		}

		this.utils.logTimeFromStart('metadata reinserted');
		this.processImage(fileobj, nexttask);
	};

	this.processimageMakePreview = function (fileobj, nexttask) {
		if (fileobj && fileobj.img.workingCanvas) {
			if (fileobj.img.previewCanvas) {
				this.processimageRotatePreview(fileobj, -1, nexttask);
				return;
			}

			var tmpCanvas = this.uploader.doc.createElement("canvas"),
							ctxTmpCanvas,
							previewCanvas = this.uploader.doc.createElement("canvas"),
							clone = null,
							prevWidth, prevHeight;

			if (fileobj.img.workingCanvas.width < this.view.previewSize && fileobj.img.workingCanvas.height < this.view.previewSize) {
				prevHeight = fileobj.img.workingCanvas.width;
				prevWidth = fileobj.img.workingCanvas.height;
			} else {
				if (fileobj.img.workingCanvas.width > fileobj.img.workingCanvas.height) {
					prevWidth = this.view.previewSize;
					prevHeight = Math.round(this.view.previewSize / fileobj.img.workingCanvas.width * fileobj.img.workingCanvas.height);

				} else {
					prevHeight = this.view.previewSize;
					prevWidth = Math.round(this.view.previewSize / fileobj.img.workingCanvas.height * fileobj.img.workingCanvas.width);
				}
			}
			tmpCanvas.width = fileobj.img.workingCanvas.width;
			tmpCanvas.height = fileobj.img.workingCanvas.height;
			ctxTmpCanvas = tmpCanvas.getContext("2d");
			ctxTmpCanvas.drawImage(fileobj.img.workingCanvas, 0, 0, tmpCanvas.width, tmpCanvas.height);

			while ((tmpCanvas.width / 2) > prevWidth) {
				clone = tmpCanvas.cloneNode(true);
				clone.getContext('2d').drawImage(tmpCanvas, 0, 0);
				tmpCanvas.width = clone.width / 2;
				tmpCanvas.height = clone.height / 2;
				ctxTmpCanvas.drawImage(clone, 0, 0, clone.width / 2, clone.height / 2);
			}

			previewCanvas.width = prevWidth;
			previewCanvas.height = prevHeight;
			previewCanvas.getContext('2d').drawImage(tmpCanvas, 0, 0, previewCanvas.width, previewCanvas.height);

			fileobj.img.previewCanvas = previewCanvas;
			fileobj.img.previewRotation = fileobj.img.actualRotation;

			// FIXME: REDESIGN LANDCSAPE CORRECTION
			if (fileobj.img.landscape.correction) {
				//this.processimageRotate(fileobj, -1, true, 0, true); //
			}

			tmpCanvas = ctxTmpCanvas = previewCanvas = clone = null;

			this.utils.logTimeFromStart('preview done');
		}

		// FIXME: REDESIGN LANDCSAPE CORRECTION
		/*
		if (fileobj.img.landscape.correction) {
			fileobj.img.orientationIsProposition = true;
			fileobj.img.landscape.correction = 0;
			this.controller.itemEditOptionsOnChange({name: (fileobj.img.landscape.correction === 90 ? 'fuOpts_addRotationRight' : 'fuOpts_addRotationLeft')}, 0, true); // IMI
		}
		*/

		this.processImage(fileobj, nexttask);

	};

	this.processimageMakePreviewNoneditable = function (fileobj, nexttask, isReaderOnload, e) {
		var task = isReaderOnload ? 'readerOnload' : 'main';
		switch (task) {
			case 'main':
				if (fileobj.isPreviewable) {
					var reader = new FileReader();
						reader.onloadend = this.processimageMakePreviewNoneditable.bind(this, fileobj, nexttask, true);
						reader.readAsDataURL(fileobj.file);
				} else {
					this.processImage(fileobj, nexttask);
				}
				break;
			case 'readerOnload':
				try {
					if((fileobj.dataUrl = e.target.result)){
						this.view.repaintItem(fileobj);
					}
				} catch (error) {}
				this.processImage(fileobj, nexttask);
				break;
		}

		this.processImage(fileobj, nexttask);
	};

	this.processimageRotatePreview = function (fileobj, deg, nexttask) {
		if (fileobj && fileobj.img.previewCanvas) {
			var targetRotation = deg === -1 ? fileobj.img.actualRotation : deg;
			var realRotation = (targetRotation - fileobj.img.previewRotation + 360) % 360;

			this.processimageRotate(fileobj, -1, true, realRotation);
			this.view.replacePreviewCanvas(fileobj);
			fileobj.img.previewRotation = targetRotation;
		}

		if (!nexttask) {
			return;
		}
		this.processImage(fileobj, nexttask);
	};

	this.processimagePostProcess = function (fileobj, nexttask) {
		fileobj.img.originalSize = fileobj.file.size;

		if (fileobj.dataArray && (fileobj.isEdited || fileobj.isLandscapeCorrected)) {
			fileobj.size = fileobj.dataArray.length;
		}

		fileobj.dataArray = null; // we recompute it from dataUrl and metas while uploading
		// WHY DO WE EMPTY dataArray JUST TO RECOMPUTE IT BEFORE UPLOADING THE IMAGE ???

		if (false && this.PROCESS_PREVIEWS_ONLY) {
			fileobj.dataUrl = null;
		}

		fileobj.img.processedWidth = 0;//fileobj.img.fullPrev.width;
		fileobj.img.processedHeight = 0; //fileobj.img.fullPrev.height;

		fileobj.img.workingCanvas = null;
		fileobj.img.fullPrev = null;

		fileobj.img.processedOptions = JSON.parse(JSON.stringify(fileobj.img.editOptions));

		// FIXME: should be obsolete
		fileobj.totalParts = Math.ceil(fileobj.size / this.sender.chunkSize);
		fileobj.lastChunkSize = fileobj.size % this.sender.chunkSize;
		//in case the last segment is "0", the real size is chunkSize, since we don't have a segment containing the remaining contents
		fileobj.lastChunkSize = fileobj.lastChunkSize ? fileobj.lastChunkSize : this.sender.chunkSize;

		this.sender.preparedFiles[fileobj.preparedFilesIndex] = fileobj;
		this.view.repaintItem(fileobj);
		this.view.repaintImageEditMessage();
		this.utils.logTimeFromStart('processing finished', false, fileobj);

		if (false && this.IS_MEMORY_MANAGMENT) {
			this.memorymanagerRegister(fileobj);
			if (this.memorymanagerIsOverflow()) {
				this.memorymanagerEmptySpace();
				//win.console.log('emptied space for this file');

				if (this.imageFilesToProcess.length) { //
					if (fileobj.img.editOptions.doEdit && this.memorymanagerIsUneditedPreviewToDelete) {
						//win.console.log('dataURLs of unedited where deleted: go on');
					} else {
						//win.console.log('processing images must be stopped: we will make previews if needed and then stop');
						this.PROCESS_PREVIEWS_ONLY = true;
					}
				}
			}
		}

		if (fileobj.isProcessSingleImage) {
			this.processSingleImage(fileobj, true);
		} else {
			this.processImage(fileobj, nexttask);
		}
	};

	this.memorymanagerRegister = function (fileobj) {

		/*var m = this.memoryManagement;
		 this.memorymanagerUnregister(fileobj);

		 if (fileobj.dataArray || fileobj.dataUrl) {
		 var size = parseInt((fileobj.dataArray ? fileobj.dataArray.length : 0)) + parseInt((fileobj.dataUrl ? fileobj.dataUrl.length : 0));

		 m[fileobj.isEdited ? 'queueEdited' : 'queueNotEdited'].push(fileobj.index);
		 m.registeredValues['o_' + fileobj.index] = size;
		 m.registeredSum += size;
		 }*/
		//win.console.log('register', m, sender.preparedFiles);
	};

	this.memorymanagerUnregister = function (fileobj) {
		/*var m = this.memoryManagement, i;

		 if (m.registeredValues['o_' + fileobj.index]) {
		 m.registeredSum -= m.registeredValues['o_' + fileobj.index];
		 m.registeredValues['o_' + fileobj.index] = 0;
		 }

		 if ((i = m.queueEdited.indexOf(fileobj.index)) !== -1) {
		 m.queueEdited.splice(i, 1);
		 }
		 if ((i = m.queueNotEdited.indexOf(fileobj.index)) !== -1) {
		 m.queueNotEdited.splice(i, 1);
		 }*/
		//win.console.log('unregister', m, sender.preparedFiles);
	};

	this.memorymanagerReregisterAll = function () {
		this.memorymanagerReset();

		for (var i = 0; i < this.sender.preparedFiles.length; i++) {
			this.memorymanagerRegister(this.sender.preparedFiles[i]);
		}
		//win.console.log('register all', self.memoryManagement, sender.preparedFiles);
	};

	this.memorymanagerReset = function () {
		this.memoryManagement = {
			registeredSum: 0,
			registeredValues: {},
			queueEdited: [],
			queueNotEdited: []
		};
	};

	this.memorymanagerIsOverflow = function () {
		return this.memoryManagement.registeredSum > this.MEMORY_LIMIT;
	};

	this.memorymanagerIsUneditedPreviewToDelete = function () {
		return this.memoryManagement.queueNotEdited.length > 0;
	};

	this.memorymanagerEmptySpace = function () {
		var m = this.memoryManagement,
						fileobj, index;

		while ((m.queueEdited.length || m.queueNotEdited.length) && m.registeredSum > this.MEMORY_LIMIT) {
			index = m.queueNotEdited.length ? m.queueNotEdited.shift() : m.queueEdited.shift();
			fileobj = this.sender.preparedFiles[index];

			this.uneditImage(fileobj.index);
			this.view.setStateItemEdit('', fileobj.index);
		}
	};

	this.jpgInsertSegments = function (dataArrayUint8, headArrayUint8, contentArrayUint8, exifSegments, setOrientationToStandard) {
		var fromCanvas = dataArrayUint8 && dataArrayUint8.byteLength > 4 && dataArrayUint8[0] === 255 && dataArrayUint8[1] === 216 &&
					dataArrayUint8[2] === 255 && dataArrayUint8[3] === 224;

		if (fromCanvas || (headArrayUint8 && contentArrayUint8)) {
			var exif;
			if(setOrientationToStandard){
				for(var i = 0; i < exifSegments.length; i++){
					exifSegments[i] = this.jpgSetOrientationToStandard(exifSegments[i], true);
				}
			}
			exif = this.utils.concatUInt8Array(exifSegments);

			if(fromCanvas){
				var pos = 0;
				if (typeof dataArrayUint8.indexOf !== 'function') { // IE11
					for (var i = 4; i < dataArrayUint8.length; i++) {
						if (dataArrayUint8[i] === 255) {
							pos = i;
							break;
						}
					}
				} else {
					pos = dataArrayUint8.indexOf(255, 4);
				}
				headArrayUint8 = dataArrayUint8.subarray(0, pos),
				contentArrayUint8 = dataArrayUint8.subarray(pos);
			}

			return this.utils.concatUInt8Array([headArrayUint8, exif, contentArrayUint8]);
		}
	};

	this.jpgSetOrientationToStandard = function (dataArr, orientationOnly, values) {
		var view = new DataView(dataArr.buffer),
			values = values || {},
			little = false,
			tagOth = 0,
			pointer0th = 0,
			pointerExif = 0,
			num0th = 0,
			numExif = 0,
			oldValue = 0;

		if (view.byteLength < 16 || view.getUint32(4, false) != 0x45786966) {
			return dataArr;
		}

		little = view.getUint16(10, false) == 0x4949;
		//top.console.log('littleendian', little); // littleendian = false seems not to work

		tagOth = view.getUint32(14, little);
		num0th = view.getUint16(tagOth + 10, little);
		pointer0th = tagOth + 12;

		for (var i = 0; i < num0th; i++) {
			if(pointer0th + (i * 12) + 10 > view.byteLength){
				break;
			}

			switch(view.getUint16(pointer0th + (i * 12), little)){
				case 0x0112: // orientation
					//top.console.log('found orientation', view.getUint16(pointer0th + (i * 12) + 8, little));
					view.setUint16(pointer0th + (i * 12) + 8, 1, little);
					i = num0th;
					break;
				case 0x8769:
					pointerExif = parseInt(view.getUint16(pointer0th + (i * 12) + 8, little));
					numExif = pointerExif + 10 + 12 < view.byteLength ? view.getUint16(pointerExif + 10, little) : 0;
					pointerExif += 12;
					//top.console.log('found Exif IFD Pointer', pointerExif, numExif);
					break;
			}
		}

		if(pointerExif && numExif){
			for (var i = 0; i < numExif; i++) {
				if(pointerExif + (i * 12) + 10 > view.byteLength){
					break;
				}

				switch(view.getUint16(pointerExif + (i * 12), little)){
					case 0xa002: //PixelXDimension
						oldValue = view.getUint16(pointerExif + (i * 12) + 8, little);
						//top.console.log('found PixelXDimension', oldValue);
//						view.setUint16(pointerExif + (i * 12) + 8, values.width ? values.width : oldValue, little);
						break;
					case 0xa003: //PixelYDimension
						oldValue = view.getUint16(pointerExif + (i * 12) + 8, little);
						//top.console.log('found PixelYDimension', view.getUint16(pointerExif + (i * 12) + 8, little));
//						view.setUint16(pointerExif + (i * 12) + 8, values.height ? values.height : oldValue, little);
						break;
				}
			}
		}

		return new Uint8Array(view.buffer);
	};

	this.jpgGetExifSegment = function (uint8array) { // unused?
		return this.jpgGetSegment(uint8array, 225);
	};

	// use this to extract a certain segment: not intended to be used several times in sequence
	this.jpgGetSegment = function (uint8array, marker) {
		var head = 0;

		while (head < uint8array.length) {
			if (uint8array[head] === 255 && uint8array[head + 1] === 218) { // SOI = Scan of Image = image data (eg. canvas works on)!
				break;
			}

			if (uint8array[head] === 255 && uint8array[head + 1] === 216) { // omit 216 (D8): SOI = Start of Image (is empty so length = 2 bytes)
				head += 2;
			} else {
				var length = uint8array[head + 2] * 256 + uint8array[head + 3],
								endPoint = head + length + 2;

				if (uint8array[head + 1] == marker) {
					return uint8array.subarray(head, endPoint);
				}

				head = endPoint;
			}
		}

		return false;
	};

	this.jpgGetAllSegmentsUpToSOS = function (uint8array) { // unused?
		var head = 0,
			segments = {},
			order = [];

		while (head < uint8array.length) {
			// each segment starts with 255 (FF) and its segment marker
			if (uint8array[head] === 255 && uint8array[head + 1] === 218) { // 218 (DA): SOS = Start of Scan = image data (canvas works on)
				//win.console.log('found SOS at pos', head, segments);
				break;
			}
			if (uint8array[head] === 255 && uint8array[head + 1] === 216) { // omit 216 (D8): SOI = Start of Image (is empty so length = 2 bytes)
				head += 2;
			} else {
				var length = uint8array[head + 2] * 256 + uint8array[head + 3],
								endPoint = head + length + 2;

				order.push(uint8array[head + 1]);
				segments[uint8array[head + 1] + '_' + head] = uint8array.subarray(head, endPoint);
				head = endPoint;
			}
		}

		return {order: order, segments: segments};
	};

	this.jpgGetSegmentsIfExist = function (uint8array, segments, splice) {
		var head = 0,
			searchObj = {},
			segmentsArrySliced = [],
			controllArr = [],
			sliced = null,
			SOSfound,
			SOSPos=0;

		for (var i = 0; i < segments.length; i++) {
			searchObj[segments[i]] = true;
		}

		while (head < uint8array.length) {
			if (uint8array[head] === 255 && uint8array[head + 1] === 218) { // SOI = Scan of Image = image data (eg. canvas works on)!
				SOSfound = true;
				SOSPos = head;
				break;
			}

			if (uint8array[head] === 255 && uint8array[head + 1] === 216) { // omit 216 (D8): SOI = Start of Image (is empty so length = 2 bytes)
				head += 2;
			} else {
				var length = uint8array[head + 2] * 256 + uint8array[head + 3],
					endPoint = head + length + 2;

				if (splice || !segments.length || searchObj[uint8array[head + 1]] === true) {
					segmentsArrySliced.push(new Uint8Array(uint8array.buffer.slice(head, endPoint)));
					controllArr.push({
						marker: uint8array[head + 1],
						head: head,
						length: (endPoint - head),
						segment: uint8array.subarray(head, endPoint),
						segmentSlice: sliced
					});
				}

				head = endPoint;
			}
		}

		if (!SOSfound) {
			return {
				success: false,
				headArray: null,
				contentArray: null,
				segments: []
			};
		}

		if(splice && controllArr.length){
			return {
				success: true,
				headArray: uint8array.subarray(0, controllArr[0].head),
				contentArray: uint8array.subarray(SOSPos),
				segments: segmentsArrySliced,
				sosPos: SOSPos
			};
		}

		return {
			success: true,
			headArray: null,
			contentArray: null,
			segments: segmentsArrySliced
		};
	};

	this.pngReinsertTextchunks = function (dataArray, pngTextChunks) {
		var combinedChunks = [];
		try {
			var chunks = this.uploader.win.extractChunks(dataArray);
			combinedChunks = [];

			combinedChunks.push(chunks.shift()); // new IHDR
			while (pngTextChunks.length) {
				combinedChunks.push(pngTextChunks.shift());
			}
			while (chunks.length) {
				combinedChunks.push(chunks.shift());
			}
		} catch (e) {
			combinedChunks = false;
		}

		var newUInt8Array = false;
		if (combinedChunks) {
			try {
				newUInt8Array = WE().layout.fileupload.encodeChunks(combinedChunks);
			} catch (e) {
				newUInt8Array = false;
			}
		}

		return newUInt8Array ? newUInt8Array : dataArray;
	};
}

function Fileupload_imageEdit_base(uploader) {
	Fileupload_imageEdit_abstract.call(this, uploader);

	this.uploader = uploader;
}
Fileupload_imageEdit_base.prototype = Object.create(Fileupload_imageEdit_abstract.prototype);
Fileupload_imageEdit_base.prototype.constructor = Fileupload_imageEdit_base;

function Fileupload_imageEdit_bindoc(uploader) {
	Fileupload_imageEdit_abstract.call(this, uploader);

	this.uploader = uploader;

	this.getImageEditIndices = function (pos, general) {
		return this.sender.preparedFiles.length ? [0] : [];
	};
}
Fileupload_imageEdit_bindoc.prototype = Object.create(Fileupload_imageEdit_abstract.prototype);
Fileupload_imageEdit_bindoc.prototype.constructor = Fileupload_imageEdit_bindoc;

function Fileupload_imageEdit_editFull(uploader) {
	Fileupload_imageEdit_abstract.call(this, uploader);

	this.uploader = uploader;

	this.processImage = function (fileobj, task, data) {
		if (!fileobj) {
			this.processNextImage();
			return;
		}

		switch (task) {
			case this.IMG_START:
				this.processImage(fileobj, this.IMG_EXTRACT_METADATA);
				break;
			case this.IMG_EXTRACT_METADATA:
				this.processimageExtractMetadata(fileobj, this.IMG_WRITE_IMAGE);
				break;
			case this.IMG_WRITE_IMAGE:
				this.processimageWriteImage(fileobj, this.IMG_INSERT_METADATA, data);
				break;
			case this.IMG_INSERT_METADATA:
				this.processimagInsertMetadata(fileobj, this.IMG_POSTPROCESS);
				break;
				/*
				 case this.IMG_MAKE_PREVIEW:
				 this.processimageMakePreview(fileobj, this.IMG_POSTPROCESS);
				 break;
				 */
			case this.IMG_POSTPROCESS:
				this.processimagePostProcess(fileobj, this.IMG_NEXT);
				break;
			case this.IMG_NEXT:
			/*falls through*/
			default:
				this.processNextImage();
		}
	};

	this.setImageEditOptionsGeneral = function () {
		var opts = this.imageEditOptions;
		opts.doEdit = false;
		opts.scaleWhat = 'pixel_l';
		opts.scale = 0;
		opts.rotate = 0;
		opts.quality = this.OPTS_QUALITY_NEUTRAL_VAL;
	};

	this.setImageEditOptionsFile = function () {
		var opts = this.imageEditOptions;
		opts.doEdit = false;
		opts.scaleWhat = 'pixel_l';
		opts.scale = 0;
		opts.rotate = 0;
		opts.quality = this.OPTS_QUALITY_NEUTRAL_VAL;
	};

	this.getImageEditIndices = function (pos, general) {
		return this.sender.preparedFiles.length ? [0] : [];
	};
}
Fileupload_imageEdit_editFull.prototype = Object.create(Fileupload_imageEdit_abstract.prototype);
Fileupload_imageEdit_editFull.prototype.constructor = Fileupload_imageEdit_editFull;

function Fileupload_imageEdit_import(uploader) {
	Fileupload_imageEdit_abstract.call(this, uploader);

	this.PRESERVE_IMG_DATAURL = true;
	this.IS_MEMORY_MANAGMENT = true;
	this.isImageEditActive = true;

	this.getImageEditIndices = function (index, general, formposition) {
		if (general) {
			var uploadItems = this.uploader.doc.getElementsByClassName('uploadItem');
			var indices = [];
			for (var i = 0; i < uploadItems.length; i++) {
				index = uploadItems[i].getAttribute('data-index');
				if (this.sender.preparedFiles[index] && this.sender.preparedFiles[index].isEditable && !this.sender.preparedFiles[index].isUploadStarted) {
					indices.push(formposition ? i : index);
				}
			}

			return indices;
		}

		if (index !== undefined && index > -1 && this.sender.preparedFiles[index] && this.sender.preparedFiles[index].isEditable) {
			return [index];
		}

		return [];
	};
}
Fileupload_imageEdit_import.prototype = Object.create(Fileupload_imageEdit_abstract.prototype);
Fileupload_imageEdit_import.prototype.constructor = Fileupload_imageEdit_import;
