/* global WE, errorHandler,treeData,drawTree, top */

/**
 * webEdition CMS
 *
 * $Rev: 8faf83012496 $
 * $Author: Lukas Imhof $
 * $Date: 2021-01-25 11:37 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

var
	hot = false,
	last = 0,
	lastUsedLoadFrame = null;

function doClickWithParameters(id, ct, table, parameters) {
	WE().layout.weEditorFrameController.openDocument(table, id, ct, '', '', '', '', '', parameters);
}

function we_repl(target, url) {
	if (!target) {
		return;
	}
	try {
		// use 2 loadframes to avoid missing cmds
		if (target.name === "load" || target.name === "load2") {
			//FIXME: try to eleminate this code and replace it by regular ajax
			if (top.lastUsedLoadFrame === target.name) {
				target = (target.name === "load" ?
					window.load2 :
					window.load);
			}
			top.lastUsedLoadFrame = target.name;
		}
	} catch (e) {
		// Nothing
	}
	if (target.location === undefined) {
		target.src = url;
	} else {
		target.location.replace(url);
	}
}

function start(table_to_load) {
	if (table_to_load) {
		we_cmd("load", table_to_load);
	}
}

function doUnloadSEEM(whichWindow) {
	// unlock all open documents
	var _usedEditors = WE().layout.weEditorFrameController.getEditorsInUse();

	var docIds = "";
	var docTables = "";

	for (var frameId in _usedEditors) {
		if (_usedEditors[frameId].EditorType !== "cockpit") {
			docIds += _usedEditors[frameId].getEditorDocumentId() + ",";
			docTables += _usedEditors[frameId].getEditorEditorTable() + ",";
		}
	}

	if (docIds) {
		top.we_cmd('we_users_unlock', docIds, WE().session.userID, docTables);
		if (top.opener) {
			top.opener.focus();

		}
	}
	//  close the SEEM-edit-include when exists
	if (top.edit_include) {
		top.edit_include.close();
	}
	try {
		WE().layout.jsWindow.prototype.closeAll();
		if (WE().layout.browserwind) {
			WE().layout.browserwind.close();
		}
	} catch (e) {

	}

	//  only when no SEEM-edit-include window is closed

	if (whichWindow !== "include") {
		if (window.opener) {
			window.opener.location.replace(WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=we_gui_loggingOut');
		}
	}
}

function doUnloadNormal(whichWindow) {
	var tinyDialog;
	if (!WE().layout.regular_logout) {

		if (window.tinyMceDialog) {
			tinyDialog = window.tinyMceDialog;
			try {
				tinyDialog.close();
			} catch (err) {
			}
		}

		if (window.tinyMceSecondaryDialog) {
			tinyDialog = window.tinyMceSecondaryDialog;
			try {
				tinyDialog.close();
			} catch (err) {
			}
		}

		try {
			WE().layout.jsWindow.prototype.closeAll();
			if (WE().layout.browserwind) {
				WE().layout.browserwind.close();
			}
		} catch (e) {
		}
		if (whichWindow !== "include") { 	// only when no SEEM-edit-include window is closed
			// FIXME: closing-actions for SEEM
			var logoutpopup = window.open(WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=we_gui_loggingOut', "webEdition", "width=350,height=70,toolbar=no,menubar=no,directories=no,location=no,resizable=no,status=no,scrollbars=no,top=300,left=500");
			if (logoutpopup) {
				logoutpopup.focus();
			}

		}
	}
}

function doUnload(whichWindow) { // triggered when webEdition-window is closed
	if (!WE().layout.weEditorFrameController.closeAllDocuments()) {
		return WE().consts.g_l.alert.unsavedChanges;
	}
	if (WE().session.seemode) {
		doUnloadSEEM(whichWindow);
	} else {
		doUnloadNormal(whichWindow);
	}
}

function we_openMediaReference(win, id) {
	id = id ? id : 0;

	if (win.we_mediaReferences && win.we_mediaReferences['id_' + id]) {
		var ref = win.we_mediaReferences['id_' + id];
		switch (ref.type) {
			case 'module':
				top.we_cmd('we_' + ref.mod + '_edit', ref.id);
				break;
			case 'cat':
				top.we_cmd('editCat', ref.id);
				break;
			default:
				if (ref.isTempPossible && ref.referencedIn === 'main' && ref.isModified) {
					WE().util.showMessage('Der Link wurde bei einer unveröffentlichten Änderung entfernt: Er existiert nur noch in der veröffentlichten Version!', WE().consts.message.WE_MESSAGE_ERROR, window);// FIXME: GL()
				} else {
					WE().layout.weEditorFrameController.openDocument(ref.table, ref.id, ref.ct);
				}
		}
	}
}

function we_showInNewTab(args, url) {
	var
		ctrl = WE().layout.weEditorFrameController,
		nextWindow;
	if ((nextWindow = ctrl.getFreeWindow())) {
		we_repl(nextWindow.getDocumentReference(), url);
		// activate tab
		var pos = (args[0] === "open_cockpit" ? 0 : undefined);
		WE().layout.multiTabs.addTab(nextWindow.getFrameId(), ' &hellip; ', ' &hellip; ', pos);
		// set Window Active and show it
		ctrl.setActiveEditorFrame(nextWindow.FrameId);
		ctrl.toggleFrames();
	} else {
		WE().util.showMessage(WE().consts.g_l.main.no_editor_left, WE().consts.message.WE_MESSAGE_INFO, window);
	}
}


function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
		args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
		url = WE().util.getWe_cmdArgsUrl(args);
	//	When coming from a we_cmd, always mark the document as opened with we !!!!
	if (WE().layout.weEditorFrameController.getActiveDocumentReference()) {
		switch (args[0]) {
			case 'edit_document':
			case 'new_document':
			case 'open_extern_document':
			case 'edit_document_with_parameters':
			case 'new_folder':
			case 'edit_folder':
				break;
			default:
			try {
				WE().layout.weEditorFrameController.getActiveDocumentReference().openedWithWE = true;
			} catch (e) {
			}
		}

	}
	/* all base commands should be added to we_webEditionCmd_base.js */
	var i, ret, mods = WE().consts.modules.jsmods;
	for (i = 0; i < mods.length; i++) {
		if (WE().we_cmd.modules[mods[i]] && (ret = WE().we_cmd.modules[mods[i]].call(caller, args, url, caller)) !== false) {
			return ret;
		}
		//if a module window is requested, we have to open it
		if (args[0] === ("we_" + mods[i] + "_edit")) {
			/* CLASSIC jsWindow */ new (WE().layout.jsWindow)(caller, url, "edit_module", WE().consts.size.dialog.bigger, WE().consts.size.dialog.medium, true, true);
			return true;
		} else if (args[0] === ("we_" + mods[i] + "_editSettings")) {
			(new (WE().layout.overlay)(caller, 'top', WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=" + mods[i] + "&pnt=settings", "customer_settings", WE().consts.size.dialog.small, WE().consts.size.dialog.smaller)).setResizeable().open();
			return true;
		}
	}
	// deal with not activated modules
	mods = WE().consts.modules.inactive;
	for (i = 0; i < mods.length; i++) {
		if (args[0] === ("we_" + mods[i] + "_edit")) {
			(new (WE().layout.overlay)(caller, 'top', url, "module_info", WE().consts.size.dialog.smaller, WE().consts.size.dialog.tiny)).setResizeable().open();
			return true;
		}
	}

	//log this, as it might be an error
	//WE().t_e('cmd opens new tab', args, url);
	//finally open new window
	//make this only a fallback - everthing should be catched, so we add an error for now!
	WE().t_e('js-command not found, open a new window', args);
	we_showInNewTab(args, url);
}

function objectAssign(target, src) {
	if (Object.assign) {
		return Object.assign(target, src);
	}
	for (var key in src) {
		if (src.hasOwnProperty(key)) {
			target[key] = src[key];
		}
	}
	return target;
}

function switchEditPage(args, url) {
	// get editor root frame of active tab
	var currentEditorRootFrame = WE().layout.weEditorFrameController.getActiveDocumentReference();
	// get visible frame for displaying editor page
	var visibleEditorFrame = WE().layout.weEditorFrameController.getVisibleEditorFrame();
	// frame where the form should be sent from
	var sendFromFrame = visibleEditorFrame;
	// set flag to true if active frame is frame nr 2 (frame for displaying editor page 1 with content editor)
	var isEditpageContent = (visibleEditorFrame === currentEditorRootFrame.frames[2]);

	// if we switch from we_base_constants::WE_EDITPAGE_CONTENT to another page
	if (isEditpageContent && args[1] !== WE().consts.global.WE_EDITPAGE_CONTENT) {
		// clean body to avoid flickering
		try {
			currentEditorRootFrame.frames[1].document.body.innerHTML = "";
		} catch (e) {
			//can be caused by not loaded content
		}
		// switch to normal frame
		WE().layout.weEditorFrameController.switchToContentEditor(false);
		// set var to new active editor frame
		visibleEditorFrame = currentEditorRootFrame.frames[1];

		// set flag to false
		isEditpageContent = false;
		// if we switch to we_base_constants::WE_EDITPAGE_CONTENT from another page
	} else if (!isEditpageContent && args[1] === WE().consts.global.WE_EDITPAGE_CONTENT) {
		// switch to content editor frame
		WE().layout.weEditorFrameController.switchToContentEditor(true);
		// set var to new active editor frame
		visibleEditorFrame = currentEditorRootFrame.frames[2];
		// set flag to false
		isEditpageContent = true;
	}

	// frame where the form should be sent to
	var _sendToFrame = visibleEditorFrame;
	// get active transaction
	var _we_activeTransaction = WE().layout.weEditorFrameController.getActiveEditorFrame().getEditorTransaction();
	// if there are parameters, attach them to the url
	if (currentEditorRootFrame.parameters) {
		url += currentEditorRootFrame.parameters;
	}

	// focus the frame
	if (_sendToFrame) {
		_sendToFrame.focus();
	}
	// if visible frame equals to editpage content and there is already content loaded
	if (isEditpageContent && visibleEditorFrame && visibleEditorFrame.weIsTextEditor !== undefined && currentEditorRootFrame.frames[2].location !== "about:blank") {
		// tell the backend the right edit page nr and break (don't send the form)
		WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?cmd=documentSetPageNr", "transaction=" + _we_activeTransaction + "&editPageNr=" + args[1]);
		if (visibleEditorFrame.reloadContent === false) {
			//disable footer, if buttons are useless
			currentEditorRootFrame.editFooter.document.getElementsByClassName("footertable")[0].style.display = (WE().consts.global.DISABLED_FOOTER_PAGES.indexOf(args[1]) >= 0 ? 'none' : 'block');
			return;
		}
		visibleEditorFrame.reloadContent = false;
	}

	if (currentEditorRootFrame) {
		if (!WE().util.we_sbmtFrm(_sendToFrame, url, sendFromFrame)) {
			// add we_transaction, if not set
			if (!args[2]) {
				args[2] = _we_activeTransaction;
			}
			url += "&we_transaction=" + args[2];
			we_repl(_sendToFrame, url);
		}
	}
	//disable footer, if buttons are useless
	currentEditorRootFrame.editFooter.document.getElementsByClassName("footertable")[0].style.display = (WE().consts.global.DISABLED_FOOTER_PAGES.indexOf(args[1]) >= 0 ? 'none' : 'block');
}

function startMsg() {
	top._console_ = new (WE().layout.messageConsoleView)("mainWindow", window);
	top._console_.register();
	window.document.body.addEventListener("onunload", top._console_.unregister);
}

function showMainWindow() {
	top.document.getElementById("loading").style.display = "none";
	top.document.getElementById("weMainDiv").style.visibility = "visible";
	top.document.getElementById("loginScreen").style.background = "";
	top.document.getElementById("loginScreen").style.display = "none";
}

function checkPwd(sufficient) {
	if (!sufficient) {
		WE().util.showMessage(WE().consts.g_l.alert.pwd_startupRegExFailed, WE().consts.message.WE_MESSAGE_ERROR);
	}
}

function updateCheck(avail, version, date) {
	/* disable update check
	 if (avail) {
	 WE().util.showMessage(WE().util.sprintf(WE().consts.g_l.alert.newWEAvailable, version, date), WE().consts.message.WE_MESSAGE_INFO, window);
	 }*/
}

function getFrameset() {//FIXME: we use this function temporary until frames in modules are obsolete
	return WE().consts.dirs.WEBEDITION_DIR + "index.php?we_cmd[0]=startWE";
}


function we_cmd_delete_single_document_question(url) {
	var ctrl = WE().layout.weEditorFrameController;
	var cType = ctrl.getActiveEditorFrame().getEditorContentType();
	var eTable = ctrl.getActiveEditorFrame().getEditorEditorTable();
	var path = ctrl.getActiveEditorFrame().getEditorDocumentPath();

	if (ctrl.getActiveDocumentReference()) {
		if (!WE().util.hasPermDelete(eTable, [
			WE().consts.contentTypes.FOLDER_COMMON,
			WE().consts.contentTypes.FOLDER_DOCUMENT,
			WE().consts.contentTypes.FOLDER_TEMPLATE,
			WE().consts.contentTypes.FOLDER_OBJECTFILE,
			WE().consts.contentTypes.FOLDER_VFILE
		].indexOf(cType) !== -1)) {
			WE().util.showMessage(WE().consts.g_l.main.no_perms_action, WE().consts.message.WE_MESSAGE_ERROR, window);
		} else {
			WE().util.showConfirm(window, '', WE().consts.g_l.main.delete_single_confirm_delete + path, ['delete_single_document', '', eTable, 1]);
		}
	} else {
		WE().util.showMessage(WE().consts.g_l.main.no_document_opened, WE().consts.message.WE_MESSAGE_ERROR, window);
	}
}

function loadCloseFolder(args) {
	top.document.getElementById('reloadTree').firstChild.classList.add('fa-spin');
	WE().util.rpc(WE().util.getWe_cmdArgsUrl(args, WE().consts.dirs.WEBEDITION_DIR + 'rpc.php?cmd=treeLoadMainTree&'), null, function (weResponse) {
		if (weResponse && weResponse.Success) {
			if (weResponse.DataArray.treeName) {
				top.document.getElementById("treeName").innerHTML = weResponse.DataArray.treeName;
			}
			if (weResponse.DataArray.items) {
				if (!weResponse.DataArray.parentFolder) {
					top.treeData.clear();
					top.treeData.add(top.Node.prototype.rootEntry(0, 'root', 'root', weResponse.DataArray.offset));
				}
				for (var i = 0; i < weResponse.DataArray.items.length; i++) {
					if (!weResponse.DataArray.parentFolder || top.treeData.indexOfEntry(weResponse.DataArray.items[i].id) < 0) {
						top.treeData.add(new top.Node(weResponse.DataArray.items[i]));
					}
				}
			}
			top.drawTree();
		}
		top.treeData.scrollToY();
	});
	window.setTimeout(function () {
		top.document.getElementById('reloadTree').firstChild.classList.remove('fa-spin');
	}, 300);
}

function we_cmd_delete_single_document(url) {
	var ctrl = WE().layout.weEditorFrameController;
	var
		cType = ctrl.getActiveEditorFrame().getEditorContentType(),
		eTable = ctrl.getActiveEditorFrame().getEditorEditorTable();

	if (!ctrl.getActiveDocumentReference()) {
		WE().util.showMessage(WE().consts.g_l.main.no_document_opened, WE().consts.message.WE_MESSAGE_ERROR, window);
		return;
	}
	if (!WE().util.hasPermDelete(eTable, [
		WE().consts.contentTypes.FOLDER_COMMON,
		WE().consts.contentTypes.FOLDER_DOCUMENT,
		WE().consts.contentTypes.FOLDER_TEMPLATE,
		WE().consts.contentTypes.FOLDER_OBJECTFILE,
		WE().consts.contentTypes.FOLDER_VFILE
	].indexOf(cType) !== -1)) {
		WE().util.showMessage(WE().consts.g_l.main.no_perms_action, WE().consts.message.WE_MESSAGE_ERROR, window);
		return;
	}
	WE().util.we_sbmtFrm(window.load, url + "&we_cmd[2]=" + ctrl.getActiveEditorFrame().getEditorEditorTable(), ctrl.getActiveDocumentReference().editFooter);
}

function wecmd_editDocument(args, url) {
	try {
		if (top.treeData) {
			top.treeData.unselectNode();
			top.treeData.selectNode(args[1], args[2]);
		}
	} catch (e) {
	}

	var ctrl = WE().layout.weEditorFrameController;
	var nextWindow, nextContent;
	if ((nextWindow = ctrl.getFreeWindow())) {
		nextContent = nextWindow.getDocumentReference();
		// activate tab and set state to loading
		WE().layout.multiTabs.addTab(nextWindow.getFrameId(), nextWindow.getFrameId(), nextWindow.getFrameId());
		// use Editor Frame
		nextWindow.initEditorFrameData(
			{
				EditorType: "model",
				EditorEditorTable: args[1],
				EditorDocumentId: args[2],
				EditorContentType: args[3]
			}
		);
		// set Window Active and show it
		ctrl.setActiveEditorFrame(nextWindow.FrameId);
		ctrl.toggleFrames();
		if (nextContent.frames && nextContent.frames[1]) {
			if (!WE().util.we_sbmtFrm(nextContent, url)) {
				top.we_repl(nextContent, url + "&frameId=" + nextWindow.getFrameId());
			}
		} else {
			top.we_repl(nextContent, url + "&frameId=" + nextWindow.getFrameId());
		}
	} else {
		WE().util.showMessage(WE().consts.g_l.main.no_editor_left, WE().consts.message.WE_MESSAGE_ERROR);
	}
}

function we_cmd_new_document(url) {
	var ctrl = WE().layout.weEditorFrameController,
		nextWindow, nextContent;
	if ((nextWindow = ctrl.getFreeWindow())) {
		nextContent = nextWindow.getDocumentReference();
		// activate tab and set it status loading ...
		WE().layout.multiTabs.addTab(nextWindow.getFrameId(), nextWindow.getFrameId(), nextWindow.getFrameId());
		nextWindow.updateEditorTab();
		// set Window Active and show it
		ctrl.setActiveEditorFrame(nextWindow.getFrameId());
		ctrl.toggleFrames();
		// load new document editor
		top.we_repl(nextContent, url + "&frameId=" + nextWindow.getFrameId());
	} else {
		WE().util.showMessage(WE().consts.g_l.main.no_editor_left, WE().consts.message.WE_MESSAGE_ERROR);
	}
}

function addToCollection(args, url) {
	if (WE().session.seemode) {
		//
	} else {
		if (WE().session.deleteMode !== args[1]) {
			WE().session.deleteMode = args[1];
		}
		if (!WE().session.deleteMode && top.treeData.state === top.treeData.tree_states.select) {
			top.treeData.setState(top.treeData.tree_states.edit);
			top.drawTree();
		}
		window.document.getElementById("bm_treeheaderDiv").classList.add('collectionSelector');
		window.document.getElementById("treetable").classList.add('collectionSelector');
		WE().layout.tree.toggle(true);
		var width = WE().layout.tree.getWidth();
		WE().layout.tree.widthBeforeDeleteMode = width;
		if (width < WE().consts.size.tree.addToCollectionWidth) {
			WE().layout.tree.setWidth(WE().consts.size.tree.addToCollectionWidth);
		}
		WE().layout.tree.storeWidth(WE().layout.tree.widthBeforeDeleteMode);

		WE().layout.sidebar.widthBeforeDeleteMode = WE().layout.sidebar.getWidth();

		if (args[2] != 1) {
			top.we_repl(document.getElementsByName("treeheader")[0], url);
		}
	}
}

function updateMainTree(select, attribs, adv) {
	if (!top.treeData || top.treeData.table !== attribs.table) {
		return;
	}
	if (select) {
		top.treeData.selection_table = attribs.table;
		top.treeData.selection = attribs.id;
	} else {
		top.treeData.unselectNode();
	}

	if (top.treeData.elements[top.treeData.indexOfEntry(attribs.parentid)]) {
		/*var visible = (top.treeData.indexOfEntry(attribs.parentid) !== -1 ?
		 top.treeData.elements[top.treeData.indexOfEntry(attribs.parentid)].open :
		 0);*/
		if (top.treeData.indexOfEntry(attribs.id) !== -1) {
			top.treeData.updateEntry(attribs);
		} else {
			top.treeData.addSort(new top.Node(top.objectAssign(attribs, adv)));
		}
		top.drawTree();
	} else if (top.treeData.indexOfEntry(attribs.id) !== -1) {
		top.treeData.deleteEntry(attribs.id);
	}
}

function getHotDocumentsString() {
	var allHotDocuments = WE().layout.weEditorFrameController.getEditorsInUse();
	var ct, ulCtElem, i;
	var ret = "";
	var hotDocumentsOfCt = {};
	for (var frameId in allHotDocuments) {
		ct = allHotDocuments[frameId].getEditorContentType();
		if (!hotDocumentsOfCt[ct]) {
			hotDocumentsOfCt[ct] = [];
		}
		hotDocumentsOfCt[ct].push(allHotDocuments[frameId]);
	}

	for (ct in hotDocumentsOfCt) {
		ulCtElem = "";

		for (i = 0; i < hotDocumentsOfCt[ct].length; i++) {
			ulCtElem += "<li>" + (hotDocumentsOfCt[ct][i].getEditorDocumentText() ?
				hotDocumentsOfCt[ct][i].getEditorDocumentPath() :
				"<em>" + WE().consts.g_l.main.untitled + "</em>") + "</li>";
		}

		ret += "<li>" + WE().consts.g_l.contentTypes[ct] + "<ul>" + ulCtElem + "</ul></li>";
	}
	return ret;
}

function collection_insertFiles(args) { // args[2] = collection id, args[3] = index of item, args[4] = position of item, args[5] = recursive, args[6] = transaction (alternative to id)
	if (args[1] === undefined || args[2] === undefined) {
		return;
	}
	var index = (args[3] !== undefined ? args[3] : -1),
		collection = parseInt(args[2]),
		transaction = args[6] ? args[6] : '',
		ids = (args[1].success !== undefined ? args[1].success : (args[1].currentID !== undefined ? [
			args[1].currentID] : args[1]));

	if ((collection || transaction) && ids) {
		var usedEditors = WE().layout.weEditorFrameController.getEditorsInUse(),
			editor = null,
			recursive = args[5] ? true : false,
			frameId, candidate;

		for (frameId in usedEditors) {
			candidate = usedEditors[frameId];
			if (candidate.getEditorEditorTable() === WE().consts.tables.VFILE_TABLE &&
						((collection && parseInt(candidate.getEditorDocumentId()) === collection) || (transaction && candidate.getEditorTransaction() === transaction)))
					{
				if (parseInt(candidate.getEditorEditPageNr()) === 1) {
					editor = candidate;
				} else {
					transaction = candidate.getEditorTransaction();
				}
				break;
			}
		}

		if (editor) {
			// editor is open so we can replace by index
			recursive = editor.getContentEditor().weCollectionEdit.getIsRecursive();
			editor.getContentEditor().weCollectionEdit.callForValidItemsAndInsert(index, -1, ids.join(), WE().consts.tables.FILE_TABLE, '', false, recursive);
		} else {
			var position = args[4] !== undefined ? args[4] : index;
			we_cmd('collection_insertFiles_rpc', ids, collection, transaction, position, recursive);
		}
	}
}

function we_cmd_move(args, url) {
	if (WE().session.seemode) {
		if (WE().session.deleteMode != args[1]) {
			WE().session.deleteMode = args[1];
		}
		if (args[2] != 1) {
			top.we_repl(WE().layout.weEditorFrameController.getActiveDocumentReference(), url);
		}
	} else {
		if (WE().session.deleteMode != args[1]) {
			WE().session.deleteMode = args[1];
		}
		if (!WE().session.deleteMode && top.treeData.state == top.treeData.tree_states.selectitem) {
			top.treeData.setState(top.treeData.tree_states.edit);
			top.drawTree();
		}
		window.document.getElementById("bm_treeheaderDiv").classList.add('moveSelector');
		window.document.getElementById("treetable").classList.add('moveSelector');
		WE().layout.tree.toggle(true);
		var width = WE().layout.tree.getWidth();

		WE().layout.tree.widthBeforeDeleteMode = width;

		if (width < WE().consts.size.tree.moveWidth) {
			WE().layout.tree.setWidth(WE().consts.size.tree.moveWidth);
		}
		WE().layout.tree.storeWidth(WE().layout.tree.widthBeforeDeleteMode);

		WE().layout.sidebar.widthBeforeDeleteMode = WE().layout.sidebar.getWidth();

		if (args[2] != 1) {
			top.we_repl(document.getElementsByName("treeheader")[0], url);
		}
	}
}

function we_cmd_delete(args, url) {
	if (WE().session.seemode) {
		if (WE().session.deleteMode != args[1]) {
			WE().session.deleteMode = args[1];
		}
		if (args[2] != 1) {
			top.we_repl(WE().layout.weEditorFrameController.getActiveDocumentReference(), url);
		}
		return;
	}
	if (WE().session.deleteMode != args[1]) {
		WE().session.deleteMode = args[1];
	}
	if (!WE().session.deleteMode && top.treeData.state == top.treeData.tree_states.select) {
		top.treeData.setState(top.treeData.tree_states.edit);
		top.drawTree();
	}
	window.document.getElementById("bm_treeheaderDiv").classList.add('deleteSelector');
	window.document.getElementById("treetable").classList.add('deleteSelector');
	WE().layout.tree.toggle(true);
	var width = WE().layout.tree.getWidth();

	WE().layout.tree.widthBeforeDeleteMode = width;

	if (width < WE().consts.size.tree.deleteWidth) {
		WE().layout.tree.setWidth(WE().consts.size.tree.deleteWidth);
	}
	WE().layout.tree.storeWidth(WE().layout.tree.widthBeforeDeleteMode);

	WE().layout.sidebar.widthBeforeDeleteMode = WE().layout.sidebar.getWidth();

	if (args[2] != 1) {
		top.we_repl(document.getElementsByName("treeheader")[0], url);
	}

}

function doSave(url, trans) {
	var _EditorFrame = WE().layout.weEditorFrameController.getEditorFrameByTransaction(trans);
	// _EditorFrame.setEditorIsHot(false);
	if (_EditorFrame.getEditorAutoRebuild()) {
		url += "&we_cmd[8]=1";
	}
	if (!WE().util.we_sbmtFrm(window.load, url)) {
		url += "&we_transaction=" + trans;
		top.we_repl(window.load, url);
	}
}

function doPublish(url, trans) {
	if (!WE().util.we_sbmtFrm(window.load, url)) {
		url += "&we_transaction=" + trans;
		top.we_repl(window.load, url);
	}
}

function doExtClick(url) {
	// split url in url and parameters !
	var parameters = "",
		_position;

	if ((_position = url.indexOf("?")) !== -1) {
		parameters = url.substring(_position);
		url = url.substring(0, _position);
	}

	WE().layout.weEditorFrameController.openDocument('', '', '', '', '', url, '', '', parameters);
}

function setFrameSize() {
	WE().layout.multiTabs.setFrameSize();
}

function startMultiEditor(startCmd) {
	WE().layout.weEditorFrameController = new (WE().layout.EditorFrameController)(startCmd);
	WE().layout.multiTabs = new (WE().layout.TabView)(window.document);
	WE().layout.weEditorFrameController.goHome();
}
