/* global top, WE, entries */

/**
 * webEdition CMS
 *
 * $Rev: 5c6473fcf82a $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-15 19:07 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

function drawNewFolder() {
	WE().util.dialogTop(window).unselectAllFiles();
	var name = window.prompt(WE().consts.g_l.fileselector.new_folder_prompt);
	if (name) {
		WE().util.dialogTop(window).fscmd.location.replace(WE().util.dialogTop(window).queryString(WE().consts.selectors.CREATEFOLDER, WE().util.dialogTop(window).fileSelect.data.currentDir) + '&we_cmd[we_FolderText]=' + name);
	}
}

function showPreview(id) {
	if (WE().util.dialogTop(window).document.getElementById('fspreviewDiv')) {
		if (!id) {
			WE().util.dialogTop(window).document.getElementById('fspreviewDiv').innerHTML = "";
			return;
		}
		WE().util.rpc(WE().util.dialogTop(window).queryString(WE().consts.selectors.PREVIEW, id), '', function (weResponse) {
			var prev = WE().util.dialogTop(window).document.getElementById('fspreviewDiv'),
				html = '<table class="default previewTable" style="width:100%">',
				previewFields,
				i;

			prev.innerHTML = "";
			if (!weResponse.Success) {
				return;
			}
			setInfoSize();
			weWriteBreadCrumb(weResponse.DataArray.path);
			if (weResponse.DataArray.imgPreview) {
				html += '<tr><td colspan="2" class="image">' + weResponse.DataArray.imgPreview + '</td></tr>';
			}
			previewFields = weResponse.DataArray.previewFields;
			for (var part in previewFields) {
				if (previewFields[part].data.length) {
					html += '<tr><td colspan="2" class="headline">' + previewFields[part].headline + '</td></tr>';
					for (i = 0; i < previewFields[part].data.length; i++) {
						html += '<tr>' + (!previewFields[part].data[i].caption ? '<td colspan="2">' : '<td>' + previewFields[part].data[i].caption + ': </td><td>') + previewFields[part].data[i].content + '</td></tr>';
					}
				}
			}

			prev.innerHTML = html + '</table>';
		});
	}
}

function setInfoSize() {
	var infoElem;
	if ((infoElem = document.getElementById("info"))) {
		infoElem.style.height = document.body.clientHeight - (document.getElementById("previewpic") ? 160 : 0) + "px";
	}
}

function weWriteBreadCrumb(BreadCrumb) {
	if (top.document.getElementById("fspath")) {
		top.document.getElementById("fspath").innerHTML = BreadCrumb;
	}
}

function doClick(id, ct) {
	if (WE().util.dialogTop(window).document.getElementById('fspreviewDiv')) {
		WE().util.dialogTop(window).document.getElementById('fspreviewDiv').innerHTML = "";
	}
	if (ct == 1) {
		if (WE().util.dialogTop(window).metaKeys.doubleClick) {
			selectFile(id);
			setDir(id);
			window.setTimeout(function () {
				WE().util.dialogTop(window).metaKeys.doubleClick = false;
			}, 400);
		}
	} else if (WE().util.dialogTop(window).fileSelect.data.currentID == id && (!WE().util.dialogTop(window).metaKeys.ctrl)) {
		selectFile(id);
	} else if (WE().util.dialogTop(window).fileSelect.options.multiple) {
		if (WE().util.dialogTop(window).metaKeys.shift) {
			var oldid = WE().util.dialogTop(window).fileSelect.data.currentID;
			var currendPos = WE().util.dialogTop(window).getPositionByID(id);
			var firstSelected = WE().util.dialogTop(window).getFirstSelected();

			if (currendPos > firstSelected) {
				WE().util.dialogTop(window).selectFilesFrom(firstSelected, currendPos);
			} else if (currendPos < firstSelected) {
				WE().util.dialogTop(window).selectFilesFrom(currendPos, firstSelected);
			} else {
				WE().util.dialogTop(window).selectFile(id);
			}
			WE().util.dialogTop(window).fileSelect.data.currentID = oldid;

		} else if (!WE().util.dialogTop(window).metaKeys.ctrl) {
			WE().util.dialogTop(window).selectFile(id);
		} else if (WE().util.dialogTop(window).isFileSelected(id)) {
			WE().util.dialogTop(window).unselectFile(id);
		} else {
			WE().util.dialogTop(window).selectFile(id);
		}
	} else {
		selectFile(id);
	}
	if (WE().util.dialogTop(window).metaKeys.ctrl) {
		WE().util.dialogTop(window).metaKeys.ctrl = 0;
	}
	if (WE().util.dialogTop(window).metaKeys.shift) {
		WE().util.dialogTop(window).metaKeys.shift = 0;
	}
}

function setDir(id) {
	var e = WE().util.dialogTop(window).getEntry(id);
	showPreview(id);
	WE().util.dialogTop(window).fscmd.location.replace(WE().util.dialogTop(window).queryString(WE().consts.selectors.SETDIR, id));
	WE().util.dialogTop(window).document.getElementById('fspath').innerHTML = e.path;
}

function selectFile(id) {
	var a = WE().util.dialogTop(window).document.getElementsByName("fname")[0];
	if (id) {
		if (id === WE().util.dialogTop(window).fileSelect.data.currentID) {
			return;
		}

		showPreview(id);
		var e = WE().util.dialogTop(window).getEntry(id);
		if (a.value != e.text &&
			a.value.indexOf(e.text + ",") === -1 &&
			a.value.indexOf("," + e.text + ",") === -1 &&
			a.value.indexOf("," + e.text + ",") === -1) {

			a.value = a.value ?
				(a.value + "," + e.text) :
				e.text;
		}
		if (WE().util.dialogTop(window).fsbody.document.getElementById("line_" + id)) {
			WE().util.dialogTop(window).fsbody.document.getElementById("line_" + id).classList.add("selected");
		}
		WE().util.dialogTop(window).fileSelect.data.currentPath = e.path;
		WE().util.dialogTop(window).fileSelect.data.currentID = id;
	} else {
		a.value = "";
		WE().util.dialogTop(window).fileSelect.data.currentPath = "";
	}
}

function elementSelected() {
	return true;
}

function addEntry(ID, text, isFolder, path, modDate, contentType) {
	entries.push({
		ID: ID,
		text: text,
		isFolder: isFolder,
		path: path,
		modDate: modDate,
		contentType: contentType
	});
}

function writeBodyDir(d, withModDate) {
	var body = '';

	for (var i = 0; i < entries.length; i++) {
		body += '<tr id="line_' + entries[i].ID + '" class="' + ((entries[i].ID == WE().util.dialogTop(window).fileSelect.data.currentID) ? 'selected' : '') + '"  onclick="return selectorOnClick(event,' + entries[i].ID + ');"' + (entries[i].isFolder ? ' onDblClick="return selectorOnDblClick(' + entries[i].ID + ');"' : '') + '>' +
			'<td class="selector selectoricon">' + WE().util.getTreeIcon(entries[i].contentType, false) + '</td>' +
			'<td class="selector cutText directory" title="' + entries[i].text + '">' + entries[i].text + '</td>' +
			(withModDate ? '<td class="selector moddate">' + entries[i].modDate + '</td>' : '') +
			'<td class="remain"></td></tr>';
	}

	d.innerHTML = '<form name="we_form" target="fscmd" method="post" action="' + WE().util.dialogTop(window).fileSelect.options.formtarget + '" >' +
		'<table class="selector sort" data-skipSort="0"><thead><tr>' + WE().util.dialogTop(window).fileSelect.options.selectorHeader + '</tr></thead><tbody>' + body + '</table></form>';
	window.fsbody.$("table.sort").each(WE().layout.applyTableSort);
}

function writeBody(d) {
	writeBodyDir(d, true);
}

function queryString(what, id) {
	return WE().util.dialogTop(window).fileSelect.options.formtarget +
		'&we_cmd[what]=' + what +
		'&we_cmd[rootDirID]=' + WE().util.dialogTop(window).fileSelect.options.rootDirID +
		'&we_cmd[open_doc]=' + WE().util.dialogTop(window).fileSelect.options.open_doc +
		'&we_cmd[table]=' + WE().util.dialogTop(window).fileSelect.options.table +
		'&we_cmd[id]=' + id;
}

function weonclick(e) {
	if (WE().util.dialogTop(window).document.getElementById('fspreviewDiv')) {
		WE().util.dialogTop(window).document.getElementById('fspreviewDiv').innerHTML = "";
	}

	WE().util.dialogTop(window).metaKeys.inputClick = false;
	if (e.altKey || e.metaKey || e.ctrlKey) {
		WE().util.dialogTop(window).metaKeys.ctrl = true;
	}
	if (e.shiftKey) {
		WE().util.dialogTop(window).metaKeys.shift = true;
	}
	if (WE().util.dialogTop(window).fileSelect.options.multiple) {
		if (!WE().util.dialogTop(window).metaKeys.shift && !WE().util.dialogTop(window).metaKeys.ctrl) {
			WE().util.dialogTop(window).unselectAllFiles();
		}
	} else {
		WE().util.dialogTop(window).unselectAllFiles();
	}
}
