/* global WE */

/**
 * webEdition CMS
 *
 * $Rev: 61506a702c2c $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-01-14 18:40 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

WE().handler.keyListener = {
	plainKey: function (evt) {
		switch (evt.keyCode) {
			case 8://backspace
				if (['INPUT', 'TEXTAREA'].indexOf(evt.target.tagName) > -1) {
					break;
				}
				WE().handler.keyListener.cancelEvent(evt);
				return true;

			case 27:// ESCAPE
				// does function closeOnEscape exist!
				if (WE().handler.keyListener.win.top.closeOnEscape !== undefined &&
					WE().handler.keyListener.win.top.closeOnEscape()) {
					WE().handler.keyListener.cancelEvent(evt);
					WE().layout.jsWindow.prototype.closeAll(WE().handler.keyListener.win);
					return true;
				}
				break;
			case 13:// ENTER
				// does function applyOnEnter exist?
				if (WE().handler.keyListener.win.top.applyOnEnter !== undefined &&
					WE().handler.keyListener.win.top.applyOnEnter(evt)) {
					WE().handler.keyListener.cancelEvent(evt);
					return true;
				}
				break;
			case 116: // F5
				WE().handler.keyListener.cancelEvent(evt);
				return true;
		}
		return false;
	},
	ctrlKey: function (evt) {
		var editor = false,
			editorType = "",
			activeEditorFrame = WE().layout.weEditorFrameController.getActiveEditorFrame(),
			usedEditors,
			pos;

		// check if an editor is open
		if (WE().handler.keyListener.win && WE().handler.keyListener.win.top && WE().handler.keyListener.win.top === WE().layout.mainFrame && //only if this was fired from WE-Main window
			WE().layout.weEditorFrameController.getActiveDocumentReference()) {
			editorType = activeEditorFrame ? activeEditorFrame.getEditorType() : '';
			if (editorType === "model") {
				editor = true;
			}
		}

		switch (evt.keyCode) {
			case 73:  // I (Open Tag-Wizard Prompt)
				if (activeEditorFrame &&
					activeEditorFrame.getEditorContentType() === "text/weTmpl" &&
					activeEditorFrame.getEditorEditPageNr() === WE().consts.global.WE_EDITPAGE_CONTENT &&
					activeEditorFrame.getContentFrame().openTagWizardPrompt !== undefined) {
					activeEditorFrame.getContentFrame().openTagWizardPrompt();
					WE().handler.keyListener.cancelEvent(evt);
					return true;
				}
				break;
			case 82: //ctrl-r = reload
				WE().handler.keyListener.cancelEvent(evt);
				return true;
			case 83: //S
				if (editor) {
					if (evt.shiftKey) { // SHIFT + S (Publish)
						window.focus(); // focus, to avoid a too late onchange of editor
						WE().handler.keyListener.cancelEvent(evt);
						activeEditorFrame.setEditorPublishWhenSave(true);
						if (activeEditorFrame.getEditorFrameWindow().frames.editFooter.we_save_document !== undefined) {
							activeEditorFrame.getEditorFrameWindow().frames.editFooter.we_save_document();
						}
					} else {// S (Save)
						window.focus();  // focus, to avoid a too late onchange of editor
						WE().handler.keyListener.cancelEvent(evt);
						activeEditorFrame.setEditorPublishWhenSave(false);
						if (activeEditorFrame.getEditorFrameWindow().frames.editFooter.we_save_document !== undefined) {
							activeEditorFrame.getEditorFrameWindow().frames.editFooter.we_save_document();
						}
					}
					return true;
				}
				if (WE().handler.keyListener.win.top.naviDiv !== undefined) {
					if (WE().handler.keyListener.win.top.content &&
						WE().handler.keyListener.win.top.content.editor &&
						WE().handler.keyListener.win.top.content.editor.edfooter &&
						WE().handler.keyListener.win.top.content.editor.edfooter.document.getElementsByClassName("weBtnSave").length ) {
						WE().handler.keyListener.cancelEvent(evt);
						//just fire the event as if the button was pressed :)
						WE().handler.keyListener.win.top.content.editor.edfooter.document.getElementsByClassName("weBtnSave")[0].click();
						return true;
					}
				} else if (WE().handler.keyListener.win.top.saveOnKeyBoard !== undefined) {
					if (WE().handler.keyListener.win.top.saveOnKeyBoard()) {
						WE().handler.keyListener.cancelEvent(evt);
						return true;
					}
				}
				break;
			case 90://Strg-z
				if (editor) {
//					console.log("strg-z canceled");
					return true;
				}
				break;
			case 188://, - prev tab
				if (!activeEditorFrame) {
					break;
				}
				usedEditors = Object.keys(WE().layout.weEditorFrameController.getEditorsInUse());
				pos = usedEditors.indexOf(activeEditorFrame.FrameId);
				if (pos >= 0) {
					pos = ((usedEditors.length + (--pos)) % usedEditors.length);
					WE().layout.weEditorFrameController.showEditor(usedEditors[pos]);
				}
				WE().handler.keyListener.cancelEvent(evt);
				return true;

			case 190://. -> next tab
				if (!activeEditorFrame) {
					break;
				}
				usedEditors = Object.keys(WE().layout.weEditorFrameController.getEditorsInUse());
				pos = usedEditors.indexOf(activeEditorFrame.FrameId);
				if (pos >= 0) {
					pos = (++pos % usedEditors.length);
					WE().layout.weEditorFrameController.showEditor(usedEditors[pos]);
				}
				WE().handler.keyListener.cancelEvent(evt);
				return true;
			case 87://W
			case 115: //F4 (closing a tab)
			case 220://#
				window.focus();  // focus, to avoid a too late onchange of editor
				if (editor || editorType === "cockpit") {
					WE().handler.keyListener.cancelEvent(evt);
					WE().layout.weEditorFrameController.closeDocument(activeEditorFrame.getFrameId());
					return true;
				}
				WE().handler.keyListener.cancelEvent(evt);
				return true;
		}

		return false;
	},
	win: null,
	cancelEvent: function (evt) {
		evt.preventDefault();
		evt.stopPropagation();
	},
	deal: function (evt, win) {
		// This function receives all events, when a key is pressed and forwards the event to
		// the first keyboardlistener ("chain of responsibility")
		if (!WE()) {
			return;
		}
		this.win = win;
		switch (evt.keyCode) {
			/*case -1:
			 WE().handler.keyListener.cancelEvent(evt);
			 return true;*/
			case 17://only Ctrl pressed
				return true;
			case 8://backspace
			case 27: // ESCAPE
			case 13: // ENTER
			case 116: // F5 - works only in FF
				return (evt.ctrlKey || evt.metaKey ?
					true :
					WE().handler.keyListener.plainKey(evt));
			case 45://ins
			case 46://del
			case 67: //C
			case 86: //V
				break;
			default:
				return (evt.ctrlKey || evt.metaKey ?
					WE().handler.keyListener.ctrlKey(evt) : true);
		}
	}
};
