/* global WE, top */

/**
 *
 * $Rev: 10d182ac6c1f $
 * $Author: Lukas Imhof $
 * $Date: 2021-04-19 14:05 +0200 $
 *
 *  * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @package    webEdition_base
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

WE().layout.overlay = function (opener, target, url, ref, w, h) {
	this.opener = opener;
	this.setTarget(target);
	this.url = url;
	/*
	 this.useIframe = useIframe;
	 this.cmd = cmd; // => to populate overlay by some we_cmd instead of iframe and url
	 */

	var maxWidth = this.getMaxWidth(),
		maxHeight = this.getMaxHeight();
	this.w = Math.min(this.scale(maxWidth, 95), w);
	this.h = Math.min(this.scale(maxHeight, 95), (h + 28)); // add 28px for overlay header
	this.x = Math.round((maxWidth - this.w) / 2);
	this.y = Math.round((maxHeight - this.h) / 2);

	this.name = ref;
};

WE().layout.overlay.prototype = {
	adminFrame: null,
	btnClose: null,
	btnMaximize: null,
	btnRestore: null,
	btns: null,
	contentDiv: null,
	contentFrame: null,
	foregroundMask: null,
	h: 0,
	header: null,
	maximized: false,
	name: '',
	overlay: null,
	payload: {},
	resizeable: false,
	title: '',
	cmdClose: '',
	w: 0,
	x: 0,
	y: 0,
	move: {
		offsetX: 0,
		offsetY: 0
	},
	boundFNs: {
		move: null,
		stopMove: null,
		resize: null,
		stopResize: null
	},
	setPayload: function (name, data) {
		this.payload[name] = data;
	},
	getPayload: function (name) {
		return this.payload[name];
	},
	setTarget: function (target) {
		// processe named targets from target paramt
		if (typeof target === 'string') {
			switch (target) {
				case 'top':
					// mainwindow
					if ((target = this.opener.top.document.getElementById('resizeFrame'))) {
						//break;
					}
					// modules
					if (this.opener.top.content && (target = this.opener.top.content/*.document.getElementById('moduleContent')*/)) {
						break;
					}
					target = top;
					break;
				case 'self':
				/* falls through */
				default:
					target = this.opener;
			}
		}

		// set target properties
		if ((this.isTargetDiv = (target.tagName && target.tagName === 'DIV' ? true : false))) {
			this.targetNode = target;
			this.targetWin = target.ownerDocument.defaultView || target.ownerDocument.parentWindow;
		} else {
			this.targetNode = target.document.body;
			this.targetWin = target;
		}

		// if opener is overlay and this.targetNode is nested in opener's targetNode we use opener's targetNode!
		if (WE().layout.isOverlay(this.opener)) {
			var parentOverlay = WE().util.dialogTop(this.opener).overlay;
			var
				parentTargetNode = parentOverlay.targetNode,
				parentTargetWin = parentOverlay.targetWin,
				useParent = false;

			if (this.targetWin === parentTargetWin) {
				useParent = parentTargetNode.contains(this.targetNode);
			} else {
				var cur = this.targetWin;
				while (cur !== cur.parent) {
					cur = cur.parent;
					if (cur === parentTargetWin) {
						useParent = true;
						break;
					}
				}
			}
			if (useParent) {
				this.targetNode = parentTargetNode;
				this.targetWin = parentTargetWin;
				this.isTargetDiv = parentOverlay.isTargetDiv;
			}
		}

		this.targetOffsetY = this.isTargetDiv ? Math.round(this.targetNode.getBoundingClientRect().top) : 0;
	},

	open: function (returnDialog) {
		// FIXME: use vars instead of properties if not referenced from other function
		this.targetNode.numOverlays = this.targetNode.numOverlays ? this.targetNode.numOverlays++ : 1;

		this.backgroundMask = document.createElement("DIV");
		this.backgroundMask.className = 'backgroundMask';
		this.backgroundMask.style.position = this.isTargetDiv ? 'absolute' : 'fixed';
		this.targetNode.appendChild(this.backgroundMask);

		this.overlay = document.createElement("DIV");
		this.overlay.className = 'overlay';
		this.overlay.id = 'overlayDiv';
		this.overlay.name = 'overlayDiv';
		this.overlay.style.position = this.isTargetDiv ? 'absolute' : 'fixed';
		this.overlay.style.top = this.y + 'px';
		this.overlay.style.left = this.x + 'px';
		this.overlay.style.height = this.h + 'px';
		this.overlay.style.width = this.w + 'px';

		this.header = document.createElement("DIV");
		this.header.className = 'header';
		this.header.addEventListener('mousedown', (this.startMove.bind(this)), false);
		this.header.innerHTML = '<span class="title">' + this.title + '</title>';
		this.overlay.appendChild(this.header);

		this.btns = document.createElement("DIV");
		this.btns.className = 'btns';
		this.overlay.appendChild(this.btns);

		if (this.resizable) {
			this.btnMaximize = document.createElement("SPAN");
			this.btnMaximize.className = 'btnMaximize btn';
			this.btnMaximize.innerHTML = '<i class="fa fa-window-maximize fa-lg "></i>';
			this.btnMaximize.addEventListener('mousedown', (this.maximize.bind(this)), true);
			this.btns.appendChild(this.btnMaximize);

			this.btnRestore = document.createElement("SPAN");
			this.btnRestore.className = 'btnRestore btn';
			this.btnRestore.innerHTML = '<i class="fa fa-window-restore fa-lg "></i>';
			this.btnRestore.addEventListener('mousedown', (this.restore.bind(this)), true);
			this.btns.appendChild(this.btnRestore);
		}

		this.btnClose = document.createElement("SPAN");
		this.btnClose.className = 'btnClose btn';
		this.btnClose.innerHTML = '<i class="fa fa-close fa-lg "></i>';
		this.btnClose.addEventListener('mousedown', (this.doClose.bind(this)), true);
		this.btns.appendChild(this.btnClose);

		this.contentDiv = document.createElement("DIV");
		this.contentDiv.className = 'contentDiv';
		this.overlay.appendChild(this.contentDiv);

		if (this.resizable) {
			var elems = ['n-resize', 'e-resize', 's-resize', 'w-resize', 'ne-resize', 'se-resize', 'sw-resize', 'nw-resize'];
			var tmp;
			for (var i = 0; i < elems.length; i++) {
				tmp = document.createElement("DIV");
				tmp.className = 'resize ' + elems[i];
				tmp.addEventListener('mousedown', (this.startResize.bind(this, elems[i])), false);
				this.overlay.appendChild(tmp);
			}
		}

		this.foregroundMask = document.createElement("DIV");
		this.foregroundMask.className = 'foregroundMask';
		this.foregroundMask.style.position = this.isTargetDiv ? 'absolute' : 'fixed';

		this.targetNode.appendChild(this.overlay);
		this.targetOverflow = this.targetNode.style.overflow;
		this.targetNode.style.overflow = 'hidden';

		this.adminFrame = document.createElement('iframe');
		this.adminFrame.className = 'adminFrame';
		this.adminFrame.overlay = this;
		this.contentDiv.appendChild(this.adminFrame);
		this.adminFrame.contentWindow.addEventListener('unload', this.doUnload.bind(this), true);

		this.contentFrame = document.createElement('IFRAME');
		this.contentFrame.className = 'content';
		this.contentFrame.name = this.name;
		this.contentFrame.setAttribute('data-overlayframe', 1);
		this.doLoad();

		if (returnDialog) {
			return this;
		}
	},

	doLoad: function (url) {
		this.url = url || this.url;
		this.contentFrame.src = this.url;
		this.contentDiv.appendChild(this.contentFrame);
		this.contentFrame.contentWindow.opener = this.opener;
		this.contentFrame.contentWindow.toggleFullscreen = this.toggleFullscreen.bind(this);
		this.contentFrame.contentWindow.resizeTo = this.resizeTo.bind(this);
		this.contentFrame.contentWindow.doClose = this.doClose.bind(this);
		this.contentFrame.contentWindow.doLoad = this.doLoad.bind(this);
		this.contentFrame.contentWindow.overlay = this;
	},

	doClose: function (e) {
		if (this.overlay && this.overlay.parentNode) {
			if (e) {
				e.stopPropagation();
			}
			if (this.cmdClose && this.opener && this.opener.we_cmd) {
				var tmp = this.cmdClose.split(',');
				this.cmdClose = '';
				this.opener.we_cmd.apply(this.opener, tmp);
			}
			this.close();
		}
	},

	doUnload: function (e) {
		//e.stopPropagation();
		// not needed
	},

	close: function (e) {
		if (this.overlay && this.overlay.parentNode) {
			this.overlay.parentNode.removeChild(this.overlay);

			this.targetNode.removeChild(this.backgroundMask);
			if (--this.targetNode.numOverlays < 1) {
				this.targetNode.style.overflow = this.targetOverflow;
			}
		}
	},

	maximize: function (e) {
		this.maximized = true;
		this.lastX = this.x;
		this.lastY = this.y;
		this.lastW = this.w;
		this.lastH = this.h;

		this.btnMaximize.style.display = 'none';
		this.btnRestore.style.display = 'inline';

		var width = this.getMaxWidth(true);
		var height = this.getMaxHeight(true);

		this.resizeTo(width, height);
	},

	getMaxWidth: function (fit) {
		return this.isTargetDiv ? this.targetNode.offsetWidth : ((this.targetWin.frameElement ? this.targetWin.frameElement.parentNode.offsetWidth :
			this.targetNode.offsetWidth) - 2);
	},

	// TODO: add resize handler and save maxHeight to this.maxHeight
	getMaxHeight: function (fit) {
		return this.isTargetDiv ? this.targetNode.clientHeight : (this.targetWin.frameElement ? this.targetWin.frameElement.parentNode.offsetHeight :
			(WE().consts.IS_FRONTEND_EDIT ? this.targetWin.innerHeight : (this.targetNode.offsetHeight - 2)));
	},

	scale: function (val, percent) {
		return Math.round((parseInt(val) * parseInt(percent)) / 100);
	},

	restore: function (e) {
		this.maximized = false;
		this.overlay.style.left = (this.x = this.lastX) + 'px';
		this.overlay.style.top = (this.y = this.lastY) + 'px';
		this.overlay.style.width = (this.w = this.lastW) + 'px';
		this.overlay.style.height = (this.h = this.lastH) + 'px';

		this.btnMaximize.style.display = 'inline';
		this.btnRestore.style.display = 'none';
	},

	toggleFullscreen: function () {
		if (this.maximized) {
			this.restore();
		} else {
			this.maximize();
		}
	},

	startMove: function (e) {
		this.header.style.cursor = 'move';
		this.foregroundMask.cursor = 'move';
		this.foregroundMask.addEventListener('mousemove', (this.boundFNs.move = this.moveOverlay.bind(this)), false);
		this.foregroundMask.addEventListener('mouseup', (this.boundFNs.stopMove = this.stopMove.bind(this)), false);
		this.targetNode.appendChild(this.foregroundMask);
		this.move.offsetX = e.clientX - this.x;
		this.move.offsetY = e.clientY - this.y;
	},
	moveOverlay: function (e) {
		this.overlay.style.left = (this.x = (e.clientX - this.move.offsetX)) + 'px';
		this.overlay.style.top = (this.y = (e.clientY - this.move.offsetY)) + 'px';
	},
	stopMove: function (e) {
		this.foregroundMask.removeEventListener('mousemove', this.boundFNs.move, false);
		this.foregroundMask.removeEventListener('mouseup', this.boundFNs.stopMove, false);
		this.foregroundMask.style.cursor = '';
		this.header.style.cursor = '';
		this.targetNode.removeChild(this.foregroundMask);
	},

	startResize: function (pos) {
		this.foregroundMask.style.cursor = pos;
		this.foregroundMask.addEventListener('mousemove', (this.boundFNs[pos] = this.resize.bind(this, pos)), false);
		this.foregroundMask.addEventListener('mouseup', (this.boundFNs.stopResize = this.stopResize.bind(this, pos)), false);
		this.targetNode.appendChild(this.foregroundMask);
	},
	resize: function (pos, e) {
		var
			diffX = e.clientX - this.x,
			diffY = e.clientY - this.y - this.targetOffsetY;

		switch (pos) {
			case 'n-resize':
				this.overlay.style.top = (this.y += diffY) + 'px';
				this.overlay.style.height = (this.h -= diffY) + 'px';
				break;
			case 's-resize':
				this.overlay.style.height = (this.h = diffY) + 'px';
				break;
			case 'w-resize':
				this.overlay.style.left = (this.x += diffX) + 'px';
				this.overlay.style.width = (this.w -= diffX) + 'px';
				break;
			case 'e-resize':
				this.overlay.style.width = (this.w = diffX) + 'px';
				break;
			case 'nw-resize':
				this.overlay.style.top = (this.y += diffY) + 'px';
				this.overlay.style.height = (this.h -= diffY) + 'px';
				this.overlay.style.left = (this.x += diffX) + 'px';
				this.overlay.style.width = (this.w -= diffX) + 'px';
				break;
			case 'sw-resize':
				this.overlay.style.left = (this.x += diffX) + 'px';
				this.overlay.style.width = (this.w -= diffX) + 'px';
				this.overlay.style.height = (this.h = diffY) + 'px';
				break;
			case 'ne-resize':
				this.overlay.style.width = (this.w = diffX) + 'px';
				this.overlay.style.top = (this.y += diffY) + 'px';
				this.overlay.style.height = (this.h -= diffY) + 'px';
				break;
			case 'se-resize':
				this.overlay.style.height = (this.h = diffY) + 'px';
				this.overlay.style.width = (this.w = diffX) + 'px';
				break;
		}


	},
	stopResize: function (pos, e) {
		this.foregroundMask.removeEventListener('mousemove', this.boundFNs[pos], false);
		this.foregroundMask.removeEventListener('mouseup', this.boundFNs.stopResize, false);
		this.foregroundMask.style.cursor = '';
		this.targetNode.removeChild(this.foregroundMask);
	},

	resizeByRatio: function (r_w, r_h) {
		this.resizeTo((this.w * r_w), (this.h * r_h));
	},

	resizeTo: function (w, h) {
		var
			maxWidth = this.getMaxWidth(),
			maxHeight = this.getMaxHeight();

		this.overlay.style.width = (this.w = Math.min(maxWidth, w)) + 'px';
		this.overlay.style.height = (this.h = Math.min(maxHeight, h)) + 'px';
		this.overlay.style.left = (this.x = Math.round((maxWidth - this.w) / 2)) + 'px';
		this.overlay.style.top = (this.y = Math.round((maxHeight - this.h) / 2)) + 'px';
	},
	setTitle: function (title) {
		this.title = title;
		return this;
	},
	setCmdClose: function (cmd) {
		this.cmdClose = cmd;
		return this;
	},
	setResizeable: function () {
		this.resizable = true;
		return this;
	}

};

WE().layout.isOverlay = function (win) {
	var
		frameElement,
		cur = win,
		firstLoop = true;

	while (cur !== cur.parent) {
		cur = (firstLoop) ? cur : cur.parent;
		firstLoop = false;
		if ((frameElement = cur.frameElement) && frameElement.getAttribute('data-overlayframe')) {
			return true;
		}
	}

	return false;
};

WE().layout.getOverlayWindow = function (win) {
	var frameElement;
	var cur = win;
	var firstLoop = true;

	while (cur !== cur.parent) {
		cur = (firstLoop) ? cur : cur.parent;
		firstLoop = false;
		if ((frameElement = cur.frameElement) && frameElement.getAttribute('data-overlayframe')) {
			return cur;
		}
	}

	return null;
};