/* global WE, errorHandler,treeData,drawTree, top */

/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
/**
 * setting is built like the unix file system privileges with the 3 options
 * see notices, see warnings, see errors
 *
 * 1 => see Notices
 * 2 => see Warnings
 * 4 => see Errors
 *
 * @param message string
 * @param prio integer one of the values 1,2,4
 * @param win object reference to the calling window
 */

WE().util.showMessage = function (message, prio, win, timeout) {
	var ab;
	var overlay;
	win = (win ? win : this.window);
	win = (win ? win : window);
	win = (overlay = WE().layout.getOverlayWindow(win)) ? overlay : win.top;

	// default is error, to avoid missing messages
	prio = prio ? prio : WE().consts.message.WE_MESSAGE_ERROR;
	message = message.replace(/\\n|\\\\n/g, '<br/>');
	if (win.$) {
		ab = win.$("#alertBox");
		if (!ab.length) {
			var alertDiv = win.document.createElement('dialog');
			alertDiv.id = "alertBox";
			win.document.body.appendChild(alertDiv);
			ab = win.$("#alertBox");
		}
		if (ab.html()) {//still a present dialog
			var el = ab.get(0);
			if (!el.stack) {
				el.stack = [];
			}
			el.stack.push(['showMessage', Array.prototype.slice.call(arguments)]);
			return;
		}
	}

	// always show in console !
	WE().layout.messageConsole.addMessage(prio, message);
	/*jshint bitwise:false */
	if (prio & WE().session.messageSettings) { // show it, if you should
		var title = "";
		var icon = "";
		switch (prio) {
			// Notice
			case WE().consts.message.WE_MESSAGE_INFO:
			case WE().consts.message.WE_MESSAGE_NOTICE:
				title = WE().consts.g_l.message_reporting.notice;
				break;

				// Warning
			case WE().consts.message.WE_MESSAGE_WARNING:
				title = WE().consts.g_l.message_reporting.warning;
				icon = '<span class="alertIcon fa-stack fa-lg alertIcon" style="color:#007de3;"><i class="fa fa-circle fa-stack-2x" ></i><i class="fa fa-info fa-stack-1x fa-inverse"></i></span>';
				break;

				// Error
			case WE().consts.message.WE_MESSAGE_ERROR:
				title = WE().consts.g_l.message_reporting.error;
				icon = '<span class="alertIcon fa-stack fa-lg alertIcon"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>';
				break;
		}
		//try Jquery
		if (win.top.top.$) {
			ab.html(icon + '<div class="message">' + message.replace(/(\n|\\n)/, "<br/>") + "</div>");
			ab.dialog({
				dialogClass: "no-close",
				modal: true,
				title: title,
				height: "auto",
				maxHeight: 400,
				width: "auto",
				maxWidth: 400,
				closeOnEscape: true,
				buttons: [{
						text: WE().consts.g_l.message_reporting.ok,
						icons: {
							primary: "fa fa-check fa-ok"
						},
						click: function () {
							this.ownerDocument.defaultView.$("#alertBox").dialog("close");
						}
					}],
				open: function () {
					$(this).parent().find('.ui-dialog-buttonpane button:eq(0)').focus();
				},
				close: function () {
					this.innerHTML = "";
					if (this.stack && this.stack.length) {
						var call = this.stack.shift();
						WE().util[call[0]].apply(window, call[1]);
					}
				}
			});
			if (timeout) {
				win.top.top.setTimeout(function (win) {
					win.$("#alertBox").dialog("close");
				}, timeout, win.top.top);
			}
		} else {
			message = title + ":\n" + message;
			win.alert(message);
		}
	}
};