/**
 * webEdition CMS
 *
 * $Rev: e07252ad5aab $
 * $Author: Nils Reibold <reibold@digirezi.de> $
 * $Date: 2021-03-09 20:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/* global WE, top */
'use strict';

function doUnload() {
	WE().layout.jsWindow.prototype.closeAll(window);
}

function selectBranch() {
	var f = document.we_form;
	var txt = f.branch;
	var sel = f.branch_select.options[f.branch_select.selectedIndex].text;
	f.cmd.value = "switchBranch";
	txt.value = sel;
	submitForm();
}

function saveField() {
	document.we_form.cmd.value = "save_field";
	submitForm();
}

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
					args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
					url = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=customer&";

	var branch, field, fields_select;
	switch (args[0]) {
		case "open_add_field":
			branch = document.we_form.branch.value;
			if(branch){
				url +="&pnt=field_editor&art=1&branch=" + branch;
				(new (WE().layout.overlay)(caller, 'top', url, "field_editor", WE().consts.size.dialog.smaller, WE().consts.size.dialog.tiny)).setTitle(WE().consts.g_l.customer.admin.add_field).open();
			} else {
				WE().util.showMessage(WE().consts.g_l.customer.admin.no_branch, WE().consts.message.WE_MESSAGE_ERROR, window);
			}
			break;
		case "open_edit_field":
			field = document.we_form.fields_select.value;
			branch = document.we_form.branch.value;
			if (field === "") {
				WE().util.showMessage(WE().consts.g_l.customer.admin.no_field, WE().consts.message.WE_MESSAGE_ERROR, window);
			} else {
				url += "&pnt=field_editor&art=2&field=" + field + "&branch=" + branch;
				(new (WE().layout.overlay)(caller, 'top', url, "field_editor", WE().consts.size.dialog.smaller, WE().consts.size.dialog.tiny)).setTitle(WE().consts.g_l.customer.admin.edit_field).open();
			}
			break;
		case "delete_field":
			field = document.we_form.fields_select.value;
			if (field === "") {
				WE().util.showMessage(WE().consts.g_l.customer.admin.no_field, WE().consts.message.WE_MESSAGE_ERROR, window);
				break;
			}
			WE().util.showConfirm(window, "", WE().consts.g_l.customer.admin.del_fild_question, ["delete_field_do"]);
			break;
		case "delete_field_do":
			document.we_form.cmd.value = "delete_field";
			submitForm();
			break;
		case "move_field_up":
		case "move_field_down":
			field = document.we_form.fields_select.value;
			branch = document.we_form.branch.value;
			if (field === "") {
				WE().util.showMessage(WE().consts.g_l.customer.admin.no_field, WE().consts.message.WE_MESSAGE_ERROR, window);
				break;
			}
			document.we_form.cmd.value = args[0];
			submitForm();

			break;
		case "add_branch":
			// clear branch select
			document.we_form.branch_select.value = "";
			document.we_form.branch.value = "";
			// clear select fields
			fields_select = document.we_form.fields_select;
			var length = fields_select.length;
			for(var i = length - 1; i >= 0; i--){
				fields_select.remove(i);
			}
			setBranchButtons(true);
			break;
		case "open_edit_branch":
			branch = document.we_form.branch_select.options[document.we_form.branch_select.selectedIndex].text;
			if (branch === "") {
				WE().util.showMessage(WE().consts.g_l.customer.admin.no_branch, WE().consts.message.WE_MESSAGE_ERROR, window);
			} else if (branch === WE().consts.g_l.customer.admin.other) {
				WE().util.showMessage(WE().consts.g_l.customer.admin.branch_no_edit, WE().consts.message.WE_MESSAGE_ERROR, window);
			} else {
				url += "&pnt=branch_editor&art=0&branch=" + branch;
				(new (WE().layout.overlay)(caller, 'top', url, "field_editor", WE().consts.size.dialog.smaller, WE().consts.size.dialog.tiny)).setTitle(WE().consts.g_l.customer.admin.edit_branch).open();
			}
			break;
		case "save_branch":
		case "save_field":
			var field_name = document.we_form.name.value;
			var type = document.we_form.field_type.value;
			var doSave = true;

			// Check Field Validity
			if (field_name === "" || field_name.match(/[^a-zA-Z0-9_]/) !== null) {
				WE().util.showMessage(WE().consts.g_l.customer.admin.we_fieldname_notValid, WE().consts.message.WE_MESSAGE_ERROR, window);
				doSave = false;
			}
			if(["select", "multiselect"].includes(type)){
				var defarr = document.we_form.field_default.value;
				defarr = defarr ? defarr.split(/\r?\n/) : [];
				var valsarr = document.we_form.field_values.value.split(/\r?\n/);

				if(checkIfDuplicateExists(valsarr)){
					WE().util.showMessage(WE().consts.g_l.customer.admin.select_duplicate_exists, WE().consts.message.WE_MESSAGE_ERROR, window);
					doSave = false;
				}

				var isSub = defarr ? defarr.every(function(val){
					return valsarr.indexOf(val) >= 0;
				}) : true;

				if(!isSub){
					WE().util.showMessage(WE().consts.g_l.customer.admin.select_duplicate_exists, WE().consts.message.WE_MESSAGE_ERROR, window);
					doSave = false;
				}

			}
			if(doSave){
				document.we_form.cmd.value = args[0];
				submitForm("_self");
			}
			// Fixme : Handle Field save as an rpc instead and show field in field-select list
			break;
		case 'submitForm':
			if (WE().util.dialogTop(window).opener.top.content && WE().util.dialogTop(window).opener.top.content.editor) { // IMPORTANT: in FF top === top.content is always true!
				submitForm('_self');
				break;
			}
			/* falls through */
		default:
			//FIXME: this js is included in 2 dialogs - this causes problems! e.g. no customer selected
			if (WE().util.dialogTop(window).opener.top.content && WE().util.dialogTop(window).opener.top.content.editor) {
				WE().util.dialogTop(window).opener.top.content.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
			} else {
				WE().util.dialogTop(window).opener.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
			}
	}
}

function fieldTypeChange(type) {

	var field = document.we_form.field.value;
	var branch = document.we_form.branch.value;
	// Store some input already made
	var name = document.getElementsByName('name')[0].value;
	var field_default = document.getElementsByName('field_default')[0].value;


	WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?mod=customer&cmd=customerFieldEditor&field_type=" + type + "&field=" + field + "&branch=" + branch, "", function (weResponse) {
		if (weResponse) {
			if (weResponse.Success === true) {
				document.getElementById('EditorRpc').outerHTML = weResponse.DataArray.data;
				document.getElementsByName('name')[0].value = name;
				// Multiselect Values need a special treatment
				if(type !== 'multiselect'){
					document.getElementsByName('field_default')[0].value = field_default;
				}
				if(["select", "multiselect"].includes(type)){
					document.we_form.field_default.oninput = sanitizeSelectInput;
					document.we_form.field_values.oninput = sanitizeSelectInput;
				}

			}
		}
	});

//this should no longer be needed since we set these attributes in php
	var encSelect = document.getElementsByName('field_encrypt')[0];
	var def = document.getElementsByName('field_default')[0];
	def.removeAttribute("required");

	switch (type) {
		case 'number':
			def.setAttribute('type', 'number');
			break;
		case 'date':
			def.setAttribute('type', 'date');
			break;
		case 'select':
		case 'multiselect':
			def.setAttribute('type', 'text');
			def.setAttribute("required", "required");
			break;
		default:
			def.setAttribute('type', 'text');
			break;
	}
}

function submitForm(target, action, method, form) {
	var f = form ? window.document.forms[form] : window.document.we_form;
	f.target = target ? target : "customer_admin";
	f.action = action ? action : WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=customer";
	f.method = method ? method : "post";

	f.submit();
}

function setFieldsButtons(pos, max) {
	document.getElementById("editFieldButton").disabled = false;
	document.getElementById("deleteFieldButton").disabled = false;
	document.getElementById("moveFieldUpButton").disabled = (pos === 0);
	document.getElementById("moveFieldDownButton").disabled = (pos === (max - 1));
}

function setBranchButtons(newState){
	document.getElementById("addBranchButton").disabled = newState;
	document.getElementById("editBranchButton").disabled = newState;
}

function sanitizeSelectInput(ev){
	this.value = this.value.replace(/,/g, "");
}

// Fixme: is using Set ok?
function checkIfDuplicateExists(w){
	return new Set(w).size !== w.length ;
}