/* global tinymce */
/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

(function () {
	var wequickbar = (function () {
		var global = tinymce.util.Tools.resolve('tinymce.PluginManager');

		var Buttons = {register: function (editor) {
				var commands = editor.getParam('weQuickbarSubcmds');
				var icon, value, text;
				var map = {
					mceWeacronym: 'ACRONYM',
					mceWeabbr: 'ABBR',
					mceCite: 'CITE',
					mceDel: 'DEL',
					mceIns: 'INS',
					mceWelang: 'LANG',
					Underline: 'UNDERLINE',
					Subscript: 'SUB',
					Superscript: 'SUP',
					Strikethrough: 'STRIKETHROUGH'
				};

				if (commands.xhtml) {
					var xhtml = [];
					var tagsXhtml = [];
					if (commands.xhtml.length) {
						for (var i = 0; i < commands.xhtml.length; i++) {
							switch (commands.xhtml[i]) {
								case 'cite':
									value = 'mceCite';
									icon = 'cite';
									tagsXhtml.push('CITE');
									text = WE().consts.g_l.tinyMceTranslationObject.tt_cite;
									break;
								case 'weabbr':
									value = 'mceWeabbr';
									icon = 'abbr';
									tagsXhtml.push('ABBR');
									text = WE().consts.g_l.tinyMceTranslationObject.tt_weabbr;
									break;
								case 'weacronym':
									value = 'mceWeacronym';
									icon = 'acronym';
									tagsXhtml.push('ACRONYM');
									text = WE().consts.g_l.tinyMceTranslationObject.tt_weacronym;
									break;
								case 'welang':
									value = 'mceWelang';
									icon = 'language';
									tagsXhtml.push('SPAN');
									text = WE().consts.g_l.tinyMceTranslationObject.tt_welang;
									break;
								case 'del':
									value = 'mceDel';
									icon = 'del';
									tagsXhtml.push('DEL');
									text = WE().consts.g_l.tinyMceTranslationObject.tt_del;
									break;
								case 'ins':
									value = 'mceIns';
									icon = 'ins';
									tagsXhtml.push('INS');
									text = WE().consts.g_l.tinyMceTranslationObject.tt_ins;
									break;
							}
							xhtml.push({
								type: 'choiceitem',
								value: value,
								icon: icon,
								text: text
							});
						}
					}
					editor.ui.registry.addSplitButton('morexhtml', {
						type: 'splitbutton',
						icon: 'more',
						tooltip: 'xhtml_' + editor.getParam('id'),
						columns: commands.xhtml.length > 4 ? 3 : commands.xhtml.length,
						fetch: function (callback) {
							callback(xhtml);
						},
						onAction: function () {
							// try to open menu!
						},
						onItemAction: function (bla, value) {
							editor.execCommand(value);
						},
						onSetup: function (buttonApi) {
							var split = editor.getParam('weWin').document.querySelector('[title="xhtml_' + editor.getParam('id') + '"]');
							split.title = 'mehr spezielle Formatierungen';
							split.removeChild(split.firstChild);
							split.firstChild.classList.add('wequickbar_chevron');

							var n = editor.dom.getParent(editor.selection.getNode(), tagsXhtml.join(','));
							if (n && (n.nodeName.toLowerCase() !== 'span')) {
								split.firstChild.classList.add('wequickbar_chevron_active');
							}
						},
						select: function (value) {
							var tag = map[value];
							var n = editor.selection.getNode();

							switch (tag) {
								case 'LANG':
									// FIXME: we need to bubble up to find outer span with lang!
									n = editor.dom.getParent(n, 'SPAN');
									return n && n.getAttribute('lang') && n.getAttribute('lang') !== '';
								default:
									n = editor.dom.getParent(n, tag);
									return n && n.nodeName.toUpperCase() === tag;
							}
						}
					});
				}

				if (commands.format) {
					var
						format = [],
						isUnderline = false,
						isStrikeThrough = false,
						tagsFormat = [];
					if (commands.format.length) {
						for (var i = 0; i < commands.format.length; i++) {
							switch (commands.format[i]) {
								case 'underline':
									value = 'Underline';
									icon = 'underline';
									tagsFormat.push('SPAN');
									isUnderline = true;
									break;
								case 'superscript':
									value = 'Superscript';
									icon = 'superscript';
									tagsFormat.push('SUP');
									break;
								case 'subscript':
									value = 'Subscript';
									icon = 'subscript';
									tagsFormat.push('SUB');
									break;
								case 'strikethrough':
									value = 'Strikethrough';
									icon = 'strike-through';
									tagsFormat.push('SPAN');
									isStrikeThrough = true;
									break;
							}
							format.push({
								type: 'choiceitem',
								value: value,
								icon: icon
							});
						}
					}

					editor.ui.registry.addSplitButton('moreformat', {
						type: 'splitbutton',
						icon: 'more',
						tooltip: 'format_' + editor.getParam('id'),
						columns: commands.format.length > 4 ? 3 : commands.format.length,
						fetch: function (callback) {
							callback(format);
						},
						onAction: function () {
							//editor.execCommand('Underline');
							// try to open menu!
						},
						onItemAction: function (bla, value) {
							editor.execCommand(value);
						},
						onSetup: function (buttonApi) {
							var split = editor.getParam('weWin').document.querySelector('[title="format_' + editor.getParam('id') + '"]');
							split.title = 'weitere Formatierungen';
							split.removeChild(split.firstChild);
							split.firstChild.classList.add('wequickbar_chevron');

							var n = editor.dom.getParent(editor.selection.getNode(), tagsFormat.join(','));
							if (n && (n.nodeName.toLowerCase() !== 'span' ||
								// FIXME: we need to bubble up to find outer span with line-through or undeline!
									(isStrikeThrough && n.getAttribute('style') && n.getAttribute('style').indexOf('line-through') !== -1) ||
									(isUnderline && n.getAttribute('style') && n.getAttribute('style').indexOf('underline') !== -1)
									)) {
								split.firstChild.classList.add('wequickbar_chevron_active');
							}
						},
						select: function (value) {
							var tag = map[value];
							var n = editor.selection.getNode();

							switch (tag) {
								case 'STRIKETHROUGH':
									// FIXME: we need to bubble up to find outer span with line-through or underline!
									n = editor.dom.getParent(n, 'SPAN');
									return n && n.getAttribute('style') && n.getAttribute('style').indexOf('line-through') !== -1;
								case 'UNDERLINE':
									// FIXME: we need to bubble up to find outer span with line-through!
									n = editor.dom.getParent(n, 'SPAN');
									return n && n.getAttribute('style') && n.getAttribute('style').indexOf('underline') !== -1;
								default:
									n = editor.dom.getParent(n, tag);
									return n && n.nodeName.toUpperCase() === tag;
							}
						}
					});
				}
			}};

		global.add('wequickbar', function (editor) {
			Buttons.register(editor);
			return {};
		});

		function Plugin() {}

		return Plugin;
	}());
})();