/**
 * webEdition CMS
 *
 * $Rev: 60ffe81902c8 $
 * $Author: Lukas Imhof $
 * $Date: 2021-02-26 18:31 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

var WeAbbrAcronymDialog = {
	tinymcePlugin: window.tinyMCEPopup,
	sel: '',
	editor: null,
	elm: '',
	isTag: false,
	tagName: '',
	mceClass: '',

	init: function () {
		switch ((this.tagName = this.tinymcePlugin.params.tag)) {
			case 'abbr':
				this.mceClass = 'mceItemWeAbbr';
				break;
			case 'acronym':
				this.mceClass = 'mceItemWeAcronym';
				break;
		}

		this.editor = this.tinymcePlugin.editor;
		this.elm = this.editor.selection.getNode();
		this.sel = this.editor.selection.getContent({format: 'text'});

		var langValue = '';
		var titleValue = '';

		if (this.sel === '') {
			this.sel = this.elm.innerHTML;
			this.isTag = true;
		} else {
			if (this.elm.nodeName === this.tagName.toUpperCase() && this.sel.trim() === this.elm.innerHTML.trim()) {
				this.isTag = true;
			}
		}

		if (this.isTag) {
			langValue = this.elm.getAttribute('lang') ? this.elm.getAttribute('lang') : '';
			titleValue = this.elm.getAttribute('title') ? this.elm.getAttribute('title') : '';
		}

		document.forms.we_form.elements['we_dialog_args[lang]'].value = langValue;
		document.forms.we_form.elements['we_dialog_args[title]'].value = titleValue;
		if(document.forms.we_form.elements.text){
			document.forms.we_form.elements.text.value = this.sel; //Selected Text to insert into glossary
		}
	},

	insert: function () {
		var langValue = document.forms.we_form.elements['we_dialog_args[lang]'].value;
		var titleValue = document.forms.we_form.elements['we_dialog_args[title]'].value;

		if (this.isTag) {
			if (titleValue !== '') {
				this.editor.selection.getNode().setAttribute('title', titleValue);
				if (langValue !== '') {
					this.editor.selection.getNode().setAttribute('lang', langValue);
				} else {
					this.editor.selection.getNode().removeAttribute('lang');
				}
			} else {
				this.editor.dom.remove(this.editor.selection.getNode(), 1);
			}
		} else {
			if (titleValue !== '') {
				var blank = '';
				var isBlank = false;
				while (this.sel.charAt(this.sel.length - 1) === ' ') {
					this.sel = this.sel.substr(0, this.sel.length - 1);
					isBlank = true;
					blank += '&nbsp;';
				}
				blank = isBlank ? blank.substr(0, blank.length - 6) + ' ' : blank;

				var visual = this.editor.settings.weHasVisual ? ' class="' + this.mceClass + '"' : '';
				var content = '<' + this.tagName + ' lang="' + langValue + '" title="' + titleValue + '"' + visual + '>' + this.sel + '</' + this.tagName + '>' + blank;
				this.editor.execCommand('mceInsertContent', false, content);
			}
		}
	}
};

function weTinyDialog_doOk() {
	WeAbbrAcronymDialog.insert();
	WE().util.dialogTop(window).doClose();
}

//window.tinyMCEPopup.onInit.add(WeacronymDialog.init, WeacronymDialog);
$(function () {
	WeAbbrAcronymDialog.init();
});