/* global tinyMCEPopup, tinymce,top, WE, tinyMCE */

/**
 * webEdition CMS
 *
 * $Rev: 6510d0038c12 $
 * $Author: Lukas Imhof $
 * $Date: 2021-03-26 12:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

if (WE().consts.IS_FRONTEND_EDIT) {
	var consts = WE().util.getDynamicVar(document, 'loadVarWeTinyMce_config', 'data-consts');

	WE().consts.g_l.tinyMceTranslationObject = consts.g_l.tinyMceTranslationObject;
	WE().consts.tables.FILE_TABLE = consts.tables.FILE_TABLE;
	WE().consts.tables.OBJECT_FILES_TABLE = consts.tables.OBJECT_FILES_TABLE;
	WE().consts.dirs.WE_JS_TINYMCE_DIR = consts.dirs.WE_JS_TINYMCE_DIR;
	WE().consts.linkPrefix.TYPE_INT_PREFIX = consts.linkPrefix.TYPE_INT_PREFIX;
	WE().consts.linkPrefix.TYPE_OBJ_PREFIX = consts.linkPrefix.TYPE_OBJ_PREFIX;
}

WE().layout.we_tinyMCE = WE().layout.we_tinyMCE ? WE().layout.we_tinyMCE : {
	'do': {},
	functions: {},
	setup: {},
	vars: {},
	overwrite: {}
};

function TinyMceConfObject(args) {
	//top.console.log('args', args);
	this.weIsFrontend = args.weIsFrontend;
	this.weEditorType = args.weEditorType;
	this.weName = args.weName;
	this.weFieldName = args.weFieldName;
	this.weFieldNameClean = args.weFieldNameClean;
	this.weContentCssParts = args.weContentCssParts;
	this.weWin = args.weWin;
	this.weEditarea = null;
	this.weIsFullscreen = false;
	this.weQuickbarSubcmds = args.weQuickbarSubcmds;
	this.weIsStyleSelectOpen = false;
	this.weIsFormatSelectOpen = false;
	this.weIsFontsizeSelectOpen = false;

	this.weProcessedStyles = [];
	this.weProcessedFormats = {
		byFormat: {},
		byName: {},
		formats: []
	};

	//this.weContextmenuCommands = args.weContextmenuCommands;
	//this.weDialogProperties = args.weDialogProperties;
	this.weImageStartID = args.weImageStartID;

	this.weGalleryTemplates = args.weGalleryTemplates;
	this.weRemoveFirstParagraph = args.weRemoveFirstParagraph;

	this.weTinyParams = args.weTinyParams;
	this.wePluginClasses = args.wePluginClasses;

	this.weSynchronizeHot = {
		doSyncHot: true,
		isEditorHot: false
	};
	this.weColorIcon = {
		textColor: {
			color: 'black',
			stroke: 'gray',
			fill: 'none',
			height: 5,
			y: 22
		},
		backColor: {
			char: 'black',
			stroke: 'gray',
			fill: 'white'
		}
	};

	this.doctype = '<!DOCTYPE html>';
	this.language = args.language;
	this.selector = 'textarea.' + args.weSelectorClass;
	//this.force_br_newlines = true;
	//this.forced_root_block = '';
	this.branding = false;
	//this.skin = "oxide";

	this.width = args.size.width;
	this.height = args.size.height;
	this.menubar = false;

	this.plugins = args.pluginsInt + ',quickbars';
	this.external_plugins = args.pluginsExt;

//	this.weBlockImportCss = args.weBlockImportCss;
	// set some controls
	//ed.execCommand("mceWevisualaid", true);
	this.weSelects = args.weSelects;
	this.suppertedBlockformats = ['p', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'pre', 'address'];
	if (this.weSelects.fontsizeselect.formatted.length) {
		this.fontsize_formats = this.weSelects.fontsizeselect.formatted.join(' ');
	}
	if (this.weSelects.blockselect.formatted.length) {
		this.block_formats = this.weSelects.blockselect.formatted.join(';');
	}
	this.style_formats = this.weSelects.styleselect.formatted.length ? this.weSelects.styleselect.formatted : [];
	//
	//preview_styles => could be interesting
	this.style_formats_merge = false;
	this.style_formats_autohide = true;

	this.importcss_append = true;
	this.importcss_file_filter = ''; // to be filled in setup
	this.importcss_merge_classes = true;
	// custom node filters

	//this.toolbar_sticky = true; // what's that?

	//this.toolbar_drawer = "sliding";
	this.accessibility_warnings = false; // checked
	// add listeners for events ed has no public events for

	this.table_toolbar = args.table_toolbar;
	this.table_class_list = args.table_class_list;
	this.table_row_class_list = args.table_row_class_list;
	this.table_cell_class_list = args.table_cell_class_list;
	this.table_tab_navigation = true;
	this.table_appearance_options = true;

	this.table_style_by_css = false;
	this.table_default_styles = {};
	this.table_default_attributes = {};
	this.table_header_type = 'sectionCells';
	//this.table_advtab = false;

	this.table_grid = true;
	this.visual = false;
	// FIXME: replace copy/cut by keyDown
	this.paste_as_text = true;
	this.paste_data_images = false;
	this.paste_filter_drop = false;
	//this.paste_enable_default_filters = false;
	//this.paste_word_valid_elements = "b,strong,i,em,h1,h2";

	this.toolbar = args.toolbar;
	this.toolbar_drawer = false; //false | "sliding" | "floating"
//this.contextmenu = args.toolbar;
//this.contextmenu_never_use_native = true;
	this.contextmenu = false;
	this.nonbreaking_wrap = false;

	this.quickbars_selection_toolbar = args.quickbars_selection_toolbar;
	this.quickbars_insert_toolbar = false;
//this.inline = true;
//this.selector = 'div.' + args.weSelectorClass;
	// when not inlineTrue: get content from opener document => move to fn

	// overwrite some tiny core functions

	// call custom init if there: we_tinyMCE_FIELDNAME_init(ed)

	this.relative_urls = false;
	this.body_class = (args.weClassName ? args.weClassName + ' ' : '') + 'wetextarea tiny-wetextarea wetextarea-' + args.weOrigName;


	/*
	 this.convert_urls = false; //important!
	 this.force_p_newlines = 0; // value 0 instead of true (!) prevents adding additional lines with <p>&nbsp</p> when inlineedit="true"
	 this.entity_encoding = 'named';
	 this.entities = '160,nbsp';
	 this.fix_list_elements = true;
	 this.element_format = args.element_format;
	 */

	this.extended_valid_elements = '@[we-tiny],we-gallery[id|tmpl|class]';
	this.custom_elements = 'we-gallery';
	this.paste_auto_cleanup_on_paste = true;
	this.templates = args.templates;
	//this.force_br_newlines = true;
	//this.forced_root_block = '';


	this.codesample_dialog_width = 600;
	this.codesample_languages = [
		{text: 'HTML/XML', value: 'markup'},
		{text: 'JavaScript', value: 'javascript'},
		{text: 'CSS', value: 'css'},
		{text: 'PHP', value: 'php'}
	];

	this.icons = 'weicons';

	this.setup = WE().layout.we_tinyMCE.setup.setupEditor;
}

WE().layout.we_tinyMCE.getTinyConfObject = function (args) {
	if (true) {
		return new TinyMceConfObject(args);
	} else {
		// static test config
		return {
			selector: 'textarea.' + args.weSelectorClass,
			toolbar: 'anchor | styleselect | visualaid | visualblocks',
			plugins: 'anchor visualblocks',
			menubar: false,
			/*
			 formats: [
			 {title: 'Red cell', selector: 'td', classes: 'red'},
			 {title: 'Red text', inline: 'span', styles: {color: '#ff0000'}}
			 ]
			 */
			formats: {
				// Changes the default format for h1 to have a class of heading
				h1: {block: 'h1', classes: 'heading'}
			},
			style_formats: [
				// Adds the h1 format defined above to style_formats
				{title: 'My heading', format: 'h1'}
			]
		};
	}

};
